import { log } from '../../utils';
import type { S3Event } from 'aws-lambda';
import { config } from '../../config';
import type { AthenaInteractionEventTypeResponse } from '../interactions';
import { getQueryMetadata } from '../utils';
import * as mongodb from '../../utils/mongodb';
import { ReconciliationTypes } from '../types';
import { handleInteractionQuery } from './handleInteractionQuery';
import { handleTaskDataQuery } from './handleTaskDataQuery';
import type { AthenaTaskDataEventTypeResponse } from '../actuals';
import type { AthenaWorkstreamMetricsEventTypeResponse } from '../workstream/AthenaWorkstreamMetricEventTypeResponse';
import { handleWorkstreamMetricsQuery } from './handleWorkstreamMetricsQuery';
import { handleV2InteractionQuery } from './handleV2InteractionQuery';
import { AthenaExpress } from 'athena-express-plus';
import { Athena } from '@aws-sdk/client-athena';
import { S3 } from '@aws-sdk/client-s3';

export const doHandleAthenaS3Events = async (event: S3Event): Promise<void> => {
    if (!config.athena.db) throw new Error('Athena DB not specified in config');
    log.info('Processing record - doHandleAthenaS3Events', { event });

    let fileKey = '';
    event.Records.map((record) => (fileKey = record.s3.object.key));
    if (!fileKey.endsWith('metadata')) {
        return;
    }

    await mongodb.init(config.db);

    fileKey = fileKey.replace(/\.[^.]+/g, '');

    log.info(`Running query - doHandleAthenaS3Events`, { fileKey });

    // Get metadata for the query
    const queryMetadata = await getQueryMetadata({ queryExecutionId: fileKey });
    log.debug(`Query metadata results - doHandleAthenaS3Events`, { fileKey, queryMetadata });

    if (queryMetadata.metricType === ReconciliationTypes.TaskData) {
        const athenaExpress = new AthenaExpress<AthenaTaskDataEventTypeResponse>({
            s3: new S3({ region: config.aws.region }),
            athena: new Athena({ region: config.aws.region }),
            s3Bucket: `s3://${config.athena.athenaQueriesBucket}/`,
            db: config.athena.db,
            waitForResults: false,
        });

        const results = await athenaExpress.query(fileKey);

        log.debug(`Found athena results for ${queryMetadata.metricType} - doHandleAthenaS3Events`, { results });
        if (results.Items) await handleTaskDataQuery(results, queryMetadata, event);
    } else if (queryMetadata.metricType === ReconciliationTypes.Interaction) {
        const athenaExpress = new AthenaExpress<AthenaInteractionEventTypeResponse>({
            s3: new S3({ region: config.aws.region }),
            athena: new Athena({ region: config.aws.region }),
            s3Bucket: `s3://${config.athena.athenaQueriesBucket}/`,
            db: config.athena.db,
            waitForResults: false,
        });

        const results = await athenaExpress.query(fileKey);

        log.debug(`Found athena results for ${queryMetadata.metricType} - doHandleAthenaS3Events`, { results });
        // Get list of modified interactions
        if (results.Items) await handleInteractionQuery(results, queryMetadata, event);
    } else if (queryMetadata.metricType === ReconciliationTypes.Workstream) {
        const athenaExpress = new AthenaExpress<AthenaWorkstreamMetricsEventTypeResponse>({
            s3: new S3({ region: config.aws.region }),
            athena: new Athena({ region: config.aws.region }),
            s3Bucket: `s3://${config.athena.athenaQueriesBucket}/`,
            db: config.athena.db,
            waitForResults: false,
        });

        const results = await athenaExpress.query(fileKey);

        // Get list of modified workstreams
        if (results.Items) await handleWorkstreamMetricsQuery(results, queryMetadata);
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    } else if (queryMetadata.metricType === ReconciliationTypes.V2Interaction) {
        const athenaExpress = new AthenaExpress<AthenaInteractionEventTypeResponse>({
            s3: new S3({ region: config.aws.region }),
            athena: new Athena({ region: config.aws.region }),
            s3Bucket: `s3://${config.athena.athenaQueriesBucket}/`,
            db: config.athena.db,
            waitForResults: false,
        });

        const results = await athenaExpress.query(fileKey);

        log.debug(`Found athena results for ${queryMetadata.metricType} - doHandleAthenaS3Events`, { results });
        // Get list of modified interactions
        if (results.Items) await handleV2InteractionQuery(results, queryMetadata, event);
    }

    return;
};
