import type { AthenaInteractionEventTypeResponse } from '../interactions';
import {
    convertAthenaInteractionEventTypeResponseToInteractionEventType,
    deleteInteractionsFromMongo,
} from '../interactions';
import { updateColumnsToNum } from '../utils';
import { convertUpsertsToBulkWrite, log } from '../../utils';
import type { InteractionEventParams, MongoQueryWithFilter } from '../../metrics';
import calculateInteractionCreatedDiff from '../../metrics/interactions/calculateInteractionCreatedDiff';
import type { MongoInteractionMetric } from '@agyletime/performance-utils';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import type { S3Event } from 'aws-lambda';
import UserHistoryProvider from '../../userHistory/UserHistoryProvider';
import type { ReconciliationTypes } from '../types';
import { QueryResult } from 'athena-express-plus';

export async function handleInteractionQuery(
    results: QueryResult<AthenaInteractionEventTypeResponse>,
    queryMetadata: {
        queryExecutionId: string;
        organisationId: number;
        fromDate: Date;
        toDate: Date;
        metricType: ReconciliationTypes;
    },
    event: S3Event,
): Promise<void> {
    const interactionEvents = results.Items?.map((result) => {
        updateColumnsToNum(result);
        return convertAthenaInteractionEventTypeResponseToInteractionEventType(result);
    });

    // TODO: reduce the list of interactions down to only those that will change TPM data
    // Get timezones for users
    const uniqueOrgIds = interactionEvents
        ?.map((interaction) => interaction.organisationId)
        .filter((value: number, index: number, array: number[]) => array.indexOf(value) === index)
        .map((n) => Number(n));

    if (!uniqueOrgIds) return;

    const userHistory = await UserHistoryProvider.getUserHistoryForOrgIds({
        organisationIds: uniqueOrgIds,
        timestamp: new Date(),
    });

    const usersWithMostUpToDateHistory = await UserHistoryProvider.getMostUpToDateHistory({
        organisationIds: uniqueOrgIds,
    });

    // Delete TPM data
    const workstreamIds = [...new Set(interactionEvents?.map((interaction) => interaction.workstreamIdentifier))];

    // TODO: only delete data that is changing
    await deleteInteractionsFromMongo({
        queryMetadata,
        workstreamIds,
    });

    // Re-upsert TPM data
    const upserts: MongoQueryWithFilter<MongoInteractionMetric>[] = [];

    log.debug('Iterating over interactions', { interactionEvents });

    for (const interaction of interactionEvents ?? []) {
        try {
            const cachedUser =
                userHistory.find((user) => user.userId === interaction.agent?.wfmId) ??
                usersWithMostUpToDateHistory.find((user) => user.userId === interaction.agent?.wfmId);

            if (!cachedUser?.timezone)
                throw new Error(`Cached timezone not found for user id: ${interaction.agent?.wfmId}`);
            if (!cachedUser.teamId)
                throw new Error(`Cached team id not found for user id: ${interaction.agent?.wfmId}`);

            if (!interaction.agent) throw new Error(`Interaction has no agent: ${JSON.stringify(interaction)}`);
            if (!interaction.agent.wfmId) throw new Error(`Interaction has no user id: ${JSON.stringify(interaction)}`);

            interaction.agent.teamId = cachedUser.teamId;
            interaction.agent.timezone = cachedUser.timezone;

            log.debug(`Looking for users - doHandleAthenaS3Events`, {
                organisationId: interaction.organisationId,
                agent: interaction.agent,
                wfmIdType: typeof interaction.agent.wfmId,
            });

            const params: InteractionEventParams = {
                organisationId: interaction.organisationId,
                userId: interaction.agent.wfmId,
                teamId: interaction.agent.teamId,
                interactionType: interaction.type,
                workstreamIdentifier: interaction.workstreamIdentifier,
                workstreamDefinitionId: interaction.workstreamDefinitionId,
                workstreamDefinitionName:
                    interaction.workstreamDefinition?.name ??
                    `Unnamed Workstream (${interaction.workstreamDefinitionId})`,
                timezone: interaction.agent.timezone,
                taskTypeId: interaction.workloadTaskId,
            };

            upserts.push(calculateInteractionCreatedDiff({ interaction, params }));
        } catch (error: unknown) {
            log.error(`Failed to process S3 Reconciliation Event: ${(error as Error).message}`, { event });
        }
    }

    if (upserts.length > 0) {
        log.debug('Upserting metrics', { upserts });
        await MongoInteractionMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));
    }
}
