import { updateColumnsToNum } from '../utils';
import type { UserCacheType } from '../../userCache/UserCacheModel';
import { UserCacheModel } from '../../userCache/UserCacheModel';
import { isOrgEnabled, log } from '../../utils';
import { type MongoActualMetric, MongoActualMetricModel } from '@agyletime/performance-utils';
import type { S3Event } from 'aws-lambda';
import type { ReconciliationTypes } from '../types';
import type { AthenaTaskDataEventTypeResponse } from '../actuals';
import { convertAthenaTaskDataEventTypeResponseToTaskDataEventType, deleteActualsFromMongo } from '../actuals';
import { TaskDefinitionModel } from '@agyletime/performance-utils';
import { GenericActualMetricsError } from '../../errors';
import type { TaskDefinitionMongoSchema } from '@agyletime/wfm-types';
import calculateActualCreateDiff from '../../metrics/actuals/calculateActualCreateDiff';
import type { TaskDataMergedEvent } from '../../metrics';
import type { ReplaceOneModel } from 'mongoose';
import type { QueryResult } from 'athena-express-plus';

export async function handleTaskDataQuery(
    results: QueryResult<AthenaTaskDataEventTypeResponse>,
    queryMetadata: {
        queryExecutionId: string;
        organisationId: number;
        fromDate: Date;
        toDate: Date;
        metricType: ReconciliationTypes;
    },
    event: S3Event,
): Promise<void> {
    // 1. Get user cache for all relevant orgs.
    // 2. Get task definition cache for all relevant orgs.
    // 3. As we retrieve tasks, need to parse that back into shifts.
    // 4. Pump these shifts into a code similar to taskDataMergedHandler. There will be slight differences as we won't look up the mongo caches every time.

    // Re-organise tasks into their shifts
    const taskDataMergedEventMap = new Map<string, TaskDataMergedEvent>();
    const uniqueOrgIds = new Set<number>();
    const orgToUserMap = new Map<number, number[]>();
    const orgToTaskDefMap = new Map<number, number[]>();

    // Fill the maps and sets
    results.Items?.forEach((result) => {
        // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
        const shift: TaskDataMergedEvent = taskDataMergedEventMap.get(result.shiftid) || {
            tasks: [],
            shiftId: result.shiftid,
            userId: Number(result.userid),
        };
        updateColumnsToNum(result); // FIXME: This mutates
        const taskDataMerged = convertAthenaTaskDataEventTypeResponseToTaskDataEventType(result);
        shift.tasks.push(taskDataMerged);
        taskDataMergedEventMap.set(shift.shiftId, shift);
        uniqueOrgIds.add(taskDataMerged.organisationId);
        // Create org id to user id map
        // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
        const usersInOrg = orgToUserMap.get(taskDataMerged.organisationId) || [];
        usersInOrg.push(taskDataMerged.userId);
        orgToUserMap.set(taskDataMerged.organisationId, [...new Set<number>(usersInOrg)]);
        // Create org id to task def id map
        // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
        const taskDefsInOrg = orgToTaskDefMap.get(taskDataMerged.organisationId) || [];
        taskDefsInOrg.push(taskDataMerged.taskDefinitionId);
        orgToTaskDefMap.set(taskDataMerged.organisationId, [...new Set<number>(taskDefsInOrg)]);
    });

    // At this point this is close the structure of how these tasks were processed as events

    // Only get users that are in this batch of interactions
    const userLookup = new Map<string, UserCacheType>(
        (
            await UserCacheModel.aggregate<UserCacheType>([
                {
                    $match: {
                        $or: Array.from(orgToUserMap).map(([k, v]) => ({
                            organisationId: k,
                            id: { $in: v },
                        })),
                    },
                },
                {
                    $project: {
                        _id: 0,
                        id: 1,
                        timezone: 1,
                        organisationId: 1,
                        team: 1,
                        integrations: 1,
                    },
                },
            ])
        ).map((u) => [`${u.organisationId}-${u.id}`, u]),
    );

    log.debug(`User cached retrieved - doHandleAthenaS3Events`, { userLookup });
    log.debug('orgToTaskDefMap', { orgToTaskDefMap });

    const taskDefinitionsLookup = await TaskDefinitionModel.find({
        $or: Array.from(orgToTaskDefMap).map(([k, v]) => ({
            organisationId: k,
            id: { $in: v },
        })),
    }).lean();

    log.debug(`taskDefinitionsLookup`, { taskDefinitionsLookup });

    // Delete TPM data
    const shiftIds = Array.from(taskDataMergedEventMap.keys());
    await deleteActualsFromMongo({
        queryMetadata,
        shiftIds,
    });

    // Re-upsert TPM data
    const upserts: ReplaceOneModel<MongoActualMetric>[] = [];

    log.debug('Iterating over taskDataEvents', {
        taskDataMergedEventMap: Array.from(taskDataMergedEventMap.values()),
    });

    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    for (const rehydratedEvent of taskDataMergedEventMap.values() ?? []) {
        try {
            const organisationId = rehydratedEvent.tasks[0]?.organisationId;
            if (!organisationId)
                throw new GenericActualMetricsError(
                    `No organisation id ${organisationId} found in task data from Athena`,
                );
            const userFromCache = userLookup.get(`${organisationId}-${rehydratedEvent.userId}`);
            if (!userFromCache)
                throw new GenericActualMetricsError(
                    `Failed to find user id: ${rehydratedEvent.userId} from org: ${organisationId} in cache`,
                );

            const timezone = userFromCache.timezone;
            const teamId = userFromCache.team?.id;

            if (!timezone)
                throw new GenericActualMetricsError(
                    `User id: ${rehydratedEvent.userId} from org: ${organisationId} does not have a timezone in the cache`,
                );

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(organisationId)) return;

            const taskDefinitionsMap = new Map<number, TaskDefinitionMongoSchema>(
                taskDefinitionsLookup.filter((td) => td.organisationId === organisationId).map((td) => [td.id, td]),
            );
            upserts.push(
                calculateActualCreateDiff({
                    shift: rehydratedEvent,
                    params: {
                        organisationId,
                        userId: rehydratedEvent.userId,
                        shiftId: rehydratedEvent.shiftId,
                        ...(teamId && { teamId }),
                        taskDefinitionsMap,
                        timezone,
                    },
                }),
            );
        } catch (error: unknown) {
            log.error(`Failed to process S3 Reconciliation Event: ${(error as Error).message}`, { event });
        }
    }

    if (upserts.length > 0) {
        log.debug('Upserting metrics', { upserts });
        await MongoActualMetricModel.bulkWrite(
            upserts.map((upsert) => {
                return {
                    replaceOne: {
                        filter: upsert.filter,
                        upsert: true,
                        replacement: upsert.replacement,
                    },
                };
            }),
        );
    }
}
