import type { AthenaInteractionEventTypeResponse } from '../interactions';
import { convertAthenaInteractionEventTypeResponseToInteractionEventType } from '../interactions';
import type { ReconciliationTypes } from '../types';
import type { S3Event } from 'aws-lambda';
import { updateColumnsToNum } from '../utils';
import { deleteV2InteractionsFromMongo } from '../interactions/deleteV2InteractionsFromMongo';
import UserHistoryProvider from '../../userHistory/UserHistoryProvider';
import { convertV2UpdateToBulkWrite, log } from '../../utils';
import { V2MongoInteractionMetric, V2MongoInteractionMetricModel } from '@agyletime/performance-utils';
import type { UpdateOneModel } from 'mongoose';
import type { InteractionEventParams } from '../../metrics';
import calculateV2InteractionCreatedDiff from '../../metrics/interactions/calculateV2InteractionCreatedDiff';
import type { QueryResult } from 'athena-express-plus';

export async function handleV2InteractionQuery(
    results: QueryResult<AthenaInteractionEventTypeResponse>,
    queryMetadata: {
        queryExecutionId: string;
        organisationId: number;
        fromDate: Date;
        toDate: Date;
        metricType: ReconciliationTypes;
    },
    event: S3Event,
): Promise<void> {
    const interactionEvents = results.Items?.map((result) => {
        updateColumnsToNum(result);
        return convertAthenaInteractionEventTypeResponseToInteractionEventType(result);
    });

    // Get timezones for users
    const uniqueOrgIds = interactionEvents
        ?.map((interaction) => interaction.organisationId)
        .filter((value: number, index: number, array: number[]) => array.indexOf(value) === index)
        .map((n) => Number(n));

    if (!uniqueOrgIds) return;

    const userHistory = await UserHistoryProvider.getUserHistoryForOrgIds({
        organisationIds: uniqueOrgIds,
        timestamp: new Date(),
    });

    const usersWithMostUpToDateHistory = await UserHistoryProvider.getMostUpToDateHistory({
        organisationIds: uniqueOrgIds,
    });

    await deleteV2InteractionsFromMongo({ queryMetadata });

    const upserts: UpdateOneModel<V2MongoInteractionMetric>[] = [];

    for (const interaction of interactionEvents ?? []) {
        try {
            const cachedUser =
                userHistory.find((user) => user.userId === interaction.agent?.wfmId) ??
                usersWithMostUpToDateHistory.find((user) => user.userId === interaction.agent?.wfmId);

            if (!cachedUser?.timezone)
                throw new Error(`Cached timezone not found for user id: ${interaction.agent?.wfmId}`);
            if (!cachedUser.teamId)
                throw new Error(`Cached team id not found for user id: ${interaction.agent?.wfmId}`);

            if (!interaction.agent) throw new Error(`Interaction has no agent: ${JSON.stringify(interaction)}`);
            if (!interaction.agent.wfmId) throw new Error(`Interaction has no user id: ${JSON.stringify(interaction)}`);

            interaction.agent.teamId = cachedUser.teamId;
            interaction.agent.timezone = cachedUser.timezone;

            log.debug(`Looking for users - doHandleAthenaS3Events`, {
                organisationId: interaction.organisationId,
                agent: interaction.agent,
                wfmIdType: typeof interaction.agent.wfmId,
            });

            const params: InteractionEventParams = {
                organisationId: interaction.organisationId,
                userId: interaction.agent.wfmId,
                teamId: interaction.agent.teamId,
                interactionType: interaction.type,
                workstreamIdentifier: interaction.workstreamIdentifier,
                workstreamDefinitionId: interaction.workstreamDefinitionId,
                workstreamDefinitionName:
                    interaction.workstreamDefinition?.name ??
                    `Unnamed Workstream (${interaction.workstreamDefinitionId})`,
                timezone: interaction.agent.timezone,
                taskTypeId: interaction.workloadTaskId,
            };

            upserts.push(calculateV2InteractionCreatedDiff({ interaction, params }));
        } catch (error: unknown) {
            log.error(`Failed to process S3 Reconciliation Event: ${(error as Error).message}`, { event });
        }
    }

    if (upserts.length > 0) {
        log.debug('Upserting metrics', { upserts });
        await V2MongoInteractionMetricModel.bulkWrite(convertV2UpdateToBulkWrite(upserts));
    }
}
