import { updateColumnsToNum } from '../utils';
import { convertUpsertsToBulkWrite, log } from '../../utils';
import type { MongoQueryWithFilter, WorkstreamMetricEventParams } from '../../metrics';
import type { MongoWorkstreamMetric } from '@agyletime/performance-utils';
import { MongoWorkstreamMetricModel } from '@agyletime/performance-utils';
import UserHistoryProvider from '../../userHistory/UserHistoryProvider';
import type { ReconciliationTypes } from '../types';
import type { AthenaWorkstreamMetricsEventTypeResponse } from '../workstream/AthenaWorkstreamMetricEventTypeResponse';
import { convertAthenaWorkstreamMetricEventTypeResponseToWorkstreamMetricsEventType } from '../workstream/convertAthenaWorkstreamMetricEventTypeResponseToWorkstreamMetricsEventType';
import { deleteWorkstreamMetricsFromMongo } from '../workstream/deleteWorkstreamMetricsFromMongo';
import calculateFirstSolveWorkstreamMetricCreatedDiff from '../../metrics/workstream/calculateFirstSolveWorkstreamMetricCreatedDiff';
import { QueryResult } from 'athena-express-plus';

export async function handleWorkstreamMetricsQuery(
    results: QueryResult<AthenaWorkstreamMetricsEventTypeResponse>,
    queryMetadata: {
        queryExecutionId: string;
        organisationId: number;
        fromDate: Date;
        toDate: Date;
        metricType: ReconciliationTypes;
    },
): Promise<void> {
    const workstreamMetricsEvents = results.Items?.map((result) => {
        updateColumnsToNum(result);
        return convertAthenaWorkstreamMetricEventTypeResponseToWorkstreamMetricsEventType(result);
    });

    // TODO: reduce the list of workstreams down to only those that will change TPM data
    // Get timezones for users
    const uniqueOrgIds = workstreamMetricsEvents
        ?.map((workstream) => workstream.organisationId)
        .filter((value: number, index: number, array: number[]) => array.indexOf(value) === index)
        .map((n) => Number(n));

    if (!uniqueOrgIds) return;

    const userHistory = await UserHistoryProvider.getUserHistoryForOrgIds({
        organisationIds: uniqueOrgIds,
        timestamp: new Date(),
    });

    const usersWithMostUpToDateHistory = await UserHistoryProvider.getMostUpToDateHistory({
        organisationIds: uniqueOrgIds,
    });

    // Delete TPM data
    const workstreamIds = [...new Set(workstreamMetricsEvents?.map((workstream) => workstream.workstreamIdentifier))];

    // TODO: only delete data that is changing
    await deleteWorkstreamMetricsFromMongo({
        queryMetadata,
        workstreamIds,
    });

    // Re-upsert TPM data
    const upserts: MongoQueryWithFilter<MongoWorkstreamMetric>[] = [];

    log.debug('Iterating over workstreams', { workstreamMetricsEvents });

    for (const workstream of workstreamMetricsEvents ?? []) {
        try {
            const cachedUser =
                userHistory.find((user) => user.userId === workstream.firstResolvedAgentAlias.wfmId) ??
                usersWithMostUpToDateHistory.find((user) => user.userId === workstream.firstResolvedAgentAlias.wfmId);

            if (!cachedUser?.timezone)
                throw new Error(`Cached timezone not found for user id: ${workstream.firstResolvedAgentAlias.wfmId}`);
            if (!cachedUser.teamId)
                throw new Error(`Cached team id not found for user id: ${workstream.firstResolvedAgentAlias.wfmId}`);

            workstream.firstResolvedAgentAlias.teamId = cachedUser.teamId;
            workstream.firstResolvedAgentAlias.timezone = cachedUser.timezone;

            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (!workstream.firstResolvedAgentAlias)
                throw new Error(`Workstream has no first solved agent: ${JSON.stringify(workstream)}`);
            if (!workstream.firstResolvedAgentAlias.wfmId)
                throw new Error(`Workstream has no first solved user id: ${JSON.stringify(workstream)}`);

            if (!workstream.firstResolutionTime)
                throw new Error(`Workstream has no first resolution time: ${JSON.stringify(workstream)}`);

            log.debug(`Looking for users - doHandleAthenaS3Events`, {
                organisationId: workstream.organisationId,
                agent: workstream.firstResolvedAgentAlias,
                wfmIdType: typeof workstream.firstResolvedAgentAlias.wfmId,
            });

            const params: WorkstreamMetricEventParams = {
                organisationId: workstream.organisationId,
                workstreamIdentifier: workstream.workstreamIdentifier,
                workstreamDefinitionId: workstream.workstreamDefinitionId,
                workstreamDefinitionName:
                    workstream.workstreamDefinition?.name ??
                    `Unnamed Workstream (${workstream.workstreamDefinitionId})`,
            };

            upserts.push(await calculateFirstSolveWorkstreamMetricCreatedDiff({ workstream, params }));
        } catch (error: unknown) {
            log.error(`Failed to process S3 Reconciliation Event: ${(error as Error).message}`, { workstream });
        }
    }

    if (upserts.length > 0) {
        log.debug('Upserting metrics', { upserts });
        await MongoWorkstreamMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));
    }
}
