import * as mongodb from '../utils/mongodb';
import { config } from '../config';
import type { AthenaQueryMetadata } from './AthenaQueryMetadataModel';
import { AthenaQueryMetadataModel } from './AthenaQueryMetadataModel';
import type { AthenaQueryParams } from './types';
import type { AthenaQueryResponse } from './types';
import { log } from '../utils';
import { ReconciliationTypes } from './types';
import { AthenaExpress } from 'athena-express-plus';
import { Athena } from '@aws-sdk/client-athena';
import { S3 } from '@aws-sdk/client-s3';

export function generateAthenaQuery(
    { organisationId, fromDate, toDate }: AthenaQueryParams,
    type: ReconciliationTypes,
): string {
    let query = '';
    if (type === ReconciliationTypes.Interaction) {
        query = `
            SELECT *
            FROM "mongo_interactions"
            WHERE "organisation_id" = ${organisationId}
            AND "workstreamidentifier" IN (
                SELECT DISTINCT "workstreamidentifier"
                FROM "mongo_interactions" 
                WHERE "organisation_id" = ${organisationId} 
                AND "starttime" BETWEEN from_iso8601_timestamp('${fromDate.toISOString()}') AND from_iso8601_timestamp('${toDate.toISOString()}')
            )
        `;
    } else if (type === ReconciliationTypes.V2Interaction) {
        query = `
            SELECT *
            FROM "mongo_interactions"
            WHERE "organisation_id" = ${organisationId}
            AND "starttime" BETWEEN from_iso8601_timestamp('${fromDate.toISOString()}') AND from_iso8601_timestamp('${toDate.toISOString()}')
        `;
    } else if (type === ReconciliationTypes.TaskData) {
        query = `SELECT *
                FROM "mongo_taskdatas"
                WHERE "organisation_id" = ${organisationId} 
                AND "starttime" BETWEEN from_iso8601_timestamp('${fromDate.toISOString()}') AND from_iso8601_timestamp('${toDate.toISOString()}')`;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    } else if (type === ReconciliationTypes.Workstream) {
        query = `
            SELECT *
            FROM "mongo_workstreammetrics"
            WHERE "organisationid" = ${organisationId}
            AND "firstresolutiontime" BETWEEN from_iso8601_timestamp('${fromDate.toISOString()}') AND from_iso8601_timestamp('${toDate.toISOString()}')
        `;
    }

    log.debug('Generated athena query', { query });

    return query;
}

export const getQueryMetadata = async ({
    queryExecutionId,
}: {
    queryExecutionId: string;
}): Promise<AthenaQueryMetadata> => {
    await mongodb.init(config.db);

    const result = await AthenaQueryMetadataModel.findOne<AthenaQueryMetadata>({
        queryExecutionId: queryExecutionId,
    }).lean();

    if (!result) throw new Error(`AthenaQueryMetadata query not found for queryExecutionId: ${queryExecutionId}`);

    return result;
};

export const initiateAthenaQuery = async (
    params: AthenaQueryParams,
    metricType: ReconciliationTypes,
): Promise<AthenaQueryResponse> => {
    if (!config.athena.db) throw new Error('Athena DB not specified in config');

    const athenaExpress = new AthenaExpress({
        s3: new S3({ region: config.aws.region }),
        athena: new Athena({ region: config.aws.region }),
        s3Bucket: `s3://${config.athena.athenaQueriesBucket}/`,
        db: config.athena.db,
        waitForResults: false,
    });
    const query = generateAthenaQuery(params, metricType);
    const results = await athenaExpress.query(query);

    return {
        queryExecutionId: results.QueryExecutionId,
    };
};

export const runAthenaQuery = async (
    params: AthenaQueryParams,
    metricType: ReconciliationTypes,
): Promise<AthenaQueryResponse> => {
    // Initiate the athena query
    const { queryExecutionId } = await initiateAthenaQuery({ ...params }, metricType);

    // Store metadata
    await saveQueryMetadata({
        ...params,
        queryExecutionId,
        metricType,
    });

    // Return the athena request id
    return {
        queryExecutionId: queryExecutionId,
    };
};

export const saveQueryMetadata = async (params: AthenaQueryMetadata): Promise<void> => {
    await mongodb.init(config.db);

    await AthenaQueryMetadataModel.create({ ...params });
};

/**
 * Converts all bigints in an object to ints at the objects root level
 * @param {object} row - Object with bigints in it
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export const updateColumnsToNum = (row: any): any => {
    for (const property in row) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        if (typeof row[property] === 'bigint') {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            row[property] = Number(row[property]);
        }
        // file deepcode ignore UseNumberIsNan: Usage is legitamate. We are testing things that looks like a number, instead of is exactly a number - kinda like == vs ===
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-argument
        else if (!isNaN(row[property])) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-argument
            row[property] = parseInt(row[property]);
        }
        // Everything else leave as is (especially date as string)
    }
};
