import type { ActualFilterParams, TaskDataMergedEvent } from '../types';
import { toZonedTime } from 'date-fns-tz';
import { isAfter, isBefore, parseISO } from 'date-fns';
import { getDurationOfTaskMergedTask } from '../utils';
import { log } from '../../utils';
import type { FilterQuery, ReplaceOneModel } from 'mongoose';
import type { MongoActualMetric } from '@agyletime/performance-utils/lib/mongo/models/ActualMetricModel';
import type { WithoutId } from 'mongodb';

export default ({
    shift,
    params,
}: {
    shift: TaskDataMergedEvent;
    params: ActualFilterParams;
}): ReplaceOneModel<MongoActualMetric> => {
    const shiftTasks = [];
    let earliestShiftTaskAsLocal, earliestShiftTaskAsUtc, latestShiftTaskAsLocal, latestShiftTaskAsUtc;

    for (const shiftTask of shift.tasks) {
        const taskDefinition = params.taskDefinitionsMap.get(shiftTask.taskDefinitionId);

        if (!taskDefinition) {
            log.error(`Task definition: ${shiftTask.taskDefinitionId} not found for org: ${params.organisationId}`);
            continue;
        }

        if (taskDefinition.isLogout) continue;

        const taskStartTimeAsLocal = toZonedTime(parseISO(shiftTask.startTime), params.timezone);
        const taskStartTimeAsUtc = parseISO(shiftTask.startTime);

        const taskEndTimeAsLocal = shiftTask.endTime ? toZonedTime(parseISO(shiftTask.endTime), params.timezone) : null;
        const taskEndTimeAsUtc = shiftTask.endTime ? parseISO(shiftTask.endTime) : null;

        if (!earliestShiftTaskAsLocal || isBefore(taskStartTimeAsLocal, earliestShiftTaskAsLocal)) {
            earliestShiftTaskAsLocal = taskStartTimeAsLocal;
            earliestShiftTaskAsUtc = taskStartTimeAsUtc;
        }

        if (!latestShiftTaskAsLocal || (taskEndTimeAsLocal && isAfter(taskEndTimeAsLocal, latestShiftTaskAsLocal))) {
            latestShiftTaskAsLocal = taskEndTimeAsLocal;
            latestShiftTaskAsUtc = taskEndTimeAsUtc;
        }

        const duration = getDurationOfTaskMergedTask(shiftTask);
        const isProductive = taskDefinition.isProductive && !taskDefinition.isLogout;
        const isShrinkage = !taskDefinition.isProductive && !taskDefinition.isLeave;
        // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
        const isProductiveOrShrinkage = isProductive || isShrinkage;

        shiftTasks.push({
            taskStartTimeAsLocal,
            taskStartTimeAsUtc,
            ...(taskEndTimeAsLocal && {
                taskEndTimeAsLocal,
            }),
            ...(taskEndTimeAsUtc && {
                taskEndTimeAsUtc,
            }),
            actualAfterCallWorkTime_actualHours:
                taskDefinition.isAfterCallWork && isProductiveOrShrinkage ? duration : 0,
            plannedTaskDuration_actualHours: taskDefinition.isPlanned ? duration : 0,
            unplannedTaskDuration_actualHours: !taskDefinition.isPlanned && isProductiveOrShrinkage ? duration : 0,
            actualPaidShrinkageDuration_actualHours: taskDefinition.isPaid && isShrinkage ? duration : 0,
            actualPaidTasks_actualHours: taskDefinition.isPaid && isProductiveOrShrinkage ? duration : 0,
            actualProductiveTasks_actualHours: taskDefinition.isProductive ? duration : 0,
            actualShrinkageDuration_actualHours: isShrinkage ? duration : 0,
            totalActualTasks_actualHours: isProductiveOrShrinkage ? duration : 0,
            actualAvailableTime_actualHours:
                taskDefinition.isAvailableOccupancy && isProductiveOrShrinkage ? duration : 0,
            ...(shiftTask.startTime && { actualStartTime_actualHours: parseISO(shiftTask.startTime) }),
            ...(shiftTask.endTime && { actualEndTime_actualHours: parseISO(shiftTask.endTime) }),
        });
    }

    const filter: FilterQuery<MongoActualMetric> = {
        organisationId: params.organisationId,
        shiftId: params.shiftId,
        ...(params.teamId && { teamId: params.teamId }),
        userId: params.userId,
    };
    const replacement: WithoutId<MongoActualMetric> = {
        ...filter,

        ...(earliestShiftTaskAsLocal && {
            earliestShiftTaskAsLocal,
        }),

        ...(earliestShiftTaskAsUtc && {
            earliestShiftTaskAsUtc,
        }),

        ...(latestShiftTaskAsLocal && {
            latestShiftTaskAsLocal,
        }),

        ...(latestShiftTaskAsUtc && {
            latestShiftTaskAsUtc,
        }),

        // @ts-expect-error type doesn't match
        shiftTasks,
    };

    return {
        filter,
        replacement,
    };
};
