import { config } from '../../../config';
import { log } from '../../../utils';
import type { TaskDataMergedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { PlayvoxCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { TaskDefinitionModel } from '@agyletime/performance-utils';
import { MongoActualMetricModel } from '@agyletime/performance-utils';
import type { TaskDefinitionMongoSchema } from '@agyletime/wfm-types/lib/schedule/taskDefinition';
import calculateActualCreateDiff from '../calculateActualCreateDiff';
import getUserFromCache from '../../../utils/getUserFromCache';
import { BadRequestError, GenericActualMetricsError } from '../../../errors';
import type { KinesisStreamEvent } from 'aws-lambda';
import { isOrgEnabled } from '../../../utils';
import type { MongoActualMetric } from '@agyletime/performance-utils/lib/mongo/models/ActualMetricModel';

export const taskDataMergedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const replacements = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            log.debug('actuals -> taskDataMerged event ', record);

            const event = decodeKinesisPayload<PlayvoxCloudEvent<TaskDataMergedEvent>>({
                data: record.kinesis.data,
            });
            log.debug('actuals -> taskDataMerged decodedPayload', { decodedPayload: event });

            const { data } = event;
            log.debug('actuals -> taskDataMerged data', { data });
            log.info(`actuals -> taskDataMerged number of tasks: ${data.tasks.length}`);

            const organisationId = parseInt(event.plvxwfmid.toString());

            const userFromCache = await getUserFromCache({
                organisationId,
                userId: data.userId,
            });

            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (!userFromCache)
                throw new GenericActualMetricsError(
                    `Failed to find user id: ${data.userId} from org: ${organisationId} in cache`,
                );

            const timezone = userFromCache.timezone;
            const teamId = userFromCache.team?.id;

            if (!timezone)
                throw new GenericActualMetricsError(
                    `User id: ${data.userId} from org: ${organisationId} does not have a timezone in the cache`,
                );

            const taskDefinitionsMap = await getTaskDefinitionMap({
                organisationId,
                tasks: data.tasks,
            });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(organisationId)) return;

            replacements.push(
                calculateActualCreateDiff({
                    shift: data,
                    params: {
                        organisationId,
                        userId: data.userId,
                        shiftId: data.shiftId,
                        ...(teamId && { teamId }),
                        taskDefinitionsMap,
                        timezone,
                    },
                }),
            );
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (replacements.length > 0)
        await MongoActualMetricModel.bulkWrite<MongoActualMetric>(
            replacements.map((replacement) => {
                return {
                    replaceOne: {
                        filter: replacement.filter,
                        upsert: true,
                        replacement: replacement.replacement,
                    },
                };
            }),
        );

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};

const getTaskDefinitionMap = async ({
    organisationId,
    tasks,
}: {
    organisationId: number;
    tasks: TaskDataMergedEvent['tasks'];
}): Promise<Map<number, TaskDefinitionMongoSchema>> => {
    const definitions = await TaskDefinitionModel.find({
        organisationId,
        id: { $in: tasks.map((t) => t.taskDefinitionId) },
    }).lean();
    return new Map(definitions.map((d) => [d.id, d]));
};
