import type { AnyBulkWriteOperation, BulkWriteResult } from 'mongodb';
import { ObjectId } from 'mongodb';
import { ControlLogModel } from '../../../models';
import { createMongoConnection, log } from '../../../utils';
import { fetchMySqlViews } from './queries/fetchMySqlViews';
import { config } from '../../../config';
import type { IntermediateWorkstreamMetricsModel } from '@agyletime/performance-utils';
import { getSSMParametersById } from '@agyletime/wfm-aws-utils';
import type { Connection } from 'mysql';
import * as dbConnection from '@agyletime/wfm-rds-utils/lib/dbConnection';
import type { AgyleTimeIdentity } from '@agyletime/wfm-aws-utils/lib/types/events';

/**
 * The fields that are passed in from processInboundSQSEventRecords.invoke
 */
export interface SyncPerformanceViewsDto {
    organisationId: number;
    eventType?: string;
    eventSource?: string;
    eventData: {
        fromDate?: string;
        toDate?: string;
    };
    identity?: AgyleTimeIdentity;
}

export const doSyncPerformanceViews = async ({ organisationId, eventData }: SyncPerformanceViewsDto): Promise<void> => {
    const jobStartedAt = new Date();
    log.debug('doSyncPerformanceViews: Extra logging is enabled');
    const ssmParams = await getSSMParametersById(organisationId);
    if (!ssmParams) {
        log.warn('No SSM Params for Organisation', { organisationId });
        throw new Error('No SSM Params for Organisation');
    }

    if (!ssmParams.rds) {
        log.warn('No RDS Config for Organisation', { organisationId });
        throw new Error('No RDS Config for Organisation');
    }

    const db = await createMongoConnection('performance');
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (!db?.database) {
        throw new Error('Failed to connect to MongoDB');
    }

    const connection: Connection = await dbConnection.connect({
        host: ssmParams.rds.host_ro ?? ssmParams.rds.host,
        user: ssmParams.rds.user,
        password: ssmParams.rds.password,
        database: 'workactivity',
    });

    const fetchStartedAt = new Date();
    const intermediateMetrics = await fetchMySqlViews({
        connection,
        organisationId: organisationId,
        fromDate: eventData.fromDate,
        toDate: eventData.toDate,
    });
    const fetchFinishedAt = new Date();

    log.debug(`Received ${intermediateMetrics.length} mysql items`, intermediateMetrics);

    await db.collections.agent_workstream_metrics.createIndex({
        'user.id': 1,
        date: 1,
        identifier: 1,
        organisationId: 1,
    });

    await db.collections.agent_workstream_metrics.createIndex({
        organisationId: 1,
        'user.id': 1,
        'team.id': 1,
        rosteredShiftStartTime: -1,
    });

    const write: AnyBulkWriteOperation<typeof IntermediateWorkstreamMetricsModel>[] = intermediateMetrics.map((row) => {
        delete row._id;
        return {
            updateOne: {
                filter: {
                    'user.id': row.user.id,
                    date: row.date.toString(),
                    identifier: row.workstreamData.identifier,
                    organisationId,
                },
                update: {
                    $setOnInsert: {
                        _id: new ObjectId(),
                        location: config.db.location,
                        organisationId,
                    },
                    $set: {
                        ...row,
                        date: row.date.toString(),
                        updatedAt: jobStartedAt,
                    },
                },
                upsert: true,
            },
        };
    });

    let result: BulkWriteResult | undefined;
    let failed: unknown;
    let errors = 0;

    if (write.length) {
        log.debug(`Writing ${write.length} items.`, {
            organisationId,
            bulkWriteActions: write,
        });

        try {
            // @ts-expect-error TODO: Turned up strictness. Need to fix these.
            result = await db.collections.agent_workstream_metrics.bulkWrite(write);
            // @ts-expect-error TODO: Turned up strictness. Need to fix these.
            errors = result.getWriteErrorCount() || 0;
        } catch (e: unknown) {
            log.debug('Encountered an error while bulkWriting', { e });
            failed = e;
            errors = 1;
        }
    } else {
        log.debug('Nothing to write.');
    }

    try {
        const controlLog = new ControlLogModel({
            organisationId,
            jobStartedAt,
            jobFinishedAt: new Date(),
            fetchStartedAt,
            fetchFinishedAt,
            successes: failed ? 0 : write.length - errors,
            errors,
            totalItems: write.length,
        }).toDocument();

        await db.collections.agent_workstream_control_log.insertOne(controlLog);
    } catch (err: unknown) {
        log.error('Failed to insert control log', { err });
        throw err;
    }
    log.info('Completed synchronisation.', { organisationId });
};
