import { JSend, extractIdentity, typedJSONParse, formatAsJSend } from '@agyletime/wfm-aws-utils';
import { log } from '../../../../utils';
import { doAddSyncPerformanceViewsJobsToQueue } from '../doAddSyncPerformanceViewsJobsToQueue';
import moment from 'moment';
import * as Errors from '../../../../errors';
import { getSSMParameters } from '@agyletime/wfm-aws-utils';
import { AgentMetricsApiV1, isSuccess } from '@agyletime/wfm-client';
import type { SQSHandler } from 'aws-lambda';
import { APIGatewayProxyEvent } from 'aws-lambda/trigger/api-gateway-proxy';
import { config } from '../../../../config';

export const publishSyncPerformanceMetricViewsHttp = async (event: APIGatewayProxyEvent): Promise<JSend> => {
    try {
        const organisationId = extractIdentity(event).organisationId;
        if (!config.performance.syncPerformanceViewsQueueUrl) {
            throw new Errors.QueueUrlNotDefinedError();
        }

        if (typeof event.body !== 'string') {
            throw new Errors.BadRequestError('Bad event');
        }

        const body = typedJSONParse<{ fromDate?: string; toDate?: string }>(event.body);
        if (Object.keys(body).length < 2) {
            throw new Errors.BadRequestError('Need to pass in fromDate and toDate');
        }

        const fromDate = moment(body.fromDate).toISOString();
        const toDate = moment(body.toDate).toISOString();

        const result = await doAddSyncPerformanceViewsJobsToQueue({
            queueUrl: config.performance.syncPerformanceViewsQueueUrl,
            organisationId,
            fromDate,
            toDate,
        });

        return formatAsJSend({ result }, 200);
    } catch (error: unknown) {
        log.warn(`Failed triggering job`, error);
        throw error;
    }
};

interface OrgEnabledStatus {
    orgId: number;
    orgName: string;
    workstreamMetricsEnabled: boolean;
}

export const publishSyncPerformanceMetricViews: SQSHandler = async () => {
    try {
        const ssmParams = await getSSMParameters();
        const enabledStatusPromises: Promise<OrgEnabledStatus>[] = [];
        for (const ssm of ssmParams) {
            if (!ssm.api) {
                log.warn('Customer is missing API configuration', { organisationId: ssm.id });
                continue;
            }

            if (!ssm.rds || !(ssm.rds.host || ssm.rds.host_ro) || !ssm.rds.password) {
                log.warn('Customer is missing RDS configuration', { organisationId: ssm.id });
                continue;
            }

            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (ssm.api) {
                enabledStatusPromises.push(
                    AgentMetricsApiV1.fromCredentials(ssm.api)
                        .getWorkstreamMetricsEnabled()
                        .then((response) => {
                            if (isSuccess(response)) {
                                const workstreamMetricsEnabled = response.data.workstreamMetricsEnabled;
                                return {
                                    orgId: ssm.id,
                                    orgName: ssm.name,
                                    workstreamMetricsEnabled,
                                };
                            }
                            return {
                                orgId: ssm.id,
                                orgName: ssm.name,
                                workstreamMetricsEnabled: false,
                            };
                        }),
                );
            }
        }

        if (!config.performance.syncPerformanceViewsQueueUrl) throw new Errors.QueueUrlNotDefinedError();

        const enabledStatusResults = await Promise.allSettled(enabledStatusPromises);
        const syncPromises: Promise<string>[] = [];
        enabledStatusResults.forEach((c) => {
            if (c.status === 'fulfilled' && c.value.workstreamMetricsEnabled) {
                const fromDate = moment().subtract(1, 'day').toISOString();
                const toDate = moment().toISOString();

                syncPromises.push(
                    doAddSyncPerformanceViewsJobsToQueue({
                        queueUrl: config.performance.syncPerformanceViewsQueueUrl,
                        organisationId: c.value.orgId,
                        fromDate,
                        toDate,
                    }),
                );
            }
        });
        const resolutions = await Promise.allSettled(syncPromises);
        const syncResults: PromiseFulfilledResult<string>[] = [];
        for (const resolution of resolutions) {
            if (resolution.status === 'rejected') {
                log.warn('Failed to run sync job', { job: resolution });
                continue;
            }
            syncResults.push(resolution);
        }
    } catch (error: unknown) {
        log.warn(`Failed triggering job`, error);
        throw error;
    }
};
