import { log } from '../../../../utils';
import * as dbConnection from '@agyletime/wfm-rds-utils/lib/dbConnection';
import type { ObjectId } from 'mongodb';
import { typedJSONParse } from '@agyletime/wfm-aws-utils';
import type { Connection } from 'mysql';

interface FetchMySqlViewsParams {
    connection: Connection;
    organisationId: number;
    fromDate?: string | undefined;
    toDate?: string | undefined;
}

interface IntermediateWorkstreamMetricsDbQueryResult {
    user: string;
    team: string;
    workstreamData: string;
    workstreamDefinition: string;
    rosteredShift: string;
    interactions: string;
    date: Date;
    rosteredShiftStartTime: string;
    csatTotalScore: number;
    csatTotalResponses: number;
    csatTotalPositiveResponses: number;
}

interface RosteredShiftStub {
    id: string;
    mongoShiftId: string;
    startTime: string;
    endTime: string;
}

type TagsArray = { text: string }[];

interface UserStub {
    id: number;
    name: string;
    tags: string;
    email: string;
    roles?: { id: number; name: string }[];
}

interface TeamStub {
    id: number;
    name: string;
    mongoId: string;
    parentMongoId?: string;
}

interface WorkstreamDefinitionStub {
    id: number;
    identifier: string;
    name: string;
    channel: string;
}

interface OriginalWorkstreamStub {
    identifier: string;
    startTime: string;
    closedTime: string;
    firstReplyHandleTimeSecs: number;
    firstResolutionHandleTimeSecs: number;
    direction: 'Inbound' | 'Outbound' | undefined;
    isHandled: 0 | 1;
    isSolved: 0 | 1;
}

interface WorkstreamStub {
    identifier: string;
    startTime: string;
    closedTime: string;
    firstReplyHandleTimeSecs: number;
    firstResolutionHandleTimeSecs: number;
    direction: 'Inbound' | 'Outbound' | undefined;
    isHandled: boolean;
    isSolved: boolean;
}

interface OriginalInteractionStub {
    type: string;
    identifier: string;
    startTime: string;
    endTime: string;

    workEffortSecs: number;
    talkTimeSecs: number;
    consultationTimeSecs: number;
    wrapTimeSecs: number;
    holdTimeSecs: number;
    waitTimeSecs: number;

    completionStatus: string;
    contactIdentifier: string;
    statusChangeFrom: string;
    statusChangeTo: string;

    isHandle: 0 | 1;
    isPublicComment: 0 | 1;
    isPrivateComment: 0 | 1;
    isStatusChange: 0 | 1;
    isSolve: 0 | 1;
    isReassign: 0 | 1;
    isCallback: 0 | 1;
    isOverflowed: 0 | 1;
    isTransfer: 0 | 1;
    isVoicemail: 0 | 1;
    isFirstHandle: 0 | 1;
}

interface InteractionStub {
    type: string;
    identifier: string;
    startTime: string;
    endTime: string;

    workEffortSecs: number;
    talkTimeSecs: number;
    consultationTimeSecs: number;
    wrapTimeSecs: number;
    holdTimeSecs: number;
    waitTimeSecs: number;

    completionStatus: string;
    contactIdentifier: string;
    statusChangeFrom: string;
    statusChangeTo: string;

    isHandle: boolean;
    isPublicComment: boolean;
    isPrivateComment: boolean;
    isStatusChange: boolean;
    isSolve: boolean;
    isReassign: boolean;
    isCallback: boolean;
    isOverflowed: boolean;
    isTransfer: boolean;
    isVoicemail: boolean;
    isFirstHandle: boolean;
}

interface SqlViewsResult {
    _id?: ObjectId;
    user: UserStub;
    team: TeamStub;
    parent?: TeamStub | undefined;
    workstreamData: WorkstreamStub;
    workstreamDefinition: WorkstreamDefinitionStub;
    interactions: InteractionStub[];
    rosteredShift: RosteredShiftStub;
    date: string;
    rosteredShiftStartTime: Date;
    csatTotalScore: number;
    csatTotalResponses: number;
    csatTotalPositiveResponses: number;
}

interface TeamsResult {
    id: number;
    name: string;
    mongo_id: string;
    parent_mongo_id: string;
    // Excluding unused fields
}

const sql = `
    SELECT JSON_OBJECT(
                   'id', u.id,
                   'name', CONCAT(u.first_name, ' ', u.last_name),
                   'email', u.email,
                   'roles', cast((CONCAT(
                    '[',
                    GROUP_CONCAT(DISTINCT
                                 JSON_OBJECT(
                                         'id', r.id,
                                         'name', r.name
                                     )
                        ),
                    ']'
                )
                ) as JSON),
                   'tags', u.tags
               )                                               AS user,
           JSON_OBJECT(
                   'id', t.id,
                   'mongoId', t.mongo_id,
                   'parentMongoId', t.parent_mongo_id,
                   'name', t.name
               )                                               AS team,
           JSON_OBJECT(
                   'identifier', wd.identifier,
                   'startTime', wd.start_time,
                   'closedTime', wd.closed_time,
                   'isHandled', IFNULL(wd.first_reply_time > wd.start_time, false),
                   'firstReplyHandleTimeSecs', wd.first_reply_handle_time_secs,
                   'isSolved', wd.first_resolution_time IS NOT NULL,
                   'firstResolutionHandleTimeSecs', wd.first_resolution_handle_time_secs,
                   'direction', LCASE(wd.direction)
               )                                               AS workstreamData,
           JSON_OBJECT(
                   'id', w.id,
                   'identifier', w.identifier,
                   'name', w.name,
                   'channel', w.channel
               )                                               AS workstreamDefinition,
           JSON_OBJECT(
                   'id', rs.id,
                   'mongoShiftId', rs.mongo_shift_id,
                   'startTime', rs.rostered_start_time,
                   'endTime', rs.rostered_end_time
               )                                               AS rosteredShift,
           cast(CONCAT(
                   '[',
                   GROUP_CONCAT(
                           DISTINCT JSON_OBJECT(
                           'identifier', wda.identifier,
                           'type', LCASE(wda.type),
                           'startTime', wda.start_time,
                           'endTime', wda.end_time,
                           'talkTimeSecs', IFNULL(wda.talk_time_secs, false),
                           'consultationTimeSecs', IFNULL(wda.consultation_time_secs, false),
                           'workEffortSecs', COALESCE(wda.work_effort_secs, 0),
                           'wrapTimeSecs', IFNULL(wda.wrap_time_secs, false),
                           'holdTimeSecs', IFNULL(wda.hold_time_secs, false),
                           'waitTimeSecs', IFNULL(wda.wait_time_secs, false),
                           'completionStatus', LCASE(wda.completion_status),
                           'contactIdentifier', wda.contact_identifier,
                           'statusChangeFrom', LCASE(wda.status_change_from),
                           'statusChangeTo', LCASE(wda.status_change_to),
                           'isHandle', IFNULL(wda.work_effort_secs > 0, false),
                           'isPublicComment', IFNULL(wda.is_public_comment, false),
                           'isPrivateComment', IFNULL(wda.is_private_comment, false),
                           'isStatusChange', IFNULL(wda.is_status_change, false),
                           'isSolve', IFNULL(LCASE(wda.status_change_to) = 'solved', false),
                           'isReassign', IFNULL(wda.is_reassign, false),
                           'isCallback', IFNULL(wda.is_callback, false),
                           'isOverflowed', IFNULL(wda.is_overflowed, false),
                           'isTransfer', IFNULL(wda.is_transfer, false),
                           'isVoicemail', IFNULL(wda.is_voicemail, false),
                           'isFirstHandle', IFNULL(wda.work_effort_secs = wd.first_reply_handle_time_secs AND
                                                   wda.end_time = wd.first_reply_time, false)
                       )),
                   ']'
               ) AS JSON)                                      AS interactions,
           DATE(rs.rostered_start_time)                        AS date,
           rs.rostered_start_time                              AS rosteredShiftStartTime,
           IF(wd.csat_score > -1, wd.csat_score, 0)            AS csatTotalScore,
           IF(wd.csat_score > -1, 1, 0)                        AS csatTotalResponses,
           IF(wd.csat_score >= w.csat_positive_response, 1, 0) AS csatTotalPositiveResponses
    FROM workactivity.workstream_data wd
             JOIN workactivity.workstream_data_activity wda ON wda.workstream_identifier = wd.identifier
             JOIN workactivity.shift_data sd ON wda.shift_data_id = sd.id
             JOIN roster.rostered_shift rs ON rs.mongo_shift_id = sd.shift_id
             JOIN roster.workstreams w ON wd.workstream_id = w.id
             JOIN roster.user u ON u.id = wda.user_id
             JOIN roster.team t ON u.team_id = t.id
             LEFT JOIN roster.user_role ur ON u.id = ur.user_id
             LEFT JOIN roster.role r ON ur.role_id = r.id
    WHERE rs.rostered_start_time BETWEEN ? AND ?
    AND wda.start_time BETWEEN ? AND ?
    GROUP BY wd.identifier, wda.user_id, date
    ORDER BY wd.identifier DESC, wda.user_id DESC;`;

const defaultFromDate = new Date();
defaultFromDate.setDate(defaultFromDate.getDate() - 7);
const defaultToDate = new Date();

// Example of a truncated tags array:
// [{"text":"BDK-EN-B2C"},{"text":"LENOX-B2C"},{"text":"R
const parseTags = (tags: string): TagsArray => {
    try {
        return typedJSONParse<TagsArray>(tags || '[]');
    } catch {
        while (!tags.endsWith('}')) {
            tags = tags.substring(0, tags.length - 1);
        }

        return typedJSONParse<TagsArray>((tags += ']'));
    }
};

export const fetchMySqlViews = async ({
    connection,
    organisationId,
    fromDate = defaultFromDate.toISOString(),
    toDate = defaultToDate.toISOString(),
}: FetchMySqlViewsParams): Promise<SqlViewsResult[]> => {
    try {
        await dbConnection.query({ connection, sql: 'SET SESSION group_concat_max_len = 100000;', params: [] });
        const teams = await dbConnection.query<TeamsResult[]>({
            connection,
            sql: 'SELECT * FROM roster.team t;',
            params: [],
        });

        const results = await dbConnection.query<IntermediateWorkstreamMetricsDbQueryResult[]>({
            connection,
            sql,
            params: [fromDate, toDate, fromDate, toDate],
        });

        const groupedTeams: Record<string, TeamsResult> = {};

        for (const team of teams) {
            groupedTeams[team.mongo_id] = team;
        }

        const views = [];

        for (const row of results) {
            const date = row.date.toISOString().split('T')[0];
            if (!date) {
                log.warn('Date was invalid');
                continue;
            }
            const teamStub = typedJSONParse<TeamStub>(row.team);
            const user = typedJSONParse<UserStub>(row.user);
            const newTags = parseTags(user.tags);
            const parent = teamStub.parentMongoId ? groupedTeams[teamStub.parentMongoId] : undefined;
            const workstream = typedJSONParse<OriginalWorkstreamStub>(row.workstreamData);

            const result: SqlViewsResult = {
                user: Object.assign(user, { tags: newTags }),
                team: teamStub,
                parent: parent
                    ? {
                          id: parent.id,
                          name: parent.name,
                          mongoId: parent.mongo_id,
                      }
                    : undefined,
                workstreamData: {
                    identifier: workstream.identifier,
                    startTime: workstream.startTime,
                    closedTime: workstream.closedTime,
                    firstReplyHandleTimeSecs: workstream.firstReplyHandleTimeSecs,
                    firstResolutionHandleTimeSecs: workstream.firstResolutionHandleTimeSecs,
                    direction: workstream.direction,
                    isHandled: Boolean(workstream.isHandled),
                    isSolved: Boolean(workstream.isSolved),
                },
                workstreamDefinition: typedJSONParse<WorkstreamDefinitionStub>(row.workstreamDefinition),
                rosteredShift: typedJSONParse<RosteredShiftStub>(row.rosteredShift),
                interactions: typedJSONParse<OriginalInteractionStub[]>(row.interactions).map((i) => ({
                    type: i.type,
                    identifier: i.identifier,
                    startTime: i.startTime,
                    endTime: i.endTime,

                    workEffortSecs: i.workEffortSecs,
                    talkTimeSecs: i.talkTimeSecs,
                    consultationTimeSecs: i.consultationTimeSecs,
                    wrapTimeSecs: i.wrapTimeSecs,
                    holdTimeSecs: i.holdTimeSecs,
                    waitTimeSecs: i.waitTimeSecs,

                    completionStatus: i.completionStatus,
                    contactIdentifier: i.contactIdentifier,
                    statusChangeFrom: i.statusChangeFrom,
                    statusChangeTo: i.statusChangeTo,

                    isHandle: Boolean(i.isHandle),
                    isPublicComment: Boolean(i.isPublicComment),
                    isPrivateComment: Boolean(i.isPrivateComment),
                    isStatusChange: Boolean(i.isStatusChange),
                    isSolve: Boolean(i.isSolve),
                    isReassign: Boolean(i.isReassign),
                    isCallback: Boolean(i.isCallback),
                    isOverflowed: Boolean(i.isOverflowed),
                    isTransfer: Boolean(i.isTransfer),
                    isVoicemail: Boolean(i.isVoicemail),
                    isFirstHandle: Boolean(i.isFirstHandle),
                })),
                date: date,
                rosteredShiftStartTime: new Date(row.rosteredShiftStartTime),
                csatTotalScore: row.csatTotalScore,
                csatTotalResponses: row.csatTotalResponses,
                csatTotalPositiveResponses: row.csatTotalPositiveResponses,
            };
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (!result) {
                const message = 'fetchMySqlViews: Invalid row';
                log.warn(message, {
                    organisationId,
                    row,
                });
                throw new Error(message);
            }

            views.push(result);
        }

        return views;
    } catch (e: unknown) {
        log.warn('Error fetching SQL views');
        throw e;
    } finally {
        await dbConnection.close({ connection });
    }
};
