import { config } from '../../config';
import { log, convertLoggedEventsToBulkWrite } from '../../utils';
import * as mongodb from '../../utils/mongodb';
import { AgyleTimeCloudEvent, decodeKinesisPayload, PlayvoxCloudEvent } from '@agyletime/wfm-aws-utils';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../utils';
import { EventDebugLogModel, type EventDebugLog } from '../../models';

export const eventDebugLogger = async <T>(eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const eventsToLog: EventDebugLog[] = [];

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<PlayvoxCloudEvent<T> | AgyleTimeCloudEvent<T>>({
                data: record.kinesis.data,
            });
            log.debug('eventDebugLogger -> decodedPayload', { decodedPayload: event });

            const organisationId =
                'agyletimeorganisationId' in event ? event.agyletimeorganisationId : event.plvxidentity?.organisationId;

            if (!organisationId) throw new Error('Event has no organisation ID');

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(organisationId.toString()))) {
                log.debug('eventDebugLogger -> org disabled', { organisationId });
                return;
            } else {
                log.debug('eventDebugLogger -> org enabled', { organisationId });
            }
            if (
                config.log.log_level === 'debug' ||
                process.env['DEBUG_EVENT_COLLECTION'] === 'true' ||
                config.agyletime.clusterName === 'alpha' ||
                config.agyletime.clusterName === 'gamma'
            ) {
                const debugEvent: EventDebugLog = {
                    organisationId: parseInt(organisationId.toString()),
                    date: new Date(),
                    type: mapPlayvoxEventTypeToPerformanceMetricType(event.type),
                    event: event,
                };

                eventsToLog.push(debugEvent);
            }
        } catch (e: unknown) {
            log.error(e);
        }
    }

    if (eventsToLog.length > 0) await EventDebugLogModel.bulkWrite(convertLoggedEventsToBulkWrite(eventsToLog));
};

const mapPlayvoxEventTypeToPerformanceMetricType = (eventType: string): string => {
    switch (eventType) {
        case 'com.playvox.workactivity.interaction.created':
            return 'InteractionCreated';
        case 'com.playvox.workactivity.interaction.updated':
            return 'InteractionUpdated';
        case 'com.playvox.workactivity.interaction.deleted':
            return 'InteractionDeleted';
        case 'com.playvox.workactivity.workstream.metrics.created':
            return 'WorkstreamMetricCreated';
        case 'com.playvox.workactivity.workstream.metrics.updated':
            return 'WorkstreamMetricUpdated';
        case 'com.playvox.workactivity.workstream.metrics.deleted':
            return 'WorkstreamMetricDeleted';
        case 'TaskData.merged':
            return 'TaskDataMerged';
        case 'agyletime.schedule.shifts.PublishedShiftAdded':
            return 'ShiftCreated';
        case 'agyletime.schedule.shifts.PublishedShiftUpdated':
            return 'ShiftUpdated';
        case 'agyletime.schedule.shifts.PublishedShiftRemoved':
            return 'ShiftDeleted';
        default:
            return eventType;
    }
};
