import type { InteractionEventParams, MongoQueryWithFilter } from '../types';
import { config } from '../../config';
import { log } from '../../utils';
import { toZonedTime } from 'date-fns-tz';
import type { MongoInteractionMetric } from '@agyletime/performance-utils';
import { parseISO } from 'date-fns';
import type { InteractionEventType } from './interactionEvent.type';
import { roundDateToNearestTime } from '../../utils';
import { getHandleTime } from './utils';

export default ({
    interaction,
    params,
}: {
    interaction: InteractionEventType;
    params: InteractionEventParams;
}): MongoQueryWithFilter<MongoInteractionMetric> => {
    const interactionType = interaction.type;
    const interactionStartTimeAsLocal = toZonedTime(interaction.startTime, params.timezone);
    const timeBucketAsLocal = roundDateToNearestTime(interactionStartTimeAsLocal, config.mongoBucketSizeInMinutes);
    const timeBucketAsUtc = roundDateToNearestTime(parseISO(interaction.startTime), config.mongoBucketSizeInMinutes);

    // # of Interactions metrics
    let chatInteractionIncrement = 0,
        ticketInteractionIncrement = 0,
        inboundPhoneInteractionIncrement = 0,
        outboundPhoneInteractionIncrement = 0;

    // Total Interaction Handle Time metrics
    let chatHandleTimeIncrement = 0,
        ticketHandleTimeIncrement = 0,
        inboundPhoneHandleTimeIncrement = 0,
        outboundPhoneHandleTimeIncrement = 0;

    // Reassigns
    // let chatReassignIncrement = 0,
    let ticketReassignIncrement = 0;
    // inboundPhoneReassignIncrement = 0,
    // outboundPhoneReassignIncrement = 0;

    // Comments
    let ticketPrivateCommentsIncrement = 0,
        ticketPublicCommentsIncrement = 0;

    // Declined
    // let chatDeclinedIncrement = 0,
    let ticketDeclinedIncrement = 0,
        inboundPhoneDeclinedIncrement = 0,
        outboundPhoneDeclinedIncrement = 0;

    // Assigned
    let chatAssignedIncrement = 0;
    // inboundPhoneAssignedIncrement = 0;

    // Self-assigned
    let chatSelfAssignedIncrement = 0;

    // Missed
    let chatMissedIncrement = 0,
        inboundPhoneMissedIncrement = 0;

    // Hold time
    let inboundPhoneHoldTimeIncrement = 0,
        outboundPhoneHoldTimeIncrement = 0;

    // Talk time
    let inboundPhoneTalkTimeIncrement = 0,
        outboundPhoneTalkTimeIncrement = 0;

    // Consult time
    let inboundPhoneConsultTimeIncrement = 0,
        outboundPhoneConsultTimeIncrement = 0;

    // Wrap time
    let inboundPhoneWrapTimeIncrement = 0,
        outboundPhoneWrapTimeIncrement = 0;

    // Offered
    // let chatOfferedIncrement = 0,
    //     inboundPhoneOfferedIncrement = 0;

    const handleTime = getHandleTime(interaction);

    switch (interactionType) {
        case 'chat':
        case 'Chat':
            chatInteractionIncrement = handleTime > 0 ? -1 : 0;
            chatHandleTimeIncrement = -handleTime;
            // chatReassignIncrement = interaction.isReassign ? -1 : 0;
            // chatDeclinedIncrement = interaction.completionStatus?.toLowerCase() === 'agent_declined' ? -1 : 0;
            chatAssignedIncrement = interaction.completionStatus?.toLowerCase() === 'assigned_accepted' ? -1 : 0;
            chatSelfAssignedIncrement = interaction.completionStatus?.toLowerCase() === 'self_assigned' ? -1 : 0;
            chatMissedIncrement = interaction.completionStatus?.toLowerCase() === 'assigned_missed' ? -1 : 0;
            // chatOfferedIncrement = 0; // TBC
            break;
        case 'worklog':
        case 'Worklog':
        case 'Status_Change':
        case 'status_change':
            ticketInteractionIncrement = handleTime > 0 ? -1 : 0;
            ticketHandleTimeIncrement = -handleTime;
            ticketReassignIncrement = interaction.isReassign ? -1 : 0;
            ticketPrivateCommentsIncrement = interaction.isPrivateComment ? -1 : 0;
            ticketPublicCommentsIncrement = interaction.isPublicComment ? -1 : 0;
            ticketDeclinedIncrement = interaction.completionStatus?.toLowerCase() === 'agent_declined' ? -1 : 0;
            break;
        case 'inbound_phone':
        case 'Inbound_Phone':
            inboundPhoneInteractionIncrement = handleTime > 0 ? -1 : 0;
            inboundPhoneHandleTimeIncrement = -handleTime;
            // inboundPhoneReassignIncrement = interaction.isReassign ? -1 : 0;
            inboundPhoneDeclinedIncrement = interaction.completionStatus?.toLowerCase() === 'agent_declined' ? -1 : 0;
            // inboundPhoneAssignedIncrement = interaction.completionStatus?.toLowerCase() === 'assigned_accepted' ? -1 : 0;
            inboundPhoneMissedIncrement = interaction.completionStatus?.toLowerCase() === 'agent_missed' ? -1 : 0;
            inboundPhoneHoldTimeIncrement = -(interaction.holdTimeSecs ?? 0);
            inboundPhoneTalkTimeIncrement = -(interaction.talkTimeSecs ?? 0);
            inboundPhoneConsultTimeIncrement = -(interaction.consultationTimeSecs ?? 0);
            inboundPhoneWrapTimeIncrement = -(interaction.wrapTimeSecs ?? 0);
            // inboundPhoneOfferedIncrement = 0; // TBC
            break;
        case 'outbound_phone':
        case 'Outbound_Phone':
            outboundPhoneInteractionIncrement = handleTime > 0 ? -1 : 0;
            outboundPhoneHandleTimeIncrement = -handleTime;
            // outboundPhoneReassignIncrement = interaction.isReassign ? -1 : 0;
            outboundPhoneDeclinedIncrement = interaction.completionStatus?.toLowerCase() === 'agent_declined' ? -1 : 0;
            outboundPhoneHoldTimeIncrement = -(interaction.holdTimeSecs ?? 0);
            outboundPhoneTalkTimeIncrement = -(interaction.talkTimeSecs ?? 0);
            outboundPhoneConsultTimeIncrement = -(interaction.consultationTimeSecs ?? 0);
            outboundPhoneWrapTimeIncrement = -(interaction.wrapTimeSecs ?? 0);

            break;
        default:
            // Don't bother to update anything
            log.debug('Skipping interaction', interaction);
            break;
    }

    const set: MongoQueryWithFilter<MongoInteractionMetric>['query']['$set'] = {
        organisationId: params.organisationId,
        timeBucketAsLocal,
        timeBucketAsUtc,
        userId: params.userId,
        teamId: params.teamId,
        interactionType: params.interactionType,
        workstreamIdentifier: params.workstreamIdentifier,
        taskTypeId: params.taskTypeId,
    };

    if (params.workstreamDefinitionId) {
        set.workstreamDefinitionId = params.workstreamDefinitionId;
        set.workstreamDefinitionName =
            params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`;
    }

    return {
        filter: {
            organisationId: params.organisationId,
            timeBucketAsLocal,
            userId: params.userId,
            teamId: params.teamId,
            interactionType: params.interactionType,
            workstreamIdentifier: params.workstreamIdentifier,
        },
        query: {
            $set: set,
            $inc: {
                // # of Interactions
                numberOfInteractions_all:
                    chatInteractionIncrement +
                    ticketInteractionIncrement +
                    inboundPhoneInteractionIncrement +
                    outboundPhoneInteractionIncrement,
                numberOfInteractions_chat: chatInteractionIncrement,
                numberOfInteractions_ticket: ticketInteractionIncrement,
                numberOfInteractions_inboundPhone: inboundPhoneInteractionIncrement,
                numberOfInteractions_outboundPhone: outboundPhoneInteractionIncrement,
                // Total Interaction Handle Time
                totalInteractionHandleTime_all:
                    chatHandleTimeIncrement +
                    ticketHandleTimeIncrement +
                    inboundPhoneHandleTimeIncrement +
                    outboundPhoneHandleTimeIncrement,
                totalInteractionHandleTime_chat: chatHandleTimeIncrement,
                totalInteractionHandleTime_ticket: ticketHandleTimeIncrement,
                totalInteractionHandleTime_inboundPhone: inboundPhoneHandleTimeIncrement,
                totalInteractionHandleTime_outboundPhone: outboundPhoneHandleTimeIncrement,
                // # Reassigns
                totalReassigns_all: ticketReassignIncrement,
                totalReassigns_ticket: ticketReassignIncrement,
                // # Private Comments
                totalPrivateComments_all: ticketPrivateCommentsIncrement,
                totalPrivateComments_ticket: ticketPrivateCommentsIncrement,
                // # Public Comments
                totalPublicComments_all: ticketPublicCommentsIncrement,
                totalPublicComments_ticket: ticketPublicCommentsIncrement,
                // # Declined
                totalDeclined_all:
                    ticketDeclinedIncrement + inboundPhoneDeclinedIncrement + outboundPhoneDeclinedIncrement,
                totalDeclined_ticket: ticketDeclinedIncrement,
                totalDeclined_inboundPhone: inboundPhoneDeclinedIncrement,
                totalDeclined_outboundPhone: outboundPhoneDeclinedIncrement,
                // # Assigned
                totalAssigned_all: chatAssignedIncrement /* + inboundPhoneAssignedIncrement */,
                totalAssigned_chat: chatAssignedIncrement,
                // 'totalAssigned_inboundPhone': inboundPhoneAssignedIncrement,
                // # Self-assigned
                totalSelfAssignedInteractions_all: chatSelfAssignedIncrement,
                totalSelfAssignedInteractions_chat: chatSelfAssignedIncrement,
                // # Missed
                totalMissed_all: chatMissedIncrement + inboundPhoneMissedIncrement,
                totalMissed_chat: chatMissedIncrement,
                totalMissed_inboundPhone: inboundPhoneMissedIncrement,
                // Total Hold time
                holdTime_all: inboundPhoneHoldTimeIncrement + outboundPhoneHoldTimeIncrement,
                holdTime_inboundPhone: inboundPhoneHoldTimeIncrement,
                holdTime_outboundPhone: outboundPhoneHoldTimeIncrement,
                // Total Talk time
                talkTime_all: inboundPhoneTalkTimeIncrement + outboundPhoneTalkTimeIncrement,
                talkTime_inboundPhone: inboundPhoneTalkTimeIncrement,
                talkTime_outboundPhone: outboundPhoneTalkTimeIncrement,
                // Consult time
                consultTime_all: inboundPhoneConsultTimeIncrement + outboundPhoneConsultTimeIncrement,
                consultTime_inboundPhone: inboundPhoneConsultTimeIncrement,
                consultTime_outboundPhone: outboundPhoneConsultTimeIncrement,
                // Wrap time
                wrapTime_all: inboundPhoneWrapTimeIncrement + outboundPhoneWrapTimeIncrement,
                wrapTime_inboundPhone: inboundPhoneWrapTimeIncrement,
                wrapTime_outboundPhone: outboundPhoneWrapTimeIncrement,
                // Resolutions
                totalResolutions: interaction.statusChangeTo?.toLowerCase() === 'solved' ? -1 : 0,
            },
        },
    };
};
