import type { MongoQueryWithFilter } from '../types';
import type { InteractionEventParams } from '../types';
import { config } from '../../config';
import { isEqual, parseISO } from 'date-fns';
import { toZonedTime } from 'date-fns-tz';
import type { LowerCaseInteractionType } from '@agyletime/common-utils/models';
import { makeNumberNegative } from '../../utils';
import type { MongoInteractionMetric } from '@agyletime/performance-utils';
import type { InteractionEventType } from './interactionEvent.type';
import { roundDateToNearestTime } from '../../utils';
import { getHandleTime } from './utils';

export default ({
    previousVersion,
    currentVersion,
    params,
}: {
    previousVersion: InteractionEventType;
    currentVersion: InteractionEventType;
    params: InteractionEventParams;
}): MongoQueryWithFilter<MongoInteractionMetric>[] => {
    const previousStartTimeAsLocal = toZonedTime(previousVersion.startTime, params.timezone);
    const previousTimeBucket = roundDateToNearestTime(previousStartTimeAsLocal, config.mongoBucketSizeInMinutes);
    const previousTimeBucketUtc = roundDateToNearestTime(
        parseISO(previousVersion.startTime),
        config.mongoBucketSizeInMinutes,
    );

    const currentStartTimeAsLocal = toZonedTime(currentVersion.startTime, params.timezone);
    const currentTimeBucket = roundDateToNearestTime(currentStartTimeAsLocal, config.mongoBucketSizeInMinutes);
    const currentTimeBucketUtc = roundDateToNearestTime(
        parseISO(currentVersion.startTime),
        config.mongoBucketSizeInMinutes,
    );

    const changingBuckets = !isEqual(previousTimeBucket, currentTimeBucket);
    const validInteractionTypes = new Set(['chat', 'worklog', 'inbound_phone', 'outbound_phone', 'status_change']);
    const previousVersionType = previousVersion.type.toLowerCase() as LowerCaseInteractionType;
    const currentVersionType = currentVersion.type.toLowerCase() as LowerCaseInteractionType;
    const previousVersionHandleTime = getHandleTime(previousVersion);
    const currentVersionHandleTime = getHandleTime(currentVersion);

    // Treat undefined work effort as 0
    if (!previousVersion.workEffortSecs) previousVersion.workEffortSecs = 0;
    if (!currentVersion.workEffortSecs) currentVersion.workEffortSecs = 0;

    if (changingBuckets) {
        const setPrevious: MongoQueryWithFilter<MongoInteractionMetric>['query']['$set'] = {
            organisationId: params.organisationId,
            timeBucketAsLocal: previousTimeBucket,
            timeBucketAsUtc: previousTimeBucketUtc,
            userId: params.userId,
            teamId: params.teamId,
            interactionType: params.interactionType,
            workstreamIdentifier: params.workstreamIdentifier,
            taskTypeId: params.taskTypeId,
        };

        if (params.workstreamDefinitionId) {
            setPrevious.workstreamDefinitionId = params.workstreamDefinitionId;
            setPrevious.workstreamDefinitionName =
                params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`;
        }

        const setCurrent: MongoQueryWithFilter<MongoInteractionMetric>['query']['$set'] = {
            organisationId: params.organisationId,
            timeBucketAsLocal: previousTimeBucket,
            timeBucketAsUtc: previousTimeBucketUtc,
            userId: params.userId,
            teamId: params.teamId,
            interactionType: params.interactionType,
            workstreamIdentifier: params.workstreamIdentifier,
            taskTypeId: params.taskTypeId,
        };

        if (params.workstreamDefinitionId) {
            setCurrent.workstreamDefinitionId = params.workstreamDefinitionId;
            setCurrent.workstreamDefinitionName =
                params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`;
        }

        return [
            // Previous bucket
            {
                filter: {
                    organisationId: params.organisationId,
                    timeBucketAsLocal: previousTimeBucket,
                    userId: params.userId,
                    teamId: params.teamId,
                    interactionType: params.interactionType,
                    workstreamIdentifier: params.workstreamIdentifier,
                },
                query: {
                    $set: setPrevious,
                    $inc: {
                        // # of Interactions
                        numberOfInteractions_all: validInteractionTypes.has(previousVersionType) ? -1 : 0,
                        numberOfInteractions_chat:
                            previousVersionType === 'chat' && previousVersionHandleTime > 0 ? -1 : 0,
                        numberOfInteractions_ticket:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            previousVersionHandleTime > 0
                                ? -1
                                : 0,
                        numberOfInteractions_inboundPhone:
                            previousVersionType === 'inbound_phone' && previousVersionHandleTime > 0 ? -1 : 0,
                        numberOfInteractions_outboundPhone:
                            previousVersionType === 'outbound_phone' && previousVersionHandleTime > 0 ? -1 : 0,
                        // Total Interaction Handle Time
                        totalInteractionHandleTime_all:
                            validInteractionTypes.has(previousVersionType) && previousVersion.workEffortSecs > 0
                                ? makeNumberNegative(previousVersion.workEffortSecs)
                                : 0,
                        totalInteractionHandleTime_chat:
                            previousVersionType === 'chat' && previousVersion.workEffortSecs > 0
                                ? makeNumberNegative(previousVersion.workEffortSecs)
                                : 0,
                        totalInteractionHandleTime_ticket:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            previousVersion.workEffortSecs > 0
                                ? makeNumberNegative(previousVersion.workEffortSecs)
                                : 0,
                        totalInteractionHandleTime_inboundPhone:
                            previousVersionType === 'inbound_phone' && previousVersion.workEffortSecs > 0
                                ? makeNumberNegative(previousVersion.workEffortSecs)
                                : 0,
                        totalInteractionHandleTime_outboundPhone:
                            previousVersionType === 'outbound_phone' && previousVersion.workEffortSecs > 0
                                ? makeNumberNegative(previousVersion.workEffortSecs)
                                : 0,
                        // # Reassigns
                        totalReassigns_all:
                            validInteractionTypes.has(previousVersionType) && (previousVersion.isReassign ?? false)
                                ? -1
                                : 0,
                        totalReassigns_ticket:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            (previousVersion.isReassign ?? false)
                                ? -1
                                : 0,
                        // # Private Comments
                        totalPrivateComments_all:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            (previousVersion.isPrivateComment ?? false)
                                ? -1
                                : 0,
                        totalPrivateComments_ticket:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            (previousVersion.isPrivateComment ?? false)
                                ? -1
                                : 0,
                        // # Public Comments
                        totalPublicComments_all:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            (previousVersion.isPublicComment ?? false)
                                ? -1
                                : 0,
                        totalPublicComments_ticket:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            (previousVersion.isPublicComment ?? false)
                                ? -1
                                : 0,
                        // # Declined
                        totalDeclined_all:
                            validInteractionTypes.has(previousVersionType) &&
                            previousVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? -1
                                : 0,
                        totalDeclined_ticket:
                            (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                            previousVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? -1
                                : 0,
                        totalDeclined_inboundPhone:
                            previousVersionType === 'inbound_phone' &&
                            previousVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? -1
                                : 0,
                        totalDeclined_outboundPhone:
                            previousVersionType === 'outbound_phone' &&
                            previousVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? -1
                                : 0,
                        // # Assigned
                        totalAssigned_all:
                            (previousVersionType === 'chat' || previousVersionType === 'inbound_phone') &&
                            previousVersion.completionStatus?.toLowerCase() === 'assigned_accepted'
                                ? -1
                                : 0,
                        totalAssigned_chat:
                            previousVersionType === 'chat' &&
                            previousVersion.completionStatus?.toLowerCase() === 'assigned_accepted'
                                ? -1
                                : 0,
                        // # Self-assigned
                        totalSelfAssignedInteractions_all:
                            previousVersionType === 'chat' &&
                            previousVersion.completionStatus?.toLowerCase() === 'self_assigned'
                                ? -1
                                : 0,
                        totalSelfAssignedInteractions_chat:
                            previousVersionType === 'chat' &&
                            previousVersion.completionStatus?.toLowerCase() === 'self_assigned'
                                ? -1
                                : 0,
                        // # Missed
                        totalMissed_all:
                            (previousVersionType === 'chat' || previousVersionType === 'inbound_phone') &&
                            previousVersion.completionStatus?.toLowerCase() === 'agent_missed'
                                ? -1
                                : 0,
                        totalMissed_chat:
                            previousVersionType === 'chat' &&
                            previousVersion.completionStatus?.toLowerCase() === 'assigned_missed'
                                ? -1
                                : 0,
                        totalMissed_inboundPhone:
                            previousVersionType === 'inbound_phone' &&
                            previousVersion.completionStatus?.toLowerCase() === 'agent_missed'
                                ? -1
                                : 0,
                        holdTime_all:
                            previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.holdTimeSecs)
                                : 0,
                        holdTime_inboundPhone:
                            previousVersionType === 'inbound_phone'
                                ? makeNumberNegative(previousVersion.holdTimeSecs)
                                : 0,
                        holdTime_outboundPhone:
                            previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.holdTimeSecs)
                                : 0,
                        // Total Talk time
                        talkTime_all:
                            previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.talkTimeSecs)
                                : 0,
                        talkTime_inboundPhone:
                            previousVersionType === 'inbound_phone'
                                ? makeNumberNegative(previousVersion.talkTimeSecs)
                                : 0,
                        talkTime_outboundPhone:
                            previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.talkTimeSecs)
                                : 0,
                        // Consult time
                        consultTime_all:
                            previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.consultationTimeSecs)
                                : 0,
                        consultTime_inboundPhone:
                            previousVersionType === 'inbound_phone'
                                ? makeNumberNegative(previousVersion.consultationTimeSecs)
                                : 0,
                        consultTime_outboundPhone:
                            previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.consultationTimeSecs)
                                : 0,
                        // Wrap time
                        wrapTime_all:
                            previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.wrapTimeSecs)
                                : 0,
                        wrapTime_inboundPhone:
                            previousVersionType === 'inbound_phone'
                                ? makeNumberNegative(previousVersion.wrapTimeSecs)
                                : 0,
                        wrapTime_outboundPhone:
                            previousVersionType === 'outbound_phone'
                                ? makeNumberNegative(previousVersion.wrapTimeSecs)
                                : 0,
                        // Resolutions
                        totalResolutions: previousVersion.statusChangeTo?.toLowerCase() === 'solved' ? -1 : 0,
                    },
                },
            },
            // New bucket
            {
                filter: {
                    organisationId: params.organisationId,
                    timeBucketAsLocal: currentTimeBucket,
                    userId: params.userId,
                    teamId: params.teamId,
                    interactionType: params.interactionType,
                    workstreamIdentifier: params.workstreamIdentifier,
                },
                query: {
                    $set: setCurrent,
                    $inc: {
                        // # of Interactions
                        numberOfInteractions_all: validInteractionTypes.has(currentVersionType) ? 1 : 0,
                        numberOfInteractions_chat:
                            currentVersionType === 'chat' && currentVersionHandleTime > 0 ? 1 : 0,
                        numberOfInteractions_ticket:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            currentVersionHandleTime > 0
                                ? 1
                                : 0,
                        numberOfInteractions_inboundPhone:
                            currentVersionType === 'inbound_phone' && currentVersionHandleTime > 0 ? 1 : 0,
                        numberOfInteractions_outboundPhone:
                            currentVersionType === 'outbound_phone' && currentVersionHandleTime > 0 ? 1 : 0,
                        // Total Interaction Handle Time
                        totalInteractionHandleTime_all:
                            validInteractionTypes.has(currentVersionType) && currentVersion.workEffortSecs > 0
                                ? currentVersion.workEffortSecs
                                : 0,
                        totalInteractionHandleTime_chat:
                            currentVersionType === 'chat' && currentVersion.workEffortSecs > 0
                                ? currentVersion.workEffortSecs
                                : 0,
                        totalInteractionHandleTime_ticket:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            currentVersion.workEffortSecs > 0
                                ? currentVersion.workEffortSecs
                                : 0,
                        totalInteractionHandleTime_inboundPhone:
                            currentVersionType === 'inbound_phone' && currentVersion.workEffortSecs > 0
                                ? currentVersion.workEffortSecs
                                : 0,
                        totalInteractionHandleTime_outboundPhone:
                            currentVersionType === 'outbound_phone' && currentVersion.workEffortSecs > 0
                                ? currentVersion.workEffortSecs
                                : 0,
                        // # Reassigns
                        totalReassigns_all:
                            validInteractionTypes.has(currentVersionType) && (currentVersion.isReassign ?? false)
                                ? 1
                                : 0,
                        totalReassigns_ticket:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            (currentVersion.isReassign ?? false)
                                ? 1
                                : 0,
                        // # Private Comments
                        totalPrivateComments_all:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            (currentVersion.isPrivateComment ?? false)
                                ? 1
                                : 0,
                        totalPrivateComments_ticket:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            (currentVersion.isPrivateComment ?? false)
                                ? 1
                                : 0,
                        // # Public Comments
                        totalPublicComments_all:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            (currentVersion.isPublicComment ?? false)
                                ? 1
                                : 0,
                        totalPublicComments_ticket:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            (currentVersion.isPublicComment ?? false)
                                ? 1
                                : 0,
                        // # Declined
                        totalDeclined_all:
                            validInteractionTypes.has(currentVersionType) &&
                            currentVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? 1
                                : 0,
                        totalDeclined_ticket:
                            (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                            currentVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? 1
                                : 0,
                        totalDeclined_inboundPhone:
                            currentVersionType === 'inbound_phone' &&
                            currentVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? 1
                                : 0,
                        totalDeclined_outboundPhone:
                            currentVersionType === 'outbound_phone' &&
                            currentVersion.completionStatus?.toLowerCase() === 'agent_declined'
                                ? 1
                                : 0,
                        // # Assigned
                        totalAssigned_all:
                            (currentVersionType === 'chat' || currentVersionType === 'inbound_phone') &&
                            currentVersion.completionStatus?.toLowerCase() === 'assigned_accepted'
                                ? 1
                                : 0,
                        totalAssigned_chat:
                            currentVersionType === 'chat' &&
                            currentVersion.completionStatus?.toLowerCase() === 'assigned_accepted'
                                ? 1
                                : 0,
                        // # Self-assigned
                        totalSelfAssignedInteractions_all:
                            currentVersionType === 'chat' &&
                            currentVersion.completionStatus?.toLowerCase() === 'self_assigned'
                                ? 1
                                : 0,
                        totalSelfAssignedInteractions_chat:
                            currentVersionType === 'chat' &&
                            currentVersion.completionStatus?.toLowerCase() === 'self_assigned'
                                ? 1
                                : 0,
                        // # Missed
                        totalMissed_all:
                            (currentVersionType === 'chat' || currentVersionType === 'inbound_phone') &&
                            currentVersion.completionStatus?.toLowerCase() === 'agent_missed'
                                ? 1
                                : 0,
                        totalMissed_chat:
                            currentVersionType === 'chat' &&
                            currentVersion.completionStatus?.toLowerCase() === 'assigned_missed'
                                ? 1
                                : 0,
                        totalMissed_inboundPhone:
                            currentVersionType === 'inbound_phone' &&
                            currentVersion.completionStatus?.toLowerCase() === 'agent_missed'
                                ? 1
                                : 0,
                        // Total Hold time
                        holdTime_all:
                            currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone'
                                ? currentVersion.holdTimeSecs ?? 0
                                : 0,
                        holdTime_inboundPhone:
                            currentVersionType === 'inbound_phone' ? currentVersion.holdTimeSecs ?? 0 : 0,
                        holdTime_outboundPhone:
                            currentVersionType === 'outbound_phone' ? currentVersion.holdTimeSecs ?? 0 : 0,
                        // Total Talk time
                        talkTime_all:
                            currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone'
                                ? currentVersion.talkTimeSecs ?? 0
                                : 0,
                        talkTime_inboundPhone:
                            currentVersionType === 'inbound_phone' ? currentVersion.talkTimeSecs ?? 0 : 0,
                        talkTime_outboundPhone:
                            currentVersionType === 'outbound_phone' ? currentVersion.talkTimeSecs ?? 0 : 0,
                        // Consult time
                        consultTime_all:
                            currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone'
                                ? currentVersion.consultationTimeSecs ?? 0
                                : 0,
                        consultTime_inboundPhone:
                            currentVersionType === 'inbound_phone' ? currentVersion.consultationTimeSecs ?? 0 : 0,
                        consultTime_outboundPhone:
                            currentVersionType === 'outbound_phone' ? currentVersion.consultationTimeSecs ?? 0 : 0,
                        // Wrap time
                        wrapTime_all:
                            currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone'
                                ? currentVersion.wrapTimeSecs ?? 0
                                : 0,
                        wrapTime_inboundPhone:
                            currentVersionType === 'inbound_phone' ? currentVersion.wrapTimeSecs ?? 0 : 0,
                        wrapTime_outboundPhone:
                            currentVersionType === 'outbound_phone' ? currentVersion.wrapTimeSecs ?? 0 : 0,
                        // Resolutions
                        totalResolutions:
                            currentVersion.statusChangeTo?.toLowerCase() === 'solved' ||
                            previousVersion.statusChangeTo?.toLowerCase() === 'solved'
                                ? 1
                                : 0,
                    },
                },
            },
        ];
    } else {
        const calculateInteractionDelta = ({
            oldValue,
            newValue,
        }: {
            oldValue: boolean;
            newValue: boolean;
        }): number => {
            if (oldValue && !newValue) return -1;

            if (!oldValue && newValue) return 1;

            return 0;
        };

        const calculateHandleTimeDelta = ({ oldValue, newValue }: { oldValue: boolean; newValue: boolean }): number => {
            if (oldValue && newValue) return getHandleTime(currentVersion) - getHandleTime(previousVersion);
            if (oldValue && !newValue) return -getHandleTime(previousVersion);
            if (!oldValue && newValue) return getHandleTime(currentVersion);

            return 0;
        };

        const calculateHoldTimeDelta = ({ oldValue, newValue }: { oldValue: boolean; newValue: boolean }): number => {
            if (oldValue && newValue) return (currentVersion.holdTimeSecs ?? 0) - (previousVersion.holdTimeSecs ?? 0);
            if (oldValue && !newValue) return -(previousVersion.holdTimeSecs ?? 0);
            if (!oldValue && newValue) return currentVersion.holdTimeSecs ?? 0;

            return 0;
        };

        const calculateTalkTimeDelta = ({ oldValue, newValue }: { oldValue: boolean; newValue: boolean }): number => {
            if (oldValue && newValue) return (currentVersion.talkTimeSecs ?? 0) - (previousVersion.talkTimeSecs ?? 0);
            if (oldValue && !newValue) return -(previousVersion.talkTimeSecs ?? 0);
            if (!oldValue && newValue) return currentVersion.talkTimeSecs ?? 0;

            return 0;
        };

        const calculateConsultTimeDelta = ({
            oldValue,
            newValue,
        }: {
            oldValue: boolean;
            newValue: boolean;
        }): number => {
            if (oldValue && newValue)
                return (currentVersion.consultationTimeSecs ?? 0) - (previousVersion.consultationTimeSecs ?? 0);
            if (oldValue && !newValue) return -(previousVersion.consultationTimeSecs ?? 0);
            if (!oldValue && newValue) return currentVersion.consultationTimeSecs ?? 0;

            return 0;
        };

        const calculateWrapTimeDelta = ({ oldValue, newValue }: { oldValue: boolean; newValue: boolean }): number => {
            if (oldValue && newValue) return (currentVersion.wrapTimeSecs ?? 0) - (previousVersion.wrapTimeSecs ?? 0);
            if (oldValue && !newValue) return -(previousVersion.wrapTimeSecs ?? 0);
            if (!oldValue && newValue) return currentVersion.wrapTimeSecs ?? 0;

            return 0;
        };

        const calculateTotalResolutionsDelta = ({
            oldValue,
            newValue,
        }: {
            oldValue: boolean;
            newValue: boolean;
        }): number => {
            if (!oldValue && newValue) return 1;

            return 0;
        };

        const set: MongoQueryWithFilter<MongoInteractionMetric>['query']['$set'] = {
            organisationId: params.organisationId,
            timeBucketAsLocal: currentTimeBucket,
            timeBucketAsUtc: currentTimeBucketUtc,
            userId: params.userId,
            teamId: params.teamId,
            interactionType: params.interactionType,
            workstreamIdentifier: params.workstreamIdentifier,
            taskTypeId: params.taskTypeId,
        };

        if (params.workstreamDefinitionId) {
            set.workstreamDefinitionId = params.workstreamDefinitionId;
            set.workstreamDefinitionName =
                params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`;
        }

        return [
            {
                filter: {
                    organisationId: params.organisationId,
                    timeBucketAsLocal: currentTimeBucket,
                    userId: params.userId,
                    teamId: params.teamId,
                    interactionType: params.interactionType,
                    workstreamIdentifier: params.workstreamIdentifier,
                },
                query: {
                    $set: set,
                    $inc: {
                        // # of Interactions
                        numberOfInteractions_all: calculateInteractionDelta({
                            oldValue:
                                validInteractionTypes.has(previousVersionType) && getHandleTime(previousVersion) > 0,
                            newValue:
                                validInteractionTypes.has(currentVersionType) && getHandleTime(currentVersion) > 0,
                        }),
                        numberOfInteractions_chat: calculateInteractionDelta({
                            oldValue: previousVersionType === 'chat' && getHandleTime(previousVersion) > 0,
                            newValue: currentVersionType === 'chat' && getHandleTime(currentVersion) > 0,
                        }),
                        numberOfInteractions_ticket: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                getHandleTime(previousVersion) > 0,
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                getHandleTime(currentVersion) > 0,
                        }),
                        numberOfInteractions_inboundPhone: calculateInteractionDelta({
                            oldValue: previousVersionType === 'inbound_phone' && getHandleTime(previousVersion) > 0,
                            newValue: currentVersionType === 'inbound_phone' && getHandleTime(currentVersion) > 0,
                        }),
                        numberOfInteractions_outboundPhone: calculateInteractionDelta({
                            oldValue: previousVersionType === 'outbound_phone' && getHandleTime(previousVersion) > 0,
                            newValue: currentVersionType === 'outbound_phone' && getHandleTime(currentVersion) > 0,
                        }),
                        // Total Interaction Handle Time
                        totalInteractionHandleTime_all: calculateHandleTimeDelta({
                            oldValue:
                                validInteractionTypes.has(previousVersionType) && previousVersion.workEffortSecs > 0,
                            newValue:
                                validInteractionTypes.has(currentVersionType) && currentVersion.workEffortSecs > 0,
                        }),
                        totalInteractionHandleTime_chat: calculateHandleTimeDelta({
                            oldValue: previousVersionType === 'chat' && previousVersion.workEffortSecs > 0,
                            newValue: currentVersionType === 'chat' && currentVersion.workEffortSecs > 0,
                        }),
                        totalInteractionHandleTime_ticket: calculateHandleTimeDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                previousVersion.workEffortSecs > 0,
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                currentVersion.workEffortSecs > 0,
                        }),
                        totalInteractionHandleTime_inboundPhone: calculateHandleTimeDelta({
                            oldValue: previousVersionType === 'inbound_phone' && previousVersion.workEffortSecs > 0,
                            newValue: currentVersionType === 'inbound_phone' && currentVersion.workEffortSecs > 0,
                        }),
                        totalInteractionHandleTime_outboundPhone: calculateHandleTimeDelta({
                            oldValue: previousVersionType === 'outbound_phone' && previousVersion.workEffortSecs > 0,
                            newValue: currentVersionType === 'outbound_phone' && currentVersion.workEffortSecs > 0,
                        }),
                        // # Reassigns
                        totalReassigns_all: calculateInteractionDelta({
                            oldValue:
                                validInteractionTypes.has(previousVersionType) && (previousVersion.isReassign ?? false),
                            newValue:
                                validInteractionTypes.has(currentVersionType) && (currentVersion.isReassign ?? false),
                        }),
                        totalReassigns_ticket: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                (previousVersion.isReassign ?? false),
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                (currentVersion.isReassign ?? false),
                        }),
                        // # Private Comments
                        totalPrivateComments_all: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                (previousVersion.isPrivateComment ?? false),
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                (currentVersion.isPrivateComment ?? false),
                        }),
                        totalPrivateComments_ticket: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                (previousVersion.isPrivateComment ?? false),
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                (currentVersion.isPrivateComment ?? false),
                        }),
                        // # Public Comments
                        totalPublicComments_all: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                (previousVersion.isPublicComment ?? false),
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                (currentVersion.isPublicComment ?? false),
                        }),
                        totalPublicComments_ticket: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                (previousVersion.isPublicComment ?? false),
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                (currentVersion.isPublicComment ?? false),
                        }),
                        // # Declined
                        totalDeclined_all: calculateInteractionDelta({
                            oldValue:
                                validInteractionTypes.has(previousVersionType) &&
                                previousVersion.completionStatus?.toLowerCase() === 'agent_declined',
                            newValue:
                                validInteractionTypes.has(currentVersionType) &&
                                currentVersion.completionStatus?.toLowerCase() === 'agent_declined',
                        }),
                        totalDeclined_ticket: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'worklog' || previousVersionType === 'status_change') &&
                                previousVersion.completionStatus?.toLowerCase() === 'agent_declined',
                            newValue:
                                (currentVersionType === 'worklog' || currentVersionType === 'status_change') &&
                                currentVersion.completionStatus?.toLowerCase() === 'agent_declined',
                        }),
                        totalDeclined_inboundPhone: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'inbound_phone' &&
                                previousVersion.completionStatus?.toLowerCase() === 'agent_declined',
                            newValue:
                                currentVersionType === 'inbound_phone' &&
                                currentVersion.completionStatus?.toLowerCase() === 'agent_declined',
                        }),
                        totalDeclined_outboundPhone: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'outbound_phone' &&
                                previousVersion.completionStatus?.toLowerCase() === 'agent_declined',
                            newValue:
                                currentVersionType === 'outbound_phone' &&
                                currentVersion.completionStatus?.toLowerCase() === 'agent_declined',
                        }),
                        // # Assigned
                        totalAssigned_all: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'chat' || previousVersionType === 'inbound_phone') &&
                                previousVersion.completionStatus?.toLowerCase() === 'assigned_accepted',
                            newValue:
                                (currentVersionType === 'chat' || currentVersionType === 'inbound_phone') &&
                                currentVersion.completionStatus?.toLowerCase() === 'assigned_accepted',
                        }),
                        totalAssigned_chat: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'chat' &&
                                previousVersion.completionStatus?.toLowerCase() === 'assigned_accepted',
                            newValue:
                                currentVersionType === 'chat' &&
                                currentVersion.completionStatus?.toLowerCase() === 'assigned_accepted',
                        }),
                        // # Self-assigned
                        totalSelfAssignedInteractions_all: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'chat' &&
                                previousVersion.completionStatus?.toLowerCase() === 'self_assigned',
                            newValue:
                                currentVersionType === 'chat' &&
                                currentVersion.completionStatus?.toLowerCase() === 'self_assigned',
                        }),
                        totalSelfAssignedInteractions_chat: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'chat' &&
                                previousVersion.completionStatus?.toLowerCase() === 'self_assigned',
                            newValue:
                                currentVersionType === 'chat' &&
                                currentVersion.completionStatus?.toLowerCase() === 'self_assigned',
                        }),
                        // # Missed
                        totalMissed_all: calculateInteractionDelta({
                            oldValue:
                                (previousVersionType === 'chat' || previousVersionType === 'inbound_phone') &&
                                previousVersion.completionStatus?.toLowerCase() === 'agent_missed',
                            newValue:
                                (currentVersionType === 'chat' || currentVersionType === 'inbound_phone') &&
                                currentVersion.completionStatus?.toLowerCase() === 'agent_missed',
                        }),
                        totalMissed_chat: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'chat' &&
                                previousVersion.completionStatus?.toLowerCase() === 'assigned_missed',
                            newValue:
                                currentVersionType === 'chat' &&
                                currentVersion.completionStatus?.toLowerCase() === 'assigned_missed',
                        }),
                        totalMissed_inboundPhone: calculateInteractionDelta({
                            oldValue:
                                previousVersionType === 'inbound_phone' &&
                                previousVersion.completionStatus?.toLowerCase() === 'agent_missed',
                            newValue:
                                currentVersionType === 'inbound_phone' &&
                                currentVersion.completionStatus?.toLowerCase() === 'agent_missed',
                        }),
                        // Total Hold time
                        holdTime_all: calculateHoldTimeDelta({
                            oldValue:
                                previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone',
                            newValue: currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone',
                        }),
                        holdTime_inboundPhone: calculateHoldTimeDelta({
                            oldValue: previousVersionType === 'inbound_phone',

                            newValue: currentVersionType === 'inbound_phone',
                        }),
                        holdTime_outboundPhone: calculateHoldTimeDelta({
                            oldValue: previousVersionType === 'outbound_phone',

                            newValue: currentVersionType === 'outbound_phone',
                        }),
                        // Total Talk time
                        talkTime_all: calculateTalkTimeDelta({
                            oldValue:
                                previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone',
                            newValue: currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone',
                        }),
                        talkTime_inboundPhone: calculateTalkTimeDelta({
                            oldValue: previousVersionType === 'inbound_phone',

                            newValue: currentVersionType === 'inbound_phone',
                        }),
                        talkTime_outboundPhone: calculateTalkTimeDelta({
                            oldValue: previousVersionType === 'outbound_phone',

                            newValue: currentVersionType === 'outbound_phone',
                        }),
                        // Consult time
                        consultTime_all: calculateConsultTimeDelta({
                            oldValue:
                                previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone',
                            newValue: currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone',
                        }),
                        consultTime_inboundPhone: calculateConsultTimeDelta({
                            oldValue: previousVersionType === 'inbound_phone',

                            newValue: currentVersionType === 'inbound_phone',
                        }),
                        consultTime_outboundPhone: calculateConsultTimeDelta({
                            oldValue: previousVersionType === 'outbound_phone',

                            newValue: currentVersionType === 'outbound_phone',
                        }),
                        // Wrap time
                        wrapTime_all: calculateWrapTimeDelta({
                            oldValue:
                                previousVersionType === 'inbound_phone' || previousVersionType === 'outbound_phone',
                            newValue: currentVersionType === 'inbound_phone' || currentVersionType === 'outbound_phone',
                        }),
                        wrapTime_inboundPhone: calculateWrapTimeDelta({
                            oldValue: previousVersionType === 'inbound_phone',

                            newValue: currentVersionType === 'inbound_phone',
                        }),
                        wrapTime_outboundPhone: calculateWrapTimeDelta({
                            oldValue: previousVersionType === 'outbound_phone',

                            newValue: currentVersionType === 'outbound_phone',
                        }),
                        // Resolutions
                        totalResolutions: calculateTotalResolutionsDelta({
                            oldValue: previousVersion.statusChangeTo?.toLowerCase() === 'solved',
                            newValue: currentVersion.statusChangeTo?.toLowerCase() === 'solved',
                        }),
                    },
                },
            },
        ];
    }
};
