import { config } from '../../../config';
import { convertUpsertsToBulkWrite, log } from '../../../utils';
import calculateInteractionDeletedDiff from '../calculateInteractionDeletedDiff';
import * as mongodb from '../../../utils/mongodb';
import type { MongoInteractionMetric } from '@agyletime/performance-utils';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import type { KinesisStreamEvent } from 'aws-lambda';
import type { MongoQueryWithFilter } from '../../types';
import { isOrgEnabled } from '../../../utils';
import type { InteractionDeletedEvent } from '../interactionEvent.type';
import { extractInteractionEventParams } from '../extractInteractionEventParams';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../errors';

export const interactionDeletedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoInteractionMetric>[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionDeletedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionDeleted event ', event);

            const interaction = event.data.interaction;
            log.debug('InteractionDeleted data', { interaction });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interaction.organisationId)) return;

            const params = await extractInteractionEventParams(interaction);
            if (!params) {
                log.warn('No params returned extracted %j', { interaction });
                continue;
            }

            upserts.push(calculateInteractionDeletedDiff({ interaction, params }));
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoInteractionMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
