import { config } from '../../../config';
import { convertUpsertsToBulkWrite, log } from '../../../utils';
import calculateInteractionUpdatedDiff from '../calculateInteractionUpdatedDiff';
import * as mongodb from '../../../utils/mongodb';
import type { MongoInteractionMetric } from '@agyletime/performance-utils';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import type { KinesisStreamEvent } from 'aws-lambda';
import type { MongoQueryWithFilter } from '../../types';
import calculateInteractionCreatedDiff from '../calculateInteractionCreatedDiff';
import { isOrgEnabled } from '../../../utils';
import type { WorkstreamDefinitionsType } from '../../../workstreamDefinition';
import upsertWorkstreamDefinitions from '../../../workstreamDefinition/upsertWorkstreamDefinitions';
import { setUnnamedWorkstreamDefinitionName } from '../../utils';
import type { InteractionUpdatedEvent } from '../interactionEvent.type';
import { extractInteractionEventParams } from '../extractInteractionEventParams';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../errors';

export const interactionUpdatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoInteractionMetric>[] = [];
    const workstreamDefinitions: WorkstreamDefinitionsType[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionUpdatedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionUpdated event ', event);

            const interactionBefore = event.data.interactionBefore;
            const interactionAfter = event.data.interactionAfter;
            log.debug('InteractionUpdated data (interactionAfter)', { interactionAfter });
            log.debug('InteractionUpdated data (interactionBefore)', { interactionBefore });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interactionAfter.organisationId)) return;

            const workstreamDefinitionId =
                interactionAfter.workstreamDefinition?.id ?? interactionAfter.workstreamDefinitionId;
            if (workstreamDefinitionId) {
                workstreamDefinitions.push({
                    ...{
                        id: workstreamDefinitionId,
                        name:
                            interactionAfter.workstreamDefinition?.name ??
                            setUnnamedWorkstreamDefinitionName({ id: workstreamDefinitionId }),
                    },
                    organisationId: interactionAfter.organisationId,
                    createdDate: new Date(),
                });
            }

            const params = await extractInteractionEventParams(interactionAfter);

            if (!params) {
                log.warn('No params returned extracted %j', { interactionAfter });
                return;
            } else if (!interactionBefore.agent?.wfmId) {
                // If previous event didn't have a user id, then treat this as a creation
                upserts.push(calculateInteractionCreatedDiff({ interaction: interactionAfter, params }));
            } else {
                upserts.push(
                    // Remember there may be 2 updates returned here
                    ...calculateInteractionUpdatedDiff({
                        previousVersion: interactionBefore,
                        currentVersion: interactionAfter,
                        params,
                    }),
                );
            }
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoInteractionMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));

    if (workstreamDefinitions.length > 0) await upsertWorkstreamDefinitions(workstreamDefinitions);

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
