import { config } from '../../../config';
import { convertV2UpdateToBulkWrite, log } from '../../../utils';
import * as mongodb from '../../../utils/mongodb';
import type { KinesisStreamEvent } from 'aws-lambda';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { isOrgEnabled } from '../../../utils';
import type { WorkstreamDefinitionsType } from '../../../workstreamDefinition';
import { setUnnamedWorkstreamDefinitionName } from '../../utils';
import type { InteractionCreatedEvent } from '../interactionEvent.type';
import { extractInteractionEventParams } from '../extractInteractionEventParams';
import { BadRequestError } from '../../../errors';
import type { UpdateOneModel } from 'mongoose';
import calculateV2InteractionCreatedDiff from '../calculateV2InteractionCreatedDiff';
import type { V2MongoInteractionMetric } from '@agyletime/performance-utils';
import { V2MongoInteractionMetricModel } from '@agyletime/performance-utils';

export const v2InteractionCreatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    if (
        config.agyletime.clusterName &&
        config.agyletime.clusterName !== 'alpha' &&
        config.agyletime.clusterName !== 'gamma'
    )
        return;

    const upserts: UpdateOneModel<V2MongoInteractionMetric>[] = [];
    const workstreamDefinitions: WorkstreamDefinitionsType[] = [];
    const errors: Error[] = [];

    await mongodb.init(config.db);

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionCreatedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionCreated event ', { event });

            const interaction = event.data.interaction;
            log.debug('InteractionCreated data', { interaction });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interaction.organisationId)) return;

            const params = await extractInteractionEventParams(interaction);
            if (!params) {
                log.warn('No params returned extracted %j', { interaction });
                continue;
            }

            const workstreamDefinitionId = interaction.workstreamDefinition?.id ?? interaction.workstreamDefinitionId;
            if (workstreamDefinitionId) {
                workstreamDefinitions.push({
                    ...{
                        id: workstreamDefinitionId,
                        name:
                            interaction.workstreamDefinition?.name ??
                            setUnnamedWorkstreamDefinitionName({ id: workstreamDefinitionId }),
                    },
                    organisationId: params.organisationId,
                    createdDate: new Date(),
                });
            }

            upserts.push(calculateV2InteractionCreatedDiff({ interaction, params }));
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await V2MongoInteractionMetricModel.bulkWrite(convertV2UpdateToBulkWrite(upserts));

    // if (workstreamDefinitions.length > 0) await upsertWorkstreamDefinitions(workstreamDefinitions);

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
