import { config } from '../../../config';
import { convertV2UpdateToBulkWrite, log } from '../../../utils';
import * as mongodb from '../../../utils/mongodb';
import type { KinesisStreamEvent } from 'aws-lambda';
import { isOrgEnabled } from '../../../utils';
import type { InteractionDeletedEvent } from '../interactionEvent.type';
import { extractInteractionEventParams } from '../extractInteractionEventParams';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../errors';
import type { V2MongoInteractionMetric } from '@agyletime/performance-utils';
import { V2MongoInteractionMetricModel } from '@agyletime/performance-utils';
import calculateV2InteractionDeletedDiff from '../calculateV2InteractionDeletedDiff';
import type { UpdateOneModel } from 'mongoose';

export const v2InteractionDeletedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    if (
        config.agyletime.clusterName &&
        config.agyletime.clusterName !== 'alpha' &&
        config.agyletime.clusterName !== 'gamma'
    )
        return;

    await mongodb.init(config.db);

    const upserts: UpdateOneModel<V2MongoInteractionMetric>[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionDeletedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionDeleted event ', event);

            const interaction = event.data.interaction;
            log.debug('InteractionDeleted data', { interaction });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interaction.organisationId)) return;

            const params = await extractInteractionEventParams(interaction);
            if (!params) {
                log.warn('No params returned extracted %j', { interaction });
                continue;
            }

            upserts.push(calculateV2InteractionDeletedDiff({ interaction, params }));
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await V2MongoInteractionMetricModel.bulkWrite(convertV2UpdateToBulkWrite(upserts));

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
