import { config } from '../../../config';
import { convertV2UpdateToBulkWrite, log } from '../../../utils';
import * as mongodb from '../../../utils/mongodb';
import type { KinesisStreamEvent } from 'aws-lambda';
import { isOrgEnabled } from '../../../utils';
import type { WorkstreamDefinitionsType } from '../../../workstreamDefinition';
import { setUnnamedWorkstreamDefinitionName } from '../../utils';
import type { InteractionUpdatedEvent } from '../interactionEvent.type';
import { extractInteractionEventParams } from '../extractInteractionEventParams';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../errors';
import type { V2MongoInteractionMetric } from '@agyletime/performance-utils';
import { V2MongoInteractionMetricModel } from '@agyletime/performance-utils';
import calculateV2InteractionUpdatedDiff from '../calculateV2InteractionUpdatedDiff';
import type { UpdateOneModel } from 'mongoose';

export const v2InteractionUpdatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    if (
        config.agyletime.clusterName &&
        config.agyletime.clusterName !== 'alpha' &&
        config.agyletime.clusterName !== 'gamma'
    )
        return;

    await mongodb.init(config.db);

    const upserts: UpdateOneModel<V2MongoInteractionMetric>[] = [];
    const workstreamDefinitions: WorkstreamDefinitionsType[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionUpdatedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionUpdated event ', event);

            const interactionBefore = event.data.interactionBefore;
            const interactionAfter = event.data.interactionAfter;
            log.debug('InteractionUpdated data (interactionAfter)', { interactionAfter });
            log.debug('InteractionUpdated data (interactionBefore)', { interactionBefore });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interactionAfter.organisationId)) return;

            const workstreamDefinitionId =
                interactionAfter.workstreamDefinition?.id ?? interactionAfter.workstreamDefinitionId;
            if (workstreamDefinitionId) {
                workstreamDefinitions.push({
                    ...{
                        id: workstreamDefinitionId,
                        name:
                            interactionAfter.workstreamDefinition?.name ??
                            setUnnamedWorkstreamDefinitionName({ id: workstreamDefinitionId }),
                    },
                    organisationId: interactionAfter.organisationId,
                    createdDate: new Date(),
                });
            }

            const params = await extractInteractionEventParams(interactionAfter);

            if (!params) {
                log.warn('No params returned extracted %j', { interactionAfter });
                return;
            } else if (!interactionBefore.agent?.wfmId) {
                // If previous event didn't have a user id, then treat this as a creation
                // upserts.push(calculateV2InteractionCreatedDiff({ interaction: interactionAfter, params }));
            } else {
                upserts.push(
                    calculateV2InteractionUpdatedDiff({
                        interaction: interactionAfter,
                        params,
                    }),
                );
            }
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await V2MongoInteractionMetricModel.bulkWrite(convertV2UpdateToBulkWrite(upserts));

    // if (workstreamDefinitions.length > 0) await upsertWorkstreamDefinitions(workstreamDefinitions);

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
