import type { InteractionEventParams } from '../types';
import { InteractionEventParameterValidationError } from '../../errors';
import type { InteractionEventType } from './interactionEvent.type';
import UserHistoryProvider from '../../userHistory/UserHistoryProvider';
import { log } from '../../utils';

export const extractInteractionEventParams = async (
    interaction: InteractionEventType,
): Promise<InteractionEventParams | undefined> => {
    const interactionType = interaction.type;
    if (!interactionType) throw new InteractionEventParameterValidationError('Interaction type not supplied');

    if (!interaction.startTime)
        throw new InteractionEventParameterValidationError('Interaction start time not supplied');

    const workstreamIdentifier = interaction.workstreamIdentifier;
    if (!workstreamIdentifier) throw new InteractionEventParameterValidationError('Workstream identifier not supplied');

    const workstreamDefinitionId = interaction.workstreamDefinition?.id ?? interaction.workstreamDefinitionId;
    const workstreamDefinitionName = interaction.workstreamDefinition?.name ?? interaction.workstreamDefinitionName;
    // Not validated because we know that workstream definition id can be missing from an event

    const organisationId = interaction.organisationId;
    if (!organisationId) throw new InteractionEventParameterValidationError('Organisation ID not supplied');

    const userId = interaction.agent?.wfmId;
    if (!userId) {
        log.warn('No User ID provided in event: %j', { interaction });
        return;
    }

    let teamId = interaction.agent?.teamId;
    let timezone = interaction.agent?.timezone;

    if (!teamId || !timezone) {
        const userFromCache = await UserHistoryProvider.getUserFromCache({ organisationId, userId });

        if (!teamId) teamId = userFromCache?.teamId;
        if (!timezone) timezone = userFromCache?.timezone;
    }

    if (!teamId) throw new InteractionEventParameterValidationError('Team ID was not supplied');
    if (!timezone) throw new InteractionEventParameterValidationError('Timezone was not supplied');

    const taskTypeId = interaction.workloadTaskId;

    return {
        organisationId,
        userId,
        teamId,
        interactionType,
        workstreamIdentifier,
        workstreamDefinitionId,
        workstreamDefinitionName,
        timezone,
        taskTypeId,
    };
};
