import type { APIGatewayProxyEvent } from 'aws-lambda';
import { isBefore, isValid, parseISO } from 'date-fns';
import { BadRequestError } from '../../../errors';
import type { RebuildRosteredMetricsPostBody } from '../../rostered';
import { extractIdentity, getSSMParameters } from '@agyletime/wfm-aws-utils';
import { log } from '../../../utils';
import type { OrgDetails } from '../../types';

const ssmOrgs: OrgDetails[] = [];

export async function parseRequest(event: APIGatewayProxyEvent): Promise<{
    allOrgDetails: OrgDetails[];
    fromDate: Date;
    toDate: Date;
}> {
    const identity = extractIdentity(event);
    const organisationId = Number(identity.organisationId);
    const orgs: OrgDetails[] = [];

    if (!event.body) throw new BadRequestError('No params supplied');

    const requestBody = JSON.parse(event.body) as RebuildRosteredMetricsPostBody;
    if (!isValidPostBody(requestBody)) throw new BadRequestError('Invalid request');

    const fromDate = parseISO(requestBody.fromDate);
    if (!isValid(fromDate)) throw new BadRequestError('fromDate is invalid');
    const toDate = parseISO(requestBody.toDate);
    if (!isValid(toDate)) throw new BadRequestError('toDate is invalid');

    if (isBefore(toDate, fromDate)) throw new BadRequestError('Invalid date range');

    orgs.push(...(await getRelevantOrgs(organisationId)));

    return { allOrgDetails: orgs, fromDate, toDate };
}
export const isValidPostBody = (obj: unknown): obj is RebuildRosteredMetricsPostBody => {
    const body = obj as RebuildRosteredMetricsPostBody;
    return (
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.fromDate !== undefined &&
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.fromDate !== null &&
        body.fromDate !== '' &&
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.toDate !== undefined &&
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.toDate !== null &&
        body.toDate !== ''
    );
};

export async function getRelevantOrgs(organisationId?: number): Promise<OrgDetails[]> {
    if (!ssmOrgs.length) {
        ssmOrgs.push(...(await getSSMParameters()));
    } else {
        log.debug('getRelevantOrgs Using previously retrieved ssmOrgs', { allOrgDetails: ssmOrgs });
    }

    return ssmOrgs
        .filter((orgDetail) => !organisationId || orgDetail.id === organisationId)
        .map((orgDetail) => ({
            name: orgDetail.name,
            id: orgDetail.id,
            api: {
                agyletimeApiUrl: orgDetail.api?.agyletimeApiUrl ?? '',
                apiGatewayUrl: orgDetail.api?.apiGatewayUrl ?? '',
                organisationId: orgDetail.api?.organisationId ?? 0,
            },
        }));
}
