// noinspection JSUnusedGlobalSymbols
// Used in other Performance repos.

import type { InferSchemaType, Model } from 'mongoose';
import mongoose, { Schema } from 'mongoose';

const RosteredMetrics = {
    rosteredAvailableTime_rosteredHours: { type: Number },
    rosteredPaidShrinkageDuration_rosteredHours: { type: Number },
    rosteredPaidTasks_rosteredHours: { type: Number },
    rosteredProductiveTasks_rosteredHours: { type: Number },
    rosteredShrinkageDuration_rosteredHours: { type: Number },

    totalNumberOfShifts_numberOfShifts: { type: Number },
    additionalHoursNumberOfShifts_numberOfShifts: { type: Number },
    standardShiftNumberOfShifts_numberOfShifts: { type: Number },

    plannedLeaveDuration_leave: { type: Number },
    totalLeave_leave: { type: Number },
    unplannedLeaveDuration_leave: { type: Number },

    rosteredStartTime_rosteredHours: { type: Date },
    rosteredEndTime_rosteredHours: { type: Date },
    taskTypeId: { type: Number },
    taskTypeName: { type: String },
};

export enum PerformanceMetricTypes {
    interaction = 'interaction',
    rostered = 'rostered',
    workstream = 'workstream',
}

const schema = new Schema(
    {
        metricType: {
            type: String,
            enum: [
                PerformanceMetricTypes.interaction,
                PerformanceMetricTypes.rostered,
                PerformanceMetricTypes.workstream,
            ],
            require: true,
        },
        timeBucket: { type: Date },
        year: { type: Number, required: true },
        month: { type: Number, required: true },
        day: { type: Number, required: true },
        hour: { type: Number, required: true },
        minute: { type: Number, required: true },
        utc: {
            required: true,
            type: {
                year: { type: Number, required: true },
                month: { type: Number, required: true },
                day: { type: Number, required: true },
                hour: { type: Number, required: true },
                minute: { type: Number, required: true },
                _id: false,
            },
        },
        organisationId: { type: Number, required: true },
        userId: { type: Number },
        teamId: { type: Number },
        interactionType: { type: String },
        workstreamIdentifier: { type: String },
        // workstreamGroup: { type: String },
        workstreamDefinitionId: { type: Number },
        workstreamDefinitionName: { type: String },
        shiftId: { type: String },
        // rosteredShiftStartTime: { type: String },
        interactionMetrics: {
            // # of Interactions
            numberOfInteractions_all: { type: Number },
            numberOfInteractions_chat: { type: Number },
            numberOfInteractions_ticket: { type: Number },
            numberOfInteractions_inboundPhone: { type: Number },
            numberOfInteractions_outboundPhone: { type: Number },
            // Total Interaction Handle Time
            totalInteractionHandleTime_all: { type: Number },
            totalInteractionHandleTime_chat: { type: Number },
            totalInteractionHandleTime_ticket: { type: Number },
            totalInteractionHandleTime_inboundPhone: { type: Number },
            totalInteractionHandleTime_outboundPhone: { type: Number },
            // # Reassigns
            totalReassigns_all: { type: Number },
            totalReassigns_chat: { type: Number },
            totalReassigns_ticket: { type: Number },
            totalReassigns_inboundPhone: { type: Number },
            totalReassigns_outboundPhone: { type: Number },
            // # Private Comments
            totalPrivateComments_all: { type: Number },
            totalPrivateComments_ticket: { type: Number },
            // # Public Comments
            totalPublicComments_all: { type: Number },
            totalPublicComments_ticket: { type: Number },
            // # Declined
            totalDeclined_all: { type: Number },
            totalDeclined_chat: { type: Number },
            totalDeclined_ticket: { type: Number },
            totalDeclined_inboundPhone: { type: Number },
            totalDeclined_outboundPhone: { type: Number },
            // # Assigned
            totalAssigned_all: { type: Number },
            totalAssigned_chat: { type: Number },
            totalAssigned_inboundPhone: { type: Number },
            // # Self-assigned
            totalSelfAssignedInteractions_all: { type: Number },
            totalSelfAssignedInteractions_chat: { type: Number },
            // # Missed
            totalMissed_all: { type: Number },
            totalMissed_chat: { type: Number },
            totalMissed_inboundPhone: { type: Number },
            // Total Hold time
            holdTime_all: { type: Number },
            holdTime_inboundPhone: { type: Number },
            holdTime_outboundPhone: { type: Number },
            // Total Talk time
            talkTime_all: { type: Number },
            talkTime_inboundPhone: { type: Number },
            talkTime_outboundPhone: { type: Number },
            // Consult time
            consultTime_all: { type: Number },
            consultTime_inboundPhone: { type: Number },
            consultTime_outboundPhone: { type: Number },
            // Wrap time
            wrapTime_all: { type: Number },
            wrapTime_inboundPhone: { type: Number },
            wrapTime_outboundPhone: { type: Number },
            // Offered
            totalOffered_all: { type: Number },
            totalOffered_chat: { type: Number },
            totalOffered_inboundPhone: { type: Number },
            // Total resolutions
            totalResolutions: { type: Number },
        },
        scheduleMetrics: {
            ...RosteredMetrics,
        },
        workstreamMetrics: {
            csatPositiveResponses: { type: Number },
            csatTotalResponses: { type: Number },
        },
    },
    {
        collection: 'performance_metrics',
    },
);

// TPM queries
schema.index({
    organisationId: 1,
    teamId: 1,
    year: 1,
    month: 1,
    day: 1,
    hour: 1,
    minute: 1,
});

schema.index({
    organisationId: 1,
    teamId: 1,
    timeBucket: 1,
});

// This should be used by:
// taskDataMerged: https://bitbucket.org/agylelabs/performance-commands/src/d85e51c6068180db640dbdc104b39b82df99064e/lib/metrics/performanceMetrics/taskDataMergedHandler.ts#lib/metrics/performanceMetrics/taskDataMergedHandler.ts-56
// shiftDeleted: https://bitbucket.org/agylelabs/performance-commands/src/66acf107f56f62d130cf7e939080ca1ab1a9e944/lib/metrics/performanceMetrics/schedule/eventHandlers/shiftDeletedHandler.ts#lib/metrics/performanceMetrics/schedule/eventHandlers/shiftDeletedHandler.ts-40
schema.index({
    organisationId: 1,
    metricType: 1,
    shiftId: 1,
    userId: 1,
});

// Upsert
schema.index({
    organisationId: 1,
    userId: 1,
    teamId: 1,
    year: 1,
    month: 1,
    day: 1,
    hour: 1,
    minute: 1,
});

// UPM queries
schema.index({
    organisationId: 1,
    userId: 1,
    year: 1,
    month: 1,
    day: 1,
    hour: 1,
    minute: 1,
});
export type MongoPerformanceMetric = InferSchemaType<typeof schema>;

export type InteractionPerformanceMetric = Omit<MongoPerformanceMetric, 'scheduleMetrics' | 'workstreamMetrics'> &
    Required<Pick<MongoPerformanceMetric, 'interactionMetrics'>>;
export type SchedulePerformanceMetric = Omit<
    MongoPerformanceMetric,
    'interactionType' | 'workstreamIdentifier' | 'workstreamDefinitionId' | 'interactionMetrics' | 'workstreamMetrics'
> &
    Required<Pick<MongoPerformanceMetric, 'scheduleMetrics'>>;
export type WorkstreamMetricPerformanceMetric = Omit<
    MongoPerformanceMetric,
    'interactionType' | 'interactionMetrics' | 'scheduleMetrics'
> &
    Required<Pick<MongoPerformanceMetric, 'workstreamMetrics'>>;
export const MongoPerformanceMetricModel: Model<MongoPerformanceMetric> = mongoose.model(
    'MongoPerformanceMetricModel',
    schema,
);
