import type { Document, InsertOneModel, BulkWriteResult } from 'mongodb';
import { ObjectId } from 'mongodb';
import { log } from '../../../lib/utils';
import { config } from '../../config';
import * as mongodb from '../../utils/mongodb';
import type { Collection } from 'mongoose';
import type { MongoPerformanceMetric } from './MongoPerformanceMetricModel';
import { MongoPerformanceMetricModel, PerformanceMetricTypes } from './MongoPerformanceMetricModel';

MongoPerformanceMetricModel.schema.index({
    metricType: 1,
    migrated: 1,
});
type OriginalMongoDocument = MongoPerformanceMetric & { _id: string };
interface MigrationResults {
    metricsMigrated: number;
}
export const separateCollectionsMigration = async (): Promise<MigrationResults> => {
    const mongo = await mongodb.init(config.db);

    log.info('Finding documents to migrate.');
    const original = await getOriginalMetrics();

    const insertSets = new Map();

    log.info('Creating documents for bulk inserts');
    for (const document of original) {
        try {
            const leanDocument = await getLeanDocument(document, mongo);
            if (!leanDocument) continue;

            // We originally marked it migrated after the bulk insert.
            // But we mark it before now so as not to get stuck in a loop trying to process errored ones.
            await markOriginalDocumentAsMigrated(document);

            createInsertDocument({
                // @ts-expect-error - This is addressed above
                metricType: leanDocument.metricType,
                document: leanDocument,
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                insertSets,
            });
        } catch (error: unknown) {
            await logError('Error creating document %j', error as Error, mongo);
        }
    }

    for (const [key, set] of insertSets.entries()) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        const collection = getTargetCollection({ key, mongo });
        if (!collection) continue;

        try {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            const bulkWriteResult = await collection.bulkWrite(set, { ordered: false });
            log.info(`Bulk write result: %j`, bulkWriteResult);
            await logResults({ mongo, results: bulkWriteResult, metricCollection: collection });
        } catch (error: unknown) {
            await logError(`Error in bulkWrite: %j`, error as Error, mongo, collection);
        }
    }

    log.info(`Completed.`);

    return {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        metricsMigrated: original.length ?? 0,
    };
};

function getTargetCollection({
    key,
    mongo,
}: {
    key: PerformanceMetricTypes;
    mongo: typeof import('mongoose');
}): Collection | undefined {
    let collection;
    if (key === PerformanceMetricTypes.interaction) {
        collection = mongo.connection.useDb('performance').collection('interaction_metrics');
        log.info(`Performing bulk write for interaction_metrics`);
    } else if (key === PerformanceMetricTypes.rostered) {
        collection = mongo.connection.useDb('performance').collection('rostered_metrics');
        log.info(`Performing bulk write for rostered_metrics`);
    } else {
        log.info(`Key does not match ${PerformanceMetricTypes.interaction} or ${PerformanceMetricTypes.rostered}.`);
    }
    return collection;
}

async function markOriginalDocumentAsMigrated(document: Document): Promise<void> {
    // Mark document as migrated
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call
    document['set']('migrated', true, { strict: false });
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call
    await document['save']();
}

async function getLeanDocument(
    document: Document,
    mongo: typeof import('mongoose'),
): Promise<OriginalMongoDocument | false> {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call
    const leanDocument: OriginalMongoDocument = document['toObject']();
    if (!leanDocument.metricType) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        await logError(`No metricType on ${(leanDocument as Document)['_id'].toString()}`, undefined, mongo);
        return false;
    }
    return leanDocument;
}

async function logError(
    description: string,
    error: Error | undefined,
    mongo: typeof import('mongoose'),
    metricCollection?: Collection | undefined,
): Promise<void> {
    log.error(`${description}: %j`, error, mongo);
    const collection = mongo.connection.useDb('performance').collection('metrics_migration_log');
    await collection.insertOne({
        collection: metricCollection?.collectionName,
        errors: {
            description,
            error: { ...error },
        },
    });
}
async function logResults({
    mongo,
    results,
    metricCollection,
}: {
    mongo: typeof import('mongoose');
    results: BulkWriteResult;
    metricCollection: Collection;
}): Promise<void> {
    const collection = mongo.connection.useDb('performance').collection('metrics_migration_log');
    await collection.insertOne({
        collection: metricCollection.collectionName,
        results,
        errors: results.hasWriteErrors() && results.getWriteErrors(),
    });
}

async function getOriginalMetrics(): Promise<Document[]> {
    const query = {
        metricType: { $in: [PerformanceMetricTypes.rostered, PerformanceMetricTypes.interaction] },
        utc: { $exists: true },
        year: { $exists: true },
        month: { $exists: true },
        day: { $exists: true },
        hour: { $exists: true },
        minute: { $exists: true },
        // Items not already migrated
        migrated: { $ne: true },
    };
    log.info(`Running query: %j`, query);
    return await MongoPerformanceMetricModel.find(query).limit(Number(process.env['COLLECTION_MIGRATION_PAGE_LIMIT']));
}

function createInsertDocument({
    metricType,
    document,
    insertSets,
}: {
    metricType: PerformanceMetricTypes;
    document: OriginalMongoDocument;
    insertSets: Map<PerformanceMetricTypes, { insertOne: InsertOneModel }[]>;
}): void {
    const newDocument = {
        _id: new ObjectId(document._id),
        timeBucketAsLocal: new Date(
            Date.UTC(document.year, document.month - 1, document.day, document.hour, document.minute),
        ),
        timeBucketAsUtc: new Date(
            Date.UTC(
                document.utc.year,
                document.utc.month - 1,
                document.utc.day,
                document.utc.hour,
                document.utc.minute,
            ),
        ),
        organisationId: document.organisationId,
        userId: document.userId,
        teamId: document.teamId,
        ...(document.interactionType && { interactionType: document.interactionType }),
        ...(document.workstreamIdentifier && { workstreamIdentifier: document.workstreamIdentifier }),
        ...(document.workstreamDefinitionId && { workstreamDefinitionId: document.workstreamDefinitionId }),
        ...(document.workstreamDefinitionName && { workstreamDefinitionName: document.workstreamDefinitionName }),
        ...(document.shiftId && { shiftId: document.shiftId }),
        ...document.interactionMetrics,
        ...document.scheduleMetrics,
        ...document.workstreamMetrics,
        migrated: true,
    };

    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
    const insertSet = insertSets.get(metricType) || [];
    insertSet.push({
        insertOne: {
            document: newDocument,
        },
    });
    insertSets.set(metricType, insertSet);
}
