import type { MongoQueryWithFilter, ShiftCreatedEvent, ShiftEventParams, ShiftTask } from '../types';
import { toZonedTime } from 'date-fns-tz';
import { log } from '../../utils';
import type { MongoRosteredMetric } from '@agyletime/performance-utils';
import { config } from '../../config';
import { parseISO } from 'date-fns';
import { roundDateToNearestTime } from '../../utils';

export default ({
    newShiftEvent,
    params,
}: {
    newShiftEvent: Omit<ShiftCreatedEvent, 'roster'>;
    params: ShiftEventParams;
}): MongoQueryWithFilter<MongoRosteredMetric>[] => {
    const upserts: MongoQueryWithFilter<MongoRosteredMetric>[] = [];

    const newShift = newShiftEvent.shift;

    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    for (const shiftTask of newShift.shiftTasks ?? []) {
        const diff = calculateCreateDiffForShiftTask({ shiftTask, shift: newShift, params });
        upserts.push(diff);
    }

    return upserts;
};

export const calculateCreateDiffForShiftTask = ({
    shiftTask,
    shift,
    params,
}: {
    shiftTask: ShiftTask;
    shift: { startTime: string; endTime: string; isAdditionalHour?: boolean };
    params: ShiftEventParams;
}): MongoQueryWithFilter<MongoRosteredMetric> => {
    const rosteredTaskDuration =
        (new Date(shiftTask.endTime).getTime() - new Date(shiftTask.startTime).getTime()) / 1000;
    const shiftTaskStartTimeAsUser = toZonedTime(shiftTask.startTime, params.timezone);
    const currentTimeBucket = roundDateToNearestTime(
        new Date(shiftTaskStartTimeAsUser),
        config.mongoBucketSizeInMinutes,
    );
    const currentTimeBucketUtc = roundDateToNearestTime(parseISO(shiftTask.startTime), config.mongoBucketSizeInMinutes);
    if (Number.isNaN(currentTimeBucket.getTime())) {
        log.error('Failed to generate timebucket', params);
        throw new Error('Failed to generate timebucket', {
            cause: {
                message: 'Possibly bad params (e.g. timezone)',
                params,
            },
        });
    }

    const incrementObject: Partial<MongoRosteredMetric> = {
        rosteredAvailableTime_rosteredHours:
            shiftTask.taskType.isAvailableOccupancy && !shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,
        rosteredPaidShrinkageDuration_rosteredHours:
            shiftTask.taskType.isPaid && !shiftTask.taskType.isLeave && !shiftTask.taskType.isProductive
                ? rosteredTaskDuration
                : 0,
        rosteredPaidTasks_rosteredHours:
            shiftTask.taskType.isPaid && !shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,
        rosteredProductiveTasks_rosteredHours:
            shiftTask.taskType.isProductive && !shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,
        rosteredShrinkageDuration_rosteredHours:
            !shiftTask.taskType.isProductive && !shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,

        totalNumberOfShifts_numberOfShifts: shiftTask.taskType.isLeave ? 0 : 1,
        additionalHoursNumberOfShifts_numberOfShifts: shift.isAdditionalHour ? 1 : 0,
        standardShiftNumberOfShifts_numberOfShifts: shiftTask.taskType.additionalHoursOverride ? 0 : 1,
        plannedLeaveDuration_leave:
            shiftTask.taskType.isPlanned && shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,
        totalLeave_leave: shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,
        unplannedLeaveDuration_leave:
            !shiftTask.taskType.isPlanned && shiftTask.taskType.isLeave ? rosteredTaskDuration : 0,
    };

    const upsert = {
        filter: {
            organisationId: params.organisationId,
            userId: params.userId,
            shiftId: params.shiftId,
            rosteredStartTime_rosteredHours: new Date(shiftTask.startTime),
            rosteredEndTime_rosteredHours: new Date(shiftTask.endTime),
            taskTypeId: shiftTask.taskType.id,
            timeBucketAsUtc: currentTimeBucketUtc,
            timeBucketAsLocal: currentTimeBucket,
        },
        query: {
            $set: {
                organisationId: params.organisationId,
                userId: params.userId,
                teamId: params.teamId,
                shiftId: params.shiftId,
                rosteredStartTime_rosteredHours: new Date(shiftTask.startTime),
                rosteredEndTime_rosteredHours: new Date(shiftTask.endTime),
                taskTypeId: shiftTask.taskType.id,
                taskTypeName: shiftTask.taskType.name,
                timeBucketAsUtc: currentTimeBucketUtc,
                timeBucketAsLocal: currentTimeBucket,
            },
            $inc: incrementObject,
        },
    };
    log.debug({ upsert });
    return upsert;
};
