import type { Shift, ShiftEventParams, ShiftTask, Upsert } from '../types';
import { getDurationOfShiftTask } from '../utils';
import { config } from '../../config';
import { isEqual, parseISO } from 'date-fns';
import { calculateCreateDiffForShiftTask } from './calculateShiftCreatedDiff';
import { calculateDeleteDiffForShiftTask } from './calculateShiftDeletedDiff';
import { toZonedTime } from 'date-fns-tz';
import type { MongoRosteredMetric } from '@agyletime/performance-utils';
import { roundDateToNearestTime } from '../../utils';

export const getMissingShiftTasks = (a: ShiftTask[], b: ShiftTask[]): ShiftTask[] => {
    return a.filter((shiftTaskA) => {
        return !b.some((shiftTaskB) => shiftTaskA._id === shiftTaskB._id);
    });
};
export const getMatchingShiftTasks = (a: ShiftTask[], b: ShiftTask[]): ShiftTask[] => {
    return a.filter((shiftTaskA) => {
        return b.some((shiftTaskB) => shiftTaskB._id === shiftTaskA._id);
    });
};

export default ({
    previousVersion,
    currentVersion,
    params,
}: {
    previousVersion: Shift;
    currentVersion: Shift;
    params: ShiftEventParams;
}): Upsert<MongoRosteredMetric>[] => {
    const upserts: Upsert<MongoRosteredMetric>[] = [];

    const deletedShiftTasks: ShiftTask[] = getMissingShiftTasks(previousVersion.shiftTasks, currentVersion.shiftTasks);
    const createdShiftTasks: ShiftTask[] = getMissingShiftTasks(currentVersion.shiftTasks, previousVersion.shiftTasks);
    const updatedShiftTasks: ShiftTask[] = getMatchingShiftTasks(currentVersion.shiftTasks, previousVersion.shiftTasks);

    for (const shiftTask of createdShiftTasks) {
        const diff = calculateCreateDiffForShiftTask({ shiftTask, shift: currentVersion, params });
        upserts.push(diff);
    }
    for (const shiftTask of deletedShiftTasks) {
        const diff = calculateDeleteDiffForShiftTask({ shiftTask, shift: previousVersion, params });
        upserts.push(diff);
    }

    for (const shiftTask of updatedShiftTasks) {
        const matchingPreviousShiftTask = previousVersion.shiftTasks.find((s) => s._id == shiftTask._id);

        if (matchingPreviousShiftTask) {
            const previousShiftTaskStartTimeAsUser = toZonedTime(matchingPreviousShiftTask.startTime, params.timezone);
            const previousTimeBucket = roundDateToNearestTime(
                new Date(previousShiftTaskStartTimeAsUser),
                config.mongoBucketSizeInMinutes,
            );
            const previousTimeBucketUtc = roundDateToNearestTime(
                parseISO(matchingPreviousShiftTask.startTime),
                config.mongoBucketSizeInMinutes,
            );
            const shiftTaskStartTimeAsUser = toZonedTime(shiftTask.startTime, params.timezone);
            const currentTimeBucket = roundDateToNearestTime(
                new Date(shiftTaskStartTimeAsUser),
                config.mongoBucketSizeInMinutes,
            );
            const currentTimeBucketUtc = roundDateToNearestTime(
                parseISO(shiftTask.startTime),
                config.mongoBucketSizeInMinutes,
            );
            const changingBuckets = !isEqual(previousTimeBucket, currentTimeBucket);

            const previousRosteredTaskDuration = getDurationOfShiftTask(matchingPreviousShiftTask);
            const currentRosteredTaskDuration = getDurationOfShiftTask(shiftTask);
            const rosteredTaskDurationDelta = currentRosteredTaskDuration - previousRosteredTaskDuration;

            const wasRosteredAvailableTime =
                matchingPreviousShiftTask.taskType.isAvailableOccupancy && !matchingPreviousShiftTask.taskType.isLeave;
            const isRosteredAvailableTime = shiftTask.taskType.isAvailableOccupancy && !shiftTask.taskType.isLeave;

            const wasRosteredPaidShrinkage =
                matchingPreviousShiftTask.taskType.isPaid &&
                !matchingPreviousShiftTask.taskType.isLeave &&
                !matchingPreviousShiftTask.taskType.isProductive;
            const isRosteredPaidShrinkage =
                shiftTask.taskType.isPaid && !shiftTask.taskType.isLeave && !shiftTask.taskType.isProductive;

            const wasRosteredPaidTask =
                matchingPreviousShiftTask.taskType.isPaid && !matchingPreviousShiftTask.taskType.isLeave;
            const isRosteredPaidTask = shiftTask.taskType.isPaid && !shiftTask.taskType.isLeave;

            const wasRosteredProductiveTask =
                matchingPreviousShiftTask.taskType.isProductive && !matchingPreviousShiftTask.taskType.isLeave;
            const isRosteredProductiveTask = shiftTask.taskType.isProductive && !shiftTask.taskType.isLeave;

            const wasRosteredShrinkage =
                !matchingPreviousShiftTask.taskType.isProductive && !matchingPreviousShiftTask.taskType.isLeave;
            const isRosteredShrinkage = !shiftTask.taskType.isProductive && !shiftTask.taskType.isLeave;

            const wasPlannedScheduledLeave =
                matchingPreviousShiftTask.taskType.isPlanned && matchingPreviousShiftTask.taskType.isLeave;
            const isPlannedScheduledLeave = shiftTask.taskType.isPlanned && shiftTask.taskType.isLeave;

            const wasLeave = matchingPreviousShiftTask.taskType.isLeave;
            const isLeave = shiftTask.taskType.isLeave;

            const wasUnplannedLeave =
                !matchingPreviousShiftTask.taskType.isPlanned && matchingPreviousShiftTask.taskType.isLeave;
            const isUnplannedLeave = !shiftTask.taskType.isPlanned && shiftTask.taskType.isLeave;

            if (changingBuckets) {
                upserts.push(
                    ...[
                        // Previous bucket
                        {
                            filter: {
                                organisationId: params.organisationId,
                                userId: previousVersion.user.id,
                                shiftId: params.shiftId,
                                rosteredStartTime_rosteredHours: new Date(matchingPreviousShiftTask.startTime),
                                rosteredEndTime_rosteredHours: new Date(matchingPreviousShiftTask.endTime),
                                taskTypeId: matchingPreviousShiftTask.taskType.id,
                            },
                            query: {
                                $set: {
                                    organisationId: params.organisationId,
                                    userId: previousVersion.user.id,
                                    teamId: params.teamId,
                                    shiftId: params.shiftId,
                                    rosteredStartTime_rosteredHours: new Date(matchingPreviousShiftTask.startTime),
                                    rosteredEndTime_rosteredHours: new Date(matchingPreviousShiftTask.endTime),
                                    taskTypeId: matchingPreviousShiftTask.taskType.id,
                                    taskTypeName: matchingPreviousShiftTask.taskType.name,
                                    timeBucketAsUtc: previousTimeBucketUtc,
                                    timeBucketAsLocal: previousTimeBucket,
                                },
                                $inc: {
                                    rosteredAvailableTime_rosteredHours: wasRosteredAvailableTime
                                        ? -previousRosteredTaskDuration
                                        : 0,
                                    rosteredPaidShrinkageDuration_rosteredHours: wasRosteredPaidShrinkage
                                        ? -previousRosteredTaskDuration
                                        : 0,
                                    rosteredPaidTasks_rosteredHours: wasRosteredPaidTask
                                        ? -previousRosteredTaskDuration
                                        : 0,
                                    rosteredProductiveTasks_rosteredHours: wasRosteredProductiveTask
                                        ? -previousRosteredTaskDuration
                                        : 0,
                                    rosteredShrinkageDuration_rosteredHours: wasRosteredShrinkage
                                        ? -previousRosteredTaskDuration
                                        : 0,
                                    totalNumberOfShifts_numberOfShifts: wasLeave ? 0 : -1,
                                    additionalHoursNumberOfShifts_numberOfShifts: previousVersion.isAdditionalHour
                                        ? -1
                                        : 0,
                                    standardShiftNumberOfShifts_numberOfShifts: matchingPreviousShiftTask.taskType
                                        .additionalHoursOverride
                                        ? 0
                                        : -1,
                                    plannedLeaveDuration_leave: wasPlannedScheduledLeave
                                        ? -previousRosteredTaskDuration
                                        : 0,
                                    totalLeave_leave: wasLeave ? -previousRosteredTaskDuration : 0,
                                    unplannedLeaveDuration_leave: wasUnplannedLeave ? -previousRosteredTaskDuration : 0,
                                },
                            },
                        },
                        // New bucket
                        {
                            filter: {
                                organisationId: params.organisationId,
                                userId: params.userId,
                                shiftId: params.shiftId,
                                rosteredStartTime_rosteredHours: new Date(shiftTask.startTime),
                                rosteredEndTime_rosteredHours: new Date(shiftTask.endTime),
                                taskTypeId: shiftTask.taskType.id,
                                timeBucketAsUtc: currentTimeBucketUtc,
                                timeBucketAsLocal: currentTimeBucket,
                            },
                            query: {
                                $set: {
                                    organisationId: params.organisationId,
                                    userId: params.userId,
                                    teamId: params.teamId,
                                    shiftId: params.shiftId,
                                    rosteredStartTime_rosteredHours: new Date(currentVersion.startTime),
                                    rosteredEndTime_rosteredHours: new Date(currentVersion.endTime),
                                    taskTypeId: shiftTask.taskType.id,
                                    taskTypeName: shiftTask.taskType.name,
                                    timeBucketAsUtc: currentTimeBucketUtc,
                                    timeBucketAsLocal: currentTimeBucket,
                                },
                                $inc: {
                                    rosteredAvailableTime_rosteredHours: isRosteredAvailableTime
                                        ? currentRosteredTaskDuration
                                        : 0,
                                    rosteredPaidShrinkageDuration_rosteredHours: isRosteredPaidShrinkage
                                        ? currentRosteredTaskDuration
                                        : 0,
                                    rosteredPaidTasks_rosteredHours: isRosteredPaidTask
                                        ? currentRosteredTaskDuration
                                        : 0,
                                    rosteredProductiveTasks_rosteredHours: isRosteredProductiveTask
                                        ? currentRosteredTaskDuration
                                        : 0,
                                    rosteredShrinkageDuration_rosteredHours: isRosteredShrinkage
                                        ? currentRosteredTaskDuration
                                        : 0,
                                    totalNumberOfShifts_numberOfShifts: isLeave ? 0 : 1,
                                    additionalHoursNumberOfShifts_numberOfShifts: currentVersion.isAdditionalHour
                                        ? 1
                                        : 0,
                                    standardShiftNumberOfShifts_numberOfShifts: shiftTask.taskType
                                        .additionalHoursOverride
                                        ? 0
                                        : 1,
                                    plannedLeaveDuration_leave: isPlannedScheduledLeave
                                        ? currentRosteredTaskDuration
                                        : 0,
                                    totalLeave_leave: isLeave ? currentRosteredTaskDuration : 0,
                                    unplannedLeaveDuration_leave: isUnplannedLeave ? currentRosteredTaskDuration : 0,
                                },
                            },
                        },
                    ],
                );
            } else {
                const calculateShiftDurationDelta = (oldValue: boolean, newValue: boolean): number => {
                    if (oldValue && newValue) return rosteredTaskDurationDelta;
                    if (oldValue && !newValue) return -previousRosteredTaskDuration;
                    if (!oldValue && newValue) return rosteredTaskDurationDelta + previousRosteredTaskDuration;

                    return 0;
                };

                const calculateIncrementDelta = (oldValue: boolean, newValue: boolean): number => {
                    if (oldValue && !newValue) return -1;
                    if (!oldValue && newValue) return 1;

                    return 0;
                };

                upserts.push({
                    filter: {
                        organisationId: params.organisationId,
                        userId: previousVersion.user.id,
                        shiftId: params.shiftId,
                        rosteredStartTime_rosteredHours: new Date(shiftTask.startTime),
                        rosteredEndTime_rosteredHours: new Date(shiftTask.endTime),
                        taskTypeId: shiftTask.taskType.id,
                        timeBucketAsUtc: currentTimeBucketUtc,
                        timeBucketAsLocal: currentTimeBucket,
                    },
                    query: {
                        $set: {
                            organisationId: params.organisationId,
                            userId: params.userId,
                            teamId: params.teamId,
                            shiftId: params.shiftId,
                            rosteredStartTime_rosteredHours: new Date(shiftTask.startTime),
                            rosteredEndTime_rosteredHours: new Date(shiftTask.endTime),
                            taskTypeId: shiftTask.taskType.id,
                            taskTypeName: shiftTask.taskType.name,
                            timeBucketAsUtc: currentTimeBucketUtc,
                            timeBucketAsLocal: currentTimeBucket,
                        },
                        $inc: {
                            rosteredAvailableTime_rosteredHours: calculateShiftDurationDelta(
                                wasRosteredAvailableTime,
                                isRosteredAvailableTime,
                            ),
                            rosteredPaidShrinkageDuration_rosteredHours: calculateShiftDurationDelta(
                                wasRosteredPaidShrinkage,
                                isRosteredPaidShrinkage,
                            ),
                            rosteredPaidTasks_rosteredHours: calculateShiftDurationDelta(
                                wasRosteredPaidTask,
                                isRosteredPaidTask,
                            ),
                            rosteredProductiveTasks_rosteredHours: calculateShiftDurationDelta(
                                wasRosteredProductiveTask,
                                isRosteredProductiveTask,
                            ),
                            rosteredShrinkageDuration_rosteredHours: calculateShiftDurationDelta(
                                wasRosteredShrinkage,
                                isRosteredShrinkage,
                            ),
                            totalNumberOfShifts_numberOfShifts: calculateIncrementDelta(
                                shiftTask.taskType.isLeave,
                                matchingPreviousShiftTask.taskType.isLeave,
                            ),
                            additionalHoursNumberOfShifts_numberOfShifts: calculateIncrementDelta(
                                // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
                                previousVersion.isAdditionalHour || false,
                                // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
                                currentVersion.isAdditionalHour || false,
                            ),
                            standardShiftNumberOfShifts_numberOfShifts: calculateIncrementDelta(
                                shiftTask.taskType.additionalHoursOverride,
                                matchingPreviousShiftTask.taskType.additionalHoursOverride,
                            ),
                            plannedLeaveDuration_leave: calculateShiftDurationDelta(
                                wasPlannedScheduledLeave,
                                isPlannedScheduledLeave,
                            ),
                            totalLeave_leave: calculateShiftDurationDelta(wasLeave, isLeave),
                            unplannedLeaveDuration_leave: calculateShiftDurationDelta(
                                wasUnplannedLeave,
                                isUnplannedLeave,
                            ),
                        },
                    },
                });
            }
        }
    }

    return upserts;
};
