import { config } from '../../../config';
import { log, convertUpsertsToBulkWrite } from '../../../utils';
import type { MongoQueryWithFilter, ShiftCreatedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { createShiftMetric } from '../createShiftMetrics';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../../utils';
import { MongoRosteredMetricModel, type MongoRosteredMetric } from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';

export const shiftCreatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoRosteredMetric>[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            log.debug('rostered -> shiftCreated event ', record);

            const event = decodeKinesisPayload<AgyleTimeCloudEvent<ShiftCreatedEvent>>({ data: record.kinesis.data });
            log.debug('rostered -> shiftCreated decodedPayload', { decodedPayload: event });

            const { data, agyletimeorganisationId } = event;
            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(agyletimeorganisationId.toString()))) {
                log.info('rostered -> shiftCreated org disabled', { agyletimeorganisationId });
                return;
            } else {
                log.info('rostered -> shiftCreated org enabled', { agyletimeorganisationId });
            }

            log.debug('rostered -> shiftCreated data', { data });

            upserts.push(...(await createShiftMetric(data, agyletimeorganisationId)));
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoRosteredMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
