import { config } from '../../../config';
import { log, convertDeletesToBulkWrite } from '../../../utils';
import type { Delete, ShiftDeletedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { deleteShiftMetrics } from '../deleteShiftMetrics';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../../utils';
import type { MongoRosteredMetric } from '@agyletime/performance-utils';
import { MongoRosteredMetricModel } from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';

export const shiftDeletedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const deletes: Delete<MongoRosteredMetric>[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            log.debug('rostered -> shiftDeleted event ', record);

            const event = decodeKinesisPayload<AgyleTimeCloudEvent<ShiftDeletedEvent>>({ data: record.kinesis.data });
            log.debug('rostered -> shiftDeleted decodedPayload', { decodedPayload: event });

            const { data, agyletimeorganisationId } = event;
            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(agyletimeorganisationId.toString()))) {
                log.debug('rostered -> shiftDeleted org disabled', { agyletimeorganisationId });
                return;
            } else {
                log.debug('rostered -> shiftDeleted org enabled', { agyletimeorganisationId });
            }
            log.debug('rostered -> shiftDeleted data', { data });

            const shift = data.shift;

            const shiftId = shift._id;

            deletes.push(
                deleteShiftMetrics({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: shift.user.id,
                    shiftId: shiftId,
                }),
            );
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (deletes.length > 0)
        await MongoRosteredMetricModel.bulkWrite(convertDeletesToBulkWrite(deletes), { ordered: true });
    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
