import { config } from '../../../config';
import { log, convertUpsertsToBulkWrite, convertDeletesToBulkWrite } from '../../../utils';
import type { MongoDeleteWithFilter, MongoQueryWithFilter, ShiftUpdatedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import * as getTeamForUser from '../../../utils/getTeamForUser';
import calculateShiftCreatedDiff from '../calculateShiftCreatedDiff';
import { deleteShiftMetrics } from '../deleteShiftMetrics';
import { getTimezoneForUser } from '../../../utils/getTimezoneForUser';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../../utils';
import type { MongoRosteredMetric } from '@agyletime/performance-utils';
import { MongoRosteredMetricModel } from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';

type BulkWriteUpsert = MongoQueryWithFilter<MongoRosteredMetric>;
type BulkWriteDelete = MongoDeleteWithFilter<MongoRosteredMetric>;

export const shiftUpdatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const bulkWrites = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            const upserts: BulkWriteUpsert[] = [];
            const deletes: BulkWriteDelete[] = [];
            log.debug('rostered -> shiftUpdated event ', record);

            const event = decodeKinesisPayload<AgyleTimeCloudEvent<ShiftUpdatedEvent>>({ data: record.kinesis.data });
            log.debug('rostered -> shiftUpdated decodedPayload', { decodedPayload: event });

            const { data, agyletimeorganisationId } = event;
            log.debug('rostered -> shiftUpdated data', { data });
            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(agyletimeorganisationId.toString()))) {
                log.info('rostered -> shiftUpdated org disabled', { agyletimeorganisationId });
                return;
            } else {
                log.info('rostered -> shiftUpdated org enabled', { agyletimeorganisationId });
            }

            const shift = data.shift;

            const shiftId = shift._id;

            log.debug(
                `rostered -> shiftUpdated ${shift.user.team ? 'using team from shift' : 'no team on shift, looking up'}`,
            );
            const team =
                shift.user.team ??
                (await getTeamForUser.getTeamForUser({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: shift.user.id,
                }));
            log.debug(
                `rostered -> shiftUpdated ${
                    shift.user.timezone ? 'using timezone from shift' : 'no timezone on shift, looking up'
                }`,
            );
            const timezone =
                shift.user.timezone ??
                (await getTimezoneForUser({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: shift.user.id,
                }));

            if (!team?.id || !agyletimeorganisationId || !shift.user.id) {
                log.error('Missing required data for event', {
                    team,
                    agyletimeorganisationId,
                    user: shift.user.id,
                    payload: data,
                });
                throw new Error('Missing required data for event');
            }

            const params = {
                organisationId: parseInt(agyletimeorganisationId.toString()),
                teamId: team.id,
                userId: shift.user.id,
                shiftId: shiftId,
                timezone,
            };

            deletes.push(
                deleteShiftMetrics({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: data.previousShift.user.id || shift.user.id,
                    shiftId,
                }),
            );

            upserts.push(
                ...calculateShiftCreatedDiff({
                    newShiftEvent: data,
                    params,
                }),
            );

            bulkWrites.push(...convertDeletesToBulkWrite(deletes), ...convertUpsertsToBulkWrite(upserts));
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (bulkWrites.length > 0) await MongoRosteredMetricModel.bulkWrite(bulkWrites, { ordered: true });
    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
