import { config } from '../../config';
import * as mongodb from '../../utils/mongodb';
import { convertUpsertsToBulkWrite, isOrgEnabled, log, roundDateToNearestTime } from '../../utils';
import { flushShiftMetrics } from './flushShiftMetrics';
import { createShiftMetric } from './createShiftMetrics';
import type { MongoQueryWithFilter, OrgDetails, RebuildResult, Shift } from '../types';
import { isWithinInterval } from 'date-fns';
import { type MongoRosteredMetric, MongoRosteredMetricModel } from '@agyletime/performance-utils';
import { isError, PublishedShiftApi, WfmApiClient } from '@agyletime/wfm-client';
import { BasicAuthApiClientParams } from '@agyletime/wfm-client/lib/types/agyleTimeClient/agyleTimeClient';
import type { PublishShiftResponse } from '@agyletime/wfm-client/lib/clients/schedule/types';

export async function rebuildRosteredMetricsForOrg(
    orgDetails: OrgDetails,
    fromDate: Date,
    toDate: Date,
    apiConnectionDetails: BasicAuthApiClientParams,
): Promise<RebuildResult> {
    // Check to see which organisations have been enabled
    if (!isOrgEnabled(orgDetails.id)) {
        log.info('rebuildRosteredMetricsForOrg org disabled', { orgId: orgDetails.id });
        return { error: undefined, result: undefined, disabled: true };
    }
    await mongodb.init(config.db);
    let result: RebuildResult['result'], error: RebuildResult['error'];
    const badCreates: unknown[] = [];
    const startWindow = roundDateToNearestTime(new Date(fromDate), config.mongoBucketSizeInMinutes);
    const endWindow = roundDateToNearestTime(new Date(toDate), config.mongoBucketSizeInMinutes);
    const upserts: MongoQueryWithFilter<MongoRosteredMetric>[] = [];

    try {
        const publishedShifts = (await getPublishedShifts({ orgDetails, apiConnectionDetails }))
            .map((shift) => {
                // We only care about SHIFT TASKS that start in this window
                shift.shiftTasks = shift.shiftTasks.filter((task) =>
                    isWithinInterval(new Date(task.startTime), {
                        start: new Date(fromDate),
                        end: new Date(toDate),
                    }),
                );
                return shift;
            })
            // Important to note that the start time on the shift metrics are actually on shift tasks!
            // So we want to only include shifts with tasks within window.
            // But at the same time not filter out shifts with tasks in AND out of the window.
            // It is VERY VERY VERY important that the date window publish shift/tasks retrieved matches the date window for metrics we delete
            .filter((shift) => shift.shiftTasks.length);

        log.debug('rebuildRosteredMetrics Published shifts found. Flushing metrics in this period.', {
            publishedShifts: mapShiftsForDebug(publishedShifts),
        });
        await flushShiftMetrics({
            organisationId: orgDetails.id,
            startWindow,
            endWindow,
            byUtc: true,
        });

        log.debug('rebuildRosteredMetrics Create shift metric');
        for (const shift of publishedShifts) {
            try {
                upserts.push(...(await createShiftMetric({ shift: shift as Shift }, orgDetails.id)));
            } catch (err) {
                // Catch errors on creating metrics, but don't stop creating the rest of the batch
                if (err instanceof Error) {
                    badCreates.push({
                        message: err.message,
                        stack: err.stack,
                        name: err.name,
                        cause: err.cause,
                    });
                } else {
                    badCreates.push({ error: JSON.stringify(err) });
                }
            }
        }

        if (upserts.length > 0) {
            const bulkWrites = convertUpsertsToBulkWrite(upserts);
            log.debug('rebuildRosteredMetrics', { bulkWrites });
            result = await MongoRosteredMetricModel.bulkWrite(bulkWrites);
        }

        if (badCreates.length) {
            // If there are any bad creates, log them in return alongside any successful results
            error = {
                error: badCreates,
                orgId: orgDetails.id,
            };
        }
    } catch (err: unknown) {
        log.error('Failed to rebuild schedule metrics', { err, upserts });
        if (err instanceof Error) {
            error = {
                message: err.message,
                stack: err.stack,
                name: err.name,
                cause: err.cause,
            };
        } else {
            error = { error: JSON.stringify(err), orgId: orgDetails.id };
        }
    }

    return {
        startWindow,
        endWindow,
        error,
        result,
    };
}
function mapShiftsForDebug(publishedShifts: PublishShiftResponse[]): Partial<PublishShiftResponse>[] {
    return publishedShifts.map((s) => ({
        _id: s._id,
        startTime: s.startTime,
        endTime: s.endTime,
        numberOfTasks: s.shiftTasks.length,
        user: s.user,
    }));
}
export async function getPublishedShifts({
    orgDetails,
    apiConnectionDetails,
}: {
    orgDetails: OrgDetails;
    apiConnectionDetails: BasicAuthApiClientParams;
}): Promise<PublishShiftResponse[]> {
    const apiConfig = {
        ...apiConnectionDetails,
        apiGatewayUrl: config.agyletime.api_gateway_base_url,
        organisationId: orgDetails.id,
    };

    const agyleTimeClient = new WfmApiClient(apiConfig);
    const client = new PublishedShiftApi(agyleTimeClient);

    log.debug('rebuildRosteredMetrics Finding published shifts for org', { orgDetails });

    const response = await client.getPublishedShiftApi();

    if (isError(response)) throw new Error(`Error calling getPublishedShiftApi: ${response.message}`);

    log.debug('rebuildRosteredMetrics Found published shifts', { numberOfPublishedShifts: response.data.length });

    return response.data;
}
