import type { ActualFilterParams, MongoQueryWithFilter, TaskDataMergedEvent, TaskMergedTask } from '../types';
import { toZonedTime } from 'date-fns-tz';
import { log } from '../../utils';
import { config } from '../../config';
import { parseISO } from 'date-fns';
import { roundDateToNearestTime } from '../../utils';
import type { MongoActualHoursWorkloadMetric } from '@agyletime/performance-utils';

export default ({
    shift,
    params,
}: {
    shift: TaskDataMergedEvent;
    params: ActualFilterParams;
}): MongoQueryWithFilter<MongoActualHoursWorkloadMetric>[] => {
    const upserts: MongoQueryWithFilter<MongoActualHoursWorkloadMetric>[] = [];

    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    for (const task of shift.tasks ?? []) {
        if (task.endTime) {
            // Only process tasks that have ended. i.e. not current.
            // We don't currently deal with real time
            const diff = calculateDiffForTask({ task: task, params });
            if (diff) upserts.push(diff);
        }
    }

    return upserts;
};

export const calculateDiffForTask = ({
    task,
    params,
}: {
    task: TaskMergedTask;
    params: ActualFilterParams;
}): MongoQueryWithFilter<MongoActualHoursWorkloadMetric> | undefined => {
    if (!task.endTime) return;
    const taskDuration = (new Date(task.endTime).getTime() - new Date(task.startTime).getTime()) / 1000;

    const shiftTaskStartTimeAsUser = toZonedTime(task.startTime, params.timezone);
    const currentTimeBucket = roundDateToNearestTime(
        new Date(shiftTaskStartTimeAsUser),
        config.mongoBucketSizeInMinutes,
    );
    const currentTimeBucketUtc = roundDateToNearestTime(parseISO(task.startTime), config.mongoBucketSizeInMinutes);
    if (Number.isNaN(currentTimeBucket.getTime())) {
        log.error('Failed to generate timebucket', params);
        throw new Error('Failed to generate timebucket', {
            cause: {
                message: 'Possibly bad params (e.g. timezone)',
                params,
            },
        });
    }

    const taskDefinition = params.taskDefinitionsMap.get(task.taskDefinitionId);
    if (!taskDefinition) {
        throw new Error('Failed to find task definition');
    }
    const isProductive = taskDefinition.isProductive && !taskDefinition.isLogout;
    const isShrinkage = !taskDefinition.isProductive && !taskDefinition.isLeave;
    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
    const isProductiveOrShrinkage = isProductive || isShrinkage;

    const incrementObject: Partial<MongoActualHoursWorkloadMetric> = {
        actualAvailableTime_actualHours:
            taskDefinition.isAvailableOccupancy && !taskDefinition.isLeave ? taskDuration : 0,
        actualAfterCallWorkTime_actualHours:
            taskDefinition.isAfterCallWork && isProductiveOrShrinkage ? taskDuration : 0,
        totalActualTasks_actualHours: isProductiveOrShrinkage ? taskDuration : 0,
    };

    const filter = {
        organisationId: params.organisationId,
        userId: params.userId,
        ...(params.teamId && { teamId: params.teamId }),
        workloadTaskId: task.taskDefinitionId,
        timeBucketAsLocal: currentTimeBucket,
        shiftId: params.shiftId,
    };
    const set = {
        ...filter,
        timeBucketAsUtc: currentTimeBucketUtc,
    };
    const upsert = {
        filter,
        query: {
            $set: set,
            $inc: incrementObject,
        },
    };
    log.debug({ upsert });
    return upsert;
};
