import type { MongoQueryWithFilter } from '../types';
import type { InteractionEventParams } from '../types';
import { config } from '../../config';
import { isEqual, parseISO } from 'date-fns';
import { toZonedTime } from 'date-fns-tz';
import { makeNumberNegative } from '../../utils';
import type { InteractionEventType } from '../interactions/interactionEvent.type';
import { roundDateToNearestTime } from '../../utils';
import { getHandleTime } from '../interactions/utils';
import type { MongoInteractionsWorkloadMetric } from '@agyletime/performance-utils';

export default ({
    previousVersion,
    currentVersion,
    params,
}: {
    previousVersion: InteractionEventType;
    currentVersion: InteractionEventType;
    params: InteractionEventParams;
}): MongoQueryWithFilter<MongoInteractionsWorkloadMetric>[] => {
    const previousStartTimeAsLocal = toZonedTime(previousVersion.startTime, params.timezone);
    const previousTimeBucket = roundDateToNearestTime(previousStartTimeAsLocal, config.mongoBucketSizeInMinutes);
    const previousTimeBucketUtc = roundDateToNearestTime(
        parseISO(previousVersion.startTime),
        config.mongoBucketSizeInMinutes,
    );

    const currentStartTimeAsLocal = toZonedTime(currentVersion.startTime, params.timezone);
    const currentTimeBucket = roundDateToNearestTime(currentStartTimeAsLocal, config.mongoBucketSizeInMinutes);
    const currentTimeBucketUtc = roundDateToNearestTime(
        parseISO(currentVersion.startTime),
        config.mongoBucketSizeInMinutes,
    );

    const changingBuckets = !isEqual(previousTimeBucket, currentTimeBucket);

    // Treat undefined work effort as 0
    if (!previousVersion.workEffortSecs) previousVersion.workEffortSecs = 0;
    if (!currentVersion.workEffortSecs) currentVersion.workEffortSecs = 0;

    if (changingBuckets) {
        const setPrevious: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>['query']['$set'] = {
            organisationId: params.organisationId,
            timeBucketAsLocal: previousTimeBucket,
            timeBucketAsUtc: previousTimeBucketUtc,
            userId: params.userId,
            teamId: params.teamId,
            workloadTaskId: params.taskTypeId,
        };

        const setCurrent: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>['query']['$set'] = {
            organisationId: params.organisationId,
            timeBucketAsLocal: previousTimeBucket,
            timeBucketAsUtc: previousTimeBucketUtc,
            userId: params.userId,
            teamId: params.teamId,
            workloadTaskId: params.taskTypeId,
        };

        return [
            // Previous bucket
            {
                filter: {
                    organisationId: params.organisationId,
                    timeBucketAsLocal: previousTimeBucket,
                    userId: params.userId,
                    teamId: params.teamId,
                    workloadTaskId: params.taskTypeId,
                },
                query: {
                    $set: setPrevious,
                    $inc: {
                        // Total Interaction Handle Time
                        totalHandleTime_interactions:
                            previousVersion.workEffortSecs > 0 ? makeNumberNegative(previousVersion.workEffortSecs) : 0,
                    },
                },
            },
            // New bucket
            {
                filter: {
                    organisationId: params.organisationId,
                    timeBucketAsLocal: currentTimeBucket,
                    userId: params.userId,
                    teamId: params.teamId,
                    workloadTaskId: params.taskTypeId,
                },
                query: {
                    $set: setCurrent,
                    $inc: {
                        // Total Interaction Handle Time
                        totalHandleTime_interactions:
                            currentVersion.workEffortSecs > 0 ? currentVersion.workEffortSecs : 0,
                    },
                },
            },
        ];
    } else {
        const calculateHandleTimeDelta = ({ oldValue, newValue }: { oldValue: boolean; newValue: boolean }): number => {
            if (oldValue && newValue) return getHandleTime(currentVersion) - getHandleTime(previousVersion);
            if (oldValue && !newValue) return -getHandleTime(previousVersion);
            if (!oldValue && newValue) return getHandleTime(currentVersion);

            return 0;
        };

        const set: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>['query']['$set'] = {
            organisationId: params.organisationId,
            timeBucketAsLocal: currentTimeBucket,
            timeBucketAsUtc: currentTimeBucketUtc,
            userId: params.userId,
            teamId: params.teamId,
            workloadTaskId: params.taskTypeId,
        };

        return [
            {
                filter: {
                    organisationId: params.organisationId,
                    timeBucketAsLocal: currentTimeBucket,
                    userId: params.userId,
                    teamId: params.teamId,
                    workloadTaskId: params.taskTypeId,
                },
                query: {
                    $set: set,
                    $inc: {
                        // Total Interaction Handle Time
                        totalHandleTime_interactions: calculateHandleTimeDelta({
                            oldValue: previousVersion.workEffortSecs > 0,
                            newValue: currentVersion.workEffortSecs > 0,
                        }),
                    },
                },
            },
        ];
    }
};
