import type { MongoQueryWithFilter, ShiftCreatedEvent, ShiftEventParams, ShiftTask } from '../types';
import { toZonedTime } from 'date-fns-tz';
import { log } from '../../utils';
import { config } from '../../config';
import { parseISO } from 'date-fns';
import { roundDateToNearestTime } from '../../utils';
import type { MongoRosteredHoursWorkloadMetric } from '@agyletime/performance-utils';

export default ({
    newShiftEvent,
    params,
}: {
    newShiftEvent: Omit<ShiftCreatedEvent, 'roster'>;
    params: ShiftEventParams;
}): MongoQueryWithFilter<MongoRosteredHoursWorkloadMetric>[] => {
    const upserts: MongoQueryWithFilter<MongoRosteredHoursWorkloadMetric>[] = [];

    const newShift = newShiftEvent.shift;

    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    for (const shiftTask of newShift.shiftTasks ?? []) {
        const diff = calculateCreateDiffForShiftTask({ shiftTask, params });
        upserts.push(diff);
    }

    return upserts;
};

export const calculateCreateDiffForShiftTask = ({
    shiftTask,
    params,
}: {
    shiftTask: ShiftTask;
    params: ShiftEventParams;
}): MongoQueryWithFilter<MongoRosteredHoursWorkloadMetric> => {
    const taskDuration = (new Date(shiftTask.endTime).getTime() - new Date(shiftTask.startTime).getTime()) / 1000;
    const shiftTaskStartTimeAsUser = toZonedTime(shiftTask.startTime, params.timezone);
    const currentTimeBucket = roundDateToNearestTime(
        new Date(shiftTaskStartTimeAsUser),
        config.mongoBucketSizeInMinutes,
    );
    const currentTimeBucketUtc = roundDateToNearestTime(parseISO(shiftTask.startTime), config.mongoBucketSizeInMinutes);
    if (Number.isNaN(currentTimeBucket.getTime())) {
        log.error('Failed to generate timebucket', params);
        throw new Error('Failed to generate timebucket', {
            cause: {
                message: 'Possibly bad params (e.g. timezone)',
                params,
            },
        });
    }

    const taskDefinition = shiftTask.taskType;
    const isProductive = taskDefinition.isProductive;
    const isShrinkage = !taskDefinition.isProductive && !taskDefinition.isLeave;
    const isProductiveOrShrinkage = isProductive || isShrinkage;

    const incrementObject: Partial<MongoRosteredHoursWorkloadMetric> = {
        rosteredAvailableTime_rosteredHours:
            shiftTask.taskType.isAvailableOccupancy && !shiftTask.taskType.isLeave ? taskDuration : 0,
        totalRosteredTasks_rosteredHours: isProductiveOrShrinkage ? taskDuration : 0,
    };

    const filter = {
        organisationId: params.organisationId,
        userId: params.userId,
        teamId: params.teamId,
        workloadTaskId: shiftTask.taskType.id,
        timeBucketAsLocal: currentTimeBucket,
        shiftId: params.shiftId,
    };
    const set = {
        ...filter,
        timeBucketAsUtc: currentTimeBucketUtc,
        shiftId: params.shiftId,
    };
    const upsert = {
        filter,
        query: {
            $set: set, // CRISTORK-TODO: Look to see if we need to make this a set (there migth be multiple shifts in same bucket)
            $inc: incrementObject,
        },
    };
    log.debug({ upsert });
    return upsert;
};
