import { config } from '../../../config';
import { log, convertUpsertsToBulkWrite } from '../../../utils';
import calculateWorkloadInteractionCreatedDiff from '../calculateWorkloadInteractionCreatedDiff';
import * as mongodb from '../../../utils/mongodb';
import type { KinesisStreamEvent } from 'aws-lambda';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import type { MongoQueryWithFilter } from '../../types';
import { isOrgEnabled } from '../../../utils';
import { extractInteractionEventParams, type InteractionCreatedEvent } from '../../interactions';
import {
    MongoInteractionsWorkloadMetricModel,
    type MongoInteractionsWorkloadMetric,
} from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';
import type { UpdateLastOccuranceParams } from '../../../taskDefinitionCache/updateLastOccurance';
import { updateLastOccurance } from '../../../taskDefinitionCache/updateLastOccurance';

export const interactionCreatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>[] = [];
    const errors: Error[] = [];
    const workloadTaskIds = new Map<number, UpdateLastOccuranceParams>();

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionCreatedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionCreated event ', { event });

            const interaction = event.data.interaction;
            log.debug('InteractionCreated data', { interaction });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interaction.organisationId)) return;

            const params = await extractInteractionEventParams(interaction);
            if (!params) {
                log.warn('No params returned extracted %j', { interaction });
                continue;
            }
            upserts.push(calculateWorkloadInteractionCreatedDiff({ interaction, params }));
            workloadTaskIds.set(interaction.workloadTaskId, {
                taskId: interaction.workloadTaskId,
                organisationId: interaction.organisationId,
            });
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoInteractionsWorkloadMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);

    if (workloadTaskIds.size > 0) await updateLastOccurance(workloadTaskIds);
};
