import { config } from '../../../config';
import { convertUpsertsToBulkWrite, log } from '../../../utils';
import calculateWorkloadInteractionUpdatedDiff from '../calculateWorkloadInteractionUpdatedDiff';
import * as mongodb from '../../../utils/mongodb';
import type { KinesisStreamEvent } from 'aws-lambda';
import type { MongoQueryWithFilter } from '../../types';
import { isOrgEnabled } from '../../../utils';
import type { InteractionUpdatedEvent } from '../../interactions';
import { extractInteractionEventParams } from '../../interactions';
import calculateWorkloadInteractionCreatedDiff from '../calculateWorkloadInteractionCreatedDiff';
import type { MongoInteractionsWorkloadMetric } from '@agyletime/performance-utils';
import { MongoInteractionsWorkloadMetricModel } from '@agyletime/performance-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../errors';
import type { UpdateLastOccuranceParams } from '../../../taskDefinitionCache/updateLastOccurance';
import { updateLastOccurance } from '../../../taskDefinitionCache/updateLastOccurance';

export const interactionUpdatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>[] = [];
    const errors: Error[] = [];
    const workloadTaskIds = new Map<number, UpdateLastOccuranceParams>();

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<InteractionUpdatedEvent>({
                data: record.kinesis.data,
            });

            log.debug('InteractionUpdated event ', event);

            const interactionBefore = event.data.interactionBefore;
            const interactionAfter = event.data.interactionAfter;
            log.debug('InteractionUpdated data (interactionAfter)', { interactionAfter });
            log.debug('InteractionUpdated data (interactionBefore)', { interactionBefore });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(interactionAfter.organisationId)) return;

            const params = await extractInteractionEventParams(interactionAfter);

            if (!params) {
                log.warn('No params returned extracted %j', { params, interactionAfter });
                return;
            } else if (!interactionBefore.agent?.wfmId) {
                // If previous event didn't have a user id, then treat this as a creation
                upserts.push(calculateWorkloadInteractionCreatedDiff({ interaction: interactionAfter, params }));
            } else {
                upserts.push(
                    // Remember there may be 2 updates returned here
                    ...calculateWorkloadInteractionUpdatedDiff({
                        previousVersion: interactionBefore,
                        currentVersion: interactionAfter,
                        params,
                    }),
                );
            }
            workloadTaskIds.set(interactionAfter.workloadTaskId, {
                taskId: interactionAfter.workloadTaskId,
                organisationId: params.organisationId,
            });
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoInteractionsWorkloadMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);

    if (workloadTaskIds.size > 0) await updateLastOccurance(workloadTaskIds);
};
