import { config } from '../../../config';
import { log, convertUpsertsToBulkWrite } from '../../../utils';
import type { MongoQueryWithFilter, ShiftCreatedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { createShiftMetric } from '../createShiftMetrics';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../../utils';
import {
    MongoRosteredHoursWorkloadMetricModel,
    type MongoRosteredHoursWorkloadMetric,
} from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';
import type { UpdateLastOccuranceParams } from '../../../taskDefinitionCache/updateLastOccurance';
import { updateLastOccurance } from '../../../taskDefinitionCache/updateLastOccurance';
import { getLatestBetweenDateAndNow } from '../../utils';

export const shiftCreatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoRosteredHoursWorkloadMetric>[] = [];
    const errors: Error[] = [];
    const workloadTaskIds = new Map<number, UpdateLastOccuranceParams>();

    for (const record of eventBatch.Records) {
        try {
            log.debug('workload -> shiftCreated event ', record);

            const event = decodeKinesisPayload<AgyleTimeCloudEvent<ShiftCreatedEvent>>({ data: record.kinesis.data });
            log.debug('workload -> shiftCreated decodedPayload', { decodedPayload: event });

            const { data, agyletimeorganisationId } = event;
            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(agyletimeorganisationId.toString()))) {
                log.info('workload -> shiftCreated org disabled', { agyletimeorganisationId });
                return;
            } else {
                log.info('workload -> shiftCreated org enabled', { agyletimeorganisationId });
            }

            log.debug('workload -> shiftCreated data', { data });

            upserts.push(...(await createShiftMetric(data, agyletimeorganisationId)));
            data.shift.shiftTasks.forEach((t) => {
                const existingEntry = workloadTaskIds.get(t.taskType.id)?.date;
                const newEntry = getLatestBetweenDateAndNow(t.endTime);
                if (!existingEntry || existingEntry < newEntry) {
                    workloadTaskIds.set(t.taskType.id, {
                        taskId: t.taskType.id,
                        date: newEntry,
                        organisationId: parseInt(agyletimeorganisationId.toString()),
                    });
                }
            });
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoRosteredHoursWorkloadMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));
    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);

    if (workloadTaskIds.size > 0) await updateLastOccurance(workloadTaskIds);
};
