import { config } from '../../../config';
import { log, convertUpsertsToBulkWrite } from '../../../utils';
import type { MongoUnsetWithFilter, ShiftDeletedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { resetRosteredHoursWorkloadMetricsForShift } from '../resetRosteredHoursWorkloadMetricsForShift';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../../utils';
import {
    MongoRosteredHoursWorkloadMetricModel,
    type MongoRosteredHoursWorkloadMetric,
} from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';
import type { UpdateLastOccuranceParams } from '../../../taskDefinitionCache/updateLastOccurance';
import { updateLastOccurance } from '../../../taskDefinitionCache/updateLastOccurance';
import { getLatestBetweenDateAndNow } from '../../utils';

export const shiftDeletedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const rosteredHoursUnsets: MongoUnsetWithFilter<MongoRosteredHoursWorkloadMetric>[] = [];
    const errors: Error[] = [];
    const workloadTaskIds = new Map<number, UpdateLastOccuranceParams>();

    for (const record of eventBatch.Records) {
        try {
            log.debug('workload -> shiftDeleted event ', record);

            const event = decodeKinesisPayload<AgyleTimeCloudEvent<ShiftDeletedEvent>>({ data: record.kinesis.data });
            log.debug('workload -> shiftDeleted decodedPayload', { decodedPayload: event });

            const { data, agyletimeorganisationId } = event;
            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(agyletimeorganisationId.toString()))) {
                log.debug('workload -> shiftDeleted org disabled', { agyletimeorganisationId });
                return;
            } else {
                log.debug('workload -> shiftDeleted org enabled', { agyletimeorganisationId });
            }
            log.debug('workload -> shiftDeleted data', { data });

            const shift = data.shift;

            const shiftId = shift._id;

            rosteredHoursUnsets.push(
                resetRosteredHoursWorkloadMetricsForShift({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: shift.user.id,
                    shiftId: shiftId,
                }),
            );
            data.shift.shiftTasks.forEach((t) => {
                const existingEntry = workloadTaskIds.get(t.taskType.id)?.date;
                const newEntry = getLatestBetweenDateAndNow(t.endTime);
                if (!existingEntry || existingEntry < newEntry) {
                    workloadTaskIds.set(t.taskType.id, {
                        taskId: t.taskType.id,
                        date: newEntry,
                        organisationId: parseInt(agyletimeorganisationId.toString()),
                    });
                }
            });
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (rosteredHoursUnsets.length > 0)
        await MongoRosteredHoursWorkloadMetricModel.bulkWrite(
            convertUpsertsToBulkWrite(rosteredHoursUnsets, { isUpsert: false, updateMany: true }),
            { ordered: true },
        );
    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);

    if (workloadTaskIds.size > 0) await updateLastOccurance(workloadTaskIds);
};
