import { config } from '../../../config';
import { log, convertUpsertsToBulkWrite } from '../../../utils';
import type { MongoQueryWithFilter, MongoUnsetWithFilter, ShiftUpdatedEvent } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import * as getTeamForUser from '../../../utils/getTeamForUser';
import calculateWorkloadRosteredHoursDiff from '../calculateWorkloadRosteredHoursDiff';
import { resetRosteredHoursWorkloadMetricsForShift } from '../resetRosteredHoursWorkloadMetricsForShift';
import { getTimezoneForUser } from '../../../utils/getTimezoneForUser';
import type { KinesisStreamEvent } from 'aws-lambda/trigger/kinesis-stream';
import { isOrgEnabled } from '../../../utils';
import {
    MongoRosteredHoursWorkloadMetricModel,
    type MongoRosteredHoursWorkloadMetric,
} from '@agyletime/performance-utils';
import { BadRequestError } from '../../../errors';
import type { UpdateLastOccuranceParams } from '../../../taskDefinitionCache/updateLastOccurance';
import { updateLastOccurance } from '../../../taskDefinitionCache/updateLastOccurance';
import { getLatestBetweenDateAndNow } from '../../utils';

type BulkWriteUpsert = MongoQueryWithFilter<MongoRosteredHoursWorkloadMetric>;
type BulkWriteUnset = MongoUnsetWithFilter<MongoRosteredHoursWorkloadMetric>;

export const shiftUpdatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const bulkWrites = [];
    const errors: Error[] = [];
    const workloadTaskIds = new Map<number, UpdateLastOccuranceParams>();

    for (const record of eventBatch.Records) {
        try {
            const upserts: BulkWriteUpsert[] = [];
            const rosteredHoursUnsets: BulkWriteUnset[] = [];
            log.debug('workload -> shiftUpdated event ', record);

            const event = decodeKinesisPayload<AgyleTimeCloudEvent<ShiftUpdatedEvent>>({ data: record.kinesis.data });
            log.debug('workload -> shiftUpdated decodedPayload', { decodedPayload: event });

            const { data, agyletimeorganisationId } = event;
            log.debug('workload -> shiftUpdated data', { data });
            // Check to see which organisations have been enabled
            if (!isOrgEnabled(parseInt(agyletimeorganisationId.toString()))) {
                log.info('workload -> shiftUpdated org disabled', { agyletimeorganisationId });
                return;
            } else {
                log.info('workload -> shiftUpdated org enabled', { agyletimeorganisationId });
            }

            const shift = data.shift;

            const shiftId = shift._id;

            log.debug(`shiftUpdated ${shift.user.team ? 'using team from shift' : 'no team on shift, looking up'}`);
            const team =
                shift.user.team ??
                (await getTeamForUser.getTeamForUser({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: shift.user.id,
                }));
            log.debug(
                `shiftUpdated ${shift.user.timezone ? 'using timezone from shift' : 'no timezone on shift, looking up'}`,
            );
            const timezone =
                shift.user.timezone ??
                (await getTimezoneForUser({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: shift.user.id,
                }));

            if (!team?.id || !agyletimeorganisationId || !shift.user.id) {
                log.error('Missing required data for event', {
                    team,
                    agyletimeorganisationId,
                    user: shift.user.id,
                    payload: data,
                });
                throw new Error('Missing required data for event');
            }

            const params = {
                organisationId: parseInt(agyletimeorganisationId.toString()),
                teamId: team.id,
                userId: shift.user.id,
                shiftId: shiftId,
                timezone,
            };

            // This unsets the rostered hours part of the metrics, which could potentially leave empty buckets of metrics if we don't have actuals or interaction data.
            // We can clean those up on rebuild/reconciliation
            // CRISTORK-TODO: Test this scenario in integration tests. Need to verify removing one shift and adding a different of same type back in will upsert correctly.
            rosteredHoursUnsets.push(
                resetRosteredHoursWorkloadMetricsForShift({
                    organisationId: parseInt(agyletimeorganisationId.toString()),
                    userId: data.previousShift.user.id || shift.user.id,
                    shiftId,
                }),
            );

            upserts.push(
                ...calculateWorkloadRosteredHoursDiff({
                    newShiftEvent: data,
                    params,
                }),
            );

            bulkWrites.push(
                ...convertUpsertsToBulkWrite(rosteredHoursUnsets, { isUpsert: false, updateMany: true }), // isUpsert false as we don't want to insert on $unset
                ...convertUpsertsToBulkWrite(upserts),
            );

            data.shift.shiftTasks.forEach((t) => {
                const existingEntry = workloadTaskIds.get(t.taskType.id)?.date;
                const newEntry = getLatestBetweenDateAndNow(t.endTime);
                if (!existingEntry || existingEntry < newEntry) {
                    workloadTaskIds.set(t.taskType.id, {
                        taskId: t.taskType.id,
                        date: newEntry,
                        organisationId: parseInt(agyletimeorganisationId.toString()),
                    });
                }
            });
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (bulkWrites.length > 0) await MongoRosteredHoursWorkloadMetricModel.bulkWrite(bulkWrites, { ordered: true });
    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);

    if (workloadTaskIds.size > 0) await updateLastOccurance(workloadTaskIds);
};
