import { config } from '../../../config';
import { convertUpsertsToBulkWrite, log } from '../../../utils';
import type { MongoQueryWithFilter, TaskDataMergedEvent, MongoUnsetWithFilter } from '../../types';
import * as mongodb from '../../../utils/mongodb';
import type { PlayvoxCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { TaskDefinitionModel } from '@agyletime/performance-utils';
import type { MongoActualHoursWorkloadMetric } from '@agyletime/performance-utils';
import { MongoActualHoursWorkloadMetricModel } from '@agyletime/performance-utils';
import type { TaskDefinitionMongoSchema } from '@agyletime/wfm-types/lib/schedule/taskDefinition';
import calculateWorkloadActualHoursDiff from '../calculateWorkloadActualHoursDiff';
import getUserFromCache from '../../../utils/getUserFromCache';
import { BadRequestError, GenericActualMetricsError } from '../../../errors';
import type { KinesisStreamEvent } from 'aws-lambda';
import { isOrgEnabled } from '../../../utils';
import type { AnyBulkWriteOperation } from 'mongoose';
import { resetActualHoursWorkloadMetricsForShift } from '../resetActualHoursWorkloadMetricsForShift';
import type { UpdateLastOccuranceParams } from '../../../taskDefinitionCache/updateLastOccurance';
import { updateLastOccurance } from '../../../taskDefinitionCache/updateLastOccurance';

type BulkWriteUpsert = MongoQueryWithFilter<MongoActualHoursWorkloadMetric>;
type BulkWriteUnset = MongoUnsetWithFilter<MongoActualHoursWorkloadMetric>;

export const taskDataMergedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const bulkWrites: AnyBulkWriteOperation[] = [];
    const errors: Error[] = [];
    const workloadTaskIds = new Map<number, UpdateLastOccuranceParams>();

    for (const record of eventBatch.Records) {
        try {
            const upserts: BulkWriteUpsert[] = [];
            const actualHoursUnsets: BulkWriteUnset[] = [];
            log.debug('workload -> taskDataMerged event ', record);
            const event = decodeKinesisPayload<PlayvoxCloudEvent<TaskDataMergedEvent>>({
                data: record.kinesis.data,
            });
            log.debug('workload -> taskDataMerged decodedPayload', { decodedPayload: event });

            const { data } = event;
            log.debug('workload -> taskDataMerged data', { data });
            log.info(`workload -> taskDataMerged number of tasks: ${data.tasks.length}`);

            const organisationId = parseInt(event.plvxwfmid.toString());

            const userFromCache = await getUserFromCache({
                organisationId,
                userId: data.userId,
            });

            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (!userFromCache)
                // CRISTORK-TODO - Create new error
                throw new GenericActualMetricsError(
                    `Failed to find user id: ${data.userId} from org: ${organisationId} in cache`,
                );

            const timezone = userFromCache.timezone;
            const teamId = userFromCache.team?.id;

            if (!timezone)
                // CRISTORK-TODO - Create new error
                throw new GenericActualMetricsError(
                    `User id: ${data.userId} from org: ${organisationId} does not have a timezone in the cache`,
                );

            const taskDefinitionsMap = await getTaskDefinitionMap({
                organisationId,
                tasks: data.tasks,
            });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(organisationId)) return;

            actualHoursUnsets.push(
                resetActualHoursWorkloadMetricsForShift({
                    organisationId,
                    userId: data.userId,
                    shiftId: data.shiftId,
                }),
            );

            // CRISTORK-DONE - Create calculateActualCreateDiff for workloads (do we rename)?
            upserts.push(
                ...calculateWorkloadActualHoursDiff({
                    shift: data,
                    params: {
                        organisationId,
                        userId: data.userId,
                        shiftId: data.shiftId,
                        ...(teamId && { teamId }),
                        taskDefinitionsMap,
                        timezone,
                    },
                }),
            );

            bulkWrites.push(
                ...convertUpsertsToBulkWrite(actualHoursUnsets, { isUpsert: false, updateMany: true }), // isUpsert false as we don't want to insert on $unset
                ...convertUpsertsToBulkWrite(upserts),
            );

            data.tasks.forEach((t) =>
                workloadTaskIds.set(t.taskDefinitionId, { taskId: t.taskDefinitionId, organisationId }),
            );
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (bulkWrites.length > 0) await MongoActualHoursWorkloadMetricModel.bulkWrite(bulkWrites, { ordered: true });

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);

    if (workloadTaskIds.size > 0) await updateLastOccurance(workloadTaskIds);
};

const getTaskDefinitionMap = async ({
    organisationId,
    tasks,
}: {
    organisationId: number;
    tasks: TaskDataMergedEvent['tasks'];
}): Promise<Map<number, TaskDefinitionMongoSchema>> => {
    const definitions = await TaskDefinitionModel.find({
        organisationId,
        id: { $in: tasks.map((t) => t.taskDefinitionId) },
    }).lean();
    return new Map(definitions.map((d) => [d.id, d]));
};
