import type { MongoQueryWithFilter, WorkstreamMetricEventParams } from '../types';
import type { MongoWorkstreamMetric } from '@agyletime/performance-utils';
import type { WorkstreamMetricsEventType } from './workstreamMetricEvent.type';
import { toZonedTime } from 'date-fns-tz';
import { config } from '../../config';
import { roundDateToNearestTime } from '../../utils';
import { parseISO } from 'date-fns';
import { InteractionEventParameterValidationError, WorkstreamMetricEventParameterValidationError } from '../../errors';
import UserHistoryProvider from '../../userHistory/UserHistoryProvider';
import type { UserCacheType } from '../../userCache/UserCacheModel';
import { UserCacheModel } from '../../userCache/UserCacheModel';

export default async ({
    workstream,
    params,
}: {
    workstream: WorkstreamMetricsEventType;
    params: WorkstreamMetricEventParams;
}): Promise<MongoQueryWithFilter<MongoWorkstreamMetric>> => {
    const bucketTime = workstream.firstResolutionTime;

    let userId = workstream.firstResolvedAgentAlias.wfmId;

    if (!userId) {
        const [userFromCache]: [result: UserCacheType] = await UserCacheModel.find<UserCacheType>({
            organisationId: params.organisationId,
            'integrations.integrationAlias': workstream.firstResolvedAgentAlias.integrationAlias,
        }).lean();

        userId = userFromCache.id;
    }

    if (!userId) throw new WorkstreamMetricEventParameterValidationError('No User ID provided in event');

    let teamId = workstream.firstResolvedAgentAlias.teamId;
    let timezone = workstream.firstResolvedAgentAlias.timezone;

    if (!timezone || !teamId) {
        const userFromCache = await UserHistoryProvider.getUserFromCache({
            organisationId: params.organisationId,
            userId,
        });

        if (userFromCache) {
            if (!timezone) timezone = userFromCache.timezone;
            if (!teamId) teamId = userFromCache.teamId;
        }
    }

    if (!timezone) throw new InteractionEventParameterValidationError('Timezone was not supplied');
    if (!teamId) throw new WorkstreamMetricEventParameterValidationError('Team ID was not supplied');

    const timeBucketAsLocal = roundDateToNearestTime(
        toZonedTime(bucketTime, timezone),
        config.mongoBucketSizeInMinutes,
    );
    const timeBucketAsUtc = roundDateToNearestTime(parseISO(bucketTime), config.mongoBucketSizeInMinutes);

    const set: MongoQueryWithFilter<MongoWorkstreamMetric>['query']['$set'] = {
        // Metadata
        organisationId: params.organisationId,
        timeBucketAsLocal,
        timeBucketAsUtc,
        userId: userId,
        teamId: teamId,
        workstreamIdentifier: params.workstreamIdentifier,

        // Metrics
        // csatPositiveResponses: workstream.positiveCsatResponse,
        // csatTotalResponses: workstream.csatResponse,
        firstResolveUserId: workstream.firstResolvedUserId,
        firstResolveTeamId: workstream.firstResolvedTeamId,
    };

    if (params.workstreamDefinitionId) {
        set.workstreamDefinitionId = params.workstreamDefinitionId;
        set.workstreamDefinitionName =
            params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`;
    }

    return {
        filter: {
            organisationId: params.organisationId,
            workstreamIdentifier: params.workstreamIdentifier,
        },
        query: {
            $set: set,
        },
    };
};
