import { config } from '../../../config';
import { convertUpsertsToBulkWrite, isOrgEnabled, log } from '../../../utils';
import type { MongoWorkstreamMetric } from '@agyletime/performance-utils';
import { MongoWorkstreamMetricModel } from '@agyletime/performance-utils';
import type { KinesisStreamEvent } from 'aws-lambda';
import * as mongodb from '../../../utils/mongodb';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import type { MongoQueryWithFilter } from '../../types';
import type { WorkstreamMetricCreatedEvent } from '../workstreamMetricEvent.type';
import calculateFirstSolveWorkstreamMetricCreatedDiff from '../calculateFirstSolveWorkstreamMetricCreatedDiff';
import extractWorkstreamMetricsEventParams from '../extractWorkstreamMetricsEventParams';
import type { WorkstreamDefinitionsType } from '../../../workstreamDefinition';
import upsertWorkstreamDefinitions from '../../../workstreamDefinition/upsertWorkstreamDefinitions';
import { setUnnamedWorkstreamDefinitionName } from '../../utils';
import { BadRequestError } from '../../../errors';
import { eventDebugLogger } from '../../eventHandlers/eventDebugLogger';

export const workstreamMetricCreatedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const upserts: MongoQueryWithFilter<MongoWorkstreamMetric>[] = [];
    const workstreamDefinitions: WorkstreamDefinitionsType[] = [];
    const errors: Error[] = [];

    await eventDebugLogger<WorkstreamMetricCreatedEvent>(eventBatch);

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<WorkstreamMetricCreatedEvent>({
                data: record.kinesis.data,
            });

            log.debug('WorkstreamMetricCreated event ', { event });

            const workstream = event.data.workstreamMetrics;
            log.debug(`WorkstreamMetricCreated data ${workstream.workstreamIdentifier}`, { workstream });

            const params = extractWorkstreamMetricsEventParams({ workstream });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(params.organisationId)) return;

            workstreamDefinitions.push({
                ...(workstream.workstreamDefinition ?? {
                    id: workstream.workstreamDefinitionId,
                    name: setUnnamedWorkstreamDefinitionName({ id: workstream.workstreamDefinitionId }),
                }),
                organisationId: params.organisationId,
            });

            upserts.push(
                await calculateFirstSolveWorkstreamMetricCreatedDiff({
                    workstream,
                    params,
                }),
            );
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (upserts.length > 0) await MongoWorkstreamMetricModel.bulkWrite(convertUpsertsToBulkWrite(upserts));

    if (workstreamDefinitions.length > 0) await upsertWorkstreamDefinitions(workstreamDefinitions);

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
