import { config } from '../../../config';
import { convertDeletesToBulkWrite, isOrgEnabled, log } from '../../../utils';
import type { KinesisStreamEvent } from 'aws-lambda';
import * as mongodb from '../../../utils/mongodb';
import type { MongoWorkstreamMetric } from '@agyletime/performance-utils';
import { MongoWorkstreamMetricModel } from '@agyletime/performance-utils';
import type { WorkstreamMetricDeletedEvent } from '../workstreamMetricEvent.type';
import type { MongoDeleteWithFilter } from '../../types';
import extractWorkstreamMetricsEventParams from '../extractWorkstreamMetricsEventParams';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../errors';
import { eventDebugLogger } from '../../eventHandlers/eventDebugLogger';

export const workstreamMetricDeletedHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const deletes: MongoDeleteWithFilter<MongoWorkstreamMetric>[] = [];
    const errors: Error[] = [];

    await eventDebugLogger<WorkstreamMetricDeletedEvent>(eventBatch);

    for (const record of eventBatch.Records) {
        try {
            const event = decodeKinesisPayload<WorkstreamMetricDeletedEvent>({
                data: record.kinesis.data,
            });

            log.debug('WorkstreamMetricDeleted event ', { event });

            const workstream = event.data.workstreamMetrics;
            log.debug(`WorkstreamMetricDeleted data ${workstream.workstreamIdentifier}`, { workstream });

            const params = extractWorkstreamMetricsEventParams({ workstream });

            // Check to see which organisations have been enabled
            if (!isOrgEnabled(params.organisationId)) return;

            deletes.push({
                filter: {
                    organisationId: params.organisationId,
                    workstreamDefinitionId: params.workstreamDefinitionId,
                    workstreamIdentifier: params.workstreamIdentifier,
                },
            });
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (deletes.length > 0) await MongoWorkstreamMetricModel.bulkWrite(convertDeletesToBulkWrite(deletes));

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
