import { log } from '../../../utils';
import type { APIGatewayProxyEvent } from 'aws-lambda';
import { isBefore, isValid, parseISO } from 'date-fns';
import { ReconciliationTypes, runAthenaQuery } from '../../../athena';
import { BadRequestError, NotAllowedError } from '../../../errors';
import type { RebuildMetricsPostBody } from '../../types';
import { isValidRebuildMetricsPostBody } from '../../utils';
import { extractIdentity, formatAsJSend, JSend } from '@agyletime/wfm-aws-utils';

export const rebuildWorkstreamsHandler = async (event: APIGatewayProxyEvent): Promise<JSend> => {
    log.debug('POST %s', event.path, event);

    const identity = extractIdentity(event);
    const organisationId = Number(identity.organisationId);

    // TODO: This should be replaced by the right scopes once implemented
    if (identity.identifier !== 'admin@agyletime.com') throw new NotAllowedError('Not allowed to perform this action');

    if (!event.body) throw new BadRequestError('No params supplied');

    const requestBody = JSON.parse(event.body) as RebuildMetricsPostBody;
    if (!isValidRebuildMetricsPostBody(requestBody)) throw new BadRequestError('Invalid request');

    const fromDate = parseISO(requestBody.fromDate);
    if (!isValid(fromDate)) throw new BadRequestError('fromDate is invalid');
    const toDate = parseISO(requestBody.toDate);
    if (!isValid(toDate)) throw new BadRequestError('toDate is invalid');

    if (isBefore(toDate, fromDate)) throw new BadRequestError('Invalid date range');

    const { queryExecutionId } = await runAthenaQuery(
        { organisationId, fromDate, toDate },
        ReconciliationTypes.Workstream,
    );

    return formatAsJSend(
        {
            message: `rebuildWorkstreams was called with orgId: ${organisationId}, fromDate: ${fromDate.toISOString()}, toDate: ${toDate.toISOString()}`,
            queryExecutionId: queryExecutionId,
        },
        200,
    );
};
