import { config } from '../../config';

import { log } from '../../utils/log';
import type { KinesisStreamEvent } from 'aws-lambda';
import type { TaskDefinitionMongoSchema } from '@agyletime/wfm-types';
import * as mongodb from '../../utils/mongodb';
import { TaskDefinitionModel } from '@agyletime/performance-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../errors';

interface TaskDefinitionEvent {
    agyletimeorganisationid: string | number;
    data: {
        taskDefinition: TaskDefinitionMongoSchema;
    };
    type: string;
}

export const handleInboundTaskDefinitionEvent = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);
    const errors: Error[] = [];

    for (const event of eventBatch.Records) {
        try {
            const plvxEvent = decodeKinesisPayload<TaskDefinitionEvent>({
                data: event.kinesis.data,
            });

            log.debug(`event payload`, { plvxEvent });

            const organisationId = parseInt(plvxEvent.agyletimeorganisationid.toString());
            const type = plvxEvent.type;
            const task = plvxEvent.data.taskDefinition;

            if (type === 'agyletime.schedule.taskDefinition.Deleted') {
                // Don't do anything for deleted ones as we want to keep the old record
            } else {
                await TaskDefinitionModel.updateOne(
                    {
                        organisationId,
                        id: task.id,
                    },
                    task,
                    {
                        upsert: true,
                    },
                );
            }
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
