import { log } from '../utils';

import { fetchAllUsers } from './utils/fetchAllUsers';
import { insertOrUpdateUsers } from './utils/insertOrUpdateUsers';
import type { OrganisationId } from '@agyletime/wfm-types';
import { getOrganisationApiParamsById, getSSMParameters, SSMParametersForOrganisation } from '@agyletime/wfm-aws-utils';
import UserHistoryProvider from '../userHistory/UserHistoryProvider';

/**
 * Fetch user from api
 * Sync to mongo
 * @param organisation
 */
export const syncUsersToDb = async (organisation: SSMParametersForOrganisation): Promise<void> => {
    const organisationId: OrganisationId = organisation.id;
    log.info(`ORG_ID [${organisationId}] Fetching user from api`, { organisationId });

    const organisationParameters = await getSSMParameters();

    const apiConnectionDetails = getOrganisationApiParamsById({
        organisationId: organisationId,
        organisationParams: organisationParameters,
    });

    if (!apiConnectionDetails) throw new Error('Unable to load org params');

    // fetch all users by orgId
    const users = await fetchAllUsers(organisationId, apiConnectionDetails);

    log.info(`ORG_ID [${organisationId}] Fetched ${users.length} users`, {
        tenantUrl: organisation.api?.agyletimeApiUrl,
    });

    // save to mongo
    if (users.length) {
        await insertOrUpdateUsers({ organisationId, users });

        const timestamp = new Date();

        const usersWithOrgId = users.map((user) => {
            return {
                ...user,
                organisationId,
            };
        });

        await UserHistoryProvider.syncUserHistory({ users: usersWithOrgId, timestamp });

        log.info(`ORG_ID [${organisationId}] Users are saved to DB`);
    }
};
