import type { AggregatedUserHistory, UserHistory, UserWithUpdatedAt } from './UserHistoryModel';
import { UserHistoryModel } from './UserHistoryModel';
import type { PipelineStage } from 'mongoose';
import { toZonedTime } from 'date-fns-tz';
import { UserMongoSchema } from '@agyletime/wfm-types/lib/organisation/user';

const getUserHistoryForOrgIds = async ({
    organisationIds,
    timestamp,
}: {
    organisationIds: number[];
    timestamp: Date;
}): Promise<AggregatedUserHistory[]> => {
    const aggregation: PipelineStage[] = [
        {
            $match: {
                organisationId: { $in: organisationIds },
                updatedAt: { $lte: timestamp },
            },
        },
        {
            $group: {
                _id: '$userId',
                users: {
                    $push: {
                        _id: '$_id',
                        teamId: '$teamId',
                        timezone: '$timezone',
                        updatedAt: '$updatedAt',
                        organisationId: '$organisationId',
                    },
                },
            },
        },
        {
            $unwind: '$users',
        },
        {
            $sort: {
                'users.updatedAt': -1,
            },
        },
        {
            $group: {
                _id: '$_id',
                teamId: {
                    $first: '$users.teamId',
                },
                timezone: {
                    $first: '$users.timezone',
                },
                organisationId: {
                    $first: '$users.organisationId',
                },
            },
        },
        {
            $addFields: {
                userId: '$_id',
            },
        },
        {
            $project: {
                _id: 0,
            },
        },
    ];

    return UserHistoryModel.aggregate<AggregatedUserHistory>(aggregation);
};

const getUserFromCache = async ({
    organisationId,
    userId,
}: {
    organisationId: number;
    userId: number;
}): Promise<AggregatedUserHistory | null> => {
    return UserHistoryModel.findOne({ organisationId, userId });
};

const getMostUpToDateHistory = async ({
    organisationIds,
}: {
    organisationIds: number[];
}): Promise<AggregatedUserHistory[]> => {
    const aggregation: PipelineStage[] = [
        {
            $match: {
                organisationId: { $in: organisationIds },
            },
        },
        {
            $group: {
                _id: '$userId',
                users: {
                    $push: {
                        _id: '$_id',
                        teamId: '$teamId',
                        timezone: '$timezone',
                        updatedAt: '$updatedAt',
                        organisationId: '$organisationId',
                    },
                },
            },
        },
        {
            $unwind: '$users',
        },
        {
            $sort: {
                'users.updatedAt': 1,
            },
        },
        {
            $group: {
                _id: '$_id',
                teamId: {
                    $first: '$users.teamId',
                },
                timezone: {
                    $first: '$users.timezone',
                },
                organisationId: {
                    $first: '$users.organisationId',
                },
            },
        },
        {
            $addFields: {
                userId: '$_id',
            },
        },
        {
            $project: {
                _id: 0,
            },
        },
    ];

    return UserHistoryModel.aggregate<AggregatedUserHistory>(aggregation);
};

const syncUserHistory = async ({ users, timestamp }: { users: UserMongoSchema[]; timestamp: Date }): Promise<void> => {
    // Get the saved history for the users
    const uniqueOrgIds = [...new Set(users.map((user) => user.organisationId))];
    const savedUserHistory = await getUserHistoryForOrgIds({ organisationIds: uniqueOrgIds, timestamp });

    // Filter out no ops
    const filteredUserHistory = [];
    for (const user of users) {
        const matchingUserFromHistory = savedUserHistory.find(
            (savedUser) => user.organisationId === savedUser.organisationId && user.id === savedUser.userId,
        );

        if (!matchingUserFromHistory) {
            filteredUserHistory.push(user);
        } else {
            const userHasChanged =
                (user.team.id && user.team.id !== matchingUserFromHistory.teamId) ||
                user.timezone !== matchingUserFromHistory.timezone;
            if (userHasChanged) {
                filteredUserHistory.push(user);
            }
        }
    }
    // Store the history
    const inserts = [];
    for (const user of filteredUserHistory) {
        inserts.push({
            insertOne: {
                document: {
                    organisationId: user.organisationId,
                    userId: user.id,
                    updatedAt: timestamp,
                    ...(user.team.id && { teamId: user.team.id }),
                    ...(user.timezone && { timezone: user.timezone }),
                },
            },
        });
    }

    if (inserts.length > 0) await UserHistoryModel.bulkWrite<UserHistory>(inserts);
};

const saveUserHistoryFromEvent = async (users: UserWithUpdatedAt[]): Promise<void> => {
    const updates = [];
    for (const user of users) {
        updates.push({
            updateOne: {
                filter: {
                    organisationId: user.organisationId,
                    userId: user.id,
                },
                upsert: true,
                update: [
                    {
                        $set: {
                            // Only update the date if the fields we care about are changing
                            updatedAt: {
                                $cond: {
                                    if: {
                                        $and: [
                                            { $eq: [user.timezone, '$timezone'] },
                                            { $eq: [user.team?.id, '$teamId'] },
                                        ],
                                    },
                                    then: '$updatedAt',
                                    else: toZonedTime(user.updatedAt, user.timezone ?? 'UTC'),
                                },
                            },
                            organisationId: user.organisationId,
                            userId: user.id,
                            ...(user.team?.id && { teamId: user.team.id }),
                            ...(user.timezone && { timezone: user.timezone }),
                        },
                    },
                ],
            },
        });
    }

    if (updates.length > 0) await UserHistoryModel.bulkWrite(updates);
};

export default {
    saveUserHistoryFromEvent,
    getUserHistoryForOrgIds,
    getMostUpToDateHistory,
    syncUserHistory,
    getUserFromCache,
};
