import { config } from '../../lib/config';
import { makeToken, tokenData } from '../utils/makeToken';
import type { APIGatewayEvent } from 'aws-lambda';
import { patchCustomLabel } from '../../lib/customLabels/handlers/patchCustomLabel';
import { MongoMetricsConfigCustomLabelModel } from '@agyletime/performance-utils';
import * as mongodb from '../../lib/utils/mongodb';
import { extractIdentity } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../lib/errors';

jest.createMockFromModule('@agyletime/performance-utils');
jest.mock('../../lib/utils/mongodb');

jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        extractIdentity: jest.fn(),
    };
});

config.db.url = 'test url';
const organisationId = '9';
const token = makeToken(tokenData);

const mockCustomLabel = {
    label: 'Test label',
};

const mockResult = 'Success!';

const event = {
    headers: {
        Authorization: `Bearer ${token}`,
    },
    queryStringParameters: {},
    identity: {
        accessKey: null,
        accountId: 'offlineContext_accountId',
        apiKey: 'offlineContext_apiKey',
        caller: 'offlineContext_caller',
        cognitoAuthenticationProvider: 'offlineContext_cognitoAuthenticationProvider',
        cognitoAuthenticationType: 'offlineContext_cognitoAuthenticationType',
        cognitoIdentityId: 'offlineContext_cognitoIdentityId',
        cognitoIdentityPoolId: 'offlineContext_cognitoIdentityPoolId',
        organisationId: organisationId.toString(),
        principalOrgId: null,
        sourceIp: '127.0.0.1',
        user: 'offlineContext_user',
        userAgent:
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36',
        userArn: 'offlineContext_userArn',
    },
    requestContext: {
        authorizer: { organisationId: organisationId, identifier: 'admin@agyletime.com', userId: 1 },
        httpMethod: 'PATCH',
        identity: {},
        path: 'performance-metrics/custom-label/test_field/test_sub_category/test_category',
        protocol: 'HTTP/1.1',
        resourcePath: 'custom-label/test_field/test_sub_category/test_category',
        stage: 'dev',
    },
    pathParameters: {
        field: 'test_field',
        subCategory: 'test_sub_category',
        category: 'test_category',
    },
    timestamp: '2020-08-26T22:49:48.521Z',
    body: JSON.stringify(mockCustomLabel),
} as unknown as APIGatewayEvent;

describe('HTTP Endpoint', () => {
    beforeEach(() => {
        // @ts-expect-error type mismatch
        jest.spyOn(MongoMetricsConfigCustomLabelModel, 'updateOne').mockImplementation(() => mockResult);
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
        (extractIdentity as jest.Mock).mockImplementation(() => ({
            organisationId: 9,
            identifier: 'admin@agyletime.com',
        }));
    });

    afterEach(() => {
        jest.resetAllMocks();
    });

    describe('with a valid body', () => {
        it('returns 202', async () => {
            const result = await patchCustomLabel(event);
            expect(result.statusCode).toBe(202);
        });

        it('throws error if no field/category/subCategory provided', async () => {
            const eventWithNoBody = { ...event, pathParameters: {} };

            try {
                await patchCustomLabel(eventWithNoBody);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid field');
            }
        });

        it('returns an error if the db connection fails', async () => {
            jest.spyOn(MongoMetricsConfigCustomLabelModel, 'updateOne').mockImplementation(() => {
                throw new Error('Bang!');
            });

            try {
                await patchCustomLabel(event);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as Error).message).toEqual('Bang!');
            }
        });
    });
});
