import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { MongoInteractionsWorkloadMetric, MongoInteractionsWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../lib/userCache/UserCacheModel';
import { interactionUpdatedHandler } from '../../../lib/metrics/eventHandlers/interactionUpdatedHandler';
import { MongoInteractionMetric, MongoInteractionMetricModel } from '@agyletime/performance-utils';
import { EventDebugLog, EventDebugLogModel } from '../../../lib/models';

const organisationId = 6896;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};

describe('Shift Created Handler', () => {
    beforeAll(() => {
        process.env['DEBUG_EVENT_COLLECTION'] = 'true';
    });
    afterAll(() => {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete process.env['DEBUG_EVENT_COLLECTION'];
    });

    const workEffortSecsAfter_eventOne = 32323;
    const workEffortSecsBefore_eventOne = 30000;
    const workloadTaskId_eventOne = 123;
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            interactionBefore: {
                                _id: '65418641bef04122f1706948',
                                identifier: 'ticketAudit:wat24682571149337_1698792799528',
                                organisationId: organisationId,
                                __v: 0,
                                agentAlias: 'wat6534796378649',
                                asOrganisationId: '9002',
                                asTeamId: '1',
                                asUserId: '681',
                                contactIdentifier: 'wat2310.11354293.TuJ7MrLjkY5LW',
                                createdAt: '2023-10-31T22:57:05.723Z',
                                description: 'Chat Message',
                                endTime: '2023-10-31T22:53:19.528Z',
                                isPrivateComment: false,
                                isPublicComment: false,
                                isReassign: false,
                                isStatusChange: false,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2023-10-31T22:53:19.528Z',
                                teamId: 1,
                                type: 'chat',
                                updatedAt: '2023-10-31T22:57:05.723Z',
                                userId: userOne.id,
                                workEffortSecs: workEffortSecsBefore_eventOne,
                                workstreamIdentifier: 'wat2158',
                                agent: {
                                    integrationAlias: 'wat6534796378649',
                                    wfmId: userOne.id,
                                    emailAddress: 'oaytekin@playvox.com',
                                    timezone: 'America/New_York',
                                    teamId: userOne.team?.id,
                                },
                            },
                            interactionAfter: {
                                identifier: 'ticketAudit:wat24682571149337_1698792799528',
                                organisationId: organisationId,
                                __v: 0,
                                agentAlias: 'wat6534796378649',
                                contactIdentifier: 'wat2310.11354293.TuJ7MrLjkY5LW',
                                createdAt: '2023-10-31T22:57:05.723Z',
                                description: 'Chat Message',
                                endTime: '2023-10-31T22:53:19.528Z',
                                isPrivateComment: false,
                                isPublicComment: false,
                                isReassign: false,
                                isStatusChange: false,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2023-10-31T22:53:19.528Z',
                                type: 'chat',
                                updatedAt: '2023-10-31T22:57:08.902Z',
                                userId: userOne.id,
                                workEffortSecs: workEffortSecsAfter_eventOne,
                                workstreamIdentifier: 'wat2158',
                                workloadTaskId: workloadTaskId_eventOne,
                                workstreamDefinitionId: 5,
                                agent: {
                                    integrationAlias: 'wat6534796378649',
                                    wfmId: userOne.id,
                                    emailAddress: 'oaytekin@playvox.com',
                                    timezone: 'America/New_York',
                                    teamId: userOne.team?.id,
                                },
                                workstreamDefinition: {
                                    id: 5,
                                    name: 'Catch All 1',
                                },
                            },
                            updates: [
                                {
                                    op: 'replace',
                                    path: '/updatedAt',
                                    value: '2023-10-31T22:57:08.902Z',
                                },
                                {
                                    op: 'replace',
                                    path: '/workloadTaskId',
                                    value: workloadTaskId_eventOne,
                                },
                                {
                                    op: 'replace',
                                    path: '/workstreamDefinitionId',
                                    value: 5,
                                },
                            ],
                            operation: 'update',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Single event', () => {
        let workloadDataInMongo: MongoInteractionsWorkloadMetric[] = [];
        let interactionsDataInMongo: MongoInteractionMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];

        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            await interactionUpdatedHandler(eventOne);
            workloadDataInMongo = await MongoInteractionsWorkloadMetricModel.find({ organisationId }).lean();
            interactionsDataInMongo = await MongoInteractionMetricModel.find({ organisationId }).lean();
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates total handle time correctly', () => {
            // Expect there to be 2 workstreams
            expect(workloadDataInMongo).toBeDefined();
            const expectedMetric = workloadDataInMongo.find((d) => d.workloadTaskId === workloadTaskId_eventOne);
            expect(expectedMetric?.totalHandleTime_interactions).toBe(
                workEffortSecsAfter_eventOne - workEffortSecsBefore_eventOne,
            );
            expect(interactionsDataInMongo).toBeDefined();
            expect(interactionsDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });
});
