import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { MongoRosteredHoursWorkloadMetric, MongoRosteredHoursWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../lib/userCache/UserCacheModel';
import { differenceInSeconds } from 'date-fns';
import { shiftCreatedHandler } from '../../../lib/metrics/eventHandlers/shiftCreatedHandler';
import { ShiftTask, TaskType } from '../../../lib/metrics/types';
import { MongoRosteredMetric, MongoRosteredMetricModel } from '@agyletime/performance-utils';
import { EventDebugLog, EventDebugLogModel } from '../../../lib/models';

const organisationId = 777797779;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};
const taskDefinition_isAvailableOccupancy: TaskType = {
    name: 'Is Available Occupancy Task Task',
    id: 12345,
    taskId: 12345,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: true,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};
const taskDefinition_isAfterCallWork: TaskType = {
    name: 'Is After Call Work Task',
    id: 123456,
    taskId: 123456,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};

describe('Shift Created Handler', () => {
    beforeAll(() => {
        process.env['DEBUG_EVENT_COLLECTION'] = 'true';
    });
    afterAll(() => {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete process.env['DEBUG_EVENT_COLLECTION'];
    });

    const isAvailableOccupancyTask: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T02:58:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAfterCallWorkTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T02:58:42.000Z',
        endTime: '2023-12-01T05:58:42.000Z',
        taskType: taskDefinition_isAfterCallWork,
    };
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: {
                                _id: '6566b308d2e21a9b0c9f0be6',
                                user: { id: userOne.id },
                                shiftTasks: [isAvailableOccupancyTask, isAfterCallWorkTask],
                            },
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Single event', () => {
        let workloadDataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let rosteredDataInMongo: MongoRosteredMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            await shiftCreatedHandler(eventOne);
            workloadDataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            rosteredDataInMongo = await MongoRosteredMetricModel.find({ organisationId }).lean();
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoRosteredMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            const exepctedRosteredAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.rosteredAvailableTime_rosteredHours).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask?.rosteredAvailableTime_rosteredHours).toBe(0);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });

        it('calculates total rostered hours', () => {
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            let exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAfterCallWorkTask.endTime),
                new Date(isAfterCallWorkTask.startTime),
            );
            expect(isAfterCallWorkMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });
});
