import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { MongoRosteredHoursWorkloadMetric, MongoRosteredHoursWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../lib/userCache/UserCacheModel';
import { shiftDeletedHandler } from '../../../lib/metrics/eventHandlers/shiftDeletedHandler';
import { MongoRosteredMetricModel } from '@agyletime/performance-utils/lib/mongo';
import { TaskType } from '@agyletime/wfm-client';
import { EventDebugLog, EventDebugLogModel } from '../../../lib/models';

const organisationId = 99989090909;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};

describe('Shift Created Handler', () => {
    beforeAll(() => {
        process.env['DEBUG_EVENT_COLLECTION'] = 'true';
    });
    afterAll(() => {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete process.env['DEBUG_EVENT_COLLECTION'];
    });

    const eventOne_shiftIdOne = 'event-shift-id-1';
    const eventOne_shiftIdTwo = 'event-shift-id-2';
    const taskDefinition_isAvailableOccupancy: TaskType = {
        name: 'Is Available Occupancy Task Task',
        id: 12345,
        taskId: 12345,
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: true,
        isPlanned: false,
        isProductive: false,
        isLeave: false,
        hideLeaveInAgentViews: false,
        isMergeAllowed: true,
        colour: '00000',
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        isShiftSwapAllowed: '',
    };

    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: {
                                _id: eventOne_shiftIdOne,
                                user: { id: userOne.id },
                                shiftTasks: [
                                    {
                                        _id: 'XXXXX',
                                        startTime: '2023-12-01T10:45:00.000Z',
                                        endTime: '2023-12-01T11:45:00.000Z',
                                        taskType: taskDefinition_isAvailableOccupancy,
                                    },
                                ],
                            },
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;
    const preSeedRosteredHoursMetrics = [
        {
            organisationId,
            timeBucketAsLocal: '2023-12-01T10:45:00.000Z',
            timeBucketAsUtc: '2023-12-01T10:45:00.000Z',
            userId: 73185,
            workloadTaskId: 12345,
            rosteredAvailableTime_rosteredHours: 111111111,
            shiftId: eventOne_shiftIdOne,
            totalRosteredTasks_rosteredHours: 111111111,
        },
        {
            organisationId,
            timeBucketAsLocal: '2023-12-01T10:45:00.000Z',
            timeBucketAsUtc: '2023-12-01T10:45:00.000Z',
            userId: 73185,
            workloadTaskId: 878787,
            rosteredAvailableTime_rosteredHours: 111111111,
            shiftId: eventOne_shiftIdTwo,
            totalRosteredTasks_rosteredHours: 111111111,
        },
    ];

    describe('Single event', () => {
        let workloadDataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let workloadDataInMongoBefore: MongoRosteredHoursWorkloadMetric[] = [];
        let error: Error;
        let eventDebugLog: EventDebugLog[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
            // Pre-seed rostered metrics
            await MongoRosteredHoursWorkloadMetricModel.create(preSeedRosteredHoursMetrics);

            workloadDataInMongoBefore = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            await shiftDeletedHandler(eventOne);
            try {
                workloadDataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            } catch (e: unknown) {
                error = e as Error;
            }
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoRosteredMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates rostered available time correctly', () => {
            expect(error).toBeUndefined();
            // Expect there to be 2 workstreams
            expect(workloadDataInMongo).toBeDefined();
            // The reset shouldn't change the number of documents, just unset properties.
            expect(workloadDataInMongoBefore.length).toBe(preSeedRosteredHoursMetrics.length);
            expect(workloadDataInMongo.length).toBe(preSeedRosteredHoursMetrics.length);
            const resettedRosteredMetrics = workloadDataInMongo.find((d) => d.shiftId === eventOne_shiftIdOne);
            expect(workloadDataInMongoBefore[0]?.totalRosteredTasks_rosteredHours).toBeGreaterThan(0);
            expect(workloadDataInMongoBefore[0]?.rosteredAvailableTime_rosteredHours).toBeGreaterThan(0);
            expect(resettedRosteredMetrics).toBeUndefined();
            // The metrics not marked for reset should be unchanged
            const unchangedRosteredMetrics = workloadDataInMongo.find((d) => d.shiftId === eventOne_shiftIdTwo);
            const expectedUnchangedShift = preSeedRosteredHoursMetrics.find((d) => d.shiftId === eventOne_shiftIdTwo);
            expect(unchangedRosteredMetrics?.rosteredAvailableTime_rosteredHours).toBe(
                expectedUnchangedShift?.rosteredAvailableTime_rosteredHours,
            );
            expect(unchangedRosteredMetrics?.totalRosteredTasks_rosteredHours).toBe(
                expectedUnchangedShift?.totalRosteredTasks_rosteredHours,
            );
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });
});
