import { KinesisStreamEvent } from 'aws-lambda';
import { config } from '../../../lib/config';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { MongoRosteredHoursWorkloadMetric, MongoRosteredHoursWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../lib/userCache/UserCacheModel';
import { differenceInSeconds, parseISO } from 'date-fns';
import { shiftUpdatedHandler } from '../../../lib/metrics/eventHandlers/shiftUpdatedHandler';
import { Shift, ShiftTask, ShiftUpdatedEvent, TaskType } from '../../../lib/metrics/types';
import { roundDateToNearestTime } from '../../../lib/utils';
import { MongoRosteredMetric, MongoRosteredMetricModel } from '@agyletime/performance-utils';
import { EventDebugLog, EventDebugLogModel } from '../../../lib/models';
import { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils/lib/types/events/events';

const organisationId = 8669;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};
const taskDefinition_isAvailableOccupancy: TaskType = {
    name: 'Is Available Occupancy Task Task',
    id: 12345,
    taskId: 12345,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: true,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};
const taskDefinition_isAfterCallWork: TaskType = {
    name: 'Is After Call Work Task',
    id: 123456,
    taskId: 123456,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};

describe('Shift Updated Handler', () => {
    beforeAll(() => {
        // config.agyletime.clusterName = 'alpha' // To turn on debug logging
        process.env['DEBUG_EVENT_COLLECTION'] = 'true';
    });
    afterAll(() => {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete process.env['DEBUG_EVENT_COLLECTION'];
    });

    const isAvailableOccupancyTask_eventOne: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T10:58:42.000Z',
        endTime: '2023-12-01T12:58:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAfterCallWorkTask_eventOne = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T12:58:42.000Z',
        endTime: '2023-12-01T15:58:42.000Z',
        taskType: taskDefinition_isAfterCallWork,
    };
    const shiftTasks_eventOne = [isAvailableOccupancyTask_eventOne, isAfterCallWorkTask_eventOne];
    const shift_eventOne = {
        _id: 'aaaaaaa',
        user: { id: userOne.id },
        shiftTasks: shiftTasks_eventOne,
    } as unknown as Shift;

    const singleRecordEvent: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: shift_eventOne,
                            roster: {
                                id: '',
                                version: 0,
                            },
                            previousShift: shift_eventOne,
                        },
                        specversion: '',
                        type: '',
                        source: '',
                        id: '',
                        time: '',
                        datacontenttype: 'application/json',
                    } as AgyleTimeCloudEvent<ShiftUpdatedEvent>),
                },
            },
        ],
    } as KinesisStreamEvent;
    const isAvailableOccupancyTask1_eventTwo: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T01:01:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAvailableOccupancyTask2_eventTwo: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T01:01:42.000Z',
        endTime: '2023-12-01T01:02:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAvailableOccupancyTask3_eventTwo: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T01:02:42.000Z',
        endTime: '2023-12-01T01:03:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };

    const shiftTasks_eventTwo = [
        isAvailableOccupancyTask1_eventTwo,
        isAvailableOccupancyTask2_eventTwo,
        isAvailableOccupancyTask3_eventTwo,
    ];
    const multipleRecordEvent: KinesisStreamEvent = {
        Records: [
            ...singleRecordEvent.Records, // Include records from singleRecordEvent
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: {
                                _id: 'bbbbbbbbbb',
                                user: { id: userOne.id },
                                shiftTasks: shiftTasks_eventTwo,
                            } as unknown as Shift,
                            roster: {
                                id: '',
                                version: 0,
                            },
                            previousShift: {
                                _id: 'aaaa',
                                user: { id: userOne.id },
                                shiftTasks: shiftTasks_eventTwo,
                            } as unknown as Shift,
                        },
                        specversion: '',
                        type: '',
                        source: '',
                        id: '',
                        time: '',
                        datacontenttype: 'application/json',
                    } as AgyleTimeCloudEvent<ShiftUpdatedEvent>),
                },
            },
        ],
    } as KinesisStreamEvent;

    const preSeedRosteredHoursMetric = {
        organisationId,
        timeBucketAsLocal: '2023-12-01T10:45:00.000Z',
        timeBucketAsUtc: '2023-12-01T10:45:00.000Z',
        userId: 73185,
        workloadTaskId: 12345,
        rosteredAvailableTime_rosteredHours: 111111111,
        shiftId: shift_eventOne._id,
        totalRosteredTasks_rosteredHours: 111111111,
    };

    describe('Single event metric does not exist yet', () => {
        let workloadDataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let rosteredDataInMongo: MongoRosteredMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            await shiftUpdatedHandler(singleRecordEvent);
            workloadDataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            rosteredDataInMongo = await MongoRosteredMetricModel.find({ organisationId }).lean();
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoRosteredMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            const exepctedRosteredAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.rosteredAvailableTime_rosteredHours).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask_eventOne = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask_eventOne?.rosteredAvailableTime_rosteredHours).toBe(0);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });

        it('calculates total rostered hours', () => {
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            let exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAfterCallWorkTask_eventOne.endTime),
                new Date(isAfterCallWorkTask_eventOne.startTime),
            );
            expect(isAfterCallWorkMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });

    describe('Rostered metric already exists for bucket', () => {
        let workloadDataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let workloadDataInMongoBefore: MongoRosteredHoursWorkloadMetric[] = [];
        let rosteredDataInMongo: MongoRosteredMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            // Pre-seed rostered metrics
            await MongoRosteredHoursWorkloadMetricModel.create(preSeedRosteredHoursMetric);
            await MongoRosteredMetricModel.deleteMany({ organisationId });

            workloadDataInMongoBefore = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            await shiftUpdatedHandler(singleRecordEvent);
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
            workloadDataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            rosteredDataInMongo = await MongoRosteredMetricModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoRosteredMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('unsets the previous data in mongo correctly', () => {
            const previousMetric = workloadDataInMongoBefore[0];
            expect(previousMetric?.rosteredAvailableTime_rosteredHours).toBe(
                preSeedRosteredHoursMetric.rosteredAvailableTime_rosteredHours,
            );
            expect(previousMetric?.totalRosteredTasks_rosteredHours).toBe(
                preSeedRosteredHoursMetric.totalRosteredTasks_rosteredHours,
            );
            const resettedMetrics = workloadDataInMongo.find(
                (d) => d.userId === previousMetric?.userId && d.timeBucketAsLocal === previousMetric.timeBucketAsLocal,
            );
            expect(resettedMetrics?.rosteredAvailableTime_rosteredHours).not.toBe(
                previousMetric?.rosteredAvailableTime_rosteredHours,
            );
            expect(resettedMetrics?.totalRosteredTasks_rosteredHours).not.toBe(
                previousMetric?.totalRosteredTasks_rosteredHours,
            );
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            const expectedNumberOfTimeBuckets = workloadDataInMongo.filter((d) => d.shiftId).length;
            expect(workloadDataInMongo).toBeDefined();
            expect(expectedNumberOfTimeBuckets).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) =>
                    t.workloadTaskId === taskDefinition_isAvailableOccupancy.id &&
                    t.rosteredAvailableTime_rosteredHours,
            );
            const exepctedRosteredAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.rosteredAvailableTime_rosteredHours).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask_eventOne = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask_eventOne?.rosteredAvailableTime_rosteredHours).toBe(0);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });

        it('calculates total rostered hours', () => {
            expect(workloadDataInMongo).toBeDefined();
            const expectedNumberOfTimeBuckets = workloadDataInMongo.filter((d) => d.shiftId).length;
            expect(expectedNumberOfTimeBuckets).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id && t.shiftId === shift_eventOne._id,
            );
            let exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id && t.shiftId === shift_eventOne._id,
            );
            exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAfterCallWorkTask_eventOne.endTime),
                new Date(isAfterCallWorkTask_eventOne.startTime),
            );
            expect(isAfterCallWorkMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });

    describe('Multiple events metric does not exist yet', () => {
        let workloadDataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let rosteredDataInMongo: MongoRosteredMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];

        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoRosteredMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            await shiftUpdatedHandler(multipleRecordEvent);
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
            workloadDataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            rosteredDataInMongo = await MongoRosteredMetricModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoRosteredMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(workloadDataInMongo).toBeDefined();
            const expectedNumberOfTimeBuckets = new Set([
                ...shiftTasks_eventOne.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
                ...shiftTasks_eventTwo.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
            ]).size;

            expect(workloadDataInMongo.length).toBe(expectedNumberOfTimeBuckets);
            const isAvailableOccupancyMetricSum = workloadDataInMongo
                .filter((t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id)
                .reduce((a, c) => {
                    return a + c.rosteredAvailableTime_rosteredHours;
                }, 0);
            const exepctedRosteredAvailableTime = [
                isAvailableOccupancyTask_eventOne,
                isAvailableOccupancyTask1_eventTwo,
                isAvailableOccupancyTask2_eventTwo,
                isAvailableOccupancyTask3_eventTwo,
            ].reduce((a, c) => {
                const difference = differenceInSeconds(new Date(c.endTime), new Date(c.startTime));
                return a + difference;
            }, 0);
            expect(isAvailableOccupancyMetricSum).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask_eventOne = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask_eventOne?.rosteredAvailableTime_rosteredHours).toBe(0);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });

        it('calculates total rostered hours', () => {
            expect(workloadDataInMongo).toBeDefined();
            const expectedNumberOfTimeBuckets = new Set([
                ...shiftTasks_eventOne.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
                ...shiftTasks_eventTwo.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
            ]).size;
            expect(workloadDataInMongo.length).toBe(expectedNumberOfTimeBuckets);
            const isAvailableOccupancyMetricSum = workloadDataInMongo
                .filter((t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id)
                .reduce((a, c) => {
                    return a + c.totalRosteredTasks_rosteredHours;
                }, 0);
            const exepctedTotalRosteredHours = [
                isAvailableOccupancyTask_eventOne,
                isAvailableOccupancyTask1_eventTwo,
                isAvailableOccupancyTask2_eventTwo,
                isAvailableOccupancyTask3_eventTwo,
            ].reduce((a, c) => {
                const difference = differenceInSeconds(new Date(c.endTime), new Date(c.startTime));
                return a + difference;
            }, 0);
            expect(isAvailableOccupancyMetricSum).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetricSum = workloadDataInMongo
                .filter((t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id)
                .reduce((a, c) => {
                    return a + c.totalRosteredTasks_rosteredHours;
                }, 0);
            expect(isAfterCallWorkMetricSum).toBe(exepctedTotalRosteredHours);
            expect(rosteredDataInMongo).toBeDefined();
            expect(rosteredDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });
});
