import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { taskDataMergedHandler } from '../../../lib/metrics/eventHandlers/taskDataMergedHandler';
import { MongoActualHoursWorkloadMetric, MongoActualHoursWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../lib/userCache/UserCacheModel';
import {
    MongoActualMetric,
    MongoActualMetricModel,
    TaskDefinitionModel,
} from '@agyletime/performance-utils/lib/mongo/models';
import { TaskDefinitionMongoSchema } from '@agyletime/wfm-types/lib/schedule/taskDefinition';
import { differenceInSeconds } from 'date-fns';
import { EventDebugLog, EventDebugLogModel } from '../../../lib/models';

const organisationId = 10191019;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};
const taskDefinition_isAvailableOccupancy: Partial<TaskDefinitionMongoSchema> = {
    name: 'Is Available Occupancy Task Task',
    identifier: 'isAvailableOccupancy',
    id: 12345,
    description: 'Is Available Occupancy Task Task',
    organisationId,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: true,
    isPlanned: false,
    isBreak: false,
    isDefaultWorkloadTask: false,
    includeInTimeOffLimits: false,
    isIdentifierRegEx: false,
    isProductive: false,
    isLeave: false,
    isLogout: true,
    hideLeaveInAgentViews: false,
    isShiftSwapAllowed: 'No',
    isAfterCallWork: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
    defaultDuration: 30,
    businessRoles: [],
};
const taskDefinition_isAfterCallWork: Partial<TaskDefinitionMongoSchema> = {
    name: 'Is After Call Work Task',
    identifier: 'isAfterCallWork',
    id: 123456,
    description: 'Is After Call Work Task',
    organisationId,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isBreak: false,
    isDefaultWorkloadTask: false,
    includeInTimeOffLimits: false,
    isIdentifierRegEx: false,
    isProductive: false,
    isLeave: false,
    isLogout: true,
    hideLeaveInAgentViews: false,
    isShiftSwapAllowed: 'No',
    isAfterCallWork: true,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
    defaultDuration: 30,
    businessRoles: [],
};
const taskDefinition_forNoEndtime: Partial<TaskDefinitionMongoSchema> = {
    name: 'Is After Call Work Task',
    identifier: 'isAfterCallWork',
    id: 1234567,
    description: 'Is After Call Work Task',
    organisationId,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isBreak: false,
    isDefaultWorkloadTask: false,
    includeInTimeOffLimits: false,
    isIdentifierRegEx: false,
    isProductive: false,
    isLeave: false,
    isLogout: true,
    hideLeaveInAgentViews: false,
    isShiftSwapAllowed: 'No',
    isAfterCallWork: true,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
    defaultDuration: 30,
    businessRoles: [],
};

describe('the series of events', () => {
    beforeAll(() => {
        process.env['DEBUG_EVENT_COLLECTION'] = 'true';
    });
    afterAll(() => {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete process.env['DEBUG_EVENT_COLLECTION'];
    });

    const isAvailableOccupancyTask = {
        _id: '65692fd72d1f23f4830597c3',
        organisationId: organisationId,
        identifier: '44f637b4-6971-9807-5637-938fdbb58406',
        __v: 0,
        agentAlias: '0053t000009YK6MAAW',
        createdAt: '2023-12-01T00:59:03.430Z',
        shiftId: '6566b308d2e21a9b0c9f0be6',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T02:58:42.000Z',
        taskDefinitionId: taskDefinition_isAvailableOccupancy.id,
        taskTypeIdentifier: 'OFFLINE',
        updatedAt: '2023-12-01T00:59:03.430Z',
        userEmail: 'wfm.test@playvox.com',
        userId: userOne.id,
    };
    const isAfterCallWorkTask = {
        _id: '65692fd72d1f23f4830597c3',
        organisationId: organisationId,
        identifier: '44f637b4-6971-9807-5637-938fdbb58406',
        __v: 0,
        agentAlias: '0053t000009YK6MAAW',
        createdAt: '2023-12-01T00:59:03.430Z',
        shiftId: '6566b308d2e21a9b0c9f0be6',
        startTime: '2023-12-01T02:58:42.000Z',
        endTime: '2023-12-01T05:58:42.000Z',
        taskDefinitionId: taskDefinition_isAfterCallWork.id,
        taskTypeIdentifier: 'OFFLINE',
        updatedAt: '2023-12-01T00:59:03.430Z',
        userEmail: 'wfm.test@playvox.com',
        userId: userOne.id,
    };
    const noEndTimeTask = {
        _id: '65692fd72d1f23f4830597c3',
        organisationId: organisationId,
        identifier: '44f637b4-6971-9807-5637-938fdbb58406',
        __v: 0,
        agentAlias: '0053t000009YK6MAAW',
        createdAt: '2023-12-01T00:59:03.430Z',
        shiftId: '6566b308d2e21a9b0c9f0be6',
        startTime: '2023-12-01T02:58:42.000Z',
        taskDefinitionId: taskDefinition_forNoEndtime.id,
        taskTypeIdentifier: 'OFFLINE',
        updatedAt: '2023-12-01T00:59:03.430Z',
        userEmail: 'wfm.test@playvox.com',
        userId: userOne.id,
    };
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shiftId: '6566b308d2e21a9b0c9f0be6',
                            userId: userOne.id,
                            tasks: [isAvailableOccupancyTask, isAfterCallWorkTask, noEndTimeTask],
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Task Data Handler', () => {
        let workloadDataInMongo: MongoActualHoursWorkloadMetric[] = [];
        let actualsDataInMongo: MongoActualMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([
                taskDefinition_isAvailableOccupancy,
                taskDefinition_isAfterCallWork,
                taskDefinition_forNoEndtime,
            ]);
            // Clear the DB before we run this test
            await MongoActualHoursWorkloadMetricModel.deleteMany({ organisationId });
            await MongoActualMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            await taskDataMergedHandler(eventOne);
            workloadDataInMongo = await MongoActualHoursWorkloadMetricModel.find({ organisationId }).lean();
            actualsDataInMongo = await MongoActualMetricModel.find({ organisationId }).lean();
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await TaskDefinitionModel.deleteMany({ organisationId });
            await MongoActualHoursWorkloadMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates actual available time correctly', () => {
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            const exepctedActualAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.actualAvailableTime_actualHours).toBe(exepctedActualAvailableTime);
            const isAfterCallWorkTask = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask?.actualAvailableTime_actualHours).toBe(0);
            expect(actualsDataInMongo).toBeDefined();
            expect(actualsDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
        it('calculates actual after call work time correctly', () => {
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            expect(isAvailableOccupancyMetric?.actualAfterCallWorkTime_actualHours).toBe(0);
            const exepctedAfterCallWorkTime = differenceInSeconds(
                new Date(isAfterCallWorkTask.endTime),
                new Date(isAfterCallWorkTask.startTime),
            );
            const isAfterCallWorkMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkMetric?.actualAfterCallWorkTime_actualHours).toBe(exepctedAfterCallWorkTime);
            expect(actualsDataInMongo).toBeDefined();
            expect(actualsDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
        it('calculates total actual hours', () => {
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            let exepctedTotalActualHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalActualTasks_actualHours).toBe(exepctedTotalActualHours);

            const isAfterCallWorkMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            exepctedTotalActualHours = differenceInSeconds(
                new Date(isAfterCallWorkTask.endTime),
                new Date(isAfterCallWorkTask.startTime),
            );
            expect(isAfterCallWorkMetric?.totalActualTasks_actualHours).toBe(exepctedTotalActualHours);
            expect(actualsDataInMongo).toBeDefined();
            expect(actualsDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
        it('should not have generated metric for task with no endtime', () => {
            expect(workloadDataInMongo).toBeDefined();
            expect(workloadDataInMongo.length).toBeGreaterThan(0);
            const noEndTimeMetric = workloadDataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_forNoEndtime.id,
            );
            expect(noEndTimeMetric).toBeUndefined();
            expect(actualsDataInMongo).toBeDefined();
            expect(actualsDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });
});
