import { UserHistory, UserHistoryModel, type UserWithUpdatedAt } from '../../../lib/userHistory';
import UserHistoryProvider from '../../../lib/userHistory/UserHistoryProvider';
import { parseISO } from 'date-fns';
import { toZonedTime } from 'date-fns-tz';
import { UserMongoSchema } from '@agyletime/wfm-types/lib/organisation/user';

const organisationId = 32857235723985;

describe('user history', () => {
    beforeEach(async () => {
        // Clear out the db before each test
        await UserHistoryModel.deleteMany({ organisationId });
    });

    describe('when updating from events', () => {
        it('saves a new record', async () => {
            const user = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                    name: '',
                },
                timezone: 'Australia/Sydney',
                updatedAt: parseISO('2024-02-09'),
            } as unknown as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([user]);

            const results = await UserHistoryModel.find({ organisationId }).lean();

            expect(results.length).toEqual(1);
            expect(results[0]!.teamId).toEqual(user.team!.id);
        });

        it("doesn't insert a record if the user/team relationship hasn't changed", async () => {
            const firstUser = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                    name: '',
                },
                email: '',
                timezone: 'Australia/Sydney',
                updatedAt: parseISO('2024-02-09'),
            } as unknown as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([firstUser]);

            const resultsBefore = await UserHistoryModel.find({ organisationId }).lean();

            expect(resultsBefore.length).toEqual(1);
            expect(resultsBefore[0]!.updatedAt).toEqual(toZonedTime(firstUser.updatedAt, firstUser.timezone ?? 'UTC'));

            const secondUser = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                    name: 'team name',
                },
                timezone: 'Australia/Sydney',
                email: 'added an email address',
                updatedAt: parseISO('2024-02-10'),
            } as unknown as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([secondUser]);

            const resultsAfter = await UserHistoryModel.find({ organisationId }).lean();

            expect(resultsAfter.length).toEqual(1);
            expect(resultsAfter[0]!.updatedAt).toEqual(toZonedTime(firstUser.updatedAt, secondUser.timezone ?? 'UTC'));
        });

        it("doesn't create a 2nd record when a new combination is saved", async () => {
            const firstUser = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                    name: '',
                },
                timezone: 'Australia/Sydney',
                email: '',
                updatedAt: parseISO('2024-02-09'),
            } as unknown as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([firstUser]);

            const resultsBefore = await UserHistoryModel.find({ organisationId }).lean();

            expect(resultsBefore.length).toEqual(1);
            expect(resultsBefore[0]!.updatedAt).toEqual(toZonedTime(firstUser.updatedAt, firstUser.timezone ?? 'UTC'));

            const secondUser = {
                organisationId,
                id: 123,
                team: {
                    id: 7888,
                    name: '',
                },
                timezone: 'Australia/Sydney',
                email: 'added an email address',
                updatedAt: parseISO('2024-02-10'),
            } as unknown as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([secondUser]);

            const resultsAfter = await UserHistoryModel.find({ organisationId }).lean();

            // Expect 1 results
            expect(resultsAfter.length).toEqual(1);

            expect(resultsAfter[0]!.teamId).toEqual(secondUser.team!.id);
        });

        it('saves a new record when the users timezone changes', async () => {
            const baseline = new UserHistoryModel<UserHistory>({
                organisationId,
                userId: 1,
                teamId: 1,
                updatedAt: parseISO('2024-02-01'),
                timezone: 'UTC',
            });

            await baseline.save();

            const before: UserHistory | null = await UserHistoryModel.findOne<UserHistory>({
                organisationId,
                userId: baseline.userId,
            }).lean();

            expect(before?.updatedAt).toEqual(baseline.updatedAt);

            const updatedAt = parseISO('2024-02-02T11:00:00.000Z');

            const userUpdate = {
                organisationId,
                id: baseline.userId,
                timezone: 'Australia/Sydney',
                team: {
                    id: baseline.teamId,
                },
                updatedAt,
            } as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([userUpdate]);

            const after: UserHistory | null = await UserHistoryModel.findOne<UserHistory>({
                organisationId,
                userId: baseline.userId,
            });

            expect(after!.updatedAt).toEqual(toZonedTime(updatedAt, userUpdate.timezone ?? 'UTC'));
        });

        it('handles a null or undefined team id', async () => {
            const firstUser = {
                organisationId,
                id: 1,
                timezone: 'Australia/Sydney',
                team: {
                    id: null,
                },
                updatedAt: parseISO('2024-02-05'),
            } as unknown as UserWithUpdatedAt;

            const secondUser = {
                organisationId,
                id: 2,
                timezone: 'Australia/Sydney',
                updatedAt: parseISO('2024-02-05'),
            } as unknown as UserWithUpdatedAt;

            const thirdUser = {
                organisationId,
                id: 3,
                timezone: 'Australia/Sydney',
                team: {
                    id: undefined,
                },
                updatedAt: parseISO('2024-02-05'),
            } as unknown as UserWithUpdatedAt;

            await UserHistoryProvider.saveUserHistoryFromEvent([firstUser, secondUser, thirdUser]);

            const results = await UserHistoryModel.find({ organisationId }).lean();

            expect(results.length).toEqual(3);
            expect(results[0]!.teamId).toBeUndefined();
            expect(results[1]!.teamId).toBeUndefined();
            expect(results[2]!.teamId).toBeUndefined();
        });
    });

    describe('when syncing from org queries', () => {
        it('saves a new record', async () => {
            const user = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                    name: '',
                },
                timezone: 'Australia/Sydney',
            } as UserMongoSchema;

            const timestamp = parseISO('2024-02-09');
            await UserHistoryProvider.syncUserHistory({ users: [user], timestamp });

            const results = await UserHistoryModel.find({ organisationId }).lean();

            expect(results.length).toEqual(1);
            expect(results[0]!.teamId).toEqual(user.team.id);
        });

        it("doesn't insert a record if the user/team relationship hasn't changed", async () => {
            const firstUser = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                },
                timezone: 'Australia/Sydney',
            } as UserMongoSchema;

            const timestamp = parseISO('2024-02-09');

            await UserHistoryProvider.syncUserHistory({ users: [firstUser], timestamp });

            const resultsBefore = await UserHistoryModel.find({ organisationId }).lean();

            expect(resultsBefore.length).toEqual(1);
            expect(resultsBefore[0]!.updatedAt).toEqual(timestamp);

            const secondUser = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                    name: 'team name',
                },
                timezone: 'Australia/Sydney',
            } as UserMongoSchema;

            const secondTimestamp = parseISO('2024-02-10');

            await UserHistoryProvider.syncUserHistory({ users: [secondUser], timestamp: secondTimestamp });

            const resultsAfter = await UserHistoryModel.find({ organisationId }).lean();

            expect(resultsAfter.length).toEqual(1);
            expect(resultsAfter[0]!.updatedAt).toEqual(timestamp);
        });

        it('does create a 2nd record when a new combination is saved', async () => {
            const firstUser = {
                organisationId,
                id: 123,
                team: {
                    id: 5435,
                },
                timezone: 'Australia/Sydney',
            } as UserMongoSchema;

            const timestamp = parseISO('2024-02-09');

            await UserHistoryProvider.syncUserHistory({ users: [firstUser], timestamp });

            const resultsBefore = await UserHistoryModel.find({ organisationId }).lean();

            expect(resultsBefore.length).toEqual(1);
            expect(resultsBefore[0]!.updatedAt).toEqual(timestamp);

            const secondUser = {
                organisationId,
                id: 123,
                team: {
                    id: 7888,
                },
                timezone: 'Australia/Sydney',
            } as UserMongoSchema;

            const secondTimestamp = parseISO('2024-02-10');

            await UserHistoryProvider.syncUserHistory({ users: [secondUser], timestamp: secondTimestamp });

            const resultsAfter = await UserHistoryModel.find({ organisationId }).lean();

            // Expect 2 results
            expect(resultsAfter.length).toEqual(2);

            expect(resultsAfter[0]!.teamId).toEqual(firstUser.team.id);
            expect(resultsAfter[1]!.teamId).toEqual(secondUser.team.id);
        });

        it('saves a new record when the users timezone changes', async () => {
            const baseline = new UserHistoryModel<UserHistory>({
                organisationId,
                userId: 1,
                teamId: 1,
                updatedAt: parseISO('2024-02-01'),
                timezone: 'UTC',
            });

            await baseline.save();

            const before: UserHistory | null = await UserHistoryModel.findOne<UserHistory>({
                organisationId,
                userId: baseline.userId,
            }).lean();

            expect(before?.updatedAt).toEqual(baseline.updatedAt);

            const updatedAt = parseISO('2024-02-02');

            const userUpdate = {
                organisationId,
                id: baseline.userId,
                timezone: 'Australia/Sydney',
                team: {
                    id: baseline.teamId,
                },
            } as UserMongoSchema;

            await UserHistoryProvider.syncUserHistory({ users: [userUpdate], timestamp: updatedAt });

            const after = await UserHistoryModel.find<UserHistory>({
                organisationId,
                userId: baseline.userId,
            });

            expect(after.length).toEqual(2);
            expect(after[0]!.timezone).toEqual(before?.timezone);
            expect(after[1]!.timezone).toEqual(userUpdate.timezone);
        });

        it('handles a null or undefined team id', async () => {
            const firstUser = {
                organisationId,
                id: 1,
                timezone: 'Australia/Sydney',
                team: {
                    id: null,
                },
                updatedAt: parseISO('2024-02-05'),
            } as unknown as UserMongoSchema;

            const secondUser = {
                organisationId,
                id: 2,
                timezone: 'Australia/Sydney',
                team: {},
                updatedAt: parseISO('2024-02-05'),
            } as unknown as UserMongoSchema;

            const thirdUser = {
                organisationId,
                id: 3,
                timezone: 'Australia/Sydney',
                team: {
                    id: undefined,
                },
                updatedAt: parseISO('2024-02-05'),
            } as unknown as UserMongoSchema;

            await UserHistoryProvider.syncUserHistory({
                users: [firstUser, secondUser, thirdUser],
                timestamp: parseISO('2024-02-05'),
            });

            const results = await UserHistoryModel.find({ organisationId }).lean();

            expect(results.length).toEqual(3);
            expect(results[0]!.teamId).toBeUndefined();
            expect(results[1]!.teamId).toBeUndefined();
            expect(results[2]!.teamId).toBeUndefined();
        });
    });

    describe('when retrieving user history', () => {
        it('returns the latest teamId for the given date', async () => {
            const user1Team1 = new UserHistoryModel<UserHistory>({
                organisationId,
                userId: 1,
                teamId: 1,
                updatedAt: parseISO('2024-01-01'),
                timezone: 'Australia/Sydney',
            });

            const user1Team2 = new UserHistoryModel<UserHistory>({
                organisationId,
                userId: 1,
                teamId: 2,
                updatedAt: parseISO('2024-01-02'),
                timezone: 'Australia/Sydney',
            });

            const user1Team3 = new UserHistoryModel<UserHistory>({
                organisationId,
                userId: 1,
                teamId: 3,
                updatedAt: parseISO('2024-01-03'),
                timezone: 'Australia/Sydney',
            });

            await UserHistoryModel.insertMany([user1Team1, user1Team2, user1Team3]);

            const result = await UserHistoryProvider.getUserHistoryForOrgIds({
                organisationIds: [organisationId],
                timestamp: parseISO('2024-01-02T13:00:00'),
            });

            expect(result[0]!.teamId).toEqual(2);
        });

        it('returns all teamIds/timezones for users in a given list of org ids', async () => {
            const organisationIds = [3245783285, 546745745, 74343634643];
            const user1Team1 = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[0]!,
                userId: 1,
                teamId: 1,
                updatedAt: parseISO('2024-02-05'),
                timezone: 'Australia/Sydney',
            });

            const user1Team2 = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[0]!,
                userId: 1,
                teamId: 2,
                updatedAt: parseISO('2024-02-06'),
                timezone: 'Australia/Sydney',
            });

            const user2Timezone1 = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[1]!,
                userId: 2,
                teamId: 3,
                updatedAt: parseISO('2024-02-05T12:00:00.000'),
                timezone: 'Australia/Sydney',
            });

            const user2Timezone2 = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[1]!,
                userId: 2,
                teamId: 3,
                updatedAt: parseISO('2024-02-05T13:00:00.000'),
                timezone: 'Australia/Melbourne',
            });

            const user3 = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[2]!,
                userId: 3,
                teamId: 4,
                updatedAt: parseISO('2024-02-05'),
                timezone: 'Australia/Sydney',
            });

            const user4 = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[0]!,
                userId: 4,
                teamId: 4,
                updatedAt: parseISO('2024-02-05'),
                timezone: 'Australia/Sydney',
            });
            const user4OutsideDateRange = new UserHistoryModel<UserHistory>({
                organisationId: organisationIds[0]!,
                userId: 4,
                teamId: 5,
                updatedAt: parseISO('2024-02-07'),
                timezone: 'Australia/Sydney',
            });

            await UserHistoryModel.insertMany([
                user1Team1,
                user1Team2,
                user2Timezone1,
                user2Timezone2,
                user3,
                user4,
                user4OutsideDateRange,
            ]);

            const results = await UserHistoryProvider.getUserHistoryForOrgIds({
                organisationIds,
                timestamp: parseISO('2024-02-06'),
            });

            // 4 users
            expect(results.length).toEqual(4);

            const user1Result = results.find((r) => r.userId === 1);
            expect(user1Result?.teamId).toEqual(2);
            expect(user1Result?.timezone).toEqual('Australia/Sydney');

            const user2Result = results.find((r) => r.userId === 2);
            expect(user2Result?.teamId).toEqual(3);
            expect(user2Result?.timezone).toEqual('Australia/Melbourne');

            const user3Result = results.find((r) => r.userId === 3);
            expect(user3Result?.teamId).toEqual(4);
            expect(user3Result?.timezone).toEqual('Australia/Sydney');

            const user4Result = results.find((r) => r.userId === 4);
            expect(user4Result?.teamId).toEqual(4);
            expect(user4Result?.timezone).toEqual('Australia/Sydney');
        });

        it('falls back to the oldest date if none match the timestamp', async () => {
            const user1Team1 = new UserHistoryModel<UserHistory>({
                organisationId: organisationId,
                userId: 1,
                teamId: 1,
                updatedAt: parseISO('2024-02-14'),
                timezone: 'Australia/Sydney',
            });

            const user1Team2 = new UserHistoryModel<UserHistory>({
                organisationId: organisationId,
                userId: 1,
                teamId: 2,
                updatedAt: parseISO('2024-02-15'),
                timezone: 'Australia/Sydney',
            });

            await UserHistoryModel.insertMany([user1Team1, user1Team2]);

            const results = await UserHistoryProvider.getMostUpToDateHistory({
                organisationIds: [organisationId],
            });

            expect(results.length).toEqual(1);
            // Return the oldest value since none match
            expect(results[0]!.teamId).toEqual(1);
        });
    });
});
