import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../../utils/encodeKinesisBody';
import {
    MongoInteractionsWorkloadMetric,
    MongoInteractionsWorkloadMetricModel,
    TaskDefinitionModel,
    TaskDefinitionMongoSchemaWithLastOccurance,
} from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../../lib/userCache/UserCacheModel';
import { interactionCreatedHandler } from '../../../../lib/metrics/workload/eventHandlers/interactionCreatedHandler';

const organisationId = 3232323232323;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};

describe('Interaction Created Handler', () => {
    const workEffortSecs_eventOne = 32323;
    const workloadTaskId_eventOne = 123;
    const workloadTask_eventOne = {
        name: 'Is Available Occupancy Task Task',
        organisationId,
        id: workloadTaskId_eventOne,
        taskId: workloadTaskId_eventOne,
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: true,
        isPlanned: false,
        isProductive: false,
        isLeave: false,
        hideLeaveInAgentViews: false,
        isMergeAllowed: true,
        colour: '00000',
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        isShiftSwapAllowed: 'something',
    };
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            interaction: {
                                organisationId: organisationId,
                                identifier: 'ticketAudit:wat24592740335129_1698592078234',
                                __v: 0,
                                agentAlias: 'wat20274906959769',
                                contactIdentifier: 'wat2310.11354293.Tu5WwJsl4WWJ7',
                                createdAt: '2023-10-29T20:11:54.673Z',
                                description: 'Chat Message',
                                endTime: '2023-10-29T15:07:58.234Z',
                                isPrivateComment: false,
                                isPublicComment: false,
                                isReassign: false,
                                isStatusChange: false,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2023-10-29T15:07:58.234Z',
                                type: 'chat',
                                updatedAt: '2023-10-29T20:11:54.673Z',
                                userId: 1499,
                                workEffortSecs: workEffortSecs_eventOne,
                                workloadTaskId: workloadTaskId_eventOne,
                                workstreamDefinitionId: 5,
                                workstreamIdentifier: 'wat2154',
                                agent: {
                                    integrationAlias: 'wat20274906959769',
                                    wfmId: 1499,
                                    emailAddress: 'jpromero@playvox.com',
                                    timezone: 'America/New_York',
                                    teamId: 3,
                                },
                                workstreamDefinition: {
                                    id: 5,
                                    name: 'Catch All 1',
                                },
                            },
                            operation: 'insert',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Single event', () => {
        let dataInMongo: MongoInteractionsWorkloadMetric[] = [];
        let taskDefinitionsInMongo: TaskDefinitionMongoSchemaWithLastOccurance[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([workloadTask_eventOne]);
            // Clear the DB before we run this test
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });

            await interactionCreatedHandler(eventOne);
            dataInMongo = await MongoInteractionsWorkloadMetricModel.find({ organisationId }).lean();
            taskDefinitionsInMongo = await TaskDefinitionModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await TaskDefinitionModel.deleteMany({ organisationId });
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });
        });

        for (const taskDefinition of [workloadTask_eventOne]) {
            const todaysDate = new Date().toISOString().split('T')[0];
            it(`updates the lastOccurance on the task definition "${taskDefinition.name}" in cache to ${todaysDate}`, () => {
                const actualTaskDefinition = taskDefinitionsInMongo.find((t) => t.id === taskDefinition.taskId);
                const taskDefinitionDate = actualTaskDefinition?.lastOccurance.toISOString().split('T')[0];
                expect(taskDefinitionDate).toBe(todaysDate);
            });
        }

        it('calculates total handle time correctly', () => {
            // Expect there to be 2 workstreams
            expect(dataInMongo).toBeDefined();
            const expectedMetric = dataInMongo.find((d) => d.workloadTaskId === workloadTaskId_eventOne);
            expect(expectedMetric?.totalHandleTime_interactions).toBe(workEffortSecs_eventOne);
        });
    });
});
