import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../../utils/encodeKinesisBody';
import {
    MongoInteractionsWorkloadMetric,
    MongoInteractionsWorkloadMetricModel,
    TaskDefinitionModel,
    TaskDefinitionMongoSchemaWithLastOccurance,
} from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../../lib/userCache/UserCacheModel';
import { interactionUpdatedHandler } from '../../../../lib/metrics/workload/eventHandlers/interactionUpdatedHandler';

const organisationId = 3232323232325;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};

describe('Shift Created Handler', () => {
    const workEffortSecsAfter_eventOne = 32323;
    const workEffortSecsBefore_eventOne = 30000;
    const workloadTaskId_eventOne = 123;
    const workloadTask_eventOne = {
        name: 'Is Available Occupancy Task Task',
        organisationId,
        id: workloadTaskId_eventOne,
        taskId: workloadTaskId_eventOne,
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: true,
        isPlanned: false,
        isProductive: false,
        isLeave: false,
        hideLeaveInAgentViews: false,
        isMergeAllowed: true,
        colour: '00000',
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        isShiftSwapAllowed: 'something',
    };
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            interactionBefore: {
                                _id: '65418641bef04122f1706948',
                                identifier: 'ticketAudit:wat24682571149337_1698792799528',
                                organisationId: organisationId,
                                __v: 0,
                                agentAlias: 'wat6534796378649',
                                asOrganisationId: '9002',
                                asTeamId: '1',
                                asUserId: '681',
                                contactIdentifier: 'wat2310.11354293.TuJ7MrLjkY5LW',
                                createdAt: '2023-10-31T22:57:05.723Z',
                                description: 'Chat Message',
                                endTime: '2023-10-31T22:53:19.528Z',
                                isPrivateComment: false,
                                isPublicComment: false,
                                isReassign: false,
                                isStatusChange: false,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2023-10-31T22:53:19.528Z',
                                teamId: 1,
                                type: 'chat',
                                updatedAt: '2023-10-31T22:57:05.723Z',
                                userId: userOne.id,
                                workEffortSecs: workEffortSecsBefore_eventOne,
                                workstreamIdentifier: 'wat2158',
                                agent: {
                                    integrationAlias: 'wat6534796378649',
                                    wfmId: userOne.id,
                                    emailAddress: 'oaytekin@playvox.com',
                                    timezone: 'America/New_York',
                                    teamId: userOne.team?.id,
                                },
                            },
                            interactionAfter: {
                                identifier: 'ticketAudit:wat24682571149337_1698792799528',
                                organisationId: organisationId,
                                __v: 0,
                                agentAlias: 'wat6534796378649',
                                contactIdentifier: 'wat2310.11354293.TuJ7MrLjkY5LW',
                                createdAt: '2023-10-31T22:57:05.723Z',
                                description: 'Chat Message',
                                endTime: '2023-10-31T22:53:19.528Z',
                                isPrivateComment: false,
                                isPublicComment: false,
                                isReassign: false,
                                isStatusChange: false,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2023-10-31T22:53:19.528Z',
                                type: 'chat',
                                updatedAt: '2023-10-31T22:57:08.902Z',
                                userId: userOne.id,
                                workEffortSecs: workEffortSecsAfter_eventOne,
                                workstreamIdentifier: 'wat2158',
                                workloadTaskId: workloadTaskId_eventOne,
                                workstreamDefinitionId: 5,
                                agent: {
                                    integrationAlias: 'wat6534796378649',
                                    wfmId: userOne.id,
                                    emailAddress: 'oaytekin@playvox.com',
                                    timezone: 'America/New_York',
                                    teamId: userOne.team?.id,
                                },
                                workstreamDefinition: {
                                    id: 5,
                                    name: 'Catch All 1',
                                },
                            },
                            updates: [
                                {
                                    op: 'replace',
                                    path: '/updatedAt',
                                    value: '2023-10-31T22:57:08.902Z',
                                },
                                {
                                    op: 'replace',
                                    path: '/workloadTaskId',
                                    value: workloadTaskId_eventOne,
                                },
                                {
                                    op: 'replace',
                                    path: '/workstreamDefinitionId',
                                    value: 5,
                                },
                            ],
                            operation: 'update',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Single event', () => {
        let dataInMongo: MongoInteractionsWorkloadMetric[] = [];
        let taskDefinitionsInMongo: TaskDefinitionMongoSchemaWithLastOccurance[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([workloadTask_eventOne]);
            // Clear the DB before we run this test
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });

            await interactionUpdatedHandler(eventOne);
            dataInMongo = await MongoInteractionsWorkloadMetricModel.find({ organisationId }).lean();
            taskDefinitionsInMongo = await TaskDefinitionModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await TaskDefinitionModel.deleteMany({ organisationId });
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });
        });

        for (const taskDefinition of [workloadTask_eventOne]) {
            const todaysDate = new Date().toISOString().split('T')[0];
            it(`updates the lastOccurance on the task definition "${taskDefinition.name}" in cache to ${todaysDate}`, () => {
                const actualTaskDefinition = taskDefinitionsInMongo.find((t) => t.id === taskDefinition.taskId);
                expect(actualTaskDefinition).toBeDefined();
                expect(actualTaskDefinition?.lastOccurance).toBeDefined();
                const taskDefinitionDate = actualTaskDefinition?.lastOccurance.toISOString().split('T')[0];
                expect(taskDefinitionDate).toBe(todaysDate);
            });
        }

        it('calculates total handle time correctly', () => {
            // Expect there to be 2 workstreams
            expect(dataInMongo).toBeDefined();
            const expectedMetric = dataInMongo.find((d) => d.workloadTaskId === workloadTaskId_eventOne);
            expect(expectedMetric?.totalHandleTime_interactions).toBe(
                workEffortSecsAfter_eventOne - workEffortSecsBefore_eventOne,
            );
        });
    });
});
