import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../../utils/encodeKinesisBody';
import {
    MongoRosteredHoursWorkloadMetric,
    MongoRosteredHoursWorkloadMetricModel,
    TaskDefinitionModel,
    TaskDefinitionMongoSchemaWithLastOccurance,
} from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../../lib/userCache/UserCacheModel';
import { differenceInSeconds } from 'date-fns';
import { shiftCreatedHandler } from '../../../../lib/metrics/workload/eventHandlers/shiftCreatedHandler';
import { ShiftTask } from '../../../../lib/metrics/types';

const organisationId = 123124124124;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};
const taskDefinition_isAvailableOccupancy = {
    name: 'Is Available Occupancy Task Task',
    organisationId,
    id: 12345,
    taskId: 12345,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: true,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};
const taskDefinition_isAfterCallWork = {
    name: 'Is After Call Work Task',
    organisationId,
    id: 123456,
    taskId: 123456,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};

describe('Shift Created Handler', () => {
    const isAvailableOccupancyTask: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T02:58:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAfterCallWorkTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2025-12-01T02:58:42.000Z',
        endTime: '2025-12-01T05:58:42.000Z',
        taskType: taskDefinition_isAfterCallWork,
    };
    const shiftTasks = [isAvailableOccupancyTask, isAfterCallWorkTask];
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: {
                                _id: '6566b308d2e21a9b0c9f0be6',
                                user: { id: userOne.id },
                                shiftTasks,
                            },
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Single event', () => {
        let dataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let taskDefinitionsInMongo: TaskDefinitionMongoSchemaWithLastOccurance[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([taskDefinition_isAvailableOccupancy, taskDefinition_isAfterCallWork]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });

            await shiftCreatedHandler(eventOne);
            dataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            taskDefinitionsInMongo = await TaskDefinitionModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
        });

        for (const taskDefinition of [taskDefinition_isAvailableOccupancy, taskDefinition_isAfterCallWork]) {
            const expectedDates = new Map(shiftTasks.map((t) => [t.taskType.id, t.endTime]));
            const todaysDate = new Date().toISOString().split('T')[0];

            const expectedDateForDescription =
                new Date(expectedDates.get(taskDefinition.id)!).getTime() > new Date().getTime()
                    ? expectedDates.get(taskDefinition.id)!.split('T')[0]
                    : todaysDate;
            it(`updates the lastOccurance on the task definition "${taskDefinition.name}" in cache to ${expectedDateForDescription}`, () => {
                const actualTaskDefinition = taskDefinitionsInMongo.find((t) => t.id === taskDefinition.taskId);
                expect(actualTaskDefinition).toBeDefined();
                expect(actualTaskDefinition?.lastOccurance).toBeDefined();
                const taskDefinitionDate = actualTaskDefinition?.lastOccurance.toISOString().split('T')[0];
                const expectedDate =
                    new Date(expectedDates.get(actualTaskDefinition!.id)!).getTime() > new Date().getTime()
                        ? expectedDates.get(actualTaskDefinition!.id)!.split('T')[0]
                        : todaysDate;
                expect(taskDefinitionDate).toBe(expectedDate);
            });
        }

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            const exepctedRosteredAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.rosteredAvailableTime_rosteredHours).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask = dataInMongo.find((t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id);
            expect(isAfterCallWorkTask?.rosteredAvailableTime_rosteredHours).toBe(0);
        });

        it('calculates total rostered hours', () => {
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            let exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAfterCallWorkTask.endTime),
                new Date(isAfterCallWorkTask.startTime),
            );
            expect(isAfterCallWorkMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);
        });
    });
});
