import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../../utils/encodeKinesisBody';
import {
    MongoRosteredHoursWorkloadMetric,
    MongoRosteredHoursWorkloadMetricModel,
    TaskDefinitionModel,
    TaskDefinitionMongoSchemaWithLastOccurance,
} from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../../lib/userCache/UserCacheModel';
import { shiftDeletedHandler } from '../../../../lib/metrics/workload/eventHandlers/shiftDeletedHandler';
import { ShiftTask } from '@agyletime/wfm-client';

const organisationId = 124124125;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};

describe('Shift Deleted Handler', () => {
    const eventOne_shiftIdOne = 'event-shift-id-1';
    const eventOne_shiftIdTwo = 'event-shift-id-2';
    const taskDefinition_isAvailableOccupancy = {
        name: 'Is Available Occupancy Task Task',
        organisationId,
        id: 12345,
        taskId: 12345,
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: true,
        isPlanned: false,
        isProductive: false,
        isLeave: false,
        hideLeaveInAgentViews: false,
        isMergeAllowed: true,
        colour: '00000',
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        isShiftSwapAllowed: 'something',
    };
    const taskDefinition_isAfterCallWork = {
        name: 'Is After Call Work Task',
        organisationId,
        id: 123456,
        taskId: 123456,
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: false,
        isPlanned: false,
        isProductive: false,
        isLeave: false,
        hideLeaveInAgentViews: false,
        isMergeAllowed: true,
        colour: '00000',
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        isShiftSwapAllowed: 'something',
    };
    const isAvailableOccupancyTask: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T02:58:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
        optInType: '',
    };
    const isAfterCallWorkTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2025-12-01T02:58:42.000Z',
        endTime: '2025-12-01T05:58:42.000Z',
        taskType: taskDefinition_isAfterCallWork,
    };
    const shiftTasks = [isAvailableOccupancyTask, isAfterCallWorkTask];
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: {
                                _id: eventOne_shiftIdOne,
                                user: { id: userOne.id },
                                shiftTasks,
                            },
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    const preSeedRosteredHoursMetrics = [
        {
            organisationId,
            timeBucketAsLocal: '2023-12-01T10:45:00.000Z',
            timeBucketAsUtc: '2023-12-01T10:45:00.000Z',
            userId: 73185,
            workloadTaskId: taskDefinition_isAvailableOccupancy.id,
            rosteredAvailableTime_rosteredHours: 111111111,
            shiftId: eventOne_shiftIdOne,
            totalRosteredTasks_rosteredHours: 111111111,
        },
        {
            organisationId,
            timeBucketAsLocal: '2023-12-01T10:45:00.000Z',
            timeBucketAsUtc: '2023-12-01T10:45:00.000Z',
            userId: 73185,
            workloadTaskId: taskDefinition_isAfterCallWork.id,
            rosteredAvailableTime_rosteredHours: 111111111,
            shiftId: eventOne_shiftIdTwo,
            totalRosteredTasks_rosteredHours: 111111111,
        },
    ];

    describe('Single event', () => {
        let dataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let dataInMongoBefore: MongoRosteredHoursWorkloadMetric[] = [];
        let taskDefinitionsInMongo: TaskDefinitionMongoSchemaWithLastOccurance[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([taskDefinition_isAvailableOccupancy, taskDefinition_isAfterCallWork]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
            // Pre-seed rostered metrics
            await MongoRosteredHoursWorkloadMetricModel.create(preSeedRosteredHoursMetrics);

            dataInMongoBefore = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            await shiftDeletedHandler(eventOne);
            dataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            taskDefinitionsInMongo = await TaskDefinitionModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
        });

        for (const taskDefinition of [taskDefinition_isAvailableOccupancy, taskDefinition_isAfterCallWork]) {
            const expectedDates = new Map(shiftTasks.map((t) => [t.taskType.id, t.endTime]));

            const todaysDate = new Date().toISOString().split('T')[0];

            const expectedDateForDescription =
                new Date(expectedDates.get(taskDefinition.id)!).getTime() > new Date().getTime()
                    ? expectedDates.get(taskDefinition.id)!.split('T')[0]
                    : todaysDate;
            it(`updates the lastOccurance on the task definition "${taskDefinition.name}" in cache to ${expectedDateForDescription}`, () => {
                const actualTaskDefinition = taskDefinitionsInMongo.find((t) => t.id === taskDefinition.taskId);
                expect(actualTaskDefinition).toBeDefined();
                expect(actualTaskDefinition?.lastOccurance).toBeDefined();
                const taskDefinitionDate = actualTaskDefinition?.lastOccurance.toISOString().split('T')[0];
                const expectedDate =
                    new Date(expectedDates.get(actualTaskDefinition!.id)!).getTime() > new Date().getTime()
                        ? expectedDates.get(actualTaskDefinition!.id)!.split('T')[0]
                        : todaysDate;
                expect(taskDefinitionDate).toBe(expectedDate);
            });
        }

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(dataInMongo).toBeDefined();
            // The reset shouldn't change the number of documents, just unset properties.
            expect(dataInMongoBefore.length).toBe(preSeedRosteredHoursMetrics.length);
            expect(dataInMongo.length).toBe(preSeedRosteredHoursMetrics.length);
            const resettedRosteredMetrics = dataInMongo.find((d) => d.shiftId === eventOne_shiftIdOne);
            expect(dataInMongoBefore[0]?.totalRosteredTasks_rosteredHours).toBeGreaterThan(0);
            expect(dataInMongoBefore[0]?.rosteredAvailableTime_rosteredHours).toBeGreaterThan(0);
            expect(resettedRosteredMetrics).toBeUndefined();
            // The metrics not marked for reset should be unchanged
            const unchangedRosteredMetrics = dataInMongo.find((d) => d.shiftId === eventOne_shiftIdTwo);
            const expectedUnchangedShift = preSeedRosteredHoursMetrics.find((d) => d.shiftId === eventOne_shiftIdTwo);
            expect(unchangedRosteredMetrics?.rosteredAvailableTime_rosteredHours).toBe(
                expectedUnchangedShift?.rosteredAvailableTime_rosteredHours,
            );
            expect(unchangedRosteredMetrics?.totalRosteredTasks_rosteredHours).toBe(
                expectedUnchangedShift?.totalRosteredTasks_rosteredHours,
            );
        });
    });
});
