import { KinesisStreamEvent } from 'aws-lambda';
import { config } from '../../../../lib/config';
import encodeKinesisBody from '../../../utils/encodeKinesisBody';
import {
    MongoRosteredHoursWorkloadMetric,
    MongoRosteredHoursWorkloadMetricModel,
    TaskDefinitionModel,
    TaskDefinitionMongoSchemaWithLastOccurance,
} from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../../lib/userCache/UserCacheModel';
import { differenceInSeconds, parseISO } from 'date-fns';
import { shiftUpdatedHandler } from '../../../../lib/metrics/workload/eventHandlers/shiftUpdatedHandler';
import { Shift, ShiftTask, ShiftUpdatedEvent } from '../../../../lib/metrics/types';
import { roundDateToNearestTime } from '../../../../lib/utils';
import { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils/lib/types/events/events';

const organisationId = 123124124126;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};
const taskDefinition_isAvailableOccupancy = {
    name: 'Is Available Occupancy Task Task',
    organisationId,
    id: 12345,
    taskId: 12345,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: true,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};
const taskDefinition_isAfterCallWork = {
    name: 'Is After Call Work Task',
    organisationId,
    id: 123456,
    taskId: 123456,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isProductive: false,
    isLeave: false,
    hideLeaveInAgentViews: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
};

describe('Shift Updated Handler', () => {
    const isAvailableOccupancyTask_eventOne: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T10:58:42.000Z',
        endTime: '2023-12-01T12:58:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAfterCallWorkTask_eventOne = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T12:58:42.000Z',
        endTime: '2023-12-01T15:58:42.000Z',
        taskType: taskDefinition_isAfterCallWork,
    };
    const shiftTasks_eventOne = [isAvailableOccupancyTask_eventOne, isAfterCallWorkTask_eventOne];
    const shift_eventOne = {
        _id: 'aaaaaaa',
        user: { id: userOne.id },
        shiftTasks: shiftTasks_eventOne,
    } as unknown as Shift;
    const singleRecordEvent: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: shift_eventOne,
                            roster: {
                                id: '',
                                version: 0,
                            },
                            previousShift: shift_eventOne,
                        },
                        specversion: '',
                        type: '',
                        source: '',
                        id: '',
                        time: '',
                        datacontenttype: 'application/json',
                    } as AgyleTimeCloudEvent<ShiftUpdatedEvent>),
                },
            },
        ],
    } as KinesisStreamEvent;
    const isAvailableOccupancyTask1_eventTwo: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T01:01:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAvailableOccupancyTask2_eventTwo: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2023-12-01T01:01:42.000Z',
        endTime: '2023-12-01T01:02:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };
    const isAvailableOccupancyTask3_eventTwo: ShiftTask = {
        _id: '65692fd72d1f23f4830597c3',
        startTime: '2025-12-01T01:02:42.000Z',
        endTime: '2025-12-01T01:03:42.000Z',
        taskType: taskDefinition_isAvailableOccupancy,
    };

    const shiftTasks_eventTwo = [
        isAvailableOccupancyTask1_eventTwo,
        isAvailableOccupancyTask2_eventTwo,
        isAvailableOccupancyTask3_eventTwo,
    ];
    const multipleRecordEvent: KinesisStreamEvent = {
        Records: [
            ...singleRecordEvent.Records, // Include records from singleRecordEvent
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            shift: {
                                _id: 'bbbbbbbbbb',
                                user: { id: userOne.id },
                                shiftTasks: shiftTasks_eventTwo,
                            } as unknown as Shift,
                            roster: {
                                id: '',
                                version: 0,
                            },
                            previousShift: {
                                _id: 'aaaa',
                                user: { id: userOne.id },
                                shiftTasks: shiftTasks_eventTwo,
                            } as unknown as Shift,
                        },
                        specversion: '',
                        type: '',
                        source: '',
                        id: '',
                        time: '',
                        datacontenttype: 'application/json',
                    } as AgyleTimeCloudEvent<ShiftUpdatedEvent>),
                },
            },
        ],
    } as KinesisStreamEvent;

    const preSeedRosteredHoursMetric = {
        organisationId,
        timeBucketAsLocal: '2023-12-01T10:45:00.000Z',
        timeBucketAsUtc: '2023-12-01T10:45:00.000Z',
        userId: 73185,
        workloadTaskId: 12345,
        rosteredAvailableTime_rosteredHours: 111111111,
        shiftId: shift_eventOne._id,
        totalRosteredTasks_rosteredHours: 111111111,
    };

    describe('Single event metric does not exist yet', () => {
        let dataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let taskDefinitionsInMongo: TaskDefinitionMongoSchemaWithLastOccurance[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([taskDefinition_isAvailableOccupancy, taskDefinition_isAfterCallWork]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });

            await shiftUpdatedHandler(singleRecordEvent);
            dataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            taskDefinitionsInMongo = await TaskDefinitionModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
        });

        for (const taskDefinition of [...shiftTasks_eventOne]) {
            const expectedDates = new Map([...shiftTasks_eventOne].map((t) => [t.taskType.id, t.endTime]));

            const todaysDate = new Date().toISOString().split('T')[0];

            const expectedDateForDescription =
                new Date(expectedDates.get(taskDefinition.taskType.id)!).getTime() > new Date().getTime()
                    ? expectedDates.get(taskDefinition.taskType.id)!.split('T')[0]
                    : todaysDate;
            it(`updates the lastOccurance on the task definition "${taskDefinition.taskType.name}" in cache to ${expectedDateForDescription}`, () => {
                const actualTaskDefinition = taskDefinitionsInMongo.find(
                    (t) => t.id === taskDefinition.taskType.taskId,
                );
                expect(actualTaskDefinition).toBeDefined();
                expect(actualTaskDefinition?.lastOccurance).toBeDefined();
                const taskDefinitionDate = actualTaskDefinition?.lastOccurance.toISOString().split('T')[0];
                const expectedDate =
                    new Date(expectedDates.get(actualTaskDefinition!.id)!).getTime() > new Date().getTime()
                        ? expectedDates.get(actualTaskDefinition!.id)!.split('T')[0]
                        : todaysDate;
                expect(taskDefinitionDate).toBe(expectedDate);
            });
        }

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            const exepctedRosteredAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.rosteredAvailableTime_rosteredHours).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask_eventOne = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask_eventOne?.rosteredAvailableTime_rosteredHours).toBe(0);
        });

        it('calculates total rostered hours', () => {
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            let exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAfterCallWorkTask_eventOne.endTime),
                new Date(isAfterCallWorkTask_eventOne.startTime),
            );
            expect(isAfterCallWorkMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);
        });
    });

    describe('Rostered metric already exists for bucket', () => {
        let dataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        let dataInMongoBefore: MongoRosteredHoursWorkloadMetric[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });

            // Pre-seed rostered metrics
            await MongoRosteredHoursWorkloadMetricModel.create(preSeedRosteredHoursMetric);

            dataInMongoBefore = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
            await shiftUpdatedHandler(singleRecordEvent);
            dataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
        });

        it('unsets the previous data in mongo correctly', () => {
            const previousMetric = dataInMongoBefore[0];
            expect(previousMetric?.rosteredAvailableTime_rosteredHours).toBe(
                preSeedRosteredHoursMetric.rosteredAvailableTime_rosteredHours,
            );
            expect(previousMetric?.totalRosteredTasks_rosteredHours).toBe(
                preSeedRosteredHoursMetric.totalRosteredTasks_rosteredHours,
            );
            const resettedMetrics = dataInMongo.find(
                (d) => d.userId === previousMetric?.userId && d.timeBucketAsLocal === previousMetric.timeBucketAsLocal,
            );
            expect(resettedMetrics?.rosteredAvailableTime_rosteredHours).not.toBe(
                previousMetric?.rosteredAvailableTime_rosteredHours,
            );
            expect(resettedMetrics?.totalRosteredTasks_rosteredHours).not.toBe(
                previousMetric?.totalRosteredTasks_rosteredHours,
            );
        });

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            const expectedNumberOfTimeBuckets = dataInMongo.filter((d) => d.shiftId).length;
            expect(dataInMongo).toBeDefined();
            expect(expectedNumberOfTimeBuckets).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) =>
                    t.workloadTaskId === taskDefinition_isAvailableOccupancy.id &&
                    t.rosteredAvailableTime_rosteredHours,
            );
            const exepctedRosteredAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.rosteredAvailableTime_rosteredHours).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask_eventOne = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask_eventOne?.rosteredAvailableTime_rosteredHours).toBe(0);
        });

        it('calculates total rostered hours', () => {
            expect(dataInMongo).toBeDefined();
            const expectedNumberOfTimeBuckets = dataInMongo.filter((d) => d.shiftId).length;
            expect(expectedNumberOfTimeBuckets).toBe(shiftTasks_eventOne.length);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id && t.shiftId === shift_eventOne._id,
            );
            let exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask_eventOne.endTime),
                new Date(isAvailableOccupancyTask_eventOne.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id && t.shiftId === shift_eventOne._id,
            );
            exepctedTotalRosteredHours = differenceInSeconds(
                new Date(isAfterCallWorkTask_eventOne.endTime),
                new Date(isAfterCallWorkTask_eventOne.startTime),
            );
            expect(isAfterCallWorkMetric?.totalRosteredTasks_rosteredHours).toBe(exepctedTotalRosteredHours);
        });
    });

    describe('Multiple events metric does not exist yet', () => {
        let dataInMongo: MongoRosteredHoursWorkloadMetric[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });

            await shiftUpdatedHandler(multipleRecordEvent);
            dataInMongo = await MongoRosteredHoursWorkloadMetricModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoRosteredHoursWorkloadMetricModel.deleteMany({ organisationId });
        });

        it('calculates rostered available time correctly', () => {
            // Expect there to be 2 workstreams
            expect(dataInMongo).toBeDefined();
            const expectedNumberOfTimeBuckets = new Set([
                ...shiftTasks_eventOne.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
                ...shiftTasks_eventTwo.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
            ]).size;

            expect(dataInMongo.length).toBe(expectedNumberOfTimeBuckets);
            const isAvailableOccupancyMetricSum = dataInMongo
                .filter((t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id)
                .reduce((a, c) => {
                    return a + c.rosteredAvailableTime_rosteredHours;
                }, 0);
            const exepctedRosteredAvailableTime = [
                isAvailableOccupancyTask_eventOne,
                isAvailableOccupancyTask1_eventTwo,
                isAvailableOccupancyTask2_eventTwo,
                isAvailableOccupancyTask3_eventTwo,
            ].reduce((a, c) => {
                const difference = differenceInSeconds(new Date(c.endTime), new Date(c.startTime));
                return a + difference;
            }, 0);
            expect(isAvailableOccupancyMetricSum).toBe(exepctedRosteredAvailableTime);
            const isAfterCallWorkTask_eventOne = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkTask_eventOne?.rosteredAvailableTime_rosteredHours).toBe(0);
        });

        it('calculates total rostered hours', () => {
            expect(dataInMongo).toBeDefined();
            const expectedNumberOfTimeBuckets = new Set([
                ...shiftTasks_eventOne.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
                ...shiftTasks_eventTwo.map((t) =>
                    roundDateToNearestTime(parseISO(t.startTime), config.mongoBucketSizeInMinutes).toISOString(),
                ),
            ]).size;
            expect(dataInMongo.length).toBe(expectedNumberOfTimeBuckets);
            const isAvailableOccupancyMetricSum = dataInMongo
                .filter((t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id)
                .reduce((a, c) => {
                    return a + c.totalRosteredTasks_rosteredHours;
                }, 0);
            const exepctedTotalRosteredHours = [
                isAvailableOccupancyTask_eventOne,
                isAvailableOccupancyTask1_eventTwo,
                isAvailableOccupancyTask2_eventTwo,
                isAvailableOccupancyTask3_eventTwo,
            ].reduce((a, c) => {
                const difference = differenceInSeconds(new Date(c.endTime), new Date(c.startTime));
                return a + difference;
            }, 0);
            expect(isAvailableOccupancyMetricSum).toBe(exepctedTotalRosteredHours);

            const isAfterCallWorkMetricSum = dataInMongo
                .filter((t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id)
                .reduce((a, c) => {
                    return a + c.totalRosteredTasks_rosteredHours;
                }, 0);
            expect(isAfterCallWorkMetricSum).toBe(exepctedTotalRosteredHours);
        });
    });
});
