import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { MongoWorkstreamMetricModel } from '@agyletime/performance-utils';
import { workstreamMetricCreatedHandler } from '../../../lib/metrics/workstream/eventHandlers/workstreamMetricCreatedHandler';
import { UserHistoryModel } from '../../../lib/userHistory';
import { UserCacheModel } from '../../../lib/userCache/UserCacheModel';

const organisationId = 3425902;

describe('the workstream metrics', () => {
    beforeAll(async () => {
        await UserHistoryModel.create({
            organisationId,
            userId: 178867,
            teamId: 6,
            timezone: 'Australia/Sydney',
            updatedAt: new Date(),
        });

        await UserCacheModel.create({
            organisationId,
            id: 178867,
            email: 'kpeacey+zd@playvox.com',
            integrations: [
                {
                    integrationAlias: '1901111382168',
                },
            ],
        });
    });

    it('saves the event', async () => {
        const eventOne: KinesisStreamEvent = {
            Records: [
                {
                    kinesis: {
                        data: encodeKinesisBody({
                            specversion: '1.0',
                            type: 'com.playvox.workactivity.workstream.metrics.created',
                            source: '/workactivity',
                            id: '94ac6f3e-ceb7-4b18-9176-2197b5ebb6a4',
                            time: '2023-11-27T02:07:26.256Z',
                            dataschema: '0.1',
                            plvxaccountid: '',
                            plvxwfmid: 9,
                            plvxidentity: {
                                source: 'wfm',
                                organisationId,
                            },
                            datacontenttype: 'application/json',
                            data: {
                                workstreamMetrics: {
                                    organisationId,
                                    workstreamIdentifier: '10004837',
                                    __v: 0,
                                    createdAt: '2023-11-27T02:06:36.473Z',
                                    firstReplyAgentAlias: {
                                        integrationAlias: '1901111382168',
                                        wfmId: 178867,
                                        emailAddress: 'kpeacey+zd@playvox.com',
                                        timezone: 'Australia/Sydney',
                                        teamId: 6,
                                    },
                                    firstReplyEndTime: '2023-11-27T02:05:31Z',
                                    firstReplyHandleTimeSecs: 12,
                                    firstReplyTeamId: 6,
                                    firstReplyTime: '2023-11-27T02:05:31Z',
                                    firstResolutionHandleTimeSecs: 16,
                                    firstResolutionTime: '2023-11-27T02:05:50Z',
                                    firstResolvedAgentAlias: {
                                        integrationAlias: '1901111382168',
                                        wfmId: 178867,
                                        emailAddress: 'kpeacey+zd@playvox.com',
                                        timezone: 'Australia/Sydney',
                                        teamId: 6,
                                    },
                                    firstResolvedTeamId: 6,
                                    firstResolvedUserId: 178867,
                                    handleTimeBreakdownByUsers: [
                                        {
                                            agentAlias: '1901111382168',
                                            userId: 178867,
                                            firstReplyHandleTimeSecs: 12,
                                            firstResolutionHandleTimeSecs: 16,
                                            totalResolutionHandleTimeSecs: 16,
                                            totalHandleTimeSecs: 28,
                                        },
                                    ],
                                    lastResolutionHandleTimeSecs: 0,
                                    qualityScores: [],
                                    totalFirstAnswerInteractions: 1,
                                    totalHandleTime: 28,
                                    totalInteractions: 2,
                                    totalNumberOfSolvedInteractions: 1,
                                    totalPrivateComments: 0,
                                    totalPublicComments: 2,
                                    totalReassigns: 0,
                                    totalResolutionHandleTimeSecs: 16,
                                    totalTouchTimeSecs: 28,
                                    totalTouches: 2,
                                    updatedAt: '2023-11-27T02:06:36.473Z',
                                },
                                isEventPublishedToEB: false,
                            },
                        }),
                    },
                },
            ],
        } as KinesisStreamEvent;

        // Clear the DB first
        await MongoWorkstreamMetricModel.deleteMany({ organisationId });

        await workstreamMetricCreatedHandler(eventOne);

        const dataInMongo = await MongoWorkstreamMetricModel.find({ organisationId }).lean();

        expect(dataInMongo.length).toEqual(1);

        expect(dataInMongo[0]!.firstResolveTeamId).toEqual(6);
    });

    it('handles a missing first resolve team id', async () => {
        const eventOne: KinesisStreamEvent = {
            Records: [
                {
                    kinesis: {
                        data: encodeKinesisBody({
                            specversion: '1.0',
                            type: 'com.playvox.workactivity.workstream.metrics.created',
                            source: '/workactivity',
                            id: '94ac6f3e-ceb7-4b18-9176-2197b5ebb6a4',
                            time: '2023-11-27T02:07:26.256Z',
                            dataschema: '0.1',
                            plvxaccountid: '',
                            plvxwfmid: 9,
                            plvxidentity: {
                                source: 'wfm',
                                organisationId,
                            },
                            datacontenttype: 'application/json',
                            data: {
                                workstreamMetrics: {
                                    organisationId,
                                    workstreamIdentifier: '10004837',
                                    __v: 0,
                                    createdAt: '2023-11-27T02:06:36.473Z',
                                    firstReplyAgentAlias: {
                                        integrationAlias: '1901111382168',
                                        wfmId: 178867,
                                        emailAddress: 'kpeacey+zd@playvox.com',
                                        timezone: 'Australia/Sydney',
                                        teamId: 6,
                                    },
                                    firstReplyEndTime: '2023-11-27T02:05:31Z',
                                    firstReplyHandleTimeSecs: 12,
                                    firstReplyTeamId: 6,
                                    firstReplyTime: '2023-11-27T02:05:31Z',
                                    firstResolutionHandleTimeSecs: 16,
                                    firstResolutionTime: '2023-11-27T02:05:50Z',
                                    firstResolvedAgentAlias: {
                                        integrationAlias: '1901111382168',
                                        wfmId: 178867,
                                        emailAddress: 'kpeacey+zd@playvox.com',
                                        timezone: 'Australia/Sydney',
                                    },
                                    firstResolvedTeamId: 6,
                                    firstResolvedUserId: 178867,
                                    handleTimeBreakdownByUsers: [
                                        {
                                            agentAlias: '1901111382168',
                                            userId: 178867,
                                            firstReplyHandleTimeSecs: 12,
                                            firstResolutionHandleTimeSecs: 16,
                                            totalResolutionHandleTimeSecs: 16,
                                            totalHandleTimeSecs: 28,
                                        },
                                    ],
                                    lastResolutionHandleTimeSecs: 0,
                                    qualityScores: [],
                                    totalFirstAnswerInteractions: 1,
                                    totalHandleTime: 28,
                                    totalInteractions: 2,
                                    totalNumberOfSolvedInteractions: 1,
                                    totalPrivateComments: 0,
                                    totalPublicComments: 2,
                                    totalReassigns: 0,
                                    totalResolutionHandleTimeSecs: 16,
                                    totalTouchTimeSecs: 28,
                                    totalTouches: 2,
                                    updatedAt: '2023-11-27T02:06:36.473Z',
                                },
                                isEventPublishedToEB: false,
                            },
                        }),
                    },
                },
            ],
        } as KinesisStreamEvent;

        // Clear the DB first
        await MongoWorkstreamMetricModel.deleteMany({ organisationId });

        await workstreamMetricCreatedHandler(eventOne);

        const dataInMongo = await MongoWorkstreamMetricModel.find({ organisationId }).lean();

        expect(dataInMongo.length).toEqual(1);

        expect(dataInMongo[0]!.firstResolveTeamId).toEqual(6);
    });

    it('handles a missing first resolve user id', async () => {
        const eventOne: KinesisStreamEvent = {
            Records: [
                {
                    kinesis: {
                        data: encodeKinesisBody({
                            specversion: '1.0',
                            type: 'com.playvox.workactivity.workstream.metrics.created',
                            source: '/workactivity',
                            id: '94ac6f3e-ceb7-4b18-9176-2197b5ebb6a4',
                            time: '2023-11-27T02:07:26.256Z',
                            dataschema: '0.1',
                            plvxaccountid: '',
                            plvxwfmid: 9,
                            plvxidentity: {
                                source: 'wfm',
                                organisationId,
                            },
                            datacontenttype: 'application/json',
                            data: {
                                workstreamMetrics: {
                                    organisationId,
                                    workstreamIdentifier: '10004837',
                                    __v: 0,
                                    createdAt: '2023-11-27T02:06:36.473Z',
                                    firstReplyAgentAlias: {
                                        integrationAlias: '1901111382168',
                                        wfmId: 178867,
                                        emailAddress: 'kpeacey+zd@playvox.com',
                                        timezone: 'Australia/Sydney',
                                        teamId: 6,
                                    },
                                    firstReplyEndTime: '2023-11-27T02:05:31Z',
                                    firstReplyHandleTimeSecs: 12,
                                    firstReplyTeamId: 6,
                                    firstReplyTime: '2023-11-27T02:05:31Z',
                                    firstResolutionHandleTimeSecs: 16,
                                    firstResolutionTime: '2023-11-27T02:05:50Z',
                                    firstResolvedAgentAlias: {
                                        integrationAlias: '1901111382168',
                                        emailAddress: 'kpeacey+zd@playvox.com',
                                        timezone: 'Australia/Sydney',
                                        teamId: 6,
                                    },
                                    firstResolvedTeamId: 6,
                                    firstResolvedUserId: 178867,
                                    handleTimeBreakdownByUsers: [
                                        {
                                            agentAlias: '1901111382168',
                                            userId: 178867,
                                            firstReplyHandleTimeSecs: 12,
                                            firstResolutionHandleTimeSecs: 16,
                                            totalResolutionHandleTimeSecs: 16,
                                            totalHandleTimeSecs: 28,
                                        },
                                    ],
                                    lastResolutionHandleTimeSecs: 0,
                                    qualityScores: [],
                                    totalFirstAnswerInteractions: 1,
                                    totalHandleTime: 28,
                                    totalInteractions: 2,
                                    totalNumberOfSolvedInteractions: 1,
                                    totalPrivateComments: 0,
                                    totalPublicComments: 2,
                                    totalReassigns: 0,
                                    totalResolutionHandleTimeSecs: 16,
                                    totalTouchTimeSecs: 28,
                                    totalTouches: 2,
                                    updatedAt: '2023-11-27T02:06:36.473Z',
                                },
                                isEventPublishedToEB: false,
                            },
                        }),
                    },
                },
            ],
        } as KinesisStreamEvent;

        // Clear the DB first
        await MongoWorkstreamMetricModel.deleteMany({ organisationId });

        await workstreamMetricCreatedHandler(eventOne);

        const dataInMongo = await MongoWorkstreamMetricModel.find({ organisationId }).lean();

        expect(dataInMongo.length).toEqual(1);

        expect(dataInMongo[0]!.firstResolveTeamId).toEqual(6);
    });
});
