import type { InteractionEventParams, InteractionEventType } from '../../../lib/metrics';
import * as ConvertUtcTimeToLocal from '../../../lib/utils/getTimezoneForUser';
import { parseISO } from 'date-fns';
import calculateInteractionCreatedDiff from '../../../lib/metrics/interactions/calculateInteractionCreatedDiff';

let currentVersion: InteractionEventType;

const filter: InteractionEventParams = {
    organisationId: 1,
    userId: 2,
    teamId: 3,
    interactionType: '4',
    workstreamIdentifier: '5',
    timezone: 'Australia/Sydney',
    taskTypeId: 1,
};

describe('calculateInteractionCreatedDiff', () => {
    beforeEach(() => {
        currentVersion = {
            organisationId: 0,
            identifier: 'identifier',
            type: 'chat',
            startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
            endTime: new Date(2022, 1, 1, 15, 0, 0).toISOString(),
            agent: {
                integrationAlias: 'agent alias',
            },
            workEffortSecs: 10,
            workstreamIdentifier: 'aasfasfk',
            integration: {
                integrationId: 1,
                qmIntegrationId: 2,
                integrationName: 'integration',
            },
            createdAt: '2023-01-01',
            workloadTaskId: 1,
        };

        jest.spyOn(ConvertUtcTimeToLocal, 'getTimezoneForUser').mockImplementation(async () =>
            Promise.resolve('Australia/Sydney'),
        );
    });

    const createAndAssert = ({
        currentVersion,
        fieldName,
        expectedValue,
    }: {
        currentVersion: InteractionEventType;
        fieldName: string;
        expectedValue: number | undefined;
    }) => {
        const result = calculateInteractionCreatedDiff({
            interaction: currentVersion,
            params: filter,
        });
        // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
        expect((result as any)?.query.$inc[fieldName]).toEqual(expectedValue);
    };

    it('utc time buckets are correct', () => {
        const result = calculateInteractionCreatedDiff({
            interaction: currentVersion,
            params: filter,
        });
        // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
        expect((result.query.$set as any)?.timeBucketAsUtc).toEqual(parseISO(currentVersion.startTime));
    });

    it('gracefully handles an invalid interaction type', () => {
        currentVersion.type = 'Foo';

        createAndAssert({
            currentVersion,
            fieldName: 'numberOfInteractions_all',
            expectedValue: 0,
        });
    });

    describe('does not count interactions with no handle time', () => {
        it('Chat - 0 seconds', () => {
            currentVersion.type = 'Chat';
            currentVersion.workEffortSecs = 0;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_chat',
                expectedValue: 0,
            });
        });

        it('Chat - null', () => {
            currentVersion.type = 'chat';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = null;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_chat',
                expectedValue: 0,
            });
        });

        it('Chat - undefined', () => {
            currentVersion.type = 'Chat';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = undefined;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_chat',
                expectedValue: 0,
            });
        });

        it('Worklog - 0 seconds', () => {
            currentVersion.type = 'Worklog';
            currentVersion.workEffortSecs = 0;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_ticket',
                expectedValue: 0,
            });
        });

        it('Worklog - null', () => {
            currentVersion.type = 'Worklog';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = null;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_ticket',
                expectedValue: 0,
            });
        });

        it('Worklog - undefined', () => {
            currentVersion.type = 'Worklog';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = undefined;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_ticket',
                expectedValue: 0,
            });
        });

        it('Inbound Phone - 0 seconds', () => {
            currentVersion.type = 'Inbound_Phone';
            currentVersion.workEffortSecs = 0;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_inboundPhone',
                expectedValue: 0,
            });
        });

        it('Inbound Phone - null', () => {
            currentVersion.type = 'inbound_phone';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = null;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_inboundPhone',
                expectedValue: 0,
            });
        });

        it('Inbound Phone - undefined', () => {
            currentVersion.type = 'Inbound_Phone';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = undefined;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_inboundPhone',
                expectedValue: 0,
            });
        });

        it('Outbound Phone - 0 seconds', () => {
            currentVersion.type = 'Outbound_Phone';
            currentVersion.workEffortSecs = 0;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_outboundPhone',
                expectedValue: 0,
            });
        });

        it('Outbound Phone - null', () => {
            currentVersion.type = 'outbound_phone';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = null;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_outboundPhone',
                expectedValue: 0,
            });
        });

        it('Outbound Phone - undefined', () => {
            currentVersion.type = 'Outbound_Phone';
            // @ts-expect-error overriding type
            currentVersion.workEffortSecs = undefined;

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_outboundPhone',
                expectedValue: 0,
            });
        });
    });

    describe('increments every metric', () => {
        it('numberOfInteractions_all (Chat)', () => {
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_all',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_all (Ticket)', () => {
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_all',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_all (Inbound Phone)', () => {
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_all',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_all (Outbound Phone)', () => {
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_all',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_chat', () => {
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_chat',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_ticket', () => {
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_ticket',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_inboundPhone', () => {
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_inboundPhone',
                expectedValue: 1,
            });
        });

        it('numberOfInteractions_outboundPhone', () => {
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'numberOfInteractions_outboundPhone',
                expectedValue: 1,
            });
        });

        it('totalInteractionHandleTime_all (Chat)', () => {
            const workEffort = 55;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_all',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_all (Ticket)', () => {
            const workEffort = 66;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_all',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_all (Inbound Phone)', () => {
            const workEffort = 77;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_all',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_all (Outbound Phone)', () => {
            const workEffort = 88;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_all',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_chat', () => {
            const workEffort = 55;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_chat',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_ticket', () => {
            const workEffort = 66;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_ticket',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_inboundPhone', () => {
            const workEffort = 77;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_inboundPhone',
                expectedValue: workEffort,
            });
        });

        it('totalInteractionHandleTime_outboundPhone', () => {
            const workEffort = 88;
            currentVersion.workEffortSecs = workEffort;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalInteractionHandleTime_outboundPhone',
                expectedValue: workEffort,
            });
        });

        it('totalReassigns_all (Chat)', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_all',
                expectedValue: 0, // 1,
            });
        });

        it('totalReassigns_all (Ticket)', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_all',
                expectedValue: 1,
            });
        });

        it('totalReassigns_all (Inbound Phone)', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_all',
                expectedValue: 0, // 1,
            });
        });

        it('totalReassigns_all (Outbound Phone)', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_all',
                expectedValue: 0, // 1,
            });
        });

        it('totalReassigns_chat', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_chat',
                expectedValue: undefined, // 1,
            });
        });

        it('totalReassigns_ticket', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_ticket',
                expectedValue: 1,
            });
        });

        it('totalReassigns_ticket (from status_change)', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Status_Change';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_ticket',
                expectedValue: 1,
            });
        });

        it('totalReassigns_inboundPhone', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_inboundPhone',
                expectedValue: undefined, // 1,
            });
        });

        it('totalReassigns_outboundPhone', () => {
            currentVersion.isReassign = true;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalReassigns_outboundPhone',
                expectedValue: undefined, // 1,
            });
        });

        it('totalPrivateComments_all (Ticket)', () => {
            currentVersion.isPrivateComment = true;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalPrivateComments_all',
                expectedValue: 1,
            });
        });

        it('totalPrivateComments_ticket', () => {
            currentVersion.isPrivateComment = true;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalPrivateComments_ticket',
                expectedValue: 1,
            });
        });

        it('totalPublicComments_all (Ticket)', () => {
            currentVersion.isPublicComment = true;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalPublicComments_all',
                expectedValue: 1,
            });
        });

        it('totalPublicComments_ticket', () => {
            currentVersion.isPublicComment = true;
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalPublicComments_ticket',
                expectedValue: 1,
            });
        });

        it.skip('totalDeclined_all (Chat)', () => {
            currentVersion.completionStatus = 'Agent_Declined';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_all',
                expectedValue: 1,
            });
        });

        it('totalDeclined_all (Ticket)', () => {
            currentVersion.completionStatus = 'Agent_Declined';
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_all',
                expectedValue: 1,
            });
        });

        it('totalDeclined_all (Inbound Phone)', () => {
            currentVersion.completionStatus = 'agent_declined';
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_all',
                expectedValue: 1,
            });
        });

        it('totalDeclined_all (Outbound Phone)', () => {
            currentVersion.completionStatus = 'Agent_Declined';
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_all',
                expectedValue: 1,
            });
        });

        it.skip('totalDeclined_chat', () => {
            currentVersion.completionStatus = 'agent_declined';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_chat',
                expectedValue: 1,
            });
        });

        it('totalDeclined_ticket', () => {
            currentVersion.completionStatus = 'Agent_Declined';
            currentVersion.type = 'Worklog';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_ticket',
                expectedValue: 1,
            });
        });

        it('totalDeclined_inboundPhone', () => {
            currentVersion.completionStatus = 'agent_declined';
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_inboundPhone',
                expectedValue: 1,
            });
        });

        it('totalDeclined_outboundPhone', () => {
            currentVersion.completionStatus = 'Agent_Declined';
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalDeclined_outboundPhone',
                expectedValue: 1,
            });
        });

        it('totalAssigned_all (Chat)', () => {
            currentVersion.completionStatus = 'Assigned_Accepted';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalAssigned_all',
                expectedValue: 1,
            });
        });

        it.skip('totalAssigned_all (Inbound Phone)', () => {
            currentVersion.completionStatus = 'Assigned_Accepted';
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalAssigned_all',
                expectedValue: 1,
            });
        });

        it('totalAssigned_chat', () => {
            currentVersion.completionStatus = 'Assigned_Accepted';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalAssigned_chat',
                expectedValue: 1,
            });
        });

        it.skip('totalAssigned_inboundPhone', () => {
            currentVersion.completionStatus = 'Assigned_Accepted';
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalAssigned_inboundPhone',
                expectedValue: 1,
            });
        });

        it('totalSelfAssignedInteractions_all (Chat)', () => {
            currentVersion.completionStatus = 'self_assigned';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalSelfAssignedInteractions_all',
                expectedValue: 1,
            });
        });

        it('totalSelfAssignedInteractions_chat', () => {
            currentVersion.completionStatus = 'Self_Assigned';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalSelfAssignedInteractions_chat',
                expectedValue: 1,
            });
        });

        it('totalMissed_all (Chat)', () => {
            currentVersion.completionStatus = 'Assigned_Missed';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalMissed_all',
                expectedValue: 1,
            });
        });

        it('totalMissed_all (Inbound Phone)', () => {
            currentVersion.completionStatus = 'Agent_Missed';
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalMissed_all',
                expectedValue: 1,
            });
        });

        it('totalMissed_chat', () => {
            currentVersion.completionStatus = 'Assigned_Missed';
            currentVersion.type = 'Chat';

            createAndAssert({
                currentVersion,
                fieldName: 'totalMissed_chat',
                expectedValue: 1,
            });
        });

        it('totalMissed_inboundPhone', () => {
            currentVersion.completionStatus = 'Agent_Missed';
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'totalMissed_inboundPhone',
                expectedValue: 1,
            });
        });

        it('holdTime_all (Inbound Phone)', () => {
            const holdTime = 123;
            currentVersion.holdTimeSecs = holdTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'holdTime_all',
                expectedValue: holdTime,
            });
        });

        it('holdTime_all (Outbound Phone)', () => {
            const holdTime = 123;
            currentVersion.holdTimeSecs = holdTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'holdTime_all',
                expectedValue: holdTime,
            });
        });

        it('holdTime_inboundPhone', () => {
            const holdTime = 123;
            currentVersion.holdTimeSecs = holdTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'holdTime_inboundPhone',
                expectedValue: holdTime,
            });
        });

        it('holdTime_outboundPhone', () => {
            const holdTime = 123;
            currentVersion.holdTimeSecs = holdTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'holdTime_outboundPhone',
                expectedValue: holdTime,
            });
        });

        it('talkTime_all (Inbound Phone)', () => {
            const talkTime = 123;
            currentVersion.talkTimeSecs = talkTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'talkTime_all',
                expectedValue: talkTime,
            });
        });

        it('talkTime_all (Outbound Phone)', () => {
            const talkTime = 123;
            currentVersion.talkTimeSecs = talkTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'talkTime_all',
                expectedValue: talkTime,
            });
        });

        it('talkTime_inboundPhone', () => {
            const talkTime = 123;
            currentVersion.talkTimeSecs = talkTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'talkTime_inboundPhone',
                expectedValue: talkTime,
            });
        });

        it('talkTime_outboundPhone', () => {
            const talkTime = 123;
            currentVersion.talkTimeSecs = talkTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'talkTime_outboundPhone',
                expectedValue: talkTime,
            });
        });

        it('consultTime_all (Inbound Phone)', () => {
            const consultationTime = 123;
            currentVersion.consultationTimeSecs = consultationTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'consultTime_all',
                expectedValue: consultationTime,
            });
        });

        it('consultTime_all (Outbound Phone)', () => {
            const consultationTime = 123;
            currentVersion.consultationTimeSecs = consultationTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'consultTime_all',
                expectedValue: consultationTime,
            });
        });

        it('consultTime_inboundPhone', () => {
            const consultationTime = 123;
            currentVersion.consultationTimeSecs = consultationTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'consultTime_inboundPhone',
                expectedValue: consultationTime,
            });
        });

        it('consultTime_outboundPhone', () => {
            const consultationTime = 123;
            currentVersion.consultationTimeSecs = consultationTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'consultTime_outboundPhone',
                expectedValue: consultationTime,
            });
        });

        it('wrapTime_all (Inbound Phone)', () => {
            const wrapTime = 123;
            currentVersion.wrapTimeSecs = wrapTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'wrapTime_all',
                expectedValue: wrapTime,
            });
        });

        it('wrapTime_all (Outbound Phone)', () => {
            const wrapTime = 123;
            currentVersion.wrapTimeSecs = wrapTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'wrapTime_all',
                expectedValue: wrapTime,
            });
        });

        it('wrapTime_inboundPhone', () => {
            const wrapTime = 123;
            currentVersion.wrapTimeSecs = wrapTime;
            currentVersion.type = 'Inbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'wrapTime_inboundPhone',
                expectedValue: wrapTime,
            });
        });

        it('wrapTime_outboundPhone', () => {
            const wrapTime = 123;
            currentVersion.wrapTimeSecs = wrapTime;
            currentVersion.type = 'Outbound_Phone';

            createAndAssert({
                currentVersion,
                fieldName: 'wrapTime_outboundPhone',
                expectedValue: wrapTime,
            });
        });

        it('totalResolutions (solved)', () => {
            currentVersion.statusChangeTo = 'solved';

            createAndAssert({
                currentVersion,
                fieldName: 'totalResolutions',
                expectedValue: 1,
            });
        });

        it('totalResolutions (open)', () => {
            currentVersion.statusChangeTo = 'open';

            createAndAssert({
                currentVersion,
                fieldName: 'totalResolutions',
                expectedValue: 0,
            });
        });
    });
});
