import type { InteractionEventParams, InteractionEventType } from '../../../lib/metrics';
import calculateInteractionUpdatedDiff from '../../../lib/metrics/interactions/calculateInteractionUpdatedDiff';
import { addSeconds } from 'date-fns';

let currentVersion: InteractionEventType;
let previousVersion: InteractionEventType;
const startTime = new Date(2022, 1, 1, 13, 0, 0);
const endTime = new Date(2022, 1, 1, 15, 0, 0);
const timeIncrementInSeconds: number = 60 * 60;
const bucketOffsetIncrementInSeconds: number = 17 * 60;

const incrementTime = (): Date => addSeconds(endTime, timeIncrementInSeconds);
const moveToANewBucket = (): Date => addSeconds(startTime, bucketOffsetIncrementInSeconds);

const filter: InteractionEventParams = {
    organisationId: 1,
    userId: 2,
    teamId: 3,
    interactionType: '4',
    workstreamIdentifier: '5',
    timezone: 'Australia/Sydney',
    taskTypeId: 1,
};

describe('calculateInteractionUpdatedDiff', () => {
    beforeEach(() => {
        currentVersion = {
            organisationId: 0,
            identifier: 'identifier',
            type: 'chat',
            startTime: startTime.toISOString(),
            endTime: endTime.toISOString(),
            agent: {
                integrationAlias: 'agent alias',
            },
            workEffortSecs: 10,
            workstreamIdentifier: 'aasfasfk',
            integration: {
                integrationId: 1,
                qmIntegrationId: 2,
                integrationName: 'integration',
            },
            createdAt: '2023-01-01',
            workloadTaskId: 1,
        };

        previousVersion = {
            organisationId: 0,
            identifier: 'identifier',
            type: 'chat',
            startTime: startTime.toISOString(),
            endTime: endTime.toISOString(),
            agent: {
                integrationAlias: 'agent alias',
            },
            workEffortSecs: 10,
            workstreamIdentifier: 'aasfasfk',
            integration: {
                integrationId: 1,
                qmIntegrationId: 2,
                integrationName: 'integration',
            },
            createdAt: '2023-01-01',
            workloadTaskId: 1,
        };
    });

    it('gracefully handles an invalid interaction type', () => {
        currentVersion.type = 'Foo';
        currentVersion.workEffortSecs = 0;

        const results = calculateInteractionUpdatedDiff({
            previousVersion,
            currentVersion,
            params: filter,
        });

        // Expect to use the same bucket
        expect(results.length).toBe(1);

        // Expect to see 1 result
        const result = results[0];
        expect(result).toBeTruthy();

        // Expect that the invalid type isn't counted
        expect(result?.query.$inc!.numberOfInteractions_all).toEqual(-1);
    });

    describe('within the same time bucket', () => {
        describe('successfully increments', () => {
            const incrementAndAssertWithinBucket = ({
                currentVersion,
                previousVersion,
                fieldName,
                expectedValue,
            }: {
                currentVersion: InteractionEventType;
                previousVersion: InteractionEventType;
                fieldName: string;
                expectedValue: number | undefined;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.endTime = incrementTime().toISOString();

                const results = calculateInteractionUpdatedDiff({
                    previousVersion,
                    currentVersion,
                    params: filter,
                });

                // Expect to use the same bucket
                expect(results.length).toBe(1);

                // Expect to see 1 result
                const result = results[0];
                expect(result).toBeTruthy();

                // Expect that the given field has incremented by timeIncrementInSeconds
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result as any)?.query?.$inc[fieldName]).toEqual(expectedValue);
            };

            describe('interaction metrics', () => {
                it('numberOfInteractions_all (Chat)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_all (Ticket)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_all (Inbound Phone)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_all (Outbound Phone)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_chat', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_ticket', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_inboundPhone', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        expectedValue: 1,
                    });
                });

                it('numberOfInteractions_outboundPhone', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        expectedValue: 1,
                    });
                });

                it('totalInteractionHandleTime_all (Chat)', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Ticket)', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Inbound Phone)', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Outbound Phone)', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_chat', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_chat',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_ticket', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_ticket',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_inboundPhone', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_inboundPhone',
                        expectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_outboundPhone', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_outboundPhone',
                        expectedValue: workEffort,
                    });
                });

                it('totalReassigns_all (Chat)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: 1,
                    });
                });

                it('totalReassigns_all (Ticket)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: 1,
                    });
                });

                it('totalReassigns_all (Inbound Phone)', () => {
                    currentVersion.isReassign = false;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: 1,
                    });
                });

                it('totalReassigns_all (Outbound Phone)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: 1,
                    });
                });

                it('totalReassigns_chat', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_chat',
                        expectedValue: undefined, // 1,
                    });
                });

                it('totalReassigns_ticket', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        expectedValue: 1,
                    });
                });

                it('totalReassigns_ticket (from status change)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Status_Change';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Status_Change';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        expectedValue: 1,
                    });
                });

                it('totalReassigns_inboundPhone', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_inboundPhone',
                        expectedValue: undefined, // 1,
                    });
                });

                it('totalReassigns_outboundPhone', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_outboundPhone',
                        expectedValue: undefined, // 1,
                    });
                });

                it('totalPrivateComments_all (Ticket)', () => {
                    previousVersion.isPrivateComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_all',
                        expectedValue: 1,
                    });
                });

                it('totalPrivateComments_ticket', () => {
                    previousVersion.isPrivateComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_ticket',
                        expectedValue: 1,
                    });
                });

                it('totalPublicComments_all (Ticket)', () => {
                    previousVersion.isPublicComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_all',
                        expectedValue: 1,
                    });
                });

                it('totalPublicComments_ticket', () => {
                    previousVersion.isPublicComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_ticket',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_all (Chat)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'agent_declined';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_all (Ticket)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_all (Outbound Phone)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: 1,
                    });
                });

                it.skip('totalDeclined_chat', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_chat',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_ticket', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_ticket',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_inboundPhone', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_inboundPhone',
                        expectedValue: 1,
                    });
                });

                it('totalDeclined_outboundPhone', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_outboundPhone',
                        expectedValue: 1,
                    });
                });

                it('totalAssigned_all (Chat)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        expectedValue: 1,
                    });
                });

                it('totalAssigned_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'assigned_accepted';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        expectedValue: 1,
                    });
                });

                it('totalAssigned_chat', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_chat',
                        expectedValue: 1,
                    });
                });

                it.skip('totalAssigned_inboundPhone', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_inboundPhone',
                        expectedValue: 1,
                    });
                });

                it('totalSelfAssignedInteractions_all (Chat)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_all',
                        expectedValue: 1,
                    });
                });

                it('totalSelfAssignedInteractions_chat', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_chat',
                        expectedValue: 1,
                    });
                });

                it('totalMissed_all (Chat)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        expectedValue: 1,
                    });
                });

                it('totalMissed_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        expectedValue: 1,
                    });
                });

                it('totalMissed_chat', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Missed';
                    currentVersion.type = 'Chat';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_chat',
                        expectedValue: 1,
                    });
                });

                it('totalMissed_inboundPhone', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_inboundPhone',
                        expectedValue: 1,
                    });
                });

                it('holdTime_all (Inbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        expectedValue: holdTime,
                    });
                });

                it('holdTime_all (Outbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        expectedValue: holdTime,
                    });
                });

                it('holdTime_inboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_inboundPhone',
                        expectedValue: holdTime,
                    });
                });

                it('holdTime_outboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_outboundPhone',
                        expectedValue: holdTime,
                    });
                });

                it('talkTime_all (Inbound Phone)', () => {
                    const talkTime = 123;
                    currentVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        expectedValue: talkTime,
                    });
                });

                it('talkTime_all (Outbound Phone)', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        expectedValue: talkTime,
                    });
                });

                it('talkTime_inboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_inboundPhone',
                        expectedValue: talkTime,
                    });
                });

                it('talkTime_outboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_outboundPhone',
                        expectedValue: talkTime,
                    });
                });

                it('consultTime_all (Inbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        expectedValue: consultationTime,
                    });
                });

                it('consultTime_all (Outbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        expectedValue: consultationTime,
                    });
                });

                it('consultTime_inboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_inboundPhone',
                        expectedValue: consultationTime,
                    });
                });

                it('consultTime_outboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_outboundPhone',
                        expectedValue: consultationTime,
                    });
                });

                it('wrapTime_all (Inbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        expectedValue: wrapTime,
                    });
                });

                it('wrapTime_all (Outbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        expectedValue: wrapTime,
                    });
                });

                it('wrapTime_inboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_inboundPhone',
                        expectedValue: wrapTime,
                    });
                });

                it('wrapTime_outboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_outboundPhone',
                        expectedValue: wrapTime,
                    });
                });

                it('totalResolutions (solved)', () => {
                    previousVersion.statusChangeTo = 'new';
                    currentVersion.statusChangeTo = 'solved';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        expectedValue: 1,
                    });
                });

                it('totalResolutions (no change)', () => {
                    previousVersion.statusChangeTo = 'solved';
                    currentVersion.statusChangeTo = 'solved';

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        expectedValue: 0,
                    });
                });
            });

            describe('does not increase interaction count with no handle time', () => {
                it('Chat - 0 seconds', () => {
                    previousVersion.type = 'Chat';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Chat';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        expectedValue: 0,
                    });
                });

                it('Ticket - 0 seconds', () => {
                    previousVersion.type = 'Worklog';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        expectedValue: 0,
                    });
                });

                it('Inbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Inbound_Phone';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        expectedValue: 0,
                    });
                });

                it('Outbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Outbound_Phone';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        expectedValue: 0,
                    });
                });
            });

            describe('missing handle time is treated as 0', () => {
                it('for chat', () => {
                    previousVersion.type = 'Chat';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Chat';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: 0,
                    });

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_chat',
                        expectedValue: 0,
                    });
                });

                it('for ticket', () => {
                    previousVersion.type = 'Worklog';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Worklog';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: 0,
                    });

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_ticket',
                        expectedValue: 0,
                    });
                });

                it('for inbound phone', () => {
                    previousVersion.type = 'Inbound_Phone';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Inbound_Phone';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: 0,
                    });

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_inboundPhone',
                        expectedValue: 0,
                    });
                });

                it('for outbound phone', () => {
                    previousVersion.type = 'Outbound_Phone';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Outbound_Phone';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: 0,
                    });

                    incrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_outboundPhone',
                        expectedValue: 0,
                    });
                });
            });
        });

        describe('successfully decrements', () => {
            const decrementAndAssertWithinBucket = ({
                currentVersion,
                previousVersion,
                fieldName,
                expectedValue,
            }: {
                currentVersion: InteractionEventType;
                previousVersion: InteractionEventType;
                fieldName: string;
                expectedValue: number | undefined;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.endTime = incrementTime().toISOString();

                const results = calculateInteractionUpdatedDiff({
                    previousVersion,
                    currentVersion,
                    params: filter,
                });

                // Expect to use the same bucket
                expect(results.length).toBe(1);

                // Expect to see 1 result
                const result = results[0];
                expect(result).toBeTruthy();

                // Expect that the given field has incremented by timeIncrementInSeconds
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result as any)?.query?.$inc[fieldName]).toEqual(expectedValue);
            };

            describe('interaction metrics', () => {
                it('numberOfInteractions_all (Chat)', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_all (Ticket)', () => {
                    previousVersion.type = 'Worklog';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_all (Inbound Phone)', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_all (Outbound Phone)', () => {
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_chat', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_ticket', () => {
                    previousVersion.type = 'Worklog';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_inboundPhone', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        expectedValue: -1,
                    });
                });

                it('numberOfInteractions_outboundPhone', () => {
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        expectedValue: -1,
                    });
                });

                it('totalInteractionHandleTime_all (Chat)', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Ticket)', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Inbound Phone)', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Outbound Phone)', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_chat', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_chat',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_ticket', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_ticket',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_inboundPhone', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_inboundPhone',
                        expectedValue: -workEffort,
                    });
                });

                it('totalInteractionHandleTime_outboundPhone', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_outboundPhone',
                        expectedValue: -workEffort,
                    });
                });

                it('totalReassigns_all (Chat)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: -1,
                    });
                });

                it('totalReassigns_all (Ticket)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: -1,
                    });
                });

                it('totalReassigns_all (Inbound Phone)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: -1,
                    });
                });

                it('totalReassigns_all (Outbound Phone)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        expectedValue: -1,
                    });
                });

                it('totalReassigns_chat', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_chat',
                        expectedValue: undefined, // -1,
                    });
                });

                it('totalReassigns_ticket', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        expectedValue: -1,
                    });
                });

                it('totalReassigns_ticket (from status change)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Status_Change';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        expectedValue: -1,
                    });
                });

                it('totalReassigns_inboundPhone', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_inboundPhone',
                        expectedValue: undefined, // -1,
                    });
                });

                it('totalReassigns_outboundPhone', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_outboundPhone',
                        expectedValue: undefined, // -1,
                    });
                });

                it('totalPrivateComments_all (Ticket)', () => {
                    previousVersion.isPrivateComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_all',
                        expectedValue: -1,
                    });
                });

                it('totalPrivateComments_ticket', () => {
                    previousVersion.isPrivateComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_ticket',
                        expectedValue: -1,
                    });
                });

                it('totalPublicComments_all (Ticket)', () => {
                    previousVersion.isPublicComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.type = 'Chat';
                    currentVersion.isPublicComment = true;

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_all',
                        expectedValue: -1,
                    });
                });

                it('totalPublicComments_ticket', () => {
                    previousVersion.isPublicComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_ticket',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_all (Chat)', () => {
                    previousVersion.completionStatus = 'agent_declined';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_all (Ticket)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_all (Outbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        expectedValue: -1,
                    });
                });

                it.skip('totalDeclined_chat', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_chat',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_ticket', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_ticket',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_inboundPhone', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_inboundPhone',
                        expectedValue: -1,
                    });
                });

                it('totalDeclined_outboundPhone', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_outboundPhone',
                        expectedValue: -1,
                    });
                });

                it('totalAssigned_all (Chat)', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        expectedValue: -1,
                    });
                });

                it('totalAssigned_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        expectedValue: -1,
                    });
                });

                it('totalAssigned_chat', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_chat',
                        expectedValue: -1,
                    });
                });

                it.skip('totalAssigned_inboundPhone', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_inboundPhone',
                        expectedValue: -1,
                    });
                });

                it('totalSelfAssignedInteractions_all (Chat)', () => {
                    previousVersion.completionStatus = 'self_assigned';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_all',
                        expectedValue: -1,
                    });
                });

                it('totalSelfAssignedInteractions_chat', () => {
                    previousVersion.completionStatus = 'Self_Assigned';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_chat',
                        expectedValue: -1,
                    });
                });

                it('totalMissed_all (Chat)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        expectedValue: -1,
                    });
                });

                it('totalMissed_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        expectedValue: -1,
                    });
                });

                it('totalMissed_chat', () => {
                    previousVersion.completionStatus = 'Assigned_Missed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Missed';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_chat',
                        expectedValue: -1,
                    });
                });

                it('totalMissed_inboundPhone', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_inboundPhone',
                        expectedValue: -1,
                    });
                });

                it('holdTime_all (Inbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        expectedValue: -holdTime,
                    });
                });

                it('holdTime_all (Outbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        expectedValue: -holdTime,
                    });
                });

                it('holdTime_inboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_inboundPhone',
                        expectedValue: -holdTime,
                    });
                });

                it('holdTime_outboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_outboundPhone',
                        expectedValue: -holdTime,
                    });
                });

                it('talkTime_all (Inbound Phone)', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        expectedValue: -talkTime,
                    });
                });

                it('talkTime_all (Outbound Phone)', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        expectedValue: -talkTime,
                    });
                });

                it('talkTime_inboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_inboundPhone',
                        expectedValue: -talkTime,
                    });
                });

                it('talkTime_outboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_outboundPhone',
                        expectedValue: -talkTime,
                    });
                });

                it('consultTime_all (Inbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        expectedValue: -consultationTime,
                    });
                });

                it('consultTime_all (Outbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        expectedValue: -consultationTime,
                    });
                });

                it('consultTime_inboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_inboundPhone',
                        expectedValue: -consultationTime,
                    });
                });

                it('consultTime_outboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_outboundPhone',
                        expectedValue: -consultationTime,
                    });
                });

                it('wrapTime_all (Inbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        expectedValue: -wrapTime,
                    });
                });

                it('wrapTime_all (Outbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        expectedValue: -wrapTime,
                    });
                });

                it('wrapTime_inboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_inboundPhone',
                        expectedValue: -wrapTime,
                    });
                });

                it('wrapTime_outboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_outboundPhone',
                        expectedValue: -wrapTime,
                    });
                });

                it.skip('totalOffered_all (Chat)', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_all',
                        expectedValue: -999, // TODO: Calculation TBC
                    });
                });

                it.skip('totalOffered_all (Inbound Phone)', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_all',
                        expectedValue: -999, // TODO: Calculation TBC
                    });
                });

                it.skip('totalOffered_chat', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_chat',
                        expectedValue: -999, // TODO: Calculation TBC
                    });
                });

                it.skip('totalOffered_inboundPhone', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_inboundPhone',
                        expectedValue: -999, // TODO: Calculation TBC
                    });
                });
            });

            describe('decrements interaction count with no handle time', () => {
                it('Chat - 0 seconds', () => {
                    previousVersion.type = 'Chat';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Chat';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        expectedValue: -1,
                    });
                });

                it('Ticket - 0 seconds', () => {
                    previousVersion.type = 'Worklog';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        expectedValue: -1,
                    });
                });

                it('Inbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Inbound_Phone';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        expectedValue: -1,
                    });
                });

                it('Outbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Outbound_Phone';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        expectedValue: -1,
                    });
                });

                it('totalResolutions (re-opened)', () => {
                    previousVersion.statusChangeTo = 'solved';
                    currentVersion.statusChangeTo = 'open';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        expectedValue: 0, // Resolution count doesn't change
                    });
                });

                it('totalResolutions (no change)', () => {
                    previousVersion.statusChangeTo = 'solved';
                    currentVersion.statusChangeTo = 'solved';

                    decrementAndAssertWithinBucket({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        expectedValue: 0,
                    });
                });
            });
        });

        describe('successfully modifies an interaction', () => {
            const incrementAndAssertWithinBucket = ({
                currentVersion,
                previousVersion,
                fieldName,
                expectedValue,
            }: {
                currentVersion: InteractionEventType;
                previousVersion: InteractionEventType;
                fieldName: string;
                expectedValue: number | undefined;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.endTime = incrementTime().toISOString();

                const results = calculateInteractionUpdatedDiff({
                    previousVersion,
                    currentVersion,
                    params: filter,
                });

                // Expect to use the same bucket
                expect(results.length).toBe(1);

                // Expect to see 1 result
                const result = results[0];
                expect(result).toBeTruthy();

                // Expect that the given field has incremented by timeIncrementInSeconds
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result as any)?.query?.$inc[fieldName]).toEqual(expectedValue);
            };

            it('totalInteractionHandleTime_all (Chat)', () => {
                const workEffort = 55;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_all (Ticket)', () => {
                const workEffort = 66;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_all (Inbound Phone)', () => {
                const workEffort = 77;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_all (Outbound Phone)', () => {
                const workEffort = 88;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_chat', () => {
                const workEffort = 55;
                currentVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_chat',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_ticket', () => {
                const workEffort = 66;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_ticket',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_inboundPhone', () => {
                const workEffort = 77;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Chat';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_inboundPhone',
                    expectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_outboundPhone', () => {
                const workEffort = 88;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Chat';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_outboundPhone',
                    expectedValue: workEffort,
                });
            });

            it('totalReassigns_all (Chat)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    expectedValue: 1,
                });
            });

            it('totalReassigns_all (Ticket)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    expectedValue: 1,
                });
            });

            it('totalReassigns_all (Inbound Phone)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    expectedValue: 1,
                });
            });

            it('totalReassigns_all (Outbound Phone)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    expectedValue: 1,
                });
            });

            it('totalReassigns_chat', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.isReassign = true;
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_chat',
                    expectedValue: undefined, // 1,
                });
            });

            it('totalReassigns_ticket', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.isReassign = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_ticket',
                    expectedValue: 1,
                });
            });

            it('totalReassigns_ticket (from status change)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.isReassign = true;
                currentVersion.type = 'Status_Change';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_ticket',
                    expectedValue: 1,
                });
            });

            it('totalReassigns_inboundPhone', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Chat';
                currentVersion.isReassign = true;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_inboundPhone',
                    expectedValue: undefined, // 1,
                });
            });

            it('totalReassigns_outboundPhone', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Chat';
                currentVersion.isReassign = true;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_outboundPhone',
                    expectedValue: undefined, // 1,
                });
            });

            it('totalPrivateComments_all (Ticket)', () => {
                previousVersion.isPrivateComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPrivateComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPrivateComments_all',
                    expectedValue: 1,
                });
            });

            it('totalPrivateComments_ticket', () => {
                previousVersion.isPrivateComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPrivateComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPrivateComments_ticket',
                    expectedValue: 1,
                });
            });

            it('totalPublicComments_all (Ticket)', () => {
                previousVersion.isPublicComment = true;
                previousVersion.type = 'Chat';
                currentVersion.type = 'Worklog';
                currentVersion.isPublicComment = true;

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPublicComments_all',
                    expectedValue: 1,
                });
            });

            it('totalPublicComments_ticket', () => {
                previousVersion.isPublicComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPublicComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPublicComments_ticket',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_all (Chat)', () => {
                previousVersion.completionStatus = 'agent_declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_all (Ticket)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_all (Inbound Phone)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_all (Outbound Phone)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    expectedValue: 1,
                });
            });

            it.skip('totalDeclined_chat', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_chat',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_ticket', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Worklog';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_ticket',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_inboundPhone', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_inboundPhone',
                    expectedValue: 1,
                });
            });

            it('totalDeclined_outboundPhone', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_outboundPhone',
                    expectedValue: 1,
                });
            });

            it('totalAssigned_all (Chat)', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_all',
                    expectedValue: 1,
                });
            });

            it('totalAssigned_all (Inbound Phone)', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_all',
                    expectedValue: 1,
                });
            });

            it('totalAssigned_chat', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_chat',
                    expectedValue: 1,
                });
            });

            it.skip('totalAssigned_inboundPhone', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_inboundPhone',
                    expectedValue: 1,
                });
            });

            it('totalSelfAssignedInteractions_all (Chat)', () => {
                previousVersion.completionStatus = 'Self_Assigned';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Self_Assigned';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalSelfAssignedInteractions_all',
                    expectedValue: 1,
                });
            });

            it('totalSelfAssignedInteractions_chat', () => {
                previousVersion.completionStatus = 'Self_Assigned';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Self_Assigned';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalSelfAssignedInteractions_chat',
                    expectedValue: 1,
                });
            });

            it('totalMissed_all (Chat)', () => {
                previousVersion.completionStatus = 'Agent_Missed';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Missed';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_all',
                    expectedValue: 1,
                });
            });

            it('totalMissed_all (Inbound Phone)', () => {
                previousVersion.completionStatus = 'Agent_Missed';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Missed';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_all',
                    expectedValue: 1,
                });
            });

            it('totalMissed_chat', () => {
                previousVersion.completionStatus = 'Assigned_Missed';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Assigned_Missed';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_chat',
                    expectedValue: 1,
                });
            });

            it('totalMissed_inboundPhone', () => {
                previousVersion.completionStatus = 'Agent_Missed';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Agent_Missed';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_inboundPhone',
                    expectedValue: 1,
                });
            });

            it('holdTime_all (Inbound Phone)', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_all',
                    expectedValue: holdTime,
                });
            });

            it('holdTime_all (Outbound Phone)', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_all',
                    expectedValue: holdTime,
                });
            });

            it('holdTime_inboundPhone', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_inboundPhone',
                    expectedValue: holdTime,
                });
            });

            it('holdTime_outboundPhone', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_outboundPhone',
                    expectedValue: holdTime,
                });
            });

            it('talkTime_all (Inbound Phone)', () => {
                const talkTime = 123;
                currentVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_all',
                    expectedValue: talkTime,
                });
            });

            it('talkTime_all (Outbound Phone)', () => {
                const talkTime = 123;
                previousVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_all',
                    expectedValue: talkTime,
                });
            });

            it('talkTime_inboundPhone', () => {
                const talkTime = 123;
                previousVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_inboundPhone',
                    expectedValue: talkTime,
                });
            });

            it('talkTime_outboundPhone', () => {
                const talkTime = 123;
                previousVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_outboundPhone',
                    expectedValue: talkTime,
                });
            });

            it('consultTime_all (Inbound Phone)', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_all',
                    expectedValue: consultationTime,
                });
            });

            it('consultTime_all (Outbound Phone)', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_all',
                    expectedValue: consultationTime,
                });
            });

            it('consultTime_inboundPhone', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_inboundPhone',
                    expectedValue: consultationTime,
                });
            });

            it('consultTime_outboundPhone', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_outboundPhone',
                    expectedValue: consultationTime,
                });
            });

            it('wrapTime_all (Inbound Phone)', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_all',
                    expectedValue: wrapTime,
                });
            });

            it('wrapTime_all (Outbound Phone)', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_all',
                    expectedValue: wrapTime,
                });
            });

            it('wrapTime_inboundPhone', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_inboundPhone',
                    expectedValue: wrapTime,
                });
            });

            it('wrapTime_outboundPhone', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_outboundPhone',
                    expectedValue: wrapTime,
                });
            });

            it.skip('totalOffered_all (Chat)', () => {
                previousVersion.type = 'Inbound_Phone';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_all',
                    expectedValue: 999, // TODO: Calculation TBC
                });
            });

            it.skip('totalOffered_all (Inbound Phone)', () => {
                previousVersion.type = 'Chat';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_all',
                    expectedValue: 999, // TODO: Calculation TBC
                });
            });

            it.skip('totalOffered_chat', () => {
                previousVersion.type = 'Inbound_Phone';
                currentVersion.type = 'Chat';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_chat',
                    expectedValue: 999, // TODO: Calculation TBC
                });
            });

            it.skip('totalOffered_inboundPhone', () => {
                previousVersion.type = 'Chat';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertWithinBucket({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_inboundPhone',
                    expectedValue: 999, // TODO: Calculation TBC
                });
            });
        });
    });

    describe('across time buckets', () => {
        describe('successfully increments', () => {
            const incrementAndAssertAcrossBuckets = ({
                currentVersion,
                previousVersion,
                fieldName,
                previousVersionExpectedValue,
                currentVersionExpectedValue,
            }: {
                currentVersion: InteractionEventType;
                previousVersion: InteractionEventType;
                fieldName: string;
                previousVersionExpectedValue: number | undefined;
                currentVersionExpectedValue: number | undefined;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.endTime = incrementTime().toISOString();

                // Move to a new bucket
                currentVersion.startTime = moveToANewBucket().toISOString();

                const results = calculateInteractionUpdatedDiff({
                    previousVersion,
                    currentVersion,
                    params: filter,
                });

                // Expect to have 2 buckets
                expect(results.length).toBe(2);

                // Expect to see 2 results
                const upsertForPreviousBucket = results[0];
                const upsertForNewBucket = results[1];
                expect(upsertForPreviousBucket).toBeTruthy();
                expect(upsertForNewBucket).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForPreviousBucket as any)?.query?.$inc[fieldName]).toEqual(previousVersionExpectedValue);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForNewBucket as any)?.query?.$inc[fieldName]).toEqual(currentVersionExpectedValue);
            };

            describe('interaction metrics', () => {
                it('numberOfInteractions_all (Chat)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_all (Ticket)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_all (Inbound Phone)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_all (Outbound Phone)', () => {
                    previousVersion.type = 'Invalid type';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_chat', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_ticket', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_inboundPhone', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('numberOfInteractions_outboundPhone', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalInteractionHandleTime_all (Chat)', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Ticket)', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Inbound Phone)', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_all (Outbound Phone)', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_chat', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_ticket', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_inboundPhone', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalInteractionHandleTime_outboundPhone', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: workEffort,
                    });
                });

                it('totalReassigns_all (Chat)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalReassigns_all (Ticket)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalReassigns_all (Inbound Phone)', () => {
                    currentVersion.isReassign = false;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalReassigns_all (Outbound Phone)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalReassigns_chat', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_chat',
                        previousVersionExpectedValue: undefined, // 0,
                        currentVersionExpectedValue: undefined, // 1,
                    });
                });

                it('totalReassigns_ticket', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalReassigns_ticket (from status change)', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Status_Change';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Status_Change';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalReassigns_inboundPhone', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_inboundPhone',
                        previousVersionExpectedValue: undefined, // 0,
                        currentVersionExpectedValue: undefined, // 1,
                    });
                });

                it('totalReassigns_outboundPhone', () => {
                    previousVersion.isReassign = false;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_outboundPhone',
                        previousVersionExpectedValue: undefined, // 0,
                        currentVersionExpectedValue: undefined, // 1,
                    });
                });

                it('totalPrivateComments_all (Ticket)', () => {
                    previousVersion.isPrivateComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalPrivateComments_ticket', () => {
                    previousVersion.isPrivateComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalPublicComments_all (Ticket)', () => {
                    previousVersion.isPublicComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalPublicComments_ticket', () => {
                    previousVersion.isPublicComment = false;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_all (Chat)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_all (Ticket)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_all (Outbound Phone)', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it.skip('totalDeclined_chat', () => {
                    previousVersion.completionStatus = 'Customer_Hang_Up';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_ticket', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Worklog';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_inboundPhone', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalDeclined_outboundPhone', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalAssigned_all (Chat)', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalAssigned_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalAssigned_chat', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it.skip('totalAssigned_inboundPhone', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalSelfAssignedInteractions_all (Chat)', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalSelfAssignedInteractions_chat', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalMissed_all (Chat)', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalMissed_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalMissed_chat', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Missed';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalMissed_inboundPhone', () => {
                    previousVersion.completionStatus = 'Completed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('holdTime_all (Inbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: holdTime,
                    });
                });

                it('holdTime_all (Outbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: holdTime,
                    });
                });

                it('holdTime_inboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: holdTime,
                    });
                });

                it('holdTime_outboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: holdTime,
                    });
                });

                it('talkTime_all (Inbound Phone)', () => {
                    const talkTime = 123;
                    currentVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: talkTime,
                    });
                });

                it('talkTime_all (Outbound Phone)', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: talkTime,
                    });
                });

                it('talkTime_inboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: talkTime,
                    });
                });

                it('talkTime_outboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: talkTime,
                    });
                });

                it('consultTime_all (Inbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: consultationTime,
                    });
                });

                it('consultTime_all (Outbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: consultationTime,
                    });
                });

                it('consultTime_inboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: consultationTime,
                    });
                });

                it('consultTime_outboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: consultationTime,
                    });
                });

                it('wrapTime_all (Inbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: wrapTime,
                    });
                });

                it('wrapTime_all (Outbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: wrapTime,
                    });
                });

                it('wrapTime_inboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: wrapTime,
                    });
                });

                it('wrapTime_outboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = 0;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Outbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: wrapTime,
                    });
                });

                it.skip('totalOffered_all (Chat)', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 999, // TODO: Calculation TBC
                    });
                });

                it.skip('totalOffered_all (Inbound Phone)', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 999, // TODO: Calculation TBC
                    });
                });

                it.skip('totalOffered_chat', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 999, // TODO: Calculation TBC
                    });
                });

                it.skip('totalOffered_inboundPhone', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 999, // TODO: Calculation TBC
                    });
                });
            });

            describe('does not increase interaction count with no handle time', () => {
                it('Chat - 0 seconds', () => {
                    previousVersion.type = 'Chat';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Chat';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('Ticket - 0 seconds', () => {
                    previousVersion.type = 'Worklog';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('Inbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Inbound_Phone';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('Outbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Outbound_Phone';
                    previousVersion.workEffortSecs = -10;
                    currentVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });
            });

            describe('missing handle time is treated as 0', () => {
                it('for chat', () => {
                    previousVersion.type = 'Chat';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Chat';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_chat',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('for ticket', () => {
                    previousVersion.type = 'Worklog';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Worklog';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_ticket',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('for inbound phone', () => {
                    previousVersion.type = 'Inbound_Phone';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Inbound_Phone';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_inboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('for outbound phone', () => {
                    previousVersion.type = 'Outbound_Phone';
                    delete previousVersion.workEffortSecs;
                    currentVersion.type = 'Outbound_Phone';
                    delete currentVersion.workEffortSecs;

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_outboundPhone',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalResolutions (solved)', () => {
                    previousVersion.statusChangeTo = 'new';
                    currentVersion.statusChangeTo = 'solved';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        previousVersionExpectedValue: 0,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalResolutions (no change)', () => {
                    previousVersion.statusChangeTo = 'solved';
                    currentVersion.statusChangeTo = 'solved';

                    incrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 1,
                    });
                });
            });
        });

        describe('successfully decrements', () => {
            const decrementAndAssertAcrossBuckets = ({
                currentVersion,
                previousVersion,
                fieldName,
                previousVersionExpectedValue,
                currentVersionExpectedValue,
            }: {
                currentVersion: InteractionEventType;
                previousVersion: InteractionEventType;
                fieldName: string;
                previousVersionExpectedValue: number | undefined;
                currentVersionExpectedValue: number | undefined;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.endTime = incrementTime().toISOString();

                // Move to a new bucket
                currentVersion.startTime = moveToANewBucket().toISOString();

                const results = calculateInteractionUpdatedDiff({
                    previousVersion,
                    currentVersion,
                    params: filter,
                });

                // Expect to have 2 buckets
                expect(results.length).toBe(2);

                // Expect to see 2 results
                const upsertForPreviousBucket = results[0];
                const upsertForNewBucket = results[1];
                expect(upsertForPreviousBucket).toBeTruthy();
                expect(upsertForNewBucket).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForPreviousBucket as any)?.query?.$inc[fieldName]).toEqual(previousVersionExpectedValue);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForNewBucket as any)?.query?.$inc[fieldName]).toEqual(currentVersionExpectedValue);
            };

            describe('interaction metrics', () => {
                it('numberOfInteractions_all (Chat)', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_all (Ticket)', () => {
                    previousVersion.type = 'Worklog';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_all (Inbound Phone)', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_all (Outbound Phone)', () => {
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_chat', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_ticket', () => {
                    previousVersion.type = 'Worklog';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_inboundPhone', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('numberOfInteractions_outboundPhone', () => {
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_all (Chat)', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_all (Ticket)', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_all (Inbound Phone)', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_all (Outbound Phone)', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_all',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_chat', () => {
                    const workEffort = 55;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Chat';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_chat',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_ticket', () => {
                    const workEffort = 66;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_ticket',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_inboundPhone', () => {
                    const workEffort = 77;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_inboundPhone',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalInteractionHandleTime_outboundPhone', () => {
                    const workEffort = 88;
                    previousVersion.workEffortSecs = workEffort;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = workEffort;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalInteractionHandleTime_outboundPhone',
                        previousVersionExpectedValue: -workEffort,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_all (Chat)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_all (Ticket)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_all (Inbound Phone)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_all (Outbound Phone)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_chat', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Chat';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_chat',
                        previousVersionExpectedValue: undefined, // -1,
                        currentVersionExpectedValue: undefined, // 0,
                    });
                });

                it('totalReassigns_ticket', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_ticket (from status change)', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Status_Change';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalReassigns_inboundPhone', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_inboundPhone',
                        previousVersionExpectedValue: undefined, // -1,
                        currentVersionExpectedValue: undefined, // 0,
                    });
                });

                it('totalReassigns_outboundPhone', () => {
                    previousVersion.isReassign = true;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.isReassign = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalReassigns_outboundPhone',
                        previousVersionExpectedValue: undefined, // -1,
                        currentVersionExpectedValue: undefined, // 0,
                    });
                });

                it('totalPrivateComments_all (Ticket)', () => {
                    previousVersion.isPrivateComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalPrivateComments_ticket', () => {
                    previousVersion.isPrivateComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPrivateComment = true;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPrivateComments_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalPublicComments_all (Ticket)', () => {
                    previousVersion.isPublicComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalPublicComments_ticket', () => {
                    previousVersion.isPublicComment = true;
                    previousVersion.type = 'Worklog';
                    currentVersion.isPublicComment = true;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalPublicComments_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_all (Chat)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_all (Ticket)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_all (Outbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it.skip('totalDeclined_chat', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_chat',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_ticket', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Worklog';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_inboundPhone', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_inboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalDeclined_outboundPhone', () => {
                    previousVersion.completionStatus = 'Agent_Declined';
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.completionStatus = 'Agent_Declined';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalDeclined_outboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalAssigned_all (Chat)', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalAssigned_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalAssigned_chat', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_chat',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it.skip('totalAssigned_inboundPhone', () => {
                    previousVersion.completionStatus = 'Assigned_Accepted';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Assigned_Accepted';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalAssigned_inboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalSelfAssignedInteractions_all (Chat)', () => {
                    previousVersion.completionStatus = 'Self_Assigned';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Self_Assigned';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalSelfAssignedInteractions_chat', () => {
                    previousVersion.completionStatus = 'Self_Assigned';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'self_assigned';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalSelfAssignedInteractions_chat',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalMissed_all (Chat)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalMissed_all (Inbound Phone)', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_all',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalMissed_chat', () => {
                    previousVersion.completionStatus = 'Assigned_Missed';
                    previousVersion.type = 'Chat';
                    currentVersion.completionStatus = 'Assigned_Missed';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_chat',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalMissed_inboundPhone', () => {
                    previousVersion.completionStatus = 'Agent_Missed';
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.completionStatus = 'Agent_Missed';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalMissed_inboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('holdTime_all (Inbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        previousVersionExpectedValue: -holdTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('holdTime_all (Outbound Phone)', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_all',
                        previousVersionExpectedValue: -holdTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('holdTime_inboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_inboundPhone',
                        previousVersionExpectedValue: -holdTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('holdTime_outboundPhone', () => {
                    const holdTime = 123;
                    previousVersion.holdTimeSecs = holdTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.holdTimeSecs = holdTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'holdTime_outboundPhone',
                        previousVersionExpectedValue: -holdTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('talkTime_all (Inbound Phone)', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        previousVersionExpectedValue: -talkTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('talkTime_all (Outbound Phone)', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_all',
                        previousVersionExpectedValue: -talkTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('talkTime_inboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_inboundPhone',
                        previousVersionExpectedValue: -talkTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('talkTime_outboundPhone', () => {
                    const talkTime = 123;
                    previousVersion.talkTimeSecs = talkTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.talkTimeSecs = talkTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'talkTime_outboundPhone',
                        previousVersionExpectedValue: -talkTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('consultTime_all (Inbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        previousVersionExpectedValue: -consultationTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('consultTime_all (Outbound Phone)', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Invalid type';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_all',
                        previousVersionExpectedValue: -consultationTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('consultTime_inboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_inboundPhone',
                        previousVersionExpectedValue: -consultationTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('consultTime_outboundPhone', () => {
                    const consultationTime = 123;
                    previousVersion.consultationTimeSecs = consultationTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.consultationTimeSecs = consultationTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'consultTime_outboundPhone',
                        previousVersionExpectedValue: -consultationTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('wrapTime_all (Inbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        previousVersionExpectedValue: -wrapTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('wrapTime_all (Outbound Phone)', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_all',
                        previousVersionExpectedValue: -wrapTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('wrapTime_inboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_inboundPhone',
                        previousVersionExpectedValue: -wrapTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('wrapTime_outboundPhone', () => {
                    const wrapTime = 123;
                    previousVersion.wrapTimeSecs = wrapTime;
                    previousVersion.type = 'Outbound_Phone';
                    currentVersion.wrapTimeSecs = wrapTime;
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'wrapTime_outboundPhone',
                        previousVersionExpectedValue: -wrapTime,
                        currentVersionExpectedValue: 0,
                    });
                });

                it.skip('totalOffered_all (Chat)', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_all',
                        previousVersionExpectedValue: -999, // TODO: Calculation TBC
                        currentVersionExpectedValue: 0,
                    });
                });

                it.skip('totalOffered_all (Inbound Phone)', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_all',
                        previousVersionExpectedValue: -999, // TODO: Calculation TBC
                        currentVersionExpectedValue: 0,
                    });
                });

                it.skip('totalOffered_chat', () => {
                    previousVersion.type = 'Chat';
                    currentVersion.type = 'Inbound_Phone';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_chat',
                        previousVersionExpectedValue: -999, // TODO: Calculation TBC
                        currentVersionExpectedValue: 0,
                    });
                });

                it.skip('totalOffered_inboundPhone', () => {
                    previousVersion.type = 'Inbound_Phone';
                    currentVersion.type = 'Chat';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalOffered_inboundPhone',
                        previousVersionExpectedValue: -999, // TODO: Calculation TBC
                        currentVersionExpectedValue: 0,
                    });
                });
            });

            describe('decrements interaction count with no handle time', () => {
                it('Chat - 0 seconds', () => {
                    previousVersion.type = 'Chat';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Chat';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_chat',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('Ticket - 0 seconds', () => {
                    previousVersion.type = 'Worklog';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Worklog';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_ticket',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('Inbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Inbound_Phone';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Inbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_inboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('Outbound Phone - 0 seconds', () => {
                    previousVersion.type = 'Outbound_Phone';
                    previousVersion.workEffortSecs = 10;
                    currentVersion.type = 'Outbound_Phone';
                    currentVersion.workEffortSecs = 0;

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'numberOfInteractions_outboundPhone',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 0,
                    });
                });

                it('totalResolutions (solved)', () => {
                    previousVersion.statusChangeTo = 'solved';
                    currentVersion.statusChangeTo = 'open';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 1,
                    });
                });

                it('totalResolutions (no change)', () => {
                    previousVersion.statusChangeTo = 'solved';
                    currentVersion.statusChangeTo = 'solved';

                    decrementAndAssertAcrossBuckets({
                        currentVersion,
                        previousVersion,
                        fieldName: 'totalResolutions',
                        previousVersionExpectedValue: -1,
                        currentVersionExpectedValue: 1,
                    });
                });
            });
        });

        describe('successfully modifies an interaction', () => {
            const incrementAndAssertAcrossBuckets = ({
                currentVersion,
                previousVersion,
                fieldName,
                previousVersionExpectedValue,
                currentVersionExpectedValue,
            }: {
                currentVersion: InteractionEventType;
                previousVersion: InteractionEventType;
                fieldName: string;
                previousVersionExpectedValue: number | undefined;
                currentVersionExpectedValue: number | undefined;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.endTime = incrementTime().toISOString();

                // Move to a new bucket
                currentVersion.startTime = moveToANewBucket().toISOString();

                const results = calculateInteractionUpdatedDiff({
                    previousVersion,
                    currentVersion,
                    params: filter,
                });

                // Expect to have 2 buckets
                expect(results.length).toBe(2);

                // Expect to see 2 results
                const upsertForPreviousBucket = results[0];
                const upsertForNewBucket = results[1];
                expect(upsertForPreviousBucket).toBeTruthy();
                expect(upsertForNewBucket).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForPreviousBucket as any)?.query?.$inc[fieldName]).toEqual(previousVersionExpectedValue);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForNewBucket as any)?.query?.$inc[fieldName]).toEqual(currentVersionExpectedValue);
            };

            it('totalInteractionHandleTime_all (Chat)', () => {
                const workEffort = 55;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_all (Ticket)', () => {
                const workEffort = 66;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_all (Inbound Phone)', () => {
                const workEffort = 77;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_all (Outbound Phone)', () => {
                const workEffort = 88;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Invalid type';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_chat', () => {
                const workEffort = 55;
                currentVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_chat',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_ticket', () => {
                const workEffort = 66;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Chat';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_ticket',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_inboundPhone', () => {
                const workEffort = 77;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Chat';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalInteractionHandleTime_outboundPhone', () => {
                const workEffort = 88;
                previousVersion.workEffortSecs = workEffort;
                previousVersion.type = 'Chat';
                currentVersion.workEffortSecs = workEffort;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalInteractionHandleTime_outboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: workEffort,
                });
            });

            it('totalReassigns_all (Chat)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalReassigns_all (Ticket)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalReassigns_all (Inbound Phone)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalReassigns_all (Outbound Phone)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Invalid type';
                currentVersion.isReassign = true;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalReassigns_chat', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Inbound_Phone';
                currentVersion.isReassign = true;
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_chat',
                    previousVersionExpectedValue: undefined, // 0,
                    currentVersionExpectedValue: undefined, // 1,
                });
            });

            it('totalReassigns_ticket', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Chat';
                currentVersion.isReassign = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_ticket',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalReassigns_ticket (from status change)', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Chat';
                currentVersion.isReassign = true;
                currentVersion.type = 'Status_Change';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_ticket',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalReassigns_inboundPhone', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Chat';
                currentVersion.isReassign = true;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_inboundPhone',
                    previousVersionExpectedValue: undefined, // 0,
                    currentVersionExpectedValue: undefined, // 1,
                });
            });

            it('totalReassigns_outboundPhone', () => {
                previousVersion.isReassign = true;
                previousVersion.type = 'Chat';
                currentVersion.isReassign = true;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalReassigns_outboundPhone',
                    previousVersionExpectedValue: undefined, // 0,
                    currentVersionExpectedValue: undefined, // 1,
                });
            });

            it('totalPrivateComments_all (Ticket)', () => {
                previousVersion.isPrivateComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPrivateComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPrivateComments_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalPrivateComments_ticket', () => {
                previousVersion.isPrivateComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPrivateComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPrivateComments_ticket',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalPublicComments_all (Ticket)', () => {
                previousVersion.isPublicComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPublicComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPublicComments_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalPublicComments_ticket', () => {
                previousVersion.isPublicComment = true;
                previousVersion.type = 'Chat';
                currentVersion.isPublicComment = true;
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalPublicComments_ticket',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_all (Chat)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_all (Ticket)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_all (Inbound Phone)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_all (Outbound Phone)', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it.skip('totalDeclined_chat', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_chat',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_ticket', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Worklog';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_ticket',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_inboundPhone', () => {
                previousVersion.completionStatus = 'Agent_Declined';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalDeclined_outboundPhone', () => {
                previousVersion.completionStatus = 'agent_declined';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Agent_Declined';
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalDeclined_outboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalAssigned_all (Chat)', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalAssigned_all (Inbound Phone)', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalAssigned_chat', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_chat',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it.skip('totalAssigned_inboundPhone', () => {
                previousVersion.completionStatus = 'Assigned_Accepted';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Assigned_Accepted';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalAssigned_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalSelfAssignedInteractions_all (Chat)', () => {
                previousVersion.completionStatus = 'Self_Assigned';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Self_Assigned';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalSelfAssignedInteractions_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalSelfAssignedInteractions_chat', () => {
                previousVersion.completionStatus = 'Self_Assigned';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Self_Assigned';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalSelfAssignedInteractions_chat',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalMissed_all (Chat)', () => {
                previousVersion.completionStatus = 'Agent_Missed';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Missed';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalMissed_all (Inbound Phone)', () => {
                previousVersion.completionStatus = 'Agent_Missed';
                previousVersion.type = 'Invalid type';
                currentVersion.completionStatus = 'Agent_Missed';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalMissed_chat', () => {
                previousVersion.completionStatus = 'Assigned_Missed';
                previousVersion.type = 'Inbound_Phone';
                currentVersion.completionStatus = 'Assigned_Missed';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_chat',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('totalMissed_inboundPhone', () => {
                previousVersion.completionStatus = 'Agent_Missed';
                previousVersion.type = 'Chat';
                currentVersion.completionStatus = 'Agent_Missed';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalMissed_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 1,
                });
            });

            it('holdTime_all (Inbound Phone)', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: holdTime,
                });
            });

            it('holdTime_all (Outbound Phone)', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: holdTime,
                });
            });

            it('holdTime_inboundPhone', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: holdTime,
                });
            });

            it('holdTime_outboundPhone', () => {
                const holdTime = 123;
                previousVersion.holdTimeSecs = holdTime;
                previousVersion.type = 'Chat';
                currentVersion.holdTimeSecs = holdTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'holdTime_outboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: holdTime,
                });
            });

            it('talkTime_all (Inbound Phone)', () => {
                const talkTime = 123;
                currentVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: talkTime,
                });
            });

            it('talkTime_all (Outbound Phone)', () => {
                const talkTime = 123;
                previousVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: talkTime,
                });
            });

            it('talkTime_inboundPhone', () => {
                const talkTime = 123;
                previousVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: talkTime,
                });
            });

            it('talkTime_outboundPhone', () => {
                const talkTime = 123;
                previousVersion.talkTimeSecs = talkTime;
                previousVersion.type = 'Chat';
                currentVersion.talkTimeSecs = talkTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'talkTime_outboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: talkTime,
                });
            });

            it('consultTime_all (Inbound Phone)', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: consultationTime,
                });
            });

            it('consultTime_all (Outbound Phone)', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: consultationTime,
                });
            });

            it('consultTime_inboundPhone', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: consultationTime,
                });
            });

            it('consultTime_outboundPhone', () => {
                const consultationTime = 123;
                previousVersion.consultationTimeSecs = consultationTime;
                previousVersion.type = 'Chat';
                currentVersion.consultationTimeSecs = consultationTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'consultTime_outboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: consultationTime,
                });
            });

            it('wrapTime_all (Inbound Phone)', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: wrapTime,
                });
            });

            it('wrapTime_all (Outbound Phone)', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: wrapTime,
                });
            });

            it('wrapTime_inboundPhone', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: wrapTime,
                });
            });

            it('wrapTime_outboundPhone', () => {
                const wrapTime = 123;
                previousVersion.wrapTimeSecs = wrapTime;
                previousVersion.type = 'Chat';
                currentVersion.wrapTimeSecs = wrapTime;
                currentVersion.type = 'Outbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'wrapTime_outboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: wrapTime,
                });
            });

            it.skip('totalOffered_all (Chat)', () => {
                previousVersion.type = 'Inbound_Phone';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 999, // TODO: Calculation TBC
                });
            });

            it.skip('totalOffered_all (Inbound Phone)', () => {
                previousVersion.type = 'Chat';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_all',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 999, // TODO: Calculation TBC
                });
            });

            it.skip('totalOffered_chat', () => {
                previousVersion.type = 'Inbound_Phone';
                currentVersion.type = 'Chat';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_chat',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 999, // TODO: Calculation TBC
                });
            });

            it.skip('totalOffered_inboundPhone', () => {
                previousVersion.type = 'Chat';
                currentVersion.type = 'Inbound_Phone';

                incrementAndAssertAcrossBuckets({
                    currentVersion,
                    previousVersion,
                    fieldName: 'totalOffered_inboundPhone',
                    previousVersionExpectedValue: 0,
                    currentVersionExpectedValue: 999, // TODO: Calculation TBC
                });
            });
        });
    });
});
