import type { KinesisStreamEvent } from 'aws-lambda';
import { interactionCreatedHandler } from '../../../../lib/metrics/interactions/eventHandlers/interactionCreatedHandler';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import * as mongodb from '../../../../lib/utils/mongodb';
import { config } from '../../../../lib/config';
import { EventDebugLogModel } from '../../../../lib/models/EventDebugLogModel';
import { WorkstreamDefinitionsModel } from '../../../../lib/workstreamDefinition/WorkstreamDefinitionsModel';
import { InteractionCreatedEvent } from '../../../../lib/metrics';

jest.createMockFromModule('@agyletime/performance-utils');
jest.mock('../../../../lib/utils/mongodb');
jest.mock('../../../../lib/models/EventDebugLogModel');
jest.mock('../../../../lib/workstreamDefinition/WorkstreamDefinitionsModel');

const event = {
    Records: [
        {
            awsRegion: 'string',
            eventID: 'string',
            eventName: 'string',
            eventSource: 'string',
            eventSourceARN: 'string',
            eventVersion: 'string',
            invokeIdentityArn: 'string',
            kinesis: {
                approximateArrivalTimestamp: 123,
                data: 'string',
                kinesisSchemaVersion: 'string',
                partitionKey: 'string',
                sequenceNumber: 'string',
            },
        },
    ],
} as KinesisStreamEvent;

function encodeKinesisBody(data: unknown) {
    return Buffer.from(JSON.stringify(data)).toString('base64');
}

let mockBulkWrite: jest.SpyInstance;
let mockWorkstreamDefinitionsCachBulkWrite: jest.SpyInstance;
let mockEventDebugLogWrite: unknown;

describe('InteractionCreated', () => {
    beforeEach(() => {
        mockBulkWrite = jest.spyOn(MongoInteractionMetricModel, 'bulkWrite').mockImplementation();
        mockEventDebugLogWrite = jest.spyOn(EventDebugLogModel, 'bulkWrite').mockImplementation();
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
        mockWorkstreamDefinitionsCachBulkWrite = jest
            .spyOn(WorkstreamDefinitionsModel, 'bulkWrite')
            .mockImplementation();

        config.featureFlags.orgAllowListTPM = '*';
    });

    afterEach(() => {
        jest.resetAllMocks();
    });

    describe('with a valid event', () => {
        it('does not throw errors if org id and user id provided', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        workstreamDefinitionName: 'Test Workstream Definition',
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionCreatedEvent);

            await interactionCreatedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
        });

        it('will write the workstream definition name if present in event', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            const workstreamDefinitionName = 'Test Workstream Definition';
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        workstreamDefinition: {
                            name: workstreamDefinitionName,
                        },
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionCreatedEvent);

            await interactionCreatedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Workstream definition should be cached
            expect(mockWorkstreamDefinitionsCachBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            expect(mockBulkWrite.mock.calls[0][0][0].updateOne.update.$set.workstreamDefinitionName).toBe(
                workstreamDefinitionName,
            );

            // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
            const mockCall = mockWorkstreamDefinitionsCachBulkWrite.mock.calls[0][0].find(
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                (call: any) => call.updateOne.update[0].$set.name === workstreamDefinitionName,
            );
            expect(mockCall).toBeTruthy();
        });

        it('does not write to mongo if no org id provided', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const eventWithoutUser: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            eventWithoutUser.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        // organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionCreatedEvent);

            try {
                await interactionCreatedHandler(eventWithoutUser);
            } catch (e: unknown) {
                /* intentionally empty */
            }

            // Interaction should not be written to mongo
            expect(mockBulkWrite).not.toHaveBeenCalled();
            // Nothing should be logged to event debug log
            expect(mockBulkWrite).not.toHaveBeenCalled();
        });

        it('writes to the debug event log', async () => {
            config.log.log_level = 'debug';

            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const eventWithoutUser: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            eventWithoutUser.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionCreatedEvent);

            await interactionCreatedHandler(eventWithoutUser);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Event should be logged - this shouldn't happen anymore as it's in level above
            expect(mockEventDebugLogWrite).not.toHaveBeenCalledTimes(1);

            config.log.log_level = 'info';
        });

        it('throws an error if the db connection fails', async () => {
            jest.spyOn(mongodb, 'init').mockImplementation(() => {
                throw new Error('Failed to connect to MongoDB');
            });

            let error = null;

            try {
                await interactionCreatedHandler(event);
            } catch (e: unknown) {
                error = e;
            }
            expect(error).toBeTruthy();
        });
    });
});
