import type { KinesisStreamEvent } from 'aws-lambda';
import { interactionDeletedHandler } from '../../../../lib/metrics/interactions/eventHandlers/interactionDeletedHandler';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import * as mongodb from '../../../../lib/utils/mongodb';
import { log } from '../../../../lib/utils';
import { config } from '../../../../lib/config';
import { EventDebugLogModel } from '../../../../lib/models';
import { InteractionDeletedEvent } from '../../../../lib/metrics';

jest.createMockFromModule('@agyletime/performance-utils');
jest.mock('../../../../lib/utils/mongodb');
jest.mock('../../../../lib/utils/log');
jest.mock('../../../../lib/models/EventDebugLogModel');

const event = {
    Records: [
        {
            awsRegion: 'string',
            eventID: 'string',
            eventName: 'string',
            eventSource: 'string',
            eventSourceARN: 'string',
            eventVersion: 'string',
            invokeIdentityArn: 'string',
            kinesis: {
                approximateArrivalTimestamp: 123,
                data: 'string',
                kinesisSchemaVersion: 'string',
                partitionKey: 'string',
                sequenceNumber: 'string',
            },
        },
    ],
} as KinesisStreamEvent;

function encodeKinesisBody(data: unknown) {
    return Buffer.from(JSON.stringify(data)).toString('base64');
}

let mockBulkWrite: unknown;
let mockEventDebugLogWrite: unknown;

describe('InteractionCreated', () => {
    beforeEach(() => {
        mockBulkWrite = jest.spyOn(MongoInteractionMetricModel, 'bulkWrite').mockImplementation();
        mockEventDebugLogWrite = jest.spyOn(EventDebugLogModel, 'bulkWrite').mockImplementation();
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
        jest.spyOn(log, 'error').mockImplementation();

        config.featureFlags.orgAllowListTPM = '*';
    });

    afterEach(() => {
        jest.resetAllMocks();
    });

    describe('with a valid event', () => {
        it('does not throw errors if org id and user id provided', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionDeletedEvent);

            await interactionDeletedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
            // No errors should be logged
            expect(log.error).not.toHaveBeenCalled();
        });

        it('does not write to mongo if no org id provided', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const eventWithoutUser: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            eventWithoutUser.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        // organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionDeletedEvent);

            try {
                await interactionDeletedHandler(eventWithoutUser);
            } catch {
                /* intentionally empty */
            }

            // Interaction should not be written to mongo
            expect(mockBulkWrite).not.toHaveBeenCalled();
            // Nothing should be logged to event debug log
            expect(mockBulkWrite).not.toHaveBeenCalled();
        });

        it('writes to the debug event log', async () => {
            config.log.log_level = 'debug';

            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const eventWithoutUser: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            eventWithoutUser.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interaction: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    operation: 'create',
                },
            } as InteractionDeletedEvent);

            await interactionDeletedHandler(eventWithoutUser);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Event should be logged - this shouldn't happen anymore as it's in level above
            expect(mockEventDebugLogWrite).not.toHaveBeenCalledTimes(1);
            // No errors should be logged
            expect(log.error).not.toHaveBeenCalled();

            config.log.log_level = 'info';
        });

        it('throws an error if the db connection fails', async () => {
            jest.spyOn(mongodb, 'init').mockImplementation(() => {
                throw new Error('Failed to connect to MongoDB');
            });

            let error = null;

            try {
                await interactionDeletedHandler(event);
            } catch (e: unknown) {
                error = e;
            }
            expect(error).toBeTruthy();
        });
    });
});
