import type { KinesisStreamEvent } from 'aws-lambda';
import { interactionUpdatedHandler } from '../../../../lib/metrics/interactions/eventHandlers/interactionUpdatedHandler';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import * as mongodb from '../../../../lib/utils/mongodb';
import { log } from '../../../../lib/utils';
import { config } from '../../../../lib/config';
import { EventDebugLogModel } from '../../../../lib/models';
import { WorkstreamDefinitionsModel } from '../../../../lib/workstreamDefinition';
import { InteractionUpdatedEvent, MongoUpdateOneQuery } from '../../../../lib/metrics';

jest.createMockFromModule('@agyletime/performance-utils');
jest.mock('../../../../lib/utils/mongodb');
jest.mock('../../../../lib/utils/log');
jest.mock('../../../../lib/models/EventDebugLogModel');
jest.mock('../../../../lib/workstreamDefinition/WorkstreamDefinitionsModel');

const event = {
    Records: [
        {
            awsRegion: 'string',
            eventID: 'string',
            eventName: 'string',
            eventSource: 'string',
            eventSourceARN: 'string',
            eventVersion: 'string',
            invokeIdentityArn: 'string',
            kinesis: {
                approximateArrivalTimestamp: 123,
                data: 'string',
                kinesisSchemaVersion: 'string',
                partitionKey: 'string',
                sequenceNumber: 'string',
            },
        },
    ],
} as KinesisStreamEvent;

function encodeKinesisBody(data: unknown) {
    return Buffer.from(JSON.stringify(data)).toString('base64');
}

let mockBulkWrite: unknown;
let mockEventDebugLogWrite: unknown;
let mockWorkstreamDefinitionsCachBulkWrite: jest.SpyInstance;
let dataToBeUpserted: unknown[];

describe('InteractionUpdated', () => {
    beforeEach(() => {
        // @ts-expect-error type mismatch
        mockBulkWrite = jest.spyOn(MongoInteractionMetricModel, 'bulkWrite').mockImplementation((upserts) => {
            dataToBeUpserted = upserts;
        });
        mockEventDebugLogWrite = jest.spyOn(EventDebugLogModel, 'bulkWrite').mockImplementation();
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
        jest.spyOn(log, 'error').mockImplementation();
        mockWorkstreamDefinitionsCachBulkWrite = jest
            .spyOn(WorkstreamDefinitionsModel, 'bulkWrite')
            .mockImplementation();

        config.featureFlags.orgAllowListTPM = '*';
    });

    afterEach(() => {
        jest.resetAllMocks();
    });

    describe('with a valid event', () => {
        it('does not throw errors if all required fields provided', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            const workstreamDefinitionName = 'Test Workstream Definition';
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interactionBefore: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        workstreamDefinition: {
                            name: workstreamDefinitionName,
                        },
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    interactionAfter: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 99,
                        workstreamDefinitionId: 1,
                        workstreamDefinition: {
                            name: workstreamDefinitionName,
                        },
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    updates: [
                        {
                            op: 'update',
                            path: 'path',
                            value: { type: 'type' },
                        },
                    ],
                    operation: 'create',
                },
            } as InteractionUpdatedEvent);

            await interactionUpdatedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Workstream definition should be cached
            expect(mockWorkstreamDefinitionsCachBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            expect(mockWorkstreamDefinitionsCachBulkWrite.mock.calls[0][0][0].updateOne.update[0].$set.name).toBe(
                workstreamDefinitionName,
            );
            // No errors should be logged
            expect(log.error).not.toHaveBeenCalled();
        });

        it('does not write to mongo if no org id provided', async () => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            // @ts-expect error Missing Organisation ID
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interactionBefore: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    interactionAfter: {
                        // organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 99,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    updates: [
                        {
                            op: 'update',
                            path: 'path',
                            value: { type: 'type' },
                        },
                    ],
                    operation: 'create',
                },
            } as InteractionUpdatedEvent);

            try {
                await interactionUpdatedHandler(validEvent);
            } catch {
                /* intentionally empty */
            }

            // Interaction should not be written to mongo
            expect(mockBulkWrite).not.toHaveBeenCalled();
            // Nothing should be logged to event debug log
            expect(mockBulkWrite).not.toHaveBeenCalled();
        });

        it('writes to the debug event log', async () => {
            config.log.log_level = 'debug';

            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interactionBefore: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    interactionAfter: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 99,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    updates: [
                        {
                            op: 'update',
                            path: 'path',
                            value: { type: 'type' },
                        },
                    ],
                    operation: 'create',
                },
            } as InteractionUpdatedEvent);

            await interactionUpdatedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
            // No errors should be logged
            expect(log.error).not.toHaveBeenCalled();

            config.log.log_level = 'info';
        });

        it('throws an error if the db connection fails', async () => {
            jest.spyOn(mongodb, 'init').mockImplementation(() => {
                throw new Error('Failed to connect to MongoDB');
            });

            let error = null;

            try {
                await interactionUpdatedHandler(event);
            } catch (e: unknown) {
                error = e;
            }
            expect(error).toBeTruthy();
        });

        it('treats an update as a create when the previous version did not have a user id', async () => {
            const workEffortSecs = 99;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interactionBefore: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            // wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    interactionAfter: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: workEffortSecs,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    updates: [
                        {
                            op: 'update',
                            path: 'path',
                            value: { type: 'type' },
                        },
                    ],
                    operation: 'create',
                },
            } as InteractionUpdatedEvent);

            await interactionUpdatedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
            // No errors should be logged
            expect(log.error).not.toHaveBeenCalled();

            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const upsertedData = dataToBeUpserted[0] as MongoUpdateOneQuery<any>;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
            const handleTime = upsertedData.updateOne.update.$inc.totalInteractionHandleTime_all;
            expect(handleTime).toEqual(workEffortSecs);
        });

        it('treats an update as a create when the previous version did not have an agent', async () => {
            const workEffortSecs = 99;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const validEvent: KinesisStreamEvent = JSON.parse(JSON.stringify(event));
            validEvent.Records[0]!.kinesis.data = encodeKinesisBody({
                data: {
                    interactionBefore: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: 10,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        // agent: {
                        //     teamId: 1,
                        //     wfmId: 2,
                        //     integrationAlias: 'alias',
                        //     timezone: 'Australia/Sydney',
                        // },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    interactionAfter: {
                        organisationId: 1,
                        workstreamIdentifier: '123',
                        type: 'worklog',
                        workEffortSecs: workEffortSecs,
                        workstreamDefinitionId: 1,
                        identifier: 'identifier',
                        createdAt: new Date().toISOString(),
                        agent: {
                            teamId: 1,
                            wfmId: 2,
                            integrationAlias: 'alias',
                            timezone: 'Australia/Sydney',
                        },
                        integration: {
                            integrationName: 'integration',
                            integrationId: 1,
                            qmIntegrationId: 1,
                        },
                        startTime: new Date().toISOString(),
                    },
                    updates: [
                        {
                            op: 'update',
                            path: 'path',
                            value: { type: 'type' },
                        },
                    ],
                    operation: 'create',
                },
            } as InteractionUpdatedEvent);

            await interactionUpdatedHandler(validEvent);

            // Interaction should be written to mongo
            expect(mockBulkWrite).toHaveBeenCalledTimes(1);
            // Nothing should be logged to event debug log
            expect(mockEventDebugLogWrite).not.toHaveBeenCalled();
            // No errors should be logged
            expect(log.error).not.toHaveBeenCalled();

            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const upsertedData = dataToBeUpserted[0] as MongoUpdateOneQuery<any>;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
            const handleTime = upsertedData.updateOne.update.$inc.totalInteractionHandleTime_all;
            expect(handleTime).toEqual(workEffortSecs);
        });
    });
});
