import { rebuildInteractionsHandler } from '../../../../lib/metrics/interactions/reconciliationHandlers/rebuildInteractionsHandler';
import { APIGatewayProxyEvent } from 'aws-lambda';
import { makeToken, tokenData } from '../../../utils/makeToken';
import { AthenaQueryMetadataModel } from '../../../../lib/athena/AthenaQueryMetadataModel';
import * as mongodb from '../../../../lib/utils/mongodb';
import { AthenaQueryResponse } from '../../../../lib/athena/types';
import { extractIdentity } from '@agyletime/wfm-aws-utils';
import { BadRequestError } from '../../../../lib/errors';
import * as initiateAthenaQuery from '../../../../lib/athena/utils';

jest.mock('../../../../lib/utils/mongodb');

jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        extractIdentity: jest.fn(),
    };
});

const organisationId = '9';
const token = makeToken(tokenData);
let apiGatewayEvent: APIGatewayProxyEvent;
let mockWriteAthenaQueryMetadata: unknown;

describe('Rebuild interactions handler', () => {
    beforeEach(() => {
        mockWriteAthenaQueryMetadata = jest.spyOn(AthenaQueryMetadataModel, 'create').mockImplementation();
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
        jest.spyOn(initiateAthenaQuery, 'initiateAthenaQuery').mockImplementation(
            (): Promise<AthenaQueryResponse> => Promise.resolve({ queryExecutionId: '123' }),
        );
        (extractIdentity as jest.Mock).mockImplementation(() => ({
            organisationId: 9,
            identifier: 'admin@agyletime.com',
        }));

        apiGatewayEvent = {
            headers: {
                Authorization: `Bearer ${token}`,
            },
            queryStringParameters: {},
            identity: {
                accessKey: null,
                accountId: 'offlineContext_accountId',
                apiKey: 'offlineContext_apiKey',
                caller: 'offlineContext_caller',
                cognitoAuthenticationProvider: 'offlineContext_cognitoAuthenticationProvider',
                cognitoAuthenticationType: 'offlineContext_cognitoAuthenticationType',
                cognitoIdentityId: 'offlineContext_cognitoIdentityId',
                cognitoIdentityPoolId: 'offlineContext_cognitoIdentityPoolId',
                organisationId: organisationId.toString(),
                principalOrgId: null,
                sourceIp: '127.0.0.1',
                user: 'offlineContext_user',
                userAgent:
                    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36',
                userArn: 'offlineContext_userArn',
            },
            requestContext: {
                authorizer: { organisationId: organisationId, identifier: 'admin@agyletime.com', userId: 1 },
                httpMethod: 'POST',
                // @ts-expect-error Identity isn't implemented here since it's not used
                identity: {},
                path: '/performance/metrics/agent/workstream',
                protocol: 'HTTP/1.1',
                resourcePath: '/metrics/agent/workstream',
                stage: 'dev',
            },
            timestamp: '2020-08-26T22:49:48.521Z',
        };
    });
    describe('throws an error when', () => {
        it('post body is invalid', async () => {
            const mockPostBody = {
                foo: 'bar',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid request');
            }
        });

        it('post body is empty', async () => {
            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('No params supplied');
            }
        });

        it('from date is invalid', async () => {
            const mockPostBody = {
                fromDate: 'last tuesday',
                toDate: '2023-01-04',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('fromDate is invalid');
            }
        });

        it('to date is invalid', async () => {
            const mockPostBody = {
                fromDate: '2023-01-04',
                toDate: 'next tuesday',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('toDate is invalid');
            }
        });

        it('from date is not supplied', async () => {
            const mockPostBody = {
                toDate: '2023-01-04',
                fromDate: '',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid request');
            }
        });

        it('to date is not supplied', async () => {
            const mockPostBody = {
                toDate: '',
                fromDate: '2023-01-04',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid request');
            }
        });

        it('from date is after to date', async () => {
            const mockPostBody = {
                fromDate: '2023-01-05',
                toDate: '2023-01-04',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildInteractionsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid date range');
            }
        });

        it('returns 200 for a successful query', async () => {
            const mockPostBody = {
                fromDate: '2023-01-04',
                toDate: '2023-01-06',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            const response = await rebuildInteractionsHandler(apiGatewayEvent);

            // Expect an OK response
            expect(response.statusCode).toEqual(200);

            // Expect the athena query to be logged
            expect(mockWriteAthenaQueryMetadata).toHaveBeenCalled();
        });
    });
});
