import type { ShiftCreatedEvent, ShiftEventParams } from '../../../lib/metrics';
import calculateShiftCreatedDiff from '../../../lib/metrics/rostered/calculateShiftCreatedDiff';
import { parseISO } from 'date-fns';

let newShift: ShiftCreatedEvent;

const params: ShiftEventParams = {
    organisationId: 1,
    userId: 2,
    teamId: 3,
    shiftId: '4',
    timezone: 'Australia/Sydney',
};

const getRosteredTaskDuration = () => {
    return (
        (new Date(newShift.shift.shiftTasks[0]?.endTime ?? '').getTime() -
            new Date(newShift.shift.shiftTasks[0]?.startTime ?? '').getTime()) /
        1000
    );
};

describe('calculateShiftCreatedDiff', () => {
    beforeEach(() => {
        newShift = {
            roster: {
                id: '123',
                version: 1,
            },
            shift: {
                _id: 'abc',
                shiftTemplate: {
                    id: 'string',
                    name: 'string',
                },
                tags: [],
                startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
                endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
                user: {
                    id: 422,
                    email: 'test@example.org',
                },
                shiftTasks: [
                    {
                        _id: '1',
                        taskType: {
                            isPaid: false,
                            isAvailable: false,
                            isAvailableOccupancy: false,
                            isPlanned: false,
                            isMergeAllowed: false,
                            disableHandleTimeClock: false,
                            additionalHoursOverride: false,
                            id: 123,
                            name: 'name',
                            colour: 'colour',
                            isLeave: false,
                            isProductive: false,
                            hideLeaveInAgentViews: false,
                            taskId: 842,
                        },
                        startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
                        endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
                        policy: {},
                        created: 'created',
                        modified: 'modified',
                    },
                    {
                        _id: '2',
                        taskType: {
                            isPaid: false,
                            isAvailable: false,
                            isAvailableOccupancy: false,
                            isPlanned: false,
                            isMergeAllowed: false,
                            disableHandleTimeClock: false,
                            additionalHoursOverride: false,
                            id: 234,
                            name: 'name',
                            colour: 'colour',
                            isLeave: false,
                            isProductive: false,
                            hideLeaveInAgentViews: false,
                            taskId: 843,
                        },
                        startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
                        endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
                        created: 'created',
                        modified: 'modified',
                        policy: {},
                    },
                ],
                isAdditionalHour: false,
                changedByUser: {},
                createdAt: '',
                hasOptInTask: false,
                leaveRequest: {},
                updatedAt: '',
            },
        };
    });

    describe('increments every metric', () => {
        const createAndAssertDuration = ({
            newShift,
            fieldName,
            expectedValues,
        }: {
            newShift: ShiftCreatedEvent;
            fieldName: string;
            expectedValues: (number | undefined)[];
        }) => {
            const results = calculateShiftCreatedDiff({ newShiftEvent: newShift, params });

            for (let i = 0; i < results.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$inc as any)[fieldName]).toEqual(expectedValues[i]);
            }
        };

        const createAndAssertValue = ({
            newShift,
            fieldName,
            expectedValues,
        }: {
            newShift: ShiftCreatedEvent;
            fieldName: string;
            expectedValues: unknown[];
        }) => {
            const results = calculateShiftCreatedDiff({ newShiftEvent: newShift, params });

            for (let i = 0; i < results.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$set as any)?.[fieldName]).toEqual(expectedValues[i]);
            }
        };

        it('utc time buckets are correct', () => {
            const results = calculateShiftCreatedDiff({ newShiftEvent: newShift, params });
            const expectedValues = [parseISO(newShift.shift.startTime), parseISO(newShift.shift.startTime)];
            for (let i = 0; i < results.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-unsafe-call, @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$set as any)?.timeBucketAsUtc.toISOString()).toEqual(
                    expectedValues[i]?.toISOString(),
                );
            }
        });

        it('rostered start time', () => {
            createAndAssertValue({
                newShift,
                fieldName: 'rosteredStartTime_rosteredHours',
                // Expect both to use the same shift start time
                expectedValues: [new Date(newShift.shift.startTime), new Date(newShift.shift.startTime)],
            });
        });

        it('rostered end time', () => {
            createAndAssertValue({
                newShift,
                fieldName: 'rosteredEndTime_rosteredHours',
                // Expect both to use the same shift end time
                expectedValues: [new Date(newShift.shift.endTime), new Date(newShift.shift.endTime)],
            });
        });

        it('task type', () => {
            createAndAssertValue({
                newShift,
                fieldName: 'taskTypeId',
                // Expect both to use the same shift end time
                expectedValues: [newShift.shift.shiftTasks[0]?.taskType.id, newShift.shift.shiftTasks[1]?.taskType.id],
            });
        });

        it('rostered available time', () => {
            newShift.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            newShift.shift.shiftTasks[1]!.taskType.isAvailableOccupancy = false;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            createAndAssertDuration({
                newShift,
                fieldName: 'rosteredAvailableTime_rosteredHours',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('rostered paid shrinkage duration', () => {
            newShift.shift.shiftTasks[0]!.taskType.isPaid = true;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            newShift.shift.shiftTasks[0]!.taskType.isProductive = false;
            newShift.shift.shiftTasks[1]!.taskType.isPaid = false;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = true;
            newShift.shift.shiftTasks[1]!.taskType.isProductive = true;

            createAndAssertDuration({
                newShift,
                fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('rostered paid tasks', () => {
            newShift.shift.shiftTasks[0]!.taskType.isPaid = true;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            newShift.shift.shiftTasks[1]!.taskType.isPaid = false;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            createAndAssertDuration({
                newShift,
                fieldName: 'rosteredPaidTasks_rosteredHours',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('rostered productive tasks', () => {
            newShift.shift.shiftTasks[0]!.taskType.isProductive = true;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            newShift.shift.shiftTasks[1]!.taskType.isProductive = false;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            createAndAssertDuration({
                newShift,
                fieldName: 'rosteredProductiveTasks_rosteredHours',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('rostered shrinkage duration', () => {
            newShift.shift.shiftTasks[0]!.taskType.isProductive = false;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            newShift.shift.shiftTasks[1]!.taskType.isProductive = true;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            createAndAssertDuration({
                newShift,
                fieldName: 'rosteredShrinkageDuration_rosteredHours',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('total number of shifts', () => {
            newShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            createAndAssertDuration({
                newShift,
                fieldName: 'totalNumberOfShifts_numberOfShifts',
                expectedValues: [0, 1],
            });
        });

        it('additional hours number of shifts', () => {
            const fieldName = 'additionalHoursNumberOfShifts_numberOfShifts';

            // Test isAdditionalHour = true first
            newShift.shift.isAdditionalHour = true;

            let results = calculateShiftCreatedDiff({ newShiftEvent: newShift, params });
            let result = results[0];
            expect(result?.query.$inc?.[fieldName]).toEqual(1);

            // Then again with isAdditionalHour = false
            newShift.shift.isAdditionalHour = false;

            results = calculateShiftCreatedDiff({ newShiftEvent: newShift, params });
            result = results[1];
            expect(result?.query.$inc?.[fieldName]).toEqual(0);
        });

        it('standard shift number of shifts', () => {
            newShift.shift.shiftTasks[0]!.taskType.additionalHoursOverride = true;
            newShift.shift.shiftTasks[1]!.taskType.additionalHoursOverride = false;

            createAndAssertDuration({
                newShift,
                fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                expectedValues: [0, 1],
            });
        });

        it('planned scheduled leave duration', () => {
            newShift.shift.shiftTasks[0]!.taskType.isPlanned = true;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            newShift.shift.shiftTasks[1]!.taskType.isPlanned = false;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            createAndAssertDuration({
                newShift,
                fieldName: 'plannedLeaveDuration_leave',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('total leave', () => {
            newShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            createAndAssertDuration({
                newShift,
                fieldName: 'totalLeave_leave',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });

        it('unplanned leave duration', () => {
            newShift.shift.shiftTasks[0]!.taskType.isPlanned = false;
            newShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            newShift.shift.shiftTasks[1]!.taskType.isPlanned = true;
            newShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            createAndAssertDuration({
                newShift,
                fieldName: 'unplannedLeaveDuration_leave',
                expectedValues: [getRosteredTaskDuration(), 0],
            });
        });
    });

    describe('Bad params', () => {
        it('should fail when timezone is invalid', () => {
            let testResult, testError;
            try {
                testResult = calculateShiftCreatedDiff({
                    newShiftEvent: newShift,
                    params: { ...params, timezone: 'bad timezone' },
                });
            } catch (error: unknown) {
                testError = error as Error & {
                    cause: {
                        message: string;
                        params: ShiftEventParams;
                    };
                };
            }
            expect(testResult).toBeUndefined();
            expect(testError?.message).toBe('Failed to generate timebucket');
            expect(testError?.cause.params.timezone).toBe('bad timezone');
        });
    });
});
