import type { ShiftDeletedEvent, ShiftEventParams } from '../../../lib/metrics/types';
import calculateShiftDeletedDiff from '../../../lib/metrics/rostered/calculateShiftDeletedDiff';
import { parseISO } from 'date-fns';

let deletedShift: ShiftDeletedEvent;

const params: ShiftEventParams = {
    organisationId: 1,
    userId: 2,
    teamId: 3,
    shiftId: '4',
    timezone: 'Australia/Sydney',
};

const getRosteredTaskDuration = () => {
    return (
        (new Date(deletedShift.shift.shiftTasks[0]?.endTime ?? '').getTime() -
            new Date(deletedShift.shift.shiftTasks[0]?.startTime ?? '').getTime()) /
        1000
    );
};

const shift = {
    _id: 'abc',
    shiftTemplate: {
        id: 'string',
        name: 'string',
    },
    tags: [],
    startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
    endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
    user: {
        firstName: '',
        lastName: '',
        timezone: '',
        teamId: 1,
        id: 422,
        email: 'test@example.org',
    },
    location: {
        id: '',
        parentLocationId: '',
        name: 'location name',
        description: 'location description',
        color: 'location color',
    },
    shiftTasks: [
        {
            _id: '1',
            id: 1,
            taskType: {
                isPaid: false,
                isAvailable: false,
                isAvailableOccupancy: false,
                isPlanned: false,
                isMergeAllowed: false,
                disableHandleTimeClock: false,
                additionalHoursOverride: false,
                id: 123,
                name: 'name',
                colour: 'colour',
                isLeave: false,
                categoryName: 'category name',
                isProductive: false,
                categoryId: 532,
                hideLeaveInAgentViews: false,
                taskId: 842,
            },
            startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
            endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
            policy: {},
            created: 'created',
            modified: 'modified',
        },
        {
            id: 2,
            _id: '2',
            taskType: {
                isPaid: false,
                isAvailable: false,
                isAvailableOccupancy: false,
                isPlanned: false,
                isMergeAllowed: false,
                disableHandleTimeClock: false,
                additionalHoursOverride: false,
                id: 234,
                name: 'name',
                colour: 'colour',
                isLeave: false,
                categoryName: 'category name',
                isProductive: false,
                categoryId: 533,
                hideLeaveInAgentViews: false,
                taskId: 843,
            },
            startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
            endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
            created: 'created',
            modified: 'modified',
            policy: {},
        },
    ],
    workloadTaskId: 543,
    workloadTask: {
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: false,
        isPlanned: false,
        isMergeAllowed: false,
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        id: 384,
        name: 'workload task name',
        taskId: 384,
        categoryId: 1,
        categoryName: 'category',
        isProductive: false,
        isShiftSwapAllowed: 'false',
        isLeave: false,
        hideLeaveInAgentViews: false,
        colour: 'red',
    },
    workPlan: {
        id: 'workplan id',
        name: 'workplan name',
    },
    baseShiftDuration: 55,
    shiftDuration: 66,
    isAdditionalHour: false,
    changedByUser: {},
    createdAt: '',
    hasOptInTask: false,
    leaveRequest: {},
    updatedAt: '',
};

describe('calculateShiftDeletedDiff', () => {
    beforeEach(() => {
        deletedShift = {
            roster: {
                id: '123',
                version: 1,
            },
            shift: { ...shift },
            previousShift: { ...shift },
        };
    });

    describe('decrements every metric', () => {
        const deleteAndAssertDuration = ({
            deletedShift,
            fieldName,
            expectedValues,
        }: {
            deletedShift: ShiftDeletedEvent;
            fieldName: string;
            expectedValues: unknown[];
        }) => {
            const results = calculateShiftDeletedDiff({ deletedShiftEvent: deletedShift, params });

            for (let i = 0; i < results.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$inc as any)[fieldName]).toEqual(expectedValues[i]);
            }
        };

        const createAndAssertValue = ({
            deletedShift,
            fieldName,
            expectedValues,
        }: {
            deletedShift: ShiftDeletedEvent;
            fieldName: string;
            expectedValues: unknown[];
        }) => {
            const results = calculateShiftDeletedDiff({ deletedShiftEvent: deletedShift, params });

            for (let i = 0; i < results.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$set as any)?.[fieldName]).toEqual(expectedValues[i]);
            }
        };

        it('utc time buckets are correct', () => {
            const results = calculateShiftDeletedDiff({ deletedShiftEvent: deletedShift, params });
            const expectedValues = [parseISO(deletedShift.shift.startTime), parseISO(deletedShift.shift.startTime)];
            for (let i = 0; i < results.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-unsafe-call, @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$set as any)?.timeBucketAsUtc.toISOString()).toEqual(
                    expectedValues[i]?.toISOString(),
                );
            }
        });

        it('rostered start time', () => {
            createAndAssertValue({
                deletedShift,
                fieldName: 'rosteredStartTime_rosteredHours',
                // Expect both to use the same shift start time
                expectedValues: [new Date(deletedShift.shift.startTime), new Date(deletedShift.shift.startTime)],
            });
        });

        it('rostered end time', () => {
            createAndAssertValue({
                deletedShift,
                fieldName: 'rosteredEndTime_rosteredHours',
                // Expect both to use the same shift end time
                expectedValues: [new Date(deletedShift.shift.endTime), new Date(deletedShift.shift.endTime)],
            });
        });

        it('task type', () => {
            createAndAssertValue({
                deletedShift,
                fieldName: 'taskTypeId',
                // Expect both to use the same shift end time
                expectedValues: [
                    deletedShift.shift.shiftTasks[0]?.taskType.id,
                    deletedShift.shift.shiftTasks[1]?.taskType.id,
                ],
            });
        });

        it('rostered available time', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isAvailableOccupancy = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'rosteredAvailableTime_rosteredHours',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('rostered paid shrinkage duration', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isPaid = true;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            deletedShift.shift.shiftTasks[0]!.taskType.isProductive = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isPaid = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isProductive = true;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('rostered paid tasks', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isPaid = true;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isPaid = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'rosteredPaidTasks_rosteredHours',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('rostered productive tasks', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isProductive = true;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isProductive = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'rosteredProductiveTasks_rosteredHours',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('rostered shrinkage duration', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isProductive = false;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isProductive = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = true;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'rosteredShrinkageDuration_rosteredHours',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('total number of shifts', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'totalNumberOfShifts_numberOfShifts',
                expectedValues: [0, -1],
            });
        });

        it('additional hours number of shifts', () => {
            const fieldName = 'additionalHoursNumberOfShifts_numberOfShifts';

            // Test isAdditionalHour = true first
            deletedShift.shift.isAdditionalHour = true;

            let results = calculateShiftDeletedDiff({ deletedShiftEvent: deletedShift, params });
            let result = results[0];
            expect(result?.query.$inc?.[fieldName]).toEqual(-1);

            // Then again with isAdditionalHour = false
            deletedShift.shift.isAdditionalHour = false;

            results = calculateShiftDeletedDiff({ deletedShiftEvent: deletedShift, params });
            result = results[1];
            expect(result?.query.$inc?.[fieldName]).toEqual(0);
        });

        it('standard shift number of shifts', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.additionalHoursOverride = true;
            deletedShift.shift.shiftTasks[1]!.taskType.additionalHoursOverride = false;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                expectedValues: [0, -1],
            });
        });

        it('planned scheduled leave duration', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isPlanned = true;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isPlanned = false;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'plannedLeaveDuration_leave',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('total leave', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'totalLeave_leave',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });

        it('unplanned leave duration', () => {
            deletedShift.shift.shiftTasks[0]!.taskType.isPlanned = false;
            deletedShift.shift.shiftTasks[0]!.taskType.isLeave = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isPlanned = true;
            deletedShift.shift.shiftTasks[1]!.taskType.isLeave = false;

            deleteAndAssertDuration({
                deletedShift,
                fieldName: 'unplannedLeaveDuration_leave',
                expectedValues: [-getRosteredTaskDuration(), 0],
            });
        });
    });
});
