import * as getTeamForUser from '../../../lib/utils/getTeamForUser';
import * as getTimezoneForUser from '../../../lib/utils/getTimezoneForUser';
import { config } from '../../../lib/config';
import { MongoRosteredMetricModel } from '@agyletime/performance-utils';
import { BasicAuthApiClientParams } from '@agyletime/wfm-client/lib/types/agyleTimeClient/agyleTimeClient';
import { getSSMParameters, extractIdentity, getOrganisationApiParamsById } from '@agyletime/wfm-aws-utils';
import { rebuildRosteredMetricsForOrg } from '../../../lib/metrics';
import * as mongodb from '../../../lib/utils/mongodb';
import { publishedShifts } from './fixtures/publishedShifts';

jest.mock('../../../lib/utils/mongodb');
jest.createMockFromModule('@agyletime/performance-utils');

jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        getSSMParameters: jest.fn(),
        extractIdentity: jest.fn(),
        getOrganisationApiParamsById: jest.fn(),
    };
});

jest.mock('@agyletime/wfm-client');
// eslint-disable-next-line @typescript-eslint/no-var-requires,@typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
const PublishedShiftApi = require('@agyletime/wfm-client').PublishedShiftApi;

let deleteManySpy: jest.SpyInstance;
let bulkWriteSpy: jest.SpyInstance;
let getTeamForUserSpy: jest.SpyInstance;
let getTimezoneForUserSpy: jest.SpyInstance;

// TODO: Move unit test to rebuildRosteredMetrics once step function works
describe('rebuildRosteredMetricsForOrg', () => {
    describe('returns an error when', () => {
        beforeEach(() => {
            // @ts-expect-error type mismatch
            jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
            (getSSMParameters as jest.Mock).mockImplementation(() => [{ id: 9, api: { foo: 'bar' } }]);
            (extractIdentity as jest.Mock).mockImplementation(() => ({
                organisationId: 9,
                identifier: 'admin@agyletime.com',
            }));
            (getOrganisationApiParamsById as jest.Mock).mockImplementation(() => ({
                org: 'params',
            }));

            // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
            PublishedShiftApi.mockImplementation(() => {
                return {
                    getPublishedShiftApi: jest.fn().mockResolvedValue({
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                        data: JSON.parse(JSON.stringify(publishedShifts)), // Clone this as the filtering in rebuildRosteredMetricsForOrg may mutate it
                    }),
                };
            });

            deleteManySpy = jest.spyOn(MongoRosteredMetricModel, 'deleteMany').mockImplementation();
            bulkWriteSpy = jest.spyOn(MongoRosteredMetricModel, 'bulkWrite').mockImplementation();
            getTeamForUserSpy = jest
                .spyOn(getTeamForUser, 'getTeamForUser')
                // eslint-disable-next-line @typescript-eslint/require-await
                .mockImplementation(async () => ({ id: 2 }));
            getTimezoneForUserSpy = jest
                .spyOn(getTimezoneForUser, 'getTimezoneForUser')
                // eslint-disable-next-line @typescript-eslint/require-await
                .mockImplementation(async () => 'Australia/Sydney');
            config.featureFlags.orgAllowListTPM = '9';
        });
        afterEach(() => {
            jest.restoreAllMocks();
        });
        it('does not call getTeamForUser if team supplied', async () => {
            // The following dates have team set in fixture
            const fromDate = new Date('2024-01-04'),
                toDate = new Date('2024-01-05');
            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsForOrg(
                    { id: 9, name: 'dummy org' },
                    fromDate,
                    toDate,
                    {} as BasicAuthApiClientParams,
                );
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse).toBeDefined();
            expect(testError).toBeUndefined();

            expect(getTimezoneForUserSpy).toHaveBeenCalled();
            expect(getTeamForUserSpy).not.toHaveBeenCalled();
            expect(deleteManySpy).toHaveBeenCalled();
            expect(bulkWriteSpy).toHaveBeenCalled();
        });
        it('does not call getTimezoneForUserSpy if team supplied', async () => {
            // The following dates have team set in fixture
            const fromDate = new Date('2025-01-04'),
                toDate = new Date('2025-01-05');
            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsForOrg(
                    { id: 9, name: 'dummy org' },
                    fromDate,
                    toDate,
                    {} as BasicAuthApiClientParams,
                );
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse).toBeDefined();
            expect(testError).toBeUndefined();

            expect(getTimezoneForUserSpy).not.toHaveBeenCalled();
            expect(getTeamForUserSpy).toHaveBeenCalled();
            expect(deleteManySpy).toHaveBeenCalled();
            expect(bulkWriteSpy).toHaveBeenCalled();
        });
        it('is disabled if org is not part of feature flag on', async () => {
            const fromDate = new Date('2023-01-04'),
                toDate = new Date('2023-01-06');
            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsForOrg(
                    { id: 1, name: 'Test Org' },
                    fromDate,
                    toDate,
                    {} as BasicAuthApiClientParams,
                );
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse).toBeDefined();
            expect(testError).toBeUndefined();

            expect(deleteManySpy).not.toHaveBeenCalled();
            expect(bulkWriteSpy).not.toHaveBeenCalled();
            expect(testResponse?.disabled).toBeTruthy();
        });
        it('generates metrics when given org details and date range', async () => {
            const fromDate = new Date('2023-09-13'),
                toDate = new Date('2023-09-14');
            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsForOrg(
                    { id: 9, name: 'Test Org' },
                    fromDate,
                    toDate,
                    {} as BasicAuthApiClientParams,
                );
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse).toBeDefined();
            expect(testError).toBeUndefined();

            expect(deleteManySpy).toHaveBeenCalled();
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            expect(bulkWriteSpy.mock.calls[0][0].length).toBe(7);
        });
        it('does not generates metrics when shift is within date range but does not start within date range', async () => {
            const fromDate = new Date('2023-10-18'),
                toDate = new Date('2023-10-19');
            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsForOrg(
                    { id: 9, name: 'Test Org' },
                    fromDate,
                    toDate,
                    {} as BasicAuthApiClientParams,
                );
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse).toBeDefined();
            expect(testError).toBeUndefined();

            expect(deleteManySpy).toHaveBeenCalled();
            expect(bulkWriteSpy).not.toHaveBeenCalled();
        });
        it('generates metrics when shift starts within date range', async () => {
            const fromDate = new Date('2023-10-18'),
                toDate = new Date('2023-10-19');
            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsForOrg(
                    { id: 9, name: 'Test Org' },
                    fromDate,
                    toDate,
                    {} as BasicAuthApiClientParams,
                );
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse).toBeDefined();
            expect(testError).toBeUndefined();

            expect(deleteManySpy).toHaveBeenCalled();
            expect(bulkWriteSpy).not.toHaveBeenCalled();
        });
    });
});
