import { APIGatewayProxyEvent } from 'aws-lambda';
import { makeToken, tokenData } from '../../../utils/makeToken';
import { rebuildRosteredMetricsHandler } from '../../../../lib/metrics/rostered/reconciliationHandlers/rebuildRosteredMetricsHandler';
import { BadRequestError } from '../../../../lib/errors';
import { extractIdentity, getSSMParameters, invokeStepFunction } from '@agyletime/wfm-aws-utils';

const organisationId = '9';
const token = makeToken(tokenData);
let apiGatewayEvent: APIGatewayProxyEvent;
jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        extractIdentity: jest.fn(),
        getSSMParameters: jest.fn(),
        invokeStepFunction: jest.fn(),
    };
});

describe('Rebuild rostered metrics handler', () => {
    beforeEach(() => {
        (extractIdentity as jest.Mock).mockImplementation(() => ({
            organisationId: 9,
            identifier: 'admin@agyletime.com',
        }));
        (getSSMParameters as jest.Mock).mockImplementation(() => [{ id: 9, name: 'dummy org' }]);
        (invokeStepFunction as jest.Mock).mockImplementation(() => Promise.resolve());
        apiGatewayEvent = {
            headers: {
                Authorization: `Bearer ${token}`,
            },
            queryStringParameters: {},
            identity: {
                accessKey: null,
                accountId: 'offlineContext_accountId',
                apiKey: 'offlineContext_apiKey',
                caller: 'offlineContext_caller',
                cognitoAuthenticationProvider: 'offlineContext_cognitoAuthenticationProvider',
                cognitoAuthenticationType: 'offlineContext_cognitoAuthenticationType',
                cognitoIdentityId: 'offlineContext_cognitoIdentityId',
                cognitoIdentityPoolId: 'offlineContext_cognitoIdentityPoolId',
                organisationId: organisationId.toString(),
                principalOrgId: null,
                sourceIp: '127.0.0.1',
                user: 'offlineContext_user',
                userAgent:
                    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36',
                userArn: 'offlineContext_userArn',
            },
            requestContext: {
                authorizer: { organisationId: organisationId, identifier: 'admin@agyletime.com', userId: 1 },
                httpMethod: 'POST',
                // @ts-expect-error Identity isn't implemented here since it's not used
                identity: {},
                path: '/performance/metrics/agent/workstream',
                protocol: 'HTTP/1.1',
                resourcePath: '/metrics/agent/workstream',
                stage: 'dev',
            },
            timestamp: '2020-08-26T22:49:48.521Z',
        };
    });
    afterEach(() => {
        jest.restoreAllMocks();
    });
    describe('throws an error when', () => {
        it('post body is invalid', async () => {
            const mockPostBody = {
                foo: 'bar',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid request');
            }
        });

        it('post body is empty', async () => {
            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('No params supplied');
            }
        });

        it('from date is invalid', async () => {
            const mockPostBody = {
                fromDate: 'last tuesday',
                toDate: '2023-01-04',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('fromDate is invalid');
            }
        });

        it('to date is invalid', async () => {
            const mockPostBody = {
                fromDate: '2023-01-04',
                toDate: 'next tuesday',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('toDate is invalid');
            }
        });

        it('from date is not supplied', async () => {
            const mockPostBody = {
                toDate: '2023-01-04',
                fromDate: '',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid request');
            }
        });

        it('to date is not supplied', async () => {
            const mockPostBody = {
                toDate: '',
                fromDate: '2023-01-04',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid request');
            }
        });

        it('from date is after to date', async () => {
            const mockPostBody = {
                fromDate: '2023-01-05',
                toDate: '2023-01-04',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            try {
                await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as BadRequestError).message).toEqual('Invalid date range');
            }
        });

        it('returns 200 for a successful query', async () => {
            const mockPostBody = {
                fromDate: '2023-01-04',
                toDate: '2023-01-06',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse?.statusCode).toEqual(200);
            expect(testError).toBeUndefined();
            // @ts-expect-error type mismatch
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            expect(JSON.parse(testResponse?.body).data.errors.length).toBe(0);
        });
        it('returns 200 but with error list if step function errors', async () => {
            const dummyError = 'Error of some sort!';
            // eslint-disable-next-line @typescript-eslint/prefer-promise-reject-errors
            (invokeStepFunction as jest.Mock).mockImplementation(() => Promise.reject(dummyError));

            const mockPostBody = {
                fromDate: '2023-01-04',
                toDate: '2023-01-06',
            };
            apiGatewayEvent.body = JSON.stringify(mockPostBody);

            let testResponse, testError;
            try {
                testResponse = await rebuildRosteredMetricsHandler(apiGatewayEvent);
            } catch (error: unknown) {
                testError = error;
            }

            // Expect an OK response
            expect(testResponse?.statusCode).toEqual(200);
            expect(testError).toBeUndefined();
            // @ts-expect-error type mismatch
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            expect(JSON.parse(testResponse?.body).data.errors[0]).toStrictEqual({ error: dummyError });
        });
    });
});
