import { config } from '../../lib/config';
import { makeToken, tokenData } from '../utils/makeToken';
import type { APIGatewayEvent } from 'aws-lambda';
import * as Mongo from '../../lib/utils/mongodb_native';
import { postMetricsConfiguration } from '../../lib/savedConfiguration/postMetricsConfiguration';
import { extractIdentity } from '@agyletime/wfm-aws-utils';

jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        extractIdentity: jest.fn(),
    };
});

config.db.url = 'test url';
const organisationId = '9';
const token = makeToken(tokenData);

const mockMetrics = {
    fake: 'content',
};

const mockResult = 'Success!';

const event = {
    headers: {
        Authorization: `Bearer ${token}`,
    },
    queryStringParameters: {},
    identity: {
        accessKey: null,
        accountId: 'offlineContext_accountId',
        apiKey: 'offlineContext_apiKey',
        caller: 'offlineContext_caller',
        cognitoAuthenticationProvider: 'offlineContext_cognitoAuthenticationProvider',
        cognitoAuthenticationType: 'offlineContext_cognitoAuthenticationType',
        cognitoIdentityId: 'offlineContext_cognitoIdentityId',
        cognitoIdentityPoolId: 'offlineContext_cognitoIdentityPoolId',
        organisationId: organisationId.toString(),
        principalOrgId: null,
        sourceIp: '127.0.0.1',
        user: 'offlineContext_user',
        userAgent:
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36',
        userArn: 'offlineContext_userArn',
    },
    requestContext: {
        authorizer: { organisationId: organisationId, identifier: 'admin@agyletime.com', userId: 1 },
        httpMethod: 'POST',
        identity: {},
        path: '/performance/metrics/agent/workstream',
        protocol: 'HTTP/1.1',
        resourcePath: '/metrics/agent/workstream',
        stage: 'dev',
    },
    timestamp: '2020-08-26T22:49:48.521Z',
    body: JSON.stringify(mockMetrics),
} as unknown as APIGatewayEvent;

describe('HTTP Endpoint', () => {
    let db: jest.SpyInstance;

    beforeEach(() => {
        db = jest.spyOn(Mongo, 'createMongoConnection');

        db.mockImplementation(
            () =>
                // eslint-disable-next-line @typescript-eslint/no-unsafe-return
                Promise.resolve({
                    database: {},
                    collections: { metrics_configuration: { updateOne: () => mockResult } },
                }) as any, // eslint-disable-line @typescript-eslint/no-explicit-any
        );
        (extractIdentity as jest.Mock).mockImplementation(() => ({
            organisationId: 9,
            identifier: 'admin@agyletime.com',
        }));
    });

    afterEach(() => {
        db.mockReset();
    });

    describe('with a valid body', () => {
        it('returns', async () => {
            const result = await postMetricsConfiguration(event);
            expect(result.body).toContain(mockResult);
        });

        it('replaces a blank body with a blank array', async () => {
            const eventWithNoBody = { ...event, body: null };

            const result = await postMetricsConfiguration(eventWithNoBody);
            expect(result.body).toContain(mockResult);
        });

        it('returns an error if the db connection fails', async () => {
            db.mockImplementation(
                () =>
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-return, @typescript-eslint/no-explicit-any
                    Promise.resolve({ collections: { metrics_configuration: { updateOne: () => mockResult } } }) as any,
            );

            try {
                await postMetricsConfiguration(event);
            } catch (e: unknown) {
                // eslint-disable-next-line jest/no-conditional-expect
                expect((e as Error).message).toEqual('Failed to connect to MongoDB');
            }
        });
    });
});
