import type { Shift, ShiftEventParams, ShiftTask, ShiftUpdatedEvent } from '../../../lib/metrics/types';
import { addSeconds, differenceInSeconds } from 'date-fns';
import calculateShiftUpdatedDiff, {
    getMatchingShiftTasks,
    getMissingShiftTasks,
} from '../../../lib/metrics/rostered/calculateShiftUpdatedDiff';
import { calculateCreateDiffForShiftTask } from '../../../lib/metrics/rostered/calculateShiftCreatedDiff';
import { calculateDeleteDiffForShiftTask } from '../../../lib/metrics/rostered/calculateShiftDeletedDiff';
import ShiftUpdatedEventJSON from './fixtures/ShiftUpdatedEvent.json';

let event: ShiftUpdatedEvent;
const timeIncrementInSeconds: number = 60 * 60;
const timeDecrementInSeconds: number = 0.5 * 60 * 60;
const bucketOffsetIncrementInSeconds: number = 17 * 60;
const startTime = new Date(2022, 1, 1, 13, 0, 0);
const endTime = new Date(2022, 1, 1, 14, 0, 0);
const taskDurationInSeconds: number = differenceInSeconds(endTime, startTime);

const incrementTime = (endTimeValue = endTime): string =>
    addSeconds(endTimeValue, timeIncrementInSeconds).toISOString();
const decrementTime = (endTimeValue = endTime): string =>
    addSeconds(endTimeValue, -timeDecrementInSeconds).toISOString();
const moveToANewBucket = (startTimeValue = startTime): string =>
    addSeconds(startTimeValue, bucketOffsetIncrementInSeconds).toISOString();

const filter: ShiftEventParams = {
    organisationId: 1,
    userId: 2,
    teamId: 3,
    shiftId: '4',
    timezone: 'Australia/Sydney',
};

const shift = {
    _id: 'abc',
    shiftTemplate: {
        id: 'string',
        name: 'string',
    },
    tags: [],
    startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
    endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
    user: {
        firstName: '',
        lastName: '',
        timezone: '',
        teamId: 1,
        id: 422,
        email: 'test@example.org',
    },
    location: {
        id: '',
        parentLocationId: '',
        name: 'location name',
        description: 'location description',
        color: 'location color',
    },
    shiftTasks: [
        {
            _id: '1',
            id: 1,
            taskType: {
                isPaid: false,
                isAvailable: false,
                isAvailableOccupancy: false,
                isPlanned: false,
                isMergeAllowed: false,
                disableHandleTimeClock: false,
                additionalHoursOverride: false,
                id: 123,
                name: 'name',
                colour: 'colour',
                isLeave: false,
                categoryName: 'category name',
                isProductive: false,
                categoryId: 532,
                hideLeaveInAgentViews: false,
                taskId: 842,
            },
            startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
            endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
            policy: {},
            created: 'created',
            modified: 'modified',
        },
        {
            id: 2,
            _id: '2',
            taskType: {
                isPaid: false,
                isAvailable: false,
                isAvailableOccupancy: false,
                isPlanned: false,
                isMergeAllowed: false,
                disableHandleTimeClock: false,
                additionalHoursOverride: false,
                id: 234,
                name: 'name',
                colour: 'colour',
                isLeave: false,
                categoryName: 'category name',
                isProductive: false,
                categoryId: 533,
                hideLeaveInAgentViews: false,
                taskId: 843,
            },
            startTime: new Date(2022, 1, 1, 13, 0, 0).toISOString(),
            endTime: new Date(2022, 1, 1, 14, 0, 0).toISOString(),
            created: 'created',
            modified: 'modified',
            policy: {},
        },
    ],
    workloadTaskId: 543,
    workloadTask: {
        isPaid: false,
        isAvailable: false,
        isAvailableOccupancy: false,
        isPlanned: false,
        isMergeAllowed: false,
        disableHandleTimeClock: false,
        additionalHoursOverride: false,
        id: 384,
        name: 'workload task name',
        taskId: 384,
        categoryId: 1,
        categoryName: 'category',
        isProductive: false,
        isShiftSwapAllowed: 'false',
        isLeave: false,
        hideLeaveInAgentViews: false,
        colour: 'red',
    },
    workPlan: {
        id: 'workplan id',
        name: 'workplan name',
    },
    baseShiftDuration: 55,
    shiftDuration: 66,
    isAdditionalHour: false,
};

describe('shift intersection helpers', function () {
    const oldShiftTasks: ShiftTask[] = [
        // @ts-expect-error Partial type needed for testing
        {
            _id: 'matching shift task 1',
        },
        // @ts-expect-error Partial type needed for testing
        {
            _id: 'expect to be deleted',
        },
        // @ts-expect-error Partial type needed for testing
        {
            _id: 'matching shift task 2',
        },
    ];

    const newShiftTasks: ShiftTask[] = [
        // @ts-expect-error Partial type needed for testing
        {
            _id: 'matching shift task 1',
        },
        // @ts-expect-error Partial type needed for testing
        {
            _id: 'expect to be new',
        },
        // @ts-expect-error Partial type needed for testing
        {
            _id: 'matching shift task 2',
        },
    ];

    it('gets the array of tasks to be created', () => {
        const tasksToBeCreated = getMissingShiftTasks(newShiftTasks, oldShiftTasks);

        expect(tasksToBeCreated).toEqual([{ _id: 'expect to be new' }]);
    });

    it('gets the array of tasks to be deleted', () => {
        const tasksToBeDeleted = getMissingShiftTasks(oldShiftTasks, newShiftTasks);

        expect(tasksToBeDeleted).toEqual([{ _id: 'expect to be deleted' }]);
    });

    it('gets the array of tasks to be updated', () => {
        const tasksToBeUpdated = getMatchingShiftTasks(oldShiftTasks, newShiftTasks);

        expect(tasksToBeUpdated).toEqual([{ _id: 'matching shift task 1' }, { _id: 'matching shift task 2' }]);
    });

    it('gets the array of tasks to be updated, but backwards', () => {
        const tasksToBeUpdated = getMatchingShiftTasks(newShiftTasks, oldShiftTasks);

        expect(tasksToBeUpdated).toEqual([{ _id: 'matching shift task 1' }, { _id: 'matching shift task 2' }]);
    });
});

describe('shiftUpdatedEvent', () => {
    beforeEach(() => {
        event = {
            roster: {
                id: '123',
                version: 1,
            },
            // quick and dirty deep clone
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            shift: JSON.parse(JSON.stringify(shift)),
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            previousShift: JSON.parse(JSON.stringify(shift)),
        };
    });

    describe('Real Event', () => {
        const { previousShift, shift } = ShiftUpdatedEventJSON;
        const results = calculateShiftUpdatedDiff({
            previousVersion: previousShift,
            currentVersion: shift,
            params: filter,
        });
        it('should calculate a shift updated event', () => {
            expect(results).toBeTruthy();
        });

        it('utc time buckets are correct', () => {
            const expectedValues = [new Date(previousShift.startTime), new Date(shift.startTime)];
            for (let i = 0; i < expectedValues.length; i++) {
                const result = results[i];
                // eslint-disable-next-line @typescript-eslint/no-unsafe-call, @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query.$set as any)?.timeBucketAsUtc.toISOString()).toEqual(
                    expectedValues[i]?.toISOString(),
                );
            }
        });
    });

    describe('within the same time bucket', () => {
        describe('successfully increments', () => {
            const incrementAndAssertWithinBucket = ({
                currentVersion,
                previousVersion,
                fieldName,
                expectedIncrement,
            }: {
                currentVersion: Shift;
                previousVersion: Shift;
                fieldName: string;
                expectedIncrement: number;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.shiftTasks[0]!.endTime = incrementTime(new Date(currentVersion.shiftTasks[0]!.endTime));

                const results = calculateShiftUpdatedDiff({ previousVersion, currentVersion, params: filter });

                // Expect to use the same bucket. There should be 2 documents, one for each shift task
                expect(results.length).toBe(2);

                // Expect to see 1 result
                const result = results[0];
                expect(result).toBeTruthy();

                // Assert that the given field has been incremented by timeIncrementInSeconds
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query as any)?.$inc[fieldName]).toEqual(expectedIncrement);
            };

            it('rostered available time', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('rostered paid shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('rostered paid tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('rostered productive tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('rostered shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('total number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('additional hours number of shifts', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = true;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('standard shift number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('planned scheduled leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('total leave', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('unplanned leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                incrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });
        });
        describe('successfully decrements', () => {
            const decrementAndAssertWithinBucket = ({
                currentVersion,
                previousVersion,
                fieldName,
                expectedIncrement,
            }: {
                currentVersion: Shift;
                previousVersion: Shift;
                fieldName: string;
                expectedIncrement: number;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.shiftTasks[0]!.endTime = decrementTime(new Date(currentVersion.shiftTasks[0]!.endTime));

                const results = calculateShiftUpdatedDiff({ previousVersion, currentVersion, params: filter });

                // Expect to use the same bucket. There should be 2 documents, one for each shift task
                expect(results.length).toBe(2);

                // Expect to see 1 result
                const result = results[0];
                expect(result).toBeTruthy();

                // Assert that the given field has been incremented by timeIncrementInSeconds
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query as any)?.$inc[fieldName]).toEqual(expectedIncrement);
            };

            it('rostered available time', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('rostered paid shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('rostered paid tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('rostered productive tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('rostered shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('total number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('additional hours number of shifts', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = true;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('standard shift number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('planned scheduled leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('total leave', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });

            it('unplanned leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                decrementAndAssertWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: -timeDecrementInSeconds,
                });
            });
        });
        describe('modifies a shift', () => {
            const updateAndAssertShiftWithinBucket = ({
                currentVersion,
                previousVersion,
                fieldName,
                expectedIncrement,
            }: {
                currentVersion: Shift;
                previousVersion: Shift;
                fieldName: string;
                expectedIncrement: number;
            }) => {
                // Increment the end time
                currentVersion.shiftTasks[0]!.endTime = incrementTime(new Date(currentVersion.shiftTasks[0]!.endTime));

                const results = calculateShiftUpdatedDiff({ previousVersion, currentVersion, params: filter });

                // Expect to use the same bucket. There should be 2 documents, one for each shift task
                expect(results.length).toBe(2);

                // Expect to see 1 result
                const result = results[0];
                expect(result).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((result?.query as any)?.$inc[fieldName]).toEqual(expectedIncrement);
            };

            it('isAvailableOccupancy true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds + taskDurationInSeconds,
                });
            });

            it('isPaid true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPaid false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: 0,
                });
            });

            it('isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isAdditionalHour true -> true', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('isAdditionalHour true -> false', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    expectedIncrement: -1,
                });
            });

            it('isAdditionalHour false -> false', () => {
                event.previousShift.isAdditionalHour = false;
                event.shift.isAdditionalHour = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('isAdditionalHour false -> true', () => {
                event.previousShift.isAdditionalHour = false;
                event.shift.isAdditionalHour = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    expectedIncrement: 1,
                });
            });

            it('additionalHoursOverride true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = true;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('additionalHoursOverride true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = true;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    expectedIncrement: 1,
                });
            });

            it('additionalHoursOverride false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });
            });

            it('additionalHoursOverride false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    expectedIncrement: -1,
                });
            });

            it('isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'totalLeave_leave',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    expectedIncrement: 1,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    expectedIncrement: 0,
                });
            });

            it('isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    expectedIncrement: -1,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'totalLeave_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPlanned true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPlanned true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPlanned false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: timeIncrementInSeconds,
                });
            });

            it('isPlanned false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPlanned false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    currentVersion: JSON.parse(JSON.stringify(event.shift)),
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    previousVersion: JSON.parse(JSON.stringify(event.previousShift)),
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });
            });

            it('isPlanned false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPlanned false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: -taskDurationInSeconds,
                });
            });

            it('isPlanned false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });

            it('isPlanned false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    expectedIncrement: taskDurationInSeconds + timeIncrementInSeconds,
                });

                updateAndAssertShiftWithinBucket({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    expectedIncrement: 0,
                });
            });
        });
        describe('handles a shift task', () => {
            it('being added', () => {
                const newTask: ShiftTask = {
                    _id: 'new task',
                    policy: {},
                    taskType: {
                        isPaid: false,
                        isAvailable: false,
                        isAvailableOccupancy: false,
                        isPlanned: false,
                        isMergeAllowed: false,
                        disableHandleTimeClock: false,
                        additionalHoursOverride: false,
                        id: 123,
                        name: 'name',
                        colour: 'colour',
                        isLeave: false,
                        isProductive: false,
                        hideLeaveInAgentViews: false,
                        taskId: 842,
                    },
                    startTime: startTime.toISOString(),
                    endTime: endTime.toISOString(),
                    created: 'created',
                    modified: 'modified',
                };

                event.shift.shiftTasks.push(newTask);

                const expectedResult = calculateCreateDiffForShiftTask({
                    shiftTask: newTask,
                    shift: event.shift,
                    params: filter,
                });

                const result = calculateShiftUpdatedDiff({
                    previousVersion: event.previousShift,
                    currentVersion: event.shift,
                    params: filter,
                });

                expect(result).toContainEqual(expectedResult);
            });

            it('being removed', () => {
                const removedTask = event.shift.shiftTasks.pop()!;

                const expectedResult = calculateDeleteDiffForShiftTask({
                    shiftTask: removedTask,
                    shift: event.shift,
                    params: filter,
                });

                const result = calculateShiftUpdatedDiff({
                    previousVersion: event.previousShift,
                    currentVersion: event.shift,
                    params: filter,
                });

                expect(result).toContainEqual(expectedResult);
            });
        });
    });

    describe('across different time buckets', () => {
        describe('successfully increments', () => {
            const incrementAndAssertAcrossBuckets = ({
                currentVersion,
                previousVersion,
                fieldName,
                oldBucketExpectedIncrement,
                newBucketExpectedIncrement,
            }: {
                currentVersion: Shift;
                previousVersion: Shift;
                fieldName: string;
                oldBucketExpectedIncrement: number;
                newBucketExpectedIncrement: number;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                currentVersion.shiftTasks[0]!.endTime = incrementTime(new Date(currentVersion.shiftTasks[0]!.endTime));

                // Move to a new bucket
                currentVersion.shiftTasks[0]!.startTime = moveToANewBucket(
                    new Date(currentVersion.shiftTasks[0]!.startTime),
                );

                const results = calculateShiftUpdatedDiff({ previousVersion, currentVersion, params: filter });

                // Expect to have 2 buckets. There should be 3 documents, 2 for the first shift task and 1 for the second
                expect(results.length).toBe(3);

                // Expect to see 2 results
                const upsertForPreviousBucket = results[0];
                const upsertForNewBucket = results[1];
                expect(upsertForPreviousBucket).toBeTruthy();
                expect(upsertForNewBucket).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForPreviousBucket?.query as any)?.$inc[fieldName]).toEqual(oldBucketExpectedIncrement);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForNewBucket?.query as any)?.$inc[fieldName]).toEqual(newBucketExpectedIncrement);
            };

            it('rostered available time', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered paid shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered paid tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered productive tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('total number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('additional hours number of shifts', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = true;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('standard shift number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('planned scheduled leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('total leave', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('unplanned leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                incrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });
        });
        describe('successfully decrements', () => {
            const decrementAndAssertAcrossBuckets = ({
                currentVersion,
                previousVersion,
                fieldName,
                oldBucketExpectedIncrement,
                newBucketExpectedIncrement,
            }: {
                currentVersion: Shift;
                previousVersion: Shift;
                fieldName: string;
                oldBucketExpectedIncrement: number;
                newBucketExpectedIncrement: number;
            }) => {
                // Increment the end time by timeIncrementInSeconds
                event.shift.shiftTasks[0]!.endTime = decrementTime();

                // Move to a new bucket
                event.shift.shiftTasks[0]!.startTime = moveToANewBucket();

                const results = calculateShiftUpdatedDiff({ previousVersion, currentVersion, params: filter });

                // Expect to have 2 buckets. There should be 3 documents, 2 for the first shift task and 1 for the second
                expect(results.length).toBe(3);

                // Expect to see 2 results
                const upsertForPreviousBucket = results[0];
                const upsertForNewBucket = results[1];
                expect(upsertForPreviousBucket).toBeTruthy();
                expect(upsertForNewBucket).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForPreviousBucket?.query as any)?.$inc[fieldName]).toEqual(oldBucketExpectedIncrement);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForNewBucket?.query as any)?.$inc[fieldName]).toEqual(newBucketExpectedIncrement);
            };

            it('rostered available time', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered paid shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered paid tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered productive tasks', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('rostered shrinkage duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('total number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('additional hours number of shifts', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = true;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('standard shift number of shifts', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('planned scheduled leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('total leave', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('unplanned leave duration', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                decrementAndAssertAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds - timeDecrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });
        });
        describe('modifies a shift', () => {
            const updateAndAssertShiftAcrossBuckets = ({
                currentVersion,
                previousVersion,
                fieldName,
                oldBucketExpectedIncrement,
                newBucketExpectedIncrement,
            }: {
                currentVersion: Shift;
                previousVersion: Shift;
                fieldName: string;
                oldBucketExpectedIncrement: number;
                newBucketExpectedIncrement: number;
            }) => {
                // Increment the end time
                event.shift.shiftTasks[0]!.endTime = incrementTime();

                // Move to a new bucket
                event.shift.shiftTasks[0]!.startTime = moveToANewBucket();

                const results = calculateShiftUpdatedDiff({ previousVersion, currentVersion, params: filter });

                // Expect to have 2 buckets. There should be 3 documents, 2 for the first shift task and 1 for the second
                expect(results.length).toBe(3);

                // Expect to see 2 results
                const upsertForPreviousBucket = results[0];
                const upsertForNewBucket = results[1];
                expect(upsertForPreviousBucket).toBeTruthy();
                expect(upsertForNewBucket).toBeTruthy();

                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForPreviousBucket?.query as any)?.$inc[fieldName]).toEqual(oldBucketExpectedIncrement);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-member-access
                expect((upsertForNewBucket?.query as any)?.$inc[fieldName]).toEqual(newBucketExpectedIncrement);
            };

            it('isAvailableOccupancy true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isAvailableOccupancy false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isAvailableOccupancy = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isAvailableOccupancy = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredAvailableTime_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPaid false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPaid false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPaid = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPaid = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredPaidTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isProductive true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isProductive true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isProductive false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isProductive false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isProductive false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isProductive = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isProductive = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredProductiveTasks_rosteredHours',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'rosteredShrinkageDuration_rosteredHours',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAdditionalHour true -> true', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('isAdditionalHour true -> false', () => {
                event.previousShift.isAdditionalHour = true;
                event.shift.isAdditionalHour = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAdditionalHour false -> false', () => {
                event.previousShift.isAdditionalHour = false;
                event.shift.isAdditionalHour = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isAdditionalHour false -> true', () => {
                event.previousShift.isAdditionalHour = false;
                event.shift.isAdditionalHour = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'additionalHoursNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('additionalHoursOverride true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = true;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('additionalHoursOverride true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = true;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('additionalHoursOverride false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });
            });

            it('additionalHoursOverride false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.additionalHoursOverride = false;
                event.shift.shiftTasks[0]!.taskType.additionalHoursOverride = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'standardShiftNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 1,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 1,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalNumberOfShifts_numberOfShifts',
                    oldBucketExpectedIncrement: -1,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'totalLeave_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPlanned true -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned true -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned true -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned true -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned true -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPlanned true -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned true -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned true -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = true;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPlanned false -> false && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPlanned false -> false && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned false -> false && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned false -> false && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = false;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });
            });

            it('isPlanned false -> true && isLeave true -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned false -> true && isLeave true -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = true;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: -taskDurationInSeconds,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned false -> true && isLeave false -> false', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = false;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });

            it('isPlanned false -> true && isLeave false -> true', () => {
                event.previousShift.shiftTasks[0]!.taskType.isPlanned = false;
                event.previousShift.shiftTasks[0]!.taskType.isLeave = false;
                event.shift.shiftTasks[0]!.taskType.isPlanned = true;
                event.shift.shiftTasks[0]!.taskType.isLeave = true;

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'plannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement:
                        taskDurationInSeconds + timeIncrementInSeconds - bucketOffsetIncrementInSeconds,
                });

                updateAndAssertShiftAcrossBuckets({
                    currentVersion: event.shift,
                    previousVersion: event.previousShift,
                    fieldName: 'unplannedLeaveDuration_leave',
                    oldBucketExpectedIncrement: 0,
                    newBucketExpectedIncrement: 0,
                });
            });
        });
        describe('handles a shift task', () => {
            it('being added', () => {
                const newTask: ShiftTask = {
                    _id: 'new task',
                    policy: {},
                    taskType: {
                        isPaid: false,
                        isAvailable: false,
                        isAvailableOccupancy: false,
                        isPlanned: false,
                        isMergeAllowed: false,
                        disableHandleTimeClock: false,
                        additionalHoursOverride: false,
                        id: 123,
                        name: 'name',
                        colour: 'colour',
                        isLeave: false,
                        isProductive: false,
                        hideLeaveInAgentViews: false,
                        taskId: 842,
                    },
                    startTime: startTime.toISOString(),
                    endTime: endTime.toISOString(),
                    created: 'created',
                    modified: 'modified',
                };

                // Move to a new bucket
                event.shift.shiftTasks[0]!.startTime = moveToANewBucket();

                event.shift.shiftTasks.push(newTask);

                const expectedResult = calculateCreateDiffForShiftTask({
                    shiftTask: newTask,
                    shift: event.shift,
                    params: filter,
                });

                const result = calculateShiftUpdatedDiff({
                    previousVersion: event.previousShift,
                    currentVersion: event.shift,
                    params: filter,
                });

                expect(result).toContainEqual(expectedResult);
            });

            it('being removed', () => {
                const removedTask = event.shift.shiftTasks.pop()!;

                const expectedResult = calculateDeleteDiffForShiftTask({
                    shiftTask: removedTask,
                    shift: event.shift,
                    params: filter,
                });

                // Move to a new bucket
                event.shift.shiftTasks[0]!.startTime = moveToANewBucket();

                const result = calculateShiftUpdatedDiff({
                    previousVersion: event.previousShift,
                    currentVersion: event.shift,
                    params: filter,
                });

                expect(result).toContainEqual(expectedResult);
            });
        });
    });
});
